/** Klasa AI reprezentujaca Walkera (maszyne kroczaca)
 *  Wersja: Walker_k - 'policyjna' z jednym gunem
 */
public class AIWalker extends AIAttacker
{
    public NewAI cDriver = null;//#\AIWalker;
    
    /** Animacje dla Walkera (te ze zwyklego AI odpadaja)
     */
    static byte A_WALKER_NORMAL                 = 0;
    static byte A_WALKER_NORMAL2STANDBY         = 1;
    static byte A_WALKER_STANDBY                = 2;
    static byte A_WALKER_STANDBY2STANDBY_OPEN   = 3;
    static byte A_WALKER_STANDBY_OPEN           = 4;
    static byte A_WALKER_STANDBY_OPEN2STANDBY   = 5;
    static byte A_WALKER_WAKEUP                 = 6;
    static byte A_WALKER_NORMAL2UP              = 7;
    static byte A_WALKER_UP                     = 8;
    static byte A_WALKER_UP2NORMAL              = 9;
    static byte A_WALKER_NORMAL2DOWN            = 10;
    static byte A_WALKER_DOWN                   = 11;
    static byte A_WALKER_DOWN2NORMAL            = 12;
    static byte A_WALKER_ROTATE_LEFT            = 13;
    static byte A_WALKER_ROTATE_RIGHT           = 14;         
    static byte A_WALKER_WALK                   = 15;
    static byte A_WALKER_WALK_BACK              = 16;
    
    static byte LAST_WALKER_ANIM                = A_WALKER_WALK_BACK;
    
    int WalkerAnims[] = new int[ LAST_WALKER_ANIM + 1 ];
    
    
    /** Tablica z wysokosciami Walkera w zaleznosci od pozycji, w ktorej stoi
     *  (indeksowana przez _WALKER_STATE*)
     */
    float aWalkerHeight[] = { 460, 220, 490, 410, 220 };
    
    /** Dodatkowe oznaczenia stanu walkera
     */
    final static byte _WALKER_STATE_NORMAL          = 0;
    final static byte _WALKER_STATE_STANDBY         = 1;
    final static byte _WALKER_STATE_UP              = 2;
    final static byte _WALKER_STATE_DOWN            = 3;
    final static byte _WALKER_STATE_DEAD            = 4;
    
    /** Aktualny stan walkera
     */
    byte nWalkerState = _WALKER_STATE_NORMAL;

    /** Stan walkera, ktory ma byc ustawiony po skonczeniu animacji przejscia
     */
    byte nNewWalkerState = nWalkerState;
    
    /** Zapamietana wartosc nWalkerState (do UpdateMotion)
     */
    byte nOldWalkerState = -1;

    /** Znacznik, ze Walker zmienia stan
     */
    boolean bChangeWalkerState = false;
    
    /** Znacznik, ze kabina Walkera jest otwarta
     */
    boolean bIsCabinOpen = false;
    
    /** Akcje specyficzne dla walkera
     */
    final static int _ACTION_WALKER_START_WALK_ON_ENEMY             = 150;
    final static int _ACTION_WALKER_STOP_WALK                       = 151;
    final static int _ACTION_WALKER_GET_UP                          = 152;
    final static int _ACTION_WALKER_GET_NORMAL                      = 153;
    final static int _ACTION_WALKER_GET_DOWN                        = 154;
    final static int _ACTION_WALKER_START_FIRING_PRIMARY            = 155;
    final static int _ACTION_WALKER_STOP_FIRING_PRIMARY             = 156;
    final static int _ACTION_WALKER_START_FIRING_SECONDARY          = 157;
    final static int _ACTION_WALKER_STOP_FIRING_SECONDARY           = 158;
    
    /** Nazwa MeshElementu, z ktorego 'patrzy' Walker
     */
    String sWalkerEye = "Barel";

    /** Nazwa MeshElementu odpowiedzialnego za otwieranie kabiny
     */
    String sCabinElemName = "Walker Pane";
    
    /** Identyfikatory MeshElementow Walkera
     */

    /** Miejsce do podpiecia drivera */
    int nSeatID = 0;
    /** ID elementu sWalkerEye */
    int nWalkerEyeID = 0;
    /** Element, ktorego widocznosc sprawdza AI */
    int nWalkerLookAtID = 0;
    /** ID miejsca, z ktorego strzela Walker */
    int nBarelID = 0;
    
    /** Element uzywany do obrotu kabiny w poziomie 
     */
    int nWalkerHeadID = 0;
    /** Element uzywany do obrotu kabiny w pionie
     */
    int nWalkerNeckID = 0;
    /** Element uzywany do obrotu dzialka (przy nacelowywaniu) w pionie
     */
    int nWalkerGunID = 0;
    
    /** Elementy kabiny Walkera - do ustawiania czasow morphingu
     */
    int aHeadElementID[] = null;
    
    
    /** Nazwa dzwieku krokow Walkera
     */
    String sStepSound = "Data/Sounds/Walker/Walker_Walk_k";
    
    String sCabinOpenSound = "Data/Sounds/Walker/Walker_OpenCabin_k";
    String sCabinCloseSound = "Data/Sounds/Walker/Walker_CloseCabin_k";
    String sStandbySound = "Data/Sounds/Walker/Walker_Standby_k";
    String sWakeUpSound = "Data/Sounds/Walker/Walker_WakeUp_k";
    String sNormUpSound = "Data/Sounds/Walker/Walker_NormUp_k";
    String sUpNormSound = "Data/Sounds/Walker/Walker_UpNorm_k";
    String sNormDownSound = "Data/Sounds/Walker/Walker_NormDown_k";
    String sDownNormSound = "Data/Sounds/Walker/Walker_DownNorm_k";

    String sHeadLoopSound = "Data/Sounds/Walker/Walker_HeadLoop_k";
    String sGunRotateLoopSound = "Data/Sounds/Walker/Walker_GunRotateLoop_k";
    
    int nHeadLoopSoundID = 0;
    int nGunRotateLoopSoundID = 0;
    
    
    /////////////////////////////
    // Parametry ataku Walkera //
    /////////////////////////////

    /** Czas, co jaki Walker updateuje pozycje przeciwnika, gdy do niego podchodzi
     */
    float fWkrEnemyPosUpdateTime = 2.5f;
    
    /** Czas, przez jaki Walker idzie podchodzac do przeciwnika
     */
    float fWkrWalkTime = 2.f;
    /** Losowy modyfikator dla fWkrWalkTime
     */
    float fRandomWkrWalkTime = 5.f;

    /** Czas, przez jaki Walker stoi podchodzac do przeciwnika
     */
    float fWkrStandTime = 2.f;
    /** Losowy modyfikator dla fWkrStandTime
     */
    float fRandomWkrStandTime = 5.f;

    /** Czas, przez jaki Walker znajduje sie w stanie Up/Normal/Down gdy stoi w miejscu
     */
    float fWkrTimeInOneState = 1.f;
    /** Losowy modyfikator dla fWkrTimeInOneState
     */
    float fRandomWkrTimeInOneState = 3.f;

    /** Odleglosc, powyzej ktorej Walker probuje podejsc blizej do przeciwnika
     */
    public float fWkrMinMarchDistance = 700.f; //#WALKER

    /** Czas przez jaki Walker strzela idac
     */
    float fWkrPrimaryFireTime  = .5f;
    /** Losowy modyfikator dla fWkrPrimaryFireTime
     */
    float fRandomWkrPrimaryFireTime = 3.f;

    /** Czas, przez jaki Walker nie strzela idac
     */
    float fWkrPrimaryFireDelay = .5f;
    /** Losowy modyfikator dla fWkrPrimaryFireDelay
     */
    float fRandomWkrPrimaryFireDelay = 3.f;

    /** Znacznik, ze bron 'marszowa' nie wymaga nacelowania lufy
     */
    boolean bWkrPrimaryUsesGuidedMissiles = false;
    /** Czas, przez jaki Walker strzela stojac
     */
    float fWkrSecondaryFireTime  = .5f;
    /** Losowy modyfikator dla fWkrSecondaryFireTime
     */
    float fRandomWkrSecondaryFireTime = 3.f;

    /** Czas, przez jaki Walker nie strzela stojac
     */
    float fWkrSecondaryFireDelay = .5f;
    /** Losowy modyfikator dla  fWkrSecondaryFireDelay
     */
    float fRandomWkrSecondaryFireDelay = 3.f;

    /** Znacznik, ze bron 'stacjonarna' nie wymaga nacelowania lufy
     */
    boolean bWkrSecondaryUsesGuidedMissiles = false;

    
    /** Cosinus maksymalnego kata odchylenia prostej do przeciwnika i kierunku lufy, powyzej
     *  ktorego walker nie strzela (jesli bedzie za maly, to mozna bedzie podejsc blisko walkera i nie bedzie 
     *  mogl strzelac)
     */
    float fMaxTargetToBarrelCos = (float)Math.cos(Math.PI/180 * 15.f);
    
    
    
    ////////////////////
    // SMIERC WALKERA //
    ////////////////////

    /** FX odpalany po rozwalenio Walkera
     */
    String sWkrDieBlowFX = "FXBlowWalker";
    /** Dzwiek odpalany po rozwalenio Walkera
     */
    String sWkrDieBlowSound = "Data/Sounds/Explosion/WalkerDieExplosion.wav";
    
    
    
    
    int METHOD_UpdateWalkTargetPos = 0;
    
             
    AIWalker()
    {
        
        
        super();
        
        bIsEmpVulnerable = true;
        //#DEF_BEGIN
        ScreenshotIconPath = "data/textures/icons/icon_walker.png";
        //#DEF_END
        
        ROOT_BONE = "Walker Plevis";
        TORSO_BONE = "Walker Neck";
        
        MeshName = "Data/Meshes/AI/Walker_k.3da";
        sAIWeapon = "WeaponWalker_k";
        //bIsNameSprVisible = true;
        //ActiveElement      = "Bip01 Spine1";
        //bStartInSittingPose = true;
        fTwistHorizMin = -35;
        fTwistHorizMax = 35;
        fTwistVertMin = -40;
        fTwistVertMax = 50;
        fStartHealth = 3000.f;
        aMotionVelocity = new float[] { 0, 330/*swim*/,330, 330, 330, 330 };
        
        //sAnimSeqFile = "Data/Meshes/AI/Walker_k_anim.scr";
        
        fDamageThreshold = 20;
    }
    
    public void OnCreate()
    {
        InitWalker();
        METHOD_UpdateWalkTargetPos = RegisterCallMethod("UpdateWalkTargetPos");
        super.OnCreate();

		SetFlag(EFlags._FXPARTICLE_COLLISION);
        SetBlobMapSize(300);
        UpdatePathMap(true, 7);
        bRunAwayFromDanger = false;
        //ShowCoarseBox(true);
    }
    
    void InitWalker()
    {
        nWalkerEyeID = GetElementID(sWalkerEye);
        nWalkerLookAtID = GetElementID("WW_Fotel");
        nBarelID = GetElementID("Barel");
        nWalkerHeadID = GetElementID("Walker Head");
        nWalkerNeckID = GetElementID("Walker Neck");
        nWalkerGunID = GetElementID("Walker Gun");
        
        InitHeadElementID();
    }

    
    public void OnPostCreate()
    {
        CallMethodIn("InitializeDriver", 0);
    }
    
    
    void InitializeDriver()
    {
        if (cDriver != null)
        {
            nSeatID = GetElementID("WW_Fotel");
            cDriver.Freeze();
            int nDriveAnim = cDriver.GetAnimID("", "drive");
            int nDriveFrame = cDriver.GetFirstFrame(nDriveAnim);
            cDriver.SetFrame(nDriveAnim, nDriveFrame, 0);
            cDriver.SetFlag(EFlags._NO_ANIMATION | 
                            EFlags._NO_ANIMATION_REFRESH);
            AttachChild(cDriver, nSeatID);
            Vector vPos = GetElementPos(nSeatID);
            vPos.fY -= 30;
            Vector vBack = GetElementForwardVector(nSeatID);
            vBack.Mul(10);
            vPos.Sub(vBack);
            cDriver.FromUpForwardPos(GetElementUpVector(nSeatID),
                                     GetElementForwardVector(nSeatID),
                                     vPos);
            cDriver.fDamageThreshold = 1e6f;
            cDriver.ClearFlag(EFlags._CAST_SHADOWS);
            if (cDriver.ActiveWeapon != null)
                cDriver.ActiveWeapon.EnableRendering(false);
        }
    }
    
    
    void InitHeadElementID()
    {
        String aElemName[] = { "Walker Neck", "Walker Head", "Walker Gun" };
        aHeadElementID = new int[aElemName.length];
        for (int i=0; i<aElemName.length; i++)
            aHeadElementID[i] = GetElementID(aElemName[i]);
    }
    
    

    void Initialize()
    {
        super.Initialize();

        bIsInPatrolAttackStateNow = false;
        SetEllipsoidFromWalkerState(_WALKER_STATE_NORMAL);
    }

    
    void InitializeAIWeapon()
    {
        super.InitializeAIWeapon();
        
        /** Podepnij guna tam gdzie trzeba i zniknij go
         */
        AttachChild(ActiveWeapon, "Walker Gun");
        ActiveWeapon.SetPosition(Vector.Zero);
        ActiveWeapon.SetDirLocal(new Vector(1,0,0));
        ActiveWeapon.EnableRendering(false);
        ActiveWeapon.ClearFlag(EFlags._BLOCK_TRACE |
                               EFlags._BLOCK_MOVE);
    }
    
    
    public void ActivateAI(boolean bActivate, int nFreezeCause)    
    {
        super.ActivateAI(bActivate, nFreezeCause);
        if (ActiveWeapon != null)
            ActiveWeapon.EnableRendering(false);
    }

    
    /** Pusta inicjalizacja dla bipeda
     */
    void InitHumanBody()
    {
    }

    /** Pusta inicjalizacja dla gadajacej geby
     */
    void InitTalkingHead()
    {
    }
    
    public void SetTorsoAnimMorphTime(float fMorphTime)
    {
        for (int i=0; i<aHeadElementID.length; i++)
            SetAnimMorphTime(aHeadElementID[i], fMorphTime, false);
    }
    
    public void SetLegsAnimMorphTime(float fMorphTime)
    {
        // TODO: ustawiac czas dla nog
    }

    
    void SetEllipsoidFromWalkerState(byte nWalkerState)
    {
        float fHeight = aWalkerHeight[nWalkerState];
        SetEllipsoid(150.f, fHeight, fHeight * 0.5f);
    }

    
    void TakeAction(int nAction)
    {
        // Zaznacz, ze akcja zaczela sie wykonywac
        bActionInProgress = true;
        switch(nAction)
        {
        case _ACTION_WALKER_START_WALK_ON_ENEMY:
            {
                UpdateWalkTargetPos();
                CallMethodEvery(METHOD_UpdateWalkTargetPos, fWkrEnemyPosUpdateTime);
                TaskFinishedQuick();
            }
            break;
        case _ACTION_WALKER_STOP_WALK:
            {
                DisableCallMethod(METHOD_UpdateWalkTargetPos);
                StopGoTo();
                bTwistCanTurn = true;
                TaskFinishedQuick();
            }
            break;
        case _ACTION_WALKER_GET_UP:
            {
                //LogD("_ACTION_WALKER_GET_UP\n");
                nNewWalkerState = _WALKER_STATE_UP;
                bChangeWalkerState = true;
                bTwistCanTurn = false;
                PerformAnimTask( WalkerAnims[A_WALKER_NORMAL2UP], 1);
                PlaySound(sNormUpSound);
            }
            break;
        case _ACTION_WALKER_GET_NORMAL:
            {
                //LogD("_ACTION_WALKER_GET_NORMAL\n");
                nNewWalkerState = _WALKER_STATE_NORMAL;
                bChangeWalkerState = true;
                bTwistCanTurn = true;
                if (nWalkerState == _WALKER_STATE_UP)
                {
                    PerformAnimTask( WalkerAnims[A_WALKER_UP2NORMAL], 1);
                    PlaySound(sUpNormSound);
                }
                else
                {
                    PerformAnimTask( WalkerAnims[A_WALKER_DOWN2NORMAL], 1);
                    PlaySound(sDownNormSound);
                }
            }
            break;
        case _ACTION_WALKER_GET_DOWN:
            {   
                //LogD("_ACTION_WALKER_GET_DOWN\n");
                nNewWalkerState = _WALKER_STATE_DOWN;
                bChangeWalkerState = true;
                bTwistCanTurn = false;
                PerformAnimTask( WalkerAnims[A_WALKER_NORMAL2DOWN], 1);
                PlaySound(sNormDownSound);
            }
            break;
        case _ACTION_WALKER_START_FIRING_PRIMARY:
            {
                StartFiringPrimary();
                TaskFinishedQuick();
            }
            break;
        case _ACTION_WALKER_STOP_FIRING_PRIMARY:
            {
                StopFiringPrimary();
                TaskFinishedQuick();
            }
            break;
        case _ACTION_WALKER_START_FIRING_SECONDARY:
            {
                StartFiringSecondary();
                TaskFinishedQuick();
            }
            break;
        case _ACTION_WALKER_STOP_FIRING_SECONDARY:
            {
                StopFiringSecondary();
                TaskFinishedQuick();
            }
            break;
        default:
            super.TakeAction(nAction);
        }
    }    
    
    
    void CancelAction(int nAction)
    {
        if (bActionInProgress) // tylko jesli jest co odwolac
        {
            switch(nAction)
            {
            case _ACTION_WALKER_START_WALK_ON_ENEMY:
                {
                    StopGoTo();
                    bTwistCanTurn = true;
                    DisableCallMethod(METHOD_UpdateWalkTargetPos);
                }
                break;
            case _ACTION_WALKER_STOP_WALK:
                {
                }
                break;
            case _ACTION_WALKER_GET_UP:
            case _ACTION_WALKER_GET_NORMAL:
            case _ACTION_WALKER_GET_DOWN:
                {
                    bTwistCanTurn = true;
                    nWalkerState = _WALKER_STATE_NORMAL;
                    StopAnim("");
                    DisableCallMethod( METHOD_AnimationTaskFinished );
                }
                break;
            case _ACTION_WALKER_START_FIRING_PRIMARY:
            case _ACTION_WALKER_STOP_FIRING_PRIMARY:
                {
                    StopFiringPrimary();
                }
                break;
            case _ACTION_WALKER_START_FIRING_SECONDARY:
            case _ACTION_WALKER_STOP_FIRING_SECONDARY:
                {
                    StopFiringSecondary();
                }
                break;

            default:
                super.CancelAction(nAction);
            }
            // Zaznacz, ze akcja juz sie nie wykonuje
            bActionInProgress = false;
        }
    }
    
    void AnimationTaskFinished()
    {
        //LogD(bChangeWalkerState + "\n");
        if (bChangeWalkerState)
        {
            nWalkerState = nNewWalkerState; // zatwierdz zmiane stanu
        }
        super.AnimationTaskFinished();
    }
    
    void UpdateWalkTargetPos()
    {
        Vector vTargetPos = cEnemy.GetPosition();
        if (vTargetPos != null &&
            vTargetPos.Distance(GetPosition()) > fWkrMinMarchDistance)
        {
            bTwistCanTurn = false;
            GoTo(vTargetPos, MotionState.SPEED_WALK, null, _GOTO_DEFAULT);
        }
        else
        {
            bTwistCanTurn = true;
            StopGoTo();
        }
    }
    
    
    
    
    /** Zwracanie punktu do patrzenia na walkerze
     */
    public Vector GetPointToLookAt(int nPoint)
    {
        switch (nPoint)
        {
        case 0:
            return GetElementPos(nWalkerLookAtID);
        default:
            return null;
        }
    }
    
    /** Zwraca kierunek, w ktorym patrzy AI */
    public Vector GetLookDirection()
    {
        return GetElementForwardVector(nWalkerEyeID);
    }
    
    /** Zwraca punkt, z ktorego patrzy AI
     */
    Vector GetLookFromPoint()
    {
        /** Punkt na osi na wysokosci guna (musi byc na osi, zeby Walker nie kiwal glowa 
         *  przy nakierowaniu na przeciwnika)
         */
        Vector vPoint = GetPosition();
        vPoint.fY = GetElementPos(nWalkerEyeID).fY;
        return vPoint;
    }
    
    
    public Vector GetFireOrigin()
    {
        return GetElementPos(nBarelID);
    }
    
    
    void ExecuteFXBarrel(String sFXName)
    {
        ExecuteFX(this, nBarelID, sFXName, Vector.Zero);
    }
 
    
    /** Pokryte z AIAttackera
     */
    void MessageWeapon(int nState)
    {
        if ((nState & Weapon.STATE_SHOT) != 0)
        {
            //PlayAnim( Anims[A_WEA_FIRE], 1 );
            //TODO: dodac odrzut guna
        }
        if ((nState & Weapon.STATE_NO_AMMO) != 0)
        {
            ActiveWeapon.nAmmo = 100000;
        } 
        else if ((nState & (Weapon.STATE_OVERHEAT | Weapon.STATE_ERROR)) != 0)
        { 
            //za czesto strzelamy albo cos nie wyszlo 
            LogD("Shooting to often!\n");
        }
    }
    
    
    // znacznik, ze strzela bronia podstawowa (marszowa)
    boolean bIsFiringPrimary = false;
    // znacznik, ze strzela bronia dodatkowa (stacjonarna)
    boolean bIsFiringSecondary = false;
    
    /** Rozpoczecie strzelania z broni 'marszowej' (strzela, gdy Walker idzie)
     */
    void StartFiringPrimary()
    {
        if (!bIsFiringPrimary)
        {
            AddTracking();
            bIsFiringPrimary = true;
        }
    }
    
    /** Zakonczenie strzelania z broni 'marszowej'
     */
    void StopFiringPrimary()
    {
        if (bIsFiringPrimary)
        {
            RemoveTracking();
            bIsFiringPrimary = false;
        }
        PrimaryWeaponStopFire();
    }
    
    /** Rozpoczecie strzelania z broni 'stacjonarnej' (strzela, gdy Walker stoi)
     */
    void StartFiringSecondary()
    {
        if (!bIsFiringSecondary)
        {
            AddTracking();
            bIsFiringSecondary = true;
        }
    }
    
    /** Zakonczenie strzelania z broni 'stacjonarnej'
     */
    void StopFiringSecondary()
    {
        if (bIsFiringSecondary)
        {
            RemoveTracking();
            bIsFiringSecondary = false;
        }
        SecondaryWeaponStopFire();
    }

    
    //////////////////////////////////////////////////////////////////////////////////////
    // Autonomiczne strzelanie Walkera - strzela co jakis czas i tylko, jesli przeciwnik
    // na linii ognia
    //

    /** Odpalenie strzelania z 'marszowego' guna
     */
    void PrimaryWeaponStartFire()
    {
        ActiveWeapon.StartFire(Weapon.MODE_AUTO);
    }
    
    /** Zatrzymanie strzelania z 'marszowego' guna
     */
    void PrimaryWeaponStopFire()
    {
        ActiveWeapon.StopFire();
    }
    
    /** Pobranie opoznienia dla 'marszowego' guna
     */
    float GetPrimaryWeaponDelay()
    {
        return ActiveWeapon.fDelay;
    }
    
    /** Odpalenie strzelania ze 'stacjonarnego' guna
     */
    void SecondaryWeaponStartFire()
    {
        ActiveWeapon.StartFire(Weapon.MODE_AUTO);
    }
    
    /** Zatrzymanie strzelania ze 'stacjonarnego' guna
     */
    void SecondaryWeaponStopFire()
    {
        ActiveWeapon.StopFire();
    }

    /** Pobranie opoznienia dla 'stacjonarnego' guna
     */
    float GetSecondaryWeaponDelay()
    {
        return ActiveWeapon.fDelay;
    }

    
    
    /** Sprawdzenie, czy cel w przyblizeniu w osi lufy 'marszowej' 
     *  (tylko w poziomie)
     */
    boolean IsTargetDirCloseToPrimaryBarrelDir()
    {
        boolean bRet = false;
        Vector vTargetDir = cEnemy.GetPointToLookAt(nTargetPointIdx);
        if (vTargetDir == null)
        {
            vTargetDir = cEnemy.GetPointToLookAt(0);
        }
        if (vTargetDir != null)
        {
            vTargetDir.Sub(GetFireOrigin());
            vTargetDir.fY = 0;
            vTargetDir.Normalize();

            Vector vBarrelDir = GetElementForwardVector(nWalkerGunID);
            vBarrelDir.fY = 0;
            vBarrelDir.Normalize();
            //CrashLog("DBG: DirClose = " + (vTargetDir.Dot(vBarrelDir) > fMaxTargetToBarrelCos) + "\n");
            bRet = (vTargetDir.Dot(vBarrelDir) > fMaxTargetToBarrelCos);
        }
        return bRet;
    }

    
    /** Sprawdzenie, czy cel w przyblizeniu w osi lufy 'stacjonarnej'
     */
    boolean IsTargetDirCloseToSecondaryBarrelDir()
    {
        return IsTargetDirCloseToPrimaryBarrelDir();
    }

    
    float fLastPrimFireStartTime = 0,
          fLastPrimFireEndTime = 0, 
          fLastPrimFireTempStopTime = 0,
          fPrimaryFireTime = 0,
          fPrimaryFireDelay = 0;
    
    boolean bIsPrimaryFiringNow = false;

    void UpdateFiringPrimary()
    {
        //CrashLog("DBG: UpdateFiringPrimary\n");
        float fCurTime = GetTime();
        if (bIsPrimaryFiringNow)
        {
            if (fCurTime - fLastPrimFireStartTime > fPrimaryFireTime)
            {
                PrimaryWeaponStopFire();
                bIsPrimaryFiringNow = false;
                fLastPrimFireEndTime = fCurTime;
                fPrimaryFireTime = fWkrPrimaryFireTime + RandomFloat(fRandomWkrPrimaryFireTime);
            }
            else
            {
                if (!bWkrPrimaryUsesGuidedMissiles)
                {
                    boolean bDirOK = IsTargetDirCloseToPrimaryBarrelDir();
                    if (bDirOK)
                    {
                        if (fLastPrimFireTempStopTime > 0 &&
                            fCurTime - fLastPrimFireTempStopTime > GetPrimaryWeaponDelay())
                        {
                            PrimaryWeaponStartFire();
                            fLastPrimFireTempStopTime = 0; // zaznacz, ze nie jest zatrzymany
                        }
                    }
                    else
                    {
                        PrimaryWeaponStopFire();
                        fLastPrimFireTempStopTime = fCurTime;
                    }
                }
            }
        }
        else
        {
            if (fCurTime - fLastPrimFireEndTime > fPrimaryFireDelay &&
                (bWkrPrimaryUsesGuidedMissiles || IsTargetDirCloseToPrimaryBarrelDir()) && 
                IsEnemyInWeaponRange())
            {
                PrimaryWeaponStartFire();
                bIsPrimaryFiringNow = true;
                fLastPrimFireTempStopTime = 0; // zaznacz, ze nie jest zatrzymane
                fLastPrimFireStartTime = fCurTime;
                fPrimaryFireDelay = fWkrPrimaryFireDelay + RandomFloat(fRandomWkrPrimaryFireDelay);
            }
        }
    }


    
    float fLastSecFireStartTime = 0,
          fLastSecFireEndTime = 0, 
          fLastSecFireTempStopTime = 0,
          fSecondaryFireTime = 0,
          fSecondaryFireDelay = 0;
    
    boolean bIsSecondaryFiringNow = false;


    void UpdateFiringSecondary()
    {
        //CrashLog("DBG: UpdateFiringSecondary\n");

        float fCurTime = GetTime();
        if (bIsSecondaryFiringNow)
        {
            if (fCurTime - fLastSecFireStartTime > fSecondaryFireTime)
            {
                SecondaryWeaponStopFire();
                bIsSecondaryFiringNow = false;
                fLastSecFireEndTime = fCurTime;
                fSecondaryFireTime = fWkrSecondaryFireTime + RandomFloat(fRandomWkrSecondaryFireTime);
            }
            else
            {
                if (!bWkrSecondaryUsesGuidedMissiles)
                {
                    boolean bDirOK = IsTargetDirCloseToSecondaryBarrelDir();
                    if (bDirOK)
                    {
                        if (fLastSecFireTempStopTime > 0 &&
                            fCurTime - fLastSecFireTempStopTime > GetSecondaryWeaponDelay())
                        {
                            SecondaryWeaponStartFire();
                            fLastSecFireTempStopTime = 0; // zaznacz, ze nie jest zatrzymany
                        }
                    }
                    else
                    {
                        SecondaryWeaponStopFire();
                        fLastSecFireTempStopTime = fCurTime;
                    }
                }
            }
        }
        else
        {
            if (fCurTime - fLastSecFireEndTime > fSecondaryFireDelay &&
                (bWkrSecondaryUsesGuidedMissiles || IsTargetDirCloseToSecondaryBarrelDir()) && 
                IsEnemyInWeaponRange())
            {
                SecondaryWeaponStartFire();
                bIsSecondaryFiringNow = true;
                fLastSecFireTempStopTime = 0; // zaznacz, ze nie jest zatrzymane
                fLastSecFireStartTime = fCurTime;
                fSecondaryFireDelay = fWkrSecondaryFireDelay + RandomFloat(fRandomWkrSecondaryFireDelay);
            }
        }
    }
    
    /** Pokryta z bazowej i rozszerzona o update strzelania Walkera
     */
    void UpdateAllTrackings()
    {
        super.UpdateAllTrackings();

        if (bIsFiringPrimary)
            UpdateFiringPrimary();
        if (bIsFiringSecondary)
            UpdateFiringSecondary();
    }
    
    
    /** Zestaw animacji dla Walkera
     */
    void DefineAnimationSet()
	{
        SetAnimFreq("", 36.0f);
        SetAnimMorphTime(fNormalAnimMorphTime);
        SetTorsoAnimMorphTime(fNormalAnimMorphTime);

        WalkerAnims[A_WALKER_NORMAL] = GetAnimID(ROOT_BONE, "normal");
        WalkerAnims[A_WALKER_NORMAL2STANDBY] = GetAnimID(ROOT_BONE, "normal-standby");
        WalkerAnims[A_WALKER_STANDBY] = GetAnimID(ROOT_BONE, "Standby");
        WalkerAnims[A_WALKER_STANDBY2STANDBY_OPEN] = GetAnimID(sCabinElemName, "standby-standby_open");
        WalkerAnims[A_WALKER_STANDBY_OPEN] = GetAnimID(ROOT_BONE, "standby_open");
        WalkerAnims[A_WALKER_STANDBY_OPEN2STANDBY] = GetAnimID(sCabinElemName, "Standby_open-standby");
        WalkerAnims[A_WALKER_WAKEUP] = GetAnimID(ROOT_BONE, "Wakeup");
        WalkerAnims[A_WALKER_NORMAL2UP] = GetAnimID(ROOT_BONE, "normal-up");
        WalkerAnims[A_WALKER_UP] = GetAnimID(ROOT_BONE, "up");
        WalkerAnims[A_WALKER_UP2NORMAL] = GetAnimID(ROOT_BONE, "up-normal");
        WalkerAnims[A_WALKER_NORMAL2DOWN] = GetAnimID(ROOT_BONE, "normal-down");
        WalkerAnims[A_WALKER_DOWN] = GetAnimID(ROOT_BONE, "down");
        WalkerAnims[A_WALKER_DOWN2NORMAL] = GetAnimID(ROOT_BONE, "down-normal");
        WalkerAnims[A_WALKER_ROTATE_LEFT] = GetAnimID(ROOT_BONE, "rotate_left");
        WalkerAnims[A_WALKER_ROTATE_RIGHT] = GetAnimID(ROOT_BONE, "rotate_right");
        WalkerAnims[A_WALKER_WALK] = GetAnimID(ROOT_BONE, "walk");
        WalkerAnims[A_WALKER_WALK_BACK] = GetAnimID(ROOT_BONE, "walk_back");

        nUnstuckAnimID = WalkerAnims[A_WALKER_NORMAL];
        
        SetElementAnimType(nWalkerNeckID, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        SetElementAnimType(nWalkerHeadID, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        SetElementAnimType(nWalkerGunID, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false );
        
        for (int i=0; i<=A_WALKER_WALK_BACK; i++)
        {
            if (WalkerAnims[i] == -1)
            {
                CrashLog("ERROR: WalkerAnims[i] == -1, i=" + i + "\n");
            }
            //LogD(i + " " + GetAnimLength(WalkerAnims[i]) + "\n");
        }
	}

    
    public void DisableVitalActivities()
    {
        super.DisableVitalActivities();
        DisableCallMethod( METHOD_UpdateWalkTargetPos );
        if (bIsFiringPrimary)
            StopFiringPrimary();
        if (bIsFiringSecondary)
            StopFiringSecondary();
    }
    
    
    /** 
     */
    void UpdateMotion()
    {
        //LogD("\n");
        //    Thread.dumpStack();
        if (bDead) return; //zeby martwy nie wstawal

        if ( cMotion.IsEqualMoving(cOldMotion) &&
             nWalkerState == nOldWalkerState )
            return;
        
        if ( cMotion.IsMoving() )
        {
            int nAnimToPlay = -1;
            if (cMotion.IsMovingLinear())
            {
                switch (cMotion.iMoveZ)
                {
                case -1 : //backward
                    nAnimToPlay = WalkerAnims[A_WALKER_WALK_BACK];
                case 0 : //strafe
                    LogD("No strafe animation defined!\n");
                    break;
                case 1 : //forward
                    nAnimToPlay = WalkerAnims[A_WALKER_WALK];
                }
            } 
            else if (cMotion.iTurn != 0) //obroty
            {
                if (cMotion.iTurn > 0)
                    nAnimToPlay = WalkerAnims[A_WALKER_ROTATE_LEFT];
                else
                    nAnimToPlay = WalkerAnims[A_WALKER_ROTATE_RIGHT];
            }

            if (nAnimToPlay != -1)
            {
                //if (!cOldMotion.IsMovingLinear())
                if (cOldMotion.iMoveZ != cMotion.iMoveZ ||
                    cOldMotion.iTurn != cMotion.iTurn)
                {
                    PlayAnim( nAnimToPlay, -1);
                    StepDone();
                    CallMethodEvery( METHOD_StepDone, GetAnimLength(nAnimToPlay) / 2.f);
                }
            }
        }
        else
        {
            PlayStandAnim();
            if (cOldMotion.IsMoving())
                DisableCallMethod( METHOD_StepDone );
        }
        
        cOldMotion = new MotionState(cMotion);
        nOldWalkerState = nWalkerState;
    }

    
    void StepDone()
    {
        PlaySound(sStepSound);  
    }
    
    /** Puste PlaceGun - nie ma po co tego robic dla Walkera
     */
    void PlaceGun()
    {
    }
    
    /** Pobranie minimalnego kata o jaki mozna skrecic Walkera w poziomie (w prawo) 
     *  @param nTwistType rodzaj deformacji (_TWIST_TYPE_*)
     *  @return minimalny kat skretu [deg]
     */
    float GetTwistHorizMin(int nTwistType)
    {
        return fTwistHorizMin;
    }

    /** Pobranie maksymalnego kata o jaki mozna skrecic Walkera w poziomie (w lewo) 
     *  @param nTwistType rodzaj deformacji (_TWIST_TYPE_*)
     *  @return minimalny kat skretu [deg]
     */
    float GetTwistHorizMax(int nTwistType)
    {
        return fTwistHorizMax;
    }

    /** Pobranie minimalnego kata o jaki mozna skrecic Walkera w pionie (w dol) 
     *  @param nTwistType rodzaj deformacji (_TWIST_TYPE_*)
     *  @return minimalny kat skretu [deg]
     */
    float GetTwistVertMin(int nTwistType)
    {
        return fTwistVertMin;
    }

    /** Pobranie maksymalnego kata o jaki mozna skrecic Walkera w pionie (w gore) 
     *  @param nTwistType rodzaj deformacji (_TWIST_TYPE_*)
     *  @return minimalny kat skretu [deg]
     */
    float GetTwistVertMax(int nTwistType)
    {
        return fTwistVertMax;
    }

    //TODO: dodac uzaleznienie GetTwist... od stanu walkera


    // Wspolczynniki od ograniczen twista
    /** Kat o jaki ma sie podniesc glowa, zeby po skrecie na bok nie zahaczac o nogi
     */
    float fLegSafeAngleVert = -15.f;
    /** Maksymalny kat o jaki moze sie podniesc w gore gun
     */
    float fGunSafeAngleVert = -10.f;

    float fLastAngleH = 0,
          fLastAngleV = 0;
    
    float fMinSoundStartAngle = 2.f;
    /** Pokryte z bazowego - tylko obrot kabiny i ew. guna
     */
    void TwistBody(float fHorizAngle, float fVertAngle, int nTwistType)
    {   
        //CrashLog("DBG: fHorizAngle = " + fHorizAngle + " fVertAngle = " + fVertAngle + "\n");
        switch (nTwistType)
        {
        case _TWIST_TYPE_NONE:
            {
                RotateElementWithAnim(nWalkerNeckID, vAlongXAxis, 0);
                RotateElementWithAnim(nWalkerHeadID, vAlongXAxis, 0);
                RotateElementWithAnim(nWalkerGunID, vAlongXAxis, 0);
                if (Math.abs(fLastAngleH) > fMinSoundStartAngle)
                    PlayHeadRotateSound(GetAnimMorphTime());
                else 
                    StopHeadRotateSound();
                if (Math.abs(fLastAngleV) > fMinSoundStartAngle)
                    PlayGunRotateSound(GetAnimMorphTime());
                else
                    StopGunRotateSound();
                fLastAngleH = fLastAngleV = 0;
            }
            break;
        case _TWIST_TYPE_HEAD_ONLY:
        case _TWIST_TYPE_HEAD_USE_TORSO:
        case _TWIST_TYPE_TORSO:
            {
                RotateElementWithAnim(nWalkerHeadID, vAlongYAxis, fHorizAngle);
                if (Math.abs(fHorizAngle) < 15.f)
                {
                    if (fVertAngle < fGunSafeAngleVert)
                    {
                        RotateElementWithAnim(nWalkerNeckID, vAlongXAxis, fVertAngle - fGunSafeAngleVert);
                        RotateElementWithAnim(nWalkerGunID, vAlongXAxis, fGunSafeAngleVert);
                    }
                    else
                    {
                        RotateElementWithAnim(nWalkerNeckID, vAlongXAxis, 0.3f * fVertAngle);
                        RotateElementWithAnim(nWalkerGunID, vAlongXAxis, 0.7f * fVertAngle);
                    }
                }
                else
                {
                    // kabina na boku - jesli za nisko - podnies ja w gore, zeby 
                    // nie zahaczala o nogi
                    if (fVertAngle > fLegSafeAngleVert) 
                    {
                        RotateElementWithAnim(nWalkerNeckID, vAlongXAxis, fLegSafeAngleVert);
                        RotateElementWithAnim(nWalkerGunID, vAlongXAxis, fVertAngle - fLegSafeAngleVert);
                    }
                    else
                    {
                        if (fVertAngle < fGunSafeAngleVert)
                        {
                            RotateElementWithAnim(nWalkerNeckID, vAlongXAxis, fVertAngle - fGunSafeAngleVert);
                            RotateElementWithAnim(nWalkerGunID, vAlongXAxis, fGunSafeAngleVert);
                        }
                        else
                        {
                            RotateElementWithAnim(nWalkerNeckID, vAlongXAxis, 0.3f * fVertAngle);
                            RotateElementWithAnim(nWalkerGunID, vAlongXAxis, 0.7f * fVertAngle);
                        }
                    }
                }
                if (Math.abs(fLastAngleH - fHorizAngle) > fMinSoundStartAngle)
                    PlayHeadRotateSound(GetAnimMorphTime());
                else
                    StopHeadRotateSound();
                if (Math.abs(fLastAngleV - fVertAngle) > fMinSoundStartAngle)
                    PlayGunRotateSound(GetAnimMorphTime());
                else
                    StopGunRotateSound();

                fLastAngleV = fVertAngle;
                fLastAngleH = fHorizAngle;
                break;
            }
        default:
            LogD("Incorrent twist type value!\n");
            break;
        }
        nLastTwistType = nTwistType;
    }
    
    
    void PlayHeadRotateSound(float fTime)
    {
        if (nHeadLoopSoundID == 0)
            nHeadLoopSoundID = AttachSound(sHeadLoopSound);
        else
            PlaySound(nHeadLoopSoundID);
        CallMethodOnceIn("StopHeadRotateSound", fTime);
    }
    
    void StopHeadRotateSound()
    {
        StopSound(nHeadLoopSoundID);   
    }
    
    void PlayGunRotateSound(float fTime)
    {
        if (nGunRotateLoopSoundID == 0)
            nGunRotateLoopSoundID = AttachSound(sHeadLoopSound);
        else
            PlaySound(nGunRotateLoopSoundID);
        CallMethodOnceIn("StopGunRotateSound", fTime);
    }
    
    void StopGunRotateSound()
    {
        StopSound(nGunRotateLoopSoundID);   
    }
    
    /** Walker ma w dupie jak stuka w niego samochod
     */
    public void OnVehicleCollision(float fDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed)
    {
    }
    
/*    public void OnTakeDamage(float fDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nDamageType)
    {
        // TODO: Wlasna obsluga, bo obrywa inaczej niz czlowiek
        if (Assassin instanceof AIAttacker)
            ((AIAttacker)Assassin).NotifyPlayerHit(this); //powiedz mu, zeby nie strzelal do swoich
        else // AI cheat: friendly fire = false
        {
            // jesli oberwal zywy i jeszcze nie atakuje to do dziela
            if (fHealth >0 && 
                Assassin instanceof ChromeActor)
            {
                ChromeActor cActor = (ChromeActor)Assassin;
                if (cActor != cEnemy &&
                    GetTeamRelation(cActor) == _TEAM_RELATION_HOSTILE)
                {
                    nAttackCause = _ATTACK_CAUSE_HIT;
                    nTargetPointIdx = 0;
                    SeekAndDestroy(cActor);
                }
            }
            boolean wasDead = bDead;
            // TODO: mnoznik damedza
            Damage(fDamage);

            if( !bDead)
            {
                TryToScream(_SCREAMTYPE_PAIN); // zglos ostrzal
            }
            else
            {
                //liczymy frag jesli AI zmienil stan istnienia
                if (wasDead != bDead)
                {
                    TryToScream(_SCREAMTYPE_DEATH);
                    if (Assassin instanceof Pawn)
                        ((Pawn)Assassin).YouKilledMe(this);
                }
            }
        }
    }
*/    
	// poprawione OnTakeDamage //////////////////////////////////////////////////////////////
    public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
    {
        // TODO: Wlasna obsluga, bo obrywa inaczej niz czlowiek
        if (Assassin instanceof AIAttacker)
            ((AIAttacker)Assassin).NotifyPlayerHit(this); //powiedz mu, zeby nie strzelal do swoich
        else // AI cheat: friendly fire = false
        {
            // jesli oberwal zywy i jeszcze nie atakuje to do dziela
            if (fHealth >0 && 
                Assassin instanceof ChromeActor)
            {
                ChromeActor cActor = (ChromeActor)Assassin;
                if (cActor != cEnemy &&
                    GetTeamRelation(cActor) == _TEAM_RELATION_HOSTILE)
                {
                    nAttackCause = _ATTACK_CAUSE_HIT;
                    nTargetPointIdx = 0;
                    SeekAndDestroy(cActor);
                }
            }
            boolean wasDead = bDead;
            // TODO: mnoznik damedza
            Damage(fDamage, nDamageType);

            if( !bDead)
            {
                TryToScream(_SCREAMTYPE_PAIN); // zglos ostrzal
            }
            else
            {
                //liczymy frag jesli AI zmienil stan istnienia
                if (wasDead != bDead)
                {
                    TryToScream(_SCREAMTYPE_DEATH);
                    if (Assassin instanceof Pawn)
                        ((Pawn)Assassin).YouKilledMe(this);
                }
            }
        }
    }
    
    /** Otwarcie kabiny
     */
    void OpenCabin()
    {
        PlayAnim( WalkerAnims[A_WALKER_STANDBY2STANDBY_OPEN], 1);
        PlaySound(sCabinOpenSound);
        bIsCabinOpen = true;
    }
    
    /** Zamkniecei kabiny
     */
    void CloseCabin()
    {
        PlayAnim( WalkerAnims[A_WALKER_STANDBY_OPEN2STANDBY], 1);
        PlaySound(sCabinCloseSound);
        bIsCabinOpen = false;
    }
    
    
    /** Pokryty z AIAttackera
     */
    public void SeekAndDestroy(Pawn cEnemy)
    {
        // zawsze atakuj, jesli mozna
        if (cEnemy != null)
            Attack(cEnemy);
        else
            ChooseIdleState();
    }
    
    //////////////////////////////////////////
    // STANY WALKERA - POKRYTE Z BAZOWYCH   //
    //////////////////////////////////////////
    
    public class Idle extends AIAttacker.Idle
    {
        public void OnEnter()
        {
            super.OnEnter();
            cMotion.iSpeed=MotionState.SPEED_STILL;
            UpdateMotion();
        }
        
        public void OnLeave()
        {
            super.OnLeave();
            SetAnimMorphTime(fNormalAnimMorphTime); //przywroc poprzednia szybkosc
        }
        
        public void OnMakeDecision()
        {
            //NewTask(_ACTION_THINK);
        }
        
        public void OnMakeDecisionEntering()
        {
            //NewTask(_ACTION_THINK);
        }
        
        // pokryta z bazowej
        public void PlayStandAnim()
        {
            PlayAnim(WalkerAnims[A_WALKER_NORMAL], -1);
        }
        
        public void OnAnimEnd(int iAnimID)
        {
        }
    }
    
    
    public class IdleSitting extends Idle
    {
        float fOldAnimMorphTime = 0;

        {
            bNeedTimeToLeave = true; // musi wstac ze standby, co wymaga czasu
        }
        
        public void OnEnter()
        {
            super.OnEnter();
            // Ustaw w pozycji siedzacej, zeby nie morphowal ze stania
            fOldAnimMorphTime = GetAnimMorphTime();
            SetAnimMorphTime(0);
            PlayStandAnim(); // Odpal animacje siedzenia
            CallMethodIn("OnEnterDelayed", 0.0f);
        }

        
        public void OnLeave()
        {
            GetUp();
        }
        
        void OnEnterDelayed()
        {
            SetAnimMorphTime(fOldAnimMorphTime);
            SetEllipsoidFromWalkerState(_WALKER_STATE_STANDBY);
        }

        
        public void OnMakeDecision()
        {
            //NewTask(_ACTION_THINK);
        }
        
        public void OnMakeDecisionEntering()
        {
            //NewTask(_ACTION_THINK);
        }
        
        
        /** Wstawanie z trybu stand-by
         */
        public void GetUp()
        {
            if (bIsSittingBull)
            {
                PlayAnim(WalkerAnims[A_WALKER_WAKEUP], 1);
                PlaySound(sWakeUpSound);
                CallMethodOnceIn("GetUpDone", GetAnimLength(WalkerAnims[A_WALKER_WAKEUP]));
                bIsSittingBull = false;
            }
        }
        
        public void GetUpDone()
        {
            SetEllipsoidFromWalkerState(_WALKER_STATE_NORMAL);
            super.OnLeave();
        }
        
        // pokryta z bazowej
        public void PlayStandAnim()
        {
            if (bIsSittingBull)
                PlayAnim(WalkerAnims[A_WALKER_STANDBY], -1);
            else
                super.PlayStandAnim();
        }
        
        public void OnAnimEnd(int iAnimID)
        {
        }
    }
    
    
    public class Patrol extends AIAttacker.Patrol
    {
        public void PlayStandAnim()
        {
            PlayAnim(WalkerAnims[A_WALKER_NORMAL], -1);
        }
        
        public void OnAnimEnd(int iAnimID)
        {
        }
    }

    
    boolean bIsInPatrolAttackStateNow = false;
    
    public class PatrolAttack extends AIWalker.Attack
    {
        public void OnEnter()
        {
            bIsInPatrolAttackStateNow = true;
            super.OnEnter();
        }
        
        public void OnLeave()
        {
            bIsInPatrolAttackStateNow = false;
            super.OnLeave();
        }
    }
    
    
    public class Attack extends AIAttacker.Attack
    {
        float fWalkStartTime = 0,
              fWalkEndTime = 0, 
              fLastStateChangeTime = 0, 
              fWalkTime = 0,
              fStandTime = 0,
              fTimeInOneState = 0;
               
        public void OnEnter()
        {
            super.OnEnter();
            SetWalkTrackedObject(null);
        }
        
        void OnMakeDecision()
        {
            // liczymy odleglosc do wroga
            float fDistance = GetDistanceToEnemy();
            
            float fCurTime = GetTime();
            if (cMotion.IsMoving()) // jesli Walker idzie albo sie obraca
            {
                //CrashLog("DBG: Idzie\n");
                // Jesli idzie odpowiednio dlugo - zatrzymaj
                if (fCurTime - fWalkStartTime > fWalkTime)
                {
                    NewTask(_ACTION_WALKER_STOP_WALK);
                    NewTask(_ACTION_WALKER_STOP_FIRING_PRIMARY);
                    NewTask(_ACTION_WALKER_START_FIRING_SECONDARY);
                    fWalkEndTime = fCurTime;
                    fWalkTime = fWkrWalkTime + RandomFloat(fRandomWkrWalkTime);
                }
                else
                    NewTask(_ACTION_THINK);
            }
            else // jesli walker stoi w miejscu
            {
                //CrashLog("DBG: Stoi w miejscu\n");
                if (nWalkerState == _WALKER_STATE_NORMAL &&
                    fCurTime - fWalkEndTime > fStandTime &&
                    fDistance > fWkrMinMarchDistance)
                {
                    // dodanie patrolowania - zmodyfikowany kod z AIPatroler
                    if(bIsInPatrolAttackStateNow && !bEndOfPathReached)
                    {
                        AIPatrolerPathPoint cCurrentNode = (AIPatrolerPathPoint) aPath.GetPoint(nCurrentNode);
                        
                        if (cCurrentNode.fStopTimeInterval != 0)
                            NewTask(_ACTION_PPOINT_WAIT_FOR);
                        
                        if (cCurrentNode.iSpeed != cMotion.iSpeed)
                        {
                            // zmieniamy sposob poruszania sie
                            cMotion.iSpeed = cCurrentNode.iSpeed;
                            NewTask(_ACTION_CHANGE_MOTION);
                        }

                        NewTask(_ACTION_GO_NEXT_PPOINT);
                    }
                    else
                        NewTask(_ACTION_WALKER_START_WALK_ON_ENEMY);
                    NewTask(_ACTION_WALKER_START_FIRING_PRIMARY);
                    NewTask(_ACTION_WALKER_STOP_FIRING_SECONDARY);
                    fWalkStartTime = fCurTime;
                    fStandTime = fWkrStandTime + RandomFloat(fRandomWkrStandTime);
                }
                else 
                {
                    // Jesli sie zatrzymal przez np. przez update, trzeba posprzatac po strzelaniu
                    if (!bIsFiringSecondary)
                        NewTask(_ACTION_WALKER_START_FIRING_SECONDARY);
                    if (bIsFiringPrimary)
                        NewTask(_ACTION_WALKER_STOP_FIRING_PRIMARY);
                    
                    
                    //LogD("nie zamierza ruszyc\n");
                    if (fCurTime - fLastStateChangeTime > fTimeInOneState)
                    {
                        switch (nWalkerState)
                        {
                        case _WALKER_STATE_DOWN:
                        case _WALKER_STATE_UP:
                            NewTask(_ACTION_WALKER_GET_NORMAL);
                            break;
                        case _WALKER_STATE_NORMAL:
                            {
                                if (CanHappen(50))
                                    NewTask(_ACTION_WALKER_GET_UP);
                                else
                                    NewTask(_ACTION_WALKER_GET_DOWN);
                            }
                            break;
                        default: 
                            {
                                LogD("Incorrect walker state at this moment (" + nWalkerState + ")\n!");
                                NewTask(_ACTION_THINK);
                            }
                        }
                        fLastStateChangeTime = fCurTime;
                        fTimeInOneState = fWkrTimeInOneState + RandomFloat(fRandomWkrTimeInOneState);
                    }
                    else
                        NewTask(_ACTION_THINK);
                }
            }   
        }


        /** Podjecie decyzji poczatkowej co robic, zaraz po wejsciu do stanu Atackowania
         */
        void OnMakeDecisionEntering()
        {
            NewTask(_ACTION_TRACK_ENEMY_BODY);
            //OnMakeDecision();
        }

        public void PlayStandAnim()
        {
            switch (nWalkerState)
            {
            case _WALKER_STATE_NORMAL:
                PlayAnim(WalkerAnims[A_WALKER_NORMAL], -1);
                break;
            case _WALKER_STATE_DOWN:
                PlayAnim(WalkerAnims[A_WALKER_DOWN], -1);
                break;
            case _WALKER_STATE_UP:
                PlayAnim(WalkerAnims[A_WALKER_UP], -1);
                break;
            }
        }
        
        public void OnAnimEnd(int iAnimID)
        {
        }
    }


    public class SearchDynamic extends AIAttacker.SearchDynamic
    {
        public void PlayStandAnim()
        {
            PlayAnim(WalkerAnims[A_WALKER_NORMAL], -1);
        }
        
        public void OnAnimEnd(int iAnimID)
        {
        }
    }
    
    /** Nigdy nie jest aktywny
     */
    public boolean IsActive(ControlObject Activator)
    {
        return false;
    }

    public void OnPostUpdate()
    {
        if (bDead)//HACK do filmow!!!! && !bReallyDead)  // znikniecie glowy po smierci
        {
            FromUpForwardPosElement(nWalkerHeadID, Vector.Up,
                                    Vector.Forward, new Vector(0,0,-100000));
        }
        super.OnPostUpdate();
    }
    
    
    public class Die extends AIBaseState
    {
        public void OnEnter()
        {
            super.OnEnter();
            DisableVitalActivities();

            if( sWkrDieBlowFX != null)
            {
                ExecuteFX(sWkrDieBlowFX, GetElementPos(nWalkerHeadID), Vector.Up);
                ExecuteParticleSystem(sWkrDieBlowFX, GetElementPos(nWalkerHeadID), Vector.Up);
            }

            if( sWkrDieBlowSound != null)
                PlaySound(sWkrDieBlowSound);
            
            if (cDriver != null)
            {
                cDriver.DeactivateOnModuleAlways();
            }

            // TODO: wlaczamy animacje smierci
            PlayAnim( WalkerAnims[A_WALKER_NORMAL2STANDBY] , 1);
            
            //SetFlag(EFlags._NO_ANIMATION_REFRESH);
            CallMethodOnceIn( "DeathAnimEnd", 
                              GetAnimLength(WalkerAnims[A_WALKER_NORMAL2STANDBY]) + 0.5f);
        }

        void DeathAnimEnd()
        {
            // ustawienie flagi, zeby zwloki mozna bylo obszukac
            bReallyDead = true;
            

            SetEllipsoidFromWalkerState(_WALKER_STATE_DEAD);

            // wylaczamy kolizje z cialem lezacym na ziemi
            ClearFlag( EFlags._BLOCK_MOVE );

        }
        
		
		public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nDamageType)
		{
            // jemu juz nic nie zaszkodzi
		}

        
        public void PlayStandAnim()
        {
        }
        
        public void OnAnimEnd(int iAnimID)
        {
        }
    } 
    
    public class TestState extends AIBaseState
    {
        public void OnEnter()
        {
            nReadinessToSearch = 0;
            //fTwistHorizMin = -360;
            //fTwistHorizMax = 360;
            cEnemy = ((ChromeModuleSingle)GetModule()).MainPlayer;
            //EnableTracingEnemyBody();
            //PlayAnim(WalkerAnims[A_WALKER_WALK], -1);
        }
        
        public void OnReact()
        {
        }
        
        public void OnHearNoise(Actor Noisy, float fNoiseLevel, int nNoiseType)
        {
        }

        float fAngle1 = 0.f,
              fAngle2 = 0.f;
        
        void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
        {
            /*
            if ((nKeyCode==EKey._G) && (bKeyState))
            {
                fAngle1+=5f;
            }
            if ((nKeyCode==EKey._H) && (bKeyState))
            {
                fAngle1-=5f;
            }
            if ((nKeyCode==EKey._K) && (bKeyState))
            {
                fAngle2+=5f;
            }
            if ((nKeyCode==EKey._L) && (bKeyState))
            {
                fAngle2-=5f;
            }
            if ((nKeyCode==EKey._J) && (bKeyState))
            {
                SetAnimMorphTime(0.001f);
                TwistBody(fAngle1, fAngle2, _TWIST_TYPE_NONE);
                fAngle1 = 0;
                fAngle2 = 0;
            }
        
            /////////////////////////////////////
            // Animacje
            if ((nKeyCode==EKey._NUMPAD1) && (bKeyState))
            {
                cMotion.iSpeed = MotionState.SPEED_WALK;
                cMotion.iMoveZ = 1;
                UpdateMotion();
            }
    
            if ((nKeyCode==EKey._NUMPAD2) && (bKeyState))
            {
                cMotion.iSpeed = MotionState.SPEED_WALK;
                cMotion.iMoveZ = -1;
                UpdateMotion();
            }

            if ((nKeyCode==EKey._NUMPAD3) && (bKeyState))
            {
                cMotion.iSpeed = MotionState.SPEED_WALK;
                cMotion.iMoveZ = 0;
                cMotion.iTurn = 1;
                UpdateMotion();
            }

            if ((nKeyCode==EKey._NUMPAD4) && (bKeyState))
            {
                cMotion.iSpeed = MotionState.SPEED_WALK;
                cMotion.iMoveZ = 0;
                cMotion.iTurn = -1;
                UpdateMotion();
            }

            if ((nKeyCode==EKey._NUMPAD5) && (bKeyState))
            {
                cMotion.iTurn = cMotion.iMoveZ = 0;
                UpdateMotion();
            }    
    
            if ((nKeyCode==EKey._NUMPAD6) && (bKeyState))
            {
                if (bIsCabinOpen)
                    CloseCabin();
                else
                    OpenCabin();
            }    
    
            if ((nKeyCode==EKey._C) && (bKeyState))
            {
                if (nWalkerState == _WALKER_STATE_STANDBY)
                {
                    PlayAnim( WalkerAnims[ A_WALKER_WAKEUP], 1);
                    nWalkerState = _WALKER_STATE_NORMAL;
                    PlaySound(sWakeUpSound);
                }
                else
                {
                    PlayAnim( WalkerAnims[ A_WALKER_NORMAL2STANDBY], 1);
                    nWalkerState = _WALKER_STATE_STANDBY;
                    PlaySound(sStandbySound);
                }
            }    
    
            if ((nKeyCode==EKey._V) && (bKeyState))
            {
                NewTaskExclusive(_ACTION_WALKER_GET_DOWN);
            }    
    
            if ((nKeyCode==EKey._B) && (bKeyState))
            {
                NewTaskExclusive(_ACTION_WALKER_GET_NORMAL);
            }    
    
            if ((nKeyCode==EKey._N) && (bKeyState))
            {
                NewTaskExclusive(_ACTION_WALKER_GET_UP);
            }    
    
    
            if (bKeyState)
            {
                SetAnimMorphTime(0.5f);
                TwistBody(fAngle1, fAngle2, _TWIST_TYPE_TORSO);
                LogD(fAngle1 + ", " + fAngle2 + "\n");
            }
            */
        }

        public void PlayStandAnim()
        {
            switch (nWalkerState)
            {
            case _WALKER_STATE_NORMAL:
                PlayAnim(WalkerAnims[A_WALKER_NORMAL], -1);
                break;
            case _WALKER_STATE_DOWN:
                PlayAnim(WalkerAnims[A_WALKER_DOWN], -1);
                break;
            case _WALKER_STATE_UP:
                PlayAnim(WalkerAnims[A_WALKER_UP], -1);
                break;
            }
        }
        
        public void OnAnimEnd(int iAnimID)
        {
        }

    }
}
