/** Kamera wykorzystywana do sledzenia obiektow w filmach przelotkach
 */
public class CameraTracking extends CameraExtended
{
    /** Cel w jaki patrzy kamera */
    MeshObject cTarget = null;
    
    /** MeshElement w jaki patrzy kamera na celu */
    int nElementID = -1;
    
    /** Kierunek patrzenia kamery, gdy nie ma celu */
    Vector vDirection = null;
    
    Module cModule = null;
    
    void OnCreate()
    {
        //super.OnCreate();
        
        cModule = GetModule();
        
        SetState("Directional");
    }
    
    /** Przelacza kamere na podany cel.
     */
    void ChangeTarget(MeshObject target)
    {
        ChangeTarget(target, null);
    }
    
    /** Przelacza kamere na podany cel. Dodatkowo mozna podac meshElement
     * w obiekcie docelowym, zeby w niego celowac.
     */
    void ChangeTarget(MeshObject target, String element)
    {
        cTarget = target;
        nElementID = -1;
        
        if(cTarget != null)
        {
            if(element!=null) nElementID = cTarget.GetElementID(element);
            SetState("Targetting");
        }
        else
        {
            SetState("Directional");
        }
    }
	
	void Move(Vector move)
	{
		Vector vPos = GetPosition();
		vPos.Add(move);
        FromForwardUpPos(GetForwardVector(), Vector.Up, vPos);
	}
    
    public class Targetting extends State
    {
        void OnPostUpdate()
        {
            // liczymy kierunek do celu
            if(cTarget != null)
            {
                Vector vDir = new Vector(GetPosition());
                
                if( nElementID == -1)
                    vDir.Sub( cTarget.GetPosition() );
                else
                    vDir.Sub( cTarget.GetElementPos(nElementID) );
                
                //Log("[CameraTracking$Targetting.OnPostUpdate] Pos: "+ GetPosition() +" dir:"+ vDir +"\n");
                FromForwardUpPos(vDir, Vector.Up, GetPosition());
            }
        }
    }

    public class Satellite extends State
    {
		Vector vDir = new Vector(0.1f, 1.0f, 0.1f);

        void OnPostUpdate()
        {
			Vector vPos = GetPosition();
			Vector vMoveDir = new Vector(vDirection);
			vMoveDir.Normalize();
			vMoveDir.Mul(100);
			vPos.Add(vMoveDir);

			FromForwardUpPos(vDir, Vector.Up, vPos);
        }
	}

    public class Directional extends State
    {
        public void OnEnter()
        {
//            if(vDirection == null) vDirection=new Vector(1.0f, 0, 0);
//            FromForwardUpPos(vDirection, Vector.Up, GetPosition());
        }
    }

    //obraca sie wokol podanego celu w jakims promieniu
    // (lub mniejszym, jesli sa przeszkody)
    // i z podana predkoscia
    public class Rotating extends State
    {
        float fAngle;

        float fRotationSpeed;
        float fRadius;
        float fHeight;
        
        float fLastTime;

        Vector vTargetPos = new Vector(0, 0, 0);
        Vector vCameraDir = new Vector(0, 0, 0);

        public void OnEnter()
        {
            fAngle = 0;
            
            fRotationSpeed = .5f;
            fRadius = 400;
            fHeight = 175;
            
            float fTargetSize = cTarget.GetMeshSize();
            //CrashLog("fTargetSize="+fTargetSize+"\n");
            
            fRadius = fTargetSize + 200.f;
            fHeight = fTargetSize * 0.75f;
            
            fLastTime = GetTime();
        }

        Vector vPrevCamPos = new Vector();
        
        void OnPostUpdate()
        {
            //liczymy polozenie celu
            if (cTarget != null)
            {
                if (nElementID == -1)
                    vTargetPos = cTarget.GetPosition();
                else
                    vTargetPos = cTarget.GetElementPos(nElementID);
            }

            //liczymy wlasne polozenie
            vCameraDir.fX = (float)Math.sin(fAngle) * fRadius;
            vCameraDir.fZ = (float)Math.cos(fAngle) * fRadius;
            
            // sprawdzamy czy cel jest nad terenem
            float fTargetTerrainY = TerrainHeight(vTargetPos.fX, vTargetPos.fZ);
            
            if( vTargetPos.fY < fTargetTerrainY)
            {
                // cel jest pod ziemia
                vCameraDir.fY = fHeight;
            }
            else
                vCameraDir.fY = TerrainHeight(vCameraDir.fX+vTargetPos.fX, vCameraDir.fZ+vTargetPos.fZ) -
                            fTargetTerrainY + fHeight;

            //liczymy kierunek do celu
            Vector vCameraPos = new Vector(vTargetPos);
            vCameraPos.Add(vCameraDir);

            // liczmy kolizje do obiektow, zeby nie bylo wnikania w obiekty
            TraceForCollision(vTargetPos, vCameraPos, cTarget);

            // jeszcze cofamy o 10cm zeby bylo ladniej
            Vector vDir = new Vector(vCameraDir);
            vDir.Normalize();
            vDir.Mul(10.0f);
            vCameraPos.Sub(vDir);
            
            //vCameraPos.Add(vPrevCamPos);
            //vCameraPos.Mul(0.5f);
            
            FromForwardUpPos(vCameraDir, Vector.Up, vCameraPos);

            //vPrevCamPos.Set(vCameraPos);
            
            float fTime = GetAbsoluteTime();

            fAngle += (fTime - fLastTime) * fRotationSpeed;
            fLastTime= fTime;
        }
    }	
}
