/** Actor rozszerzony m.in. o informacje dla Binoculars
 */
public class ChromeActor extends Actor 
    implements ipar_ConflictSide, ipar_Health, ipar_DamageThreshold, IDestroyTarget
{
    /** Znacznik, ze obiekt jest 'zwawy' tzn jest podswietlany przez Binoculars */
    boolean bIsActiveTarget = false; // not SG

    /** Znacznik, ze obiekt jest potencjalnym celem dla AI not SG*/
    boolean bIsAITarget = false;//#ChromeActor
    
    /**Znacznik czy obiekt jest wrazliwy na bronie typu emp (electromagnetic pulse)*/
    boolean bIsEmpVulnerable = false; // not SG
    
	/** czy za ten obiekt ai moze sie schowac */
	public boolean bIsHideout = false;//#ChromeActor
    
	/** obiekt, ktory zajmuje ten hideout */
	ChromeActor cHideoutInvader = null; // SG
	
    /////////////////////////////////////////////////////////////////
    // Parametry potrzebne dla AI (i nie tylko)
    
    /** Istotnosc tego ChromeActora jesli chodzi o atak przeciwnika
     */
    public float fTargetImportance = 50;//#ChromeActor

    /** Maksymalna wartosc fTargetImportance (do normalizacji). Minimum = 0;
     */
    public static float fMaxTargetImportance = 100; // not SG
    
    
    /** Minimalny damage, ktory w jakikolwiek sposob moze zaszkodzic ChromeActorowi
     */
    public float fDamageThreshold = 0;//#ChromeActor
    
    
    /**Czy wlaczone jest cloakowanie (maskowanie) obiektu*/
    boolean m_bCloaking = false; // SG
    /**Pierwotne idiki materialow, potrzebne przy cloakowaniu obiektu*/
    int iOriginalMaterials[][] = null; // not SG
    /**Tu pamietamy materialy przed przelaczeniem na cloaking*/
    String sPrevMaterials[][] = null; // SG
    
    //////////////////////////////////////////////////
    // Identyfikacja stron konfliktu                //
    //////////////////////////////////////////////////

	public EnumConflictSide cConflictSide = new EnumConflictSide(EnumConflictSide._NEUTRAL);//#ChromeActor
	

    /** Identyfikator teamu ChromeActora
     */
    public int nTeamID = 0; // SG

    /** [pomocnicze] Ponizej jakiej wartosci nTeamID (wlacznie) 
     *  Pawn nalezy do pierwszej strony konfliktu
     */
    public int nProTeamIDsThreshold = -1000; // not SG
    
    /** [pomocnicze] Powyzej jakiej wartosci nTeamID (wlacznie) 
     *  Pawn nalezy do drugiej strony konfliktu
     */
    public int nCounterTeamIDsThreshold = 1000; // not SG

    //////////////////////////////////////////////////
    // Oznaczenia 'przyjaznosci' teamow
    final static int _TEAM_RELATION_FRIENDLY    = 1;
    final static int _TEAM_RELATION_NEUTRAL     = 0;
    final static int _TEAM_RELATION_HOSTILE     = -1;

    ///////////////////////////////////////////////////////////////////////////

	boolean m_bRecognizedAsPlayer = false;
	
    ChromeActor()
    {
    }
    
    int METHOD_ShowFromUnderGround = 0;

    // identyfikator klasy Actor z enginu
    static int nActorClassID = 0;
    
    void OnCreate()
    {
        if(nActorClassID == 0) nActorClassID = GetClassID("Actor");
        
        METHOD_ShowFromUnderGround = RegisterCallMethod("ShowFromUnderGround");
            
        super.OnCreate();
        
		bSG = true;
        
        InitOriginalMaterials();
        
        if (bIsAITarget)
            SetFlag(EFlags._AI_TARGET);
    }

    void InitOriginalMaterials()
    {
        //CrashLog(this + " InitOriginalMaterials");
        
        //sprawdzamy lody
        int nLodCount = 0;
        iOriginalMaterials = new int[4][];
        for (int i = 0; i < 4; i++)
        {
            String sMaterials[] = GetMaterials(i);
            if (sMaterials != null && sMaterials.length > 0)
            {
                iOriginalMaterials[nLodCount] = new int[sMaterials.length];
                for (int j = 0; j < sMaterials.length; j++)
                {
                    if (sMaterials[j] != null && sMaterials[j].length() > 0)
                    {
                        //sprawdzamy czy czasem juz go nie ma w tablicy
                        int nMatID = MaterialGetID(sMaterials[j]);
                        if (!IsInOriginalMaterialsTab(nMatID))
                            iOriginalMaterials[nLodCount][j] = nMatID;
                        else
                            iOriginalMaterials[nLodCount][j] = -1;
                    }
                    else
                        iOriginalMaterials[nLodCount][j] = -1;
                }
                nLodCount ++;
            }
        }
    }
    
    boolean IsInOriginalMaterialsTab(int nMaterial)
    {
        if (iOriginalMaterials != null)
        {
            for (int nLod = 0; nLod < iOriginalMaterials.length; nLod ++)
            {
                if (iOriginalMaterials[nLod] != null)
                {
                    for (int i = 0; i < iOriginalMaterials[nLod].length; i++ )
                    {
                        if (iOriginalMaterials[nLod][i] == nMaterial)
                            return true;
                    } 
                }
            }
        }
        return false;
    }
    
    public void Die() 
    {
        boolean bWasDead = bDead;
        
		super.Die();
		bIsActiveTarget = false;
        
        // Powiadomienie levelu
        if (!bWasDead && bDead && // tylko, jesli zmienil stan
            GetModule() instanceof ChromeModule)
        {
            SetCloaking(false);
            ((ChromeModule)GetModule()).ChromeActorDied(this);
        }
    }

    public void Damage(float fDamage)
    {
		if (fDamage >= fDamageThreshold)
            super.Damage(fDamage);
	}
	
    /** Zwraca jeden z punktow, od widocznosci ktorego uzaleznione jest,
     *  czy AI go widzi
     *  @param nPoint numer punktu do zwrocenia
     *  @return punkt o numerze nPoint lub null, jesli nPoint > ilosci
     *          dostepnych punktow
     *  KONIECZNIE NALEZY POKRYC W KLASACH POCHODNYCH!!!
     */
    public Vector GetPointToLookAt(int nPoint)
    {
        if (nPoint == 0)
            return GetPosition();
        else
            return null;
    }
    

    /** Zwraca pozycje, z ktorej ChromeActor strzela
     */
    public Vector GetFireOrigin()
    {
        return GetPosition();
    }
        
    /** Zwraca kierunek, w ktorym celuje bron ChromeActora
     *  (wektor strzalu bez rozrzutu)
     */
    public Vector GetAimDir()
    {
        return GetForwardVector();
    }
    
    /**Zwraca wektor strzalu
     */
    public Vector GetFireDir()
    {
        return GetForwardVector();
    }
    
    /** Callback, ktory powinien byc zawolany, gdy jakis obiekt dostanie damage od tego ChromeActora */
    public void NotifyHit(ChromeActor victim, int nDamageType)
    {
    }
    
    ////////////////////////////////////////////////////////////////////////////////
    // WIDOCZNOSC OBIEKTU
    //

    /** Zwraca maksymalna odleglosc (w [cm]), z jakiej widoczny jest obiekt
     */
    float GetVisibilityRangeThreshold(ChromeActor cObserver)
    {
        return Float.MAX_VALUE;
    }
    
    /** Zwraca 'wspolczynnik widocznosci' tego ChromeActora dla innego 
     *  w zaleznosci od jego stanu 0 - niewidoczny, 1 - widoczny
     */
    public float GetVisibilityRangeFactor(ChromeActor cObserver)
    {

        return 1;
    }
    
    /** Zwraca 'wspolczynnik latwosci trafienia' tego ChromeActora przez innego
     *  0 - ekstremalnie trudny do trafienia, 1 - bardzo latwy do trafienia
     */
    public float GetShootVisibilityFactor(ChromeActor cShooter)
    {
        return 1;
    }
    
    /** Zwraca mnoznik prawdopodobienstwa trafienia tego celu przez AI
     */
    public float GetHitChanceMult(ChromeActor cShooter)
    {
        return 1;
    }
    
    
    void SetCloaking(boolean bOn)
    {
        if (m_bCloaking != bOn)
        {
			FileChunk.Log("[ChromeActor.SetCloaking] Type = <"+bOn+"> Class = <"+getClass().getName()+"> Name <"+GetName()+">\n");
			
            m_bCloaking = bOn;
            if (bOn)
            {
                sPrevMaterials = new String[iOriginalMaterials.length][];
                
                for (int nLod = 0; nLod < iOriginalMaterials.length; nLod ++)
                {
                    sPrevMaterials[nLod] = GetMaterials(nLod);
                    for(int i=0; i < iOriginalMaterials[nLod].length; i++)
                    {
                        if (iOriginalMaterials[nLod][i] != -1)
                        {
                            ReplaceOriginalMaterial(iOriginalMaterials[nLod][i], "cloaked.mat" );
                        }
                    }
                }
            }
            else
            {
                for (int nLod = 0; nLod < iOriginalMaterials.length; nLod ++)
                {
                    for(int i = 0; i < iOriginalMaterials[nLod].length; i++)
                    {
                        if (iOriginalMaterials[nLod][i] != -1)
                        {
							ReplaceOriginalMaterial(iOriginalMaterials[nLod][i], sPrevMaterials[nLod][i] );
                        }
                    }
                }
            }
        }
    }
    
    
    //////////////////////////////////////////////////
    // Identyfikacja stron konfliktu                //
    //////////////////////////////////////////////////
    
	public final int GetConflictSide()
	{
		return cConflictSide.GetValue();
	}
	
	public final boolean IsFriend(ChromeActor chromeActor)
	{
		return !IsNeutral(chromeActor) &&
			   (GetConflictSide() == chromeActor.GetConflictSide());
	}
	
	public final boolean IsEnemy(ChromeActor chromeActor)
	{
		return !IsNeutral(chromeActor) &&
			   (GetConflictSide() != chromeActor.GetConflictSide());
	}
	
	public final boolean IsNeutral(ChromeActor chromeActor)
	{
		return (GetConflictSide() == EnumConflictSide._NEUTRAL) ||
               chromeActor.GetConflictSide() == EnumConflictSide._NEUTRAL;
	}

    public final boolean IsFriend(int nConflictSide)
	{
		return !IsNeutral(nConflictSide) &&
			   (GetConflictSide() == nConflictSide);
	}
	
	public final boolean IsEnemy(int nConflictSide)
	{
		return !IsNeutral(nConflictSide) &&
			   (GetConflictSide() != nConflictSide);
	}

    public final boolean IsNeutral(int nConflictSide)
    {
        return nConflictSide == EnumConflictSide._NEUTRAL ||
               (GetConflictSide() == EnumConflictSide._NEUTRAL);
    }

    /** Metoda podaje, czy czlonek danego teamu jest z przeciwnego, naszego 
     *  czy neutralnego teamu. Do pokrycia w klasach pochodnych w zaleznosci 
     *  od potrzeb.
     *  @param nVerifiedTeamID nTeamID sprawdzanego Pawna
     *  @return jedna ze stalych _TEAM_RELATION_* (NEUTRAL, FRIENDLY, HOSTILE)
     */
    int GetTeamRelation(ChromeActor targetToVerify)
    {
        int nResult = _TEAM_RELATION_NEUTRAL;
        
        if ( targetToVerify != null )
        {
            int nIdToVerify = targetToVerify.nTeamID;
            
            if (nTeamID <= nProTeamIDsThreshold) // nalezymy do pierwszej strony konfliktu
            {
                if (nIdToVerify <= nProTeamIDsThreshold) 
                    nResult = _TEAM_RELATION_FRIENDLY;
                else if (nIdToVerify >= nCounterTeamIDsThreshold)
                    nResult = _TEAM_RELATION_HOSTILE;
            }
            else if (nTeamID >= nCounterTeamIDsThreshold) // nalezymy do drugiej strony konfliktu
            {
                if (nIdToVerify >= nCounterTeamIDsThreshold)
                    nResult = _TEAM_RELATION_FRIENDLY;
                else if (nIdToVerify <= nProTeamIDsThreshold)
                    nResult = _TEAM_RELATION_HOSTILE;
            }
        }
        
        return nResult;    
    }
	
    int GetTeamRelation(int iTargetTeamID)
    {
        int nResult = _TEAM_RELATION_NEUTRAL;
            
        if (nTeamID <= nProTeamIDsThreshold) // nalezymy do pierwszej strony konfliktu
        {
            if (iTargetTeamID <= nProTeamIDsThreshold) 
                nResult = _TEAM_RELATION_FRIENDLY;
            else
			if (iTargetTeamID >= nCounterTeamIDsThreshold)
                nResult = _TEAM_RELATION_HOSTILE;
        }
        else if (nTeamID >= nCounterTeamIDsThreshold) // nalezymy do drugiej strony konfliktu
        {
            if (iTargetTeamID >= nCounterTeamIDsThreshold)
                nResult = _TEAM_RELATION_FRIENDLY;
            else
			if (iTargetTeamID <= nProTeamIDsThreshold)
                nResult = _TEAM_RELATION_HOSTILE;
        }
		
        return nResult;    
	}

/*
    void OnCreate()
    {
        super.OnCreate();
        
        SetUnImportanceFactor(0);
    }
*/
    /**
     * Pomocnicze funkcje do sieci
     */
    
    /** Na potrzeby sieci, do pokrycia w PlayerNetach
     * Callback z broni mowiacy o tym ze wlasnie strzelila na serwerze
     */
    void WeaponFireOnServer()
    {
    }
    
    /** Na potrzeby sieci, do pokrycia w PlayerNetach
     * Callback z broni mowiacy o tym ze wlasnie strzelila na kliencie not ownerze
     */
    void WeaponFireOnClientNotOwner()
    {
    }

    
    // TODO: napisac pozniej uniwersalna procke, ktora bedzie
    // znajdowala obiekt o podanym id i sprawdzala czy
    // obiekt ten jest podanej (w parametrze) klasy
    Weapon GetWeaponByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            Weapon weapon = (Weapon) object;
            return weapon;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from Weapon\n");
        }
        
        return null;
    }

    InvObject GetInvObjByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            InvObject invObj = (InvObject) object;
            return invObj;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from InvObject\n");
        }
        
        return null;
    }
	
	TriggerObject GetTriggerObjByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            TriggerObject triggerObj = (TriggerObject) object;
            return triggerObj;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from TriggerObject\n");
        }
        
        return null;
    }
	
	PlayerNet GetPlayerNetByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            PlayerNet player = (PlayerNet) object;
            return player;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from PlayerNet\n");
        }
        
        return null;
    }
    
    NetPlayerInfo GetPlayerInfoByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            NetPlayerInfo playerinfo = (NetPlayerInfo) object;
            return playerinfo;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from NetPlayerInfo\n");
        }
        
        return null;
    }
    
    Vehicle GetVehicleByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            Vehicle vehicle = (Vehicle) object;
            return vehicle;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from Vehicle\n");
        }
        
        return null;
    }
    
    // tmp
    Pawn GetPawnByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            Pawn pawn = (Pawn) object;
            return pawn;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from Pawn\n");
        }
        
        return null;
    }

    //////////////////////////////////////////////////////
    //// PARAMETRY FX'ow trace'ow dla wystrzalow z guna
    /////////////////////////////////////////////////////

    public float fFXSizeX = 2.0f; // SG
    public float fFXSizeY = 10000.0f; // SG
    public float fFXZero = 0f; // SG
    public float fFXPosY = 0f; // SG
    
    /** Odpala efekt struzki przy strzale z podanej pozycji. Generalnie
     * trzeba ta metode pokryc, zeby we wlasciwym meshElemencie odpalic efekt.
     * @param fDistance odleglosc na jaka sie ma struzka rozciagnac
     * @param vBeamPos punkt z ktorego struzka sie rozchodzi
     */
    void ExecuteWeaponBeam(float fDistance, Vector vBeamPos,int nElementBarrelExit)
    {
        fFXPosY = fDistance * .75f;
        fFXSizeY = fDistance;
    }
	
    boolean bShowBeams = false;
    
    LaserPointer[] aBeams = null;
    
    void CreateTestLaserBeams()
    {
        if (aBeams == null)
        {
            aBeams = new LaserPointer[4];
            aBeams[0] = Tools.CreateLaserPointer(GetModule(), LaserPointer._BEAM_GREEN);
            aBeams[1] = Tools.CreateLaserPointer(GetModule(), LaserPointer._BEAM_GREEN);
            aBeams[2] = Tools.CreateLaserPointer(GetModule(), LaserPointer._BEAM_GREEN);
            aBeams[3] = Tools.CreateLaserPointer(GetModule(), LaserPointer._BEAM_GREEN);
        }
    }

    
    int nBlastDamageSamples = 4;
    
    boolean bOldDupa = true;
    
    /** Metoda zadaje innym aktorom blastDamage. Jako inicjator damage'u podawany
     * jest aktor na rzecz ktorego metoda jest wywolana.
     * @param fBlastDamage wartosc odejmowanego damage'a w epicentrum blasta
     * @param fPhSimDamage wartosc obrazen przekazywana do symulacji fizyki po smierci "klienta", im wieksza wartosc, tym bardziej postac "fruwa"
     * @param vPos pozycja z ktorej blast jest odpalany
     * @param fBlastRange zasieg dzialania blastu
     * @param bExcludeActor flaga mowi czy blast ma byc odjety aktorowi dla ktorego jest wolana ta metdoa     
     * @param cExcludeObj obiekt na ktory ma nie dzialac blast
     */
    public void MakeBlastDamage(float fBlastDamage, float fPhSimDamage, Vector vPos, float fBlastRange, boolean bExcludeActor, Actor cExcludeObj, int nDamageType)
    {
        //TODO zrobic wyslanie do sieci efektu tluczenia szyby
        // oraz replikacje stanu szyby
        
        // mnoznik do drugiego parametru jest dodany eksperymentalnie, oznacza najwieksza szybkosc
        // z jaka moga poruszac sie odlamki szkla.
        if( FXBrokenGlass_CauseAnExplosion( vPos, fBlastDamage * 1.f, fBlastRange ) > 0 )
            PlaySound("Data/Sounds/GlassShatter.wav", vPos);
        
        // odginanie traw od wybuchu
//        GrassPusher_DropSphere(vPos, fBlastRange);

        Actor cActor = null;
        
        //ControlObject[] aObjects = GetObjectsInRadiusFromClass(fBlastRange, "Actor");
        ControlObject[] aObjects = GetObjectsInRadiusFromClass(fBlastRange, nActorClassID);
        
        Vector vSpeed = null;
        for(int i=0; i<aObjects.length; i++)
        {
            cActor = (Actor)aObjects[i]; 
            if( cExcludeObj != cActor )
            {
                // sprawdzamy czy sobie nie odejmowac energii
                if(bExcludeActor) if( cActor==this ) continue;
                
                // umarlaki nie maja co miec zabieranego damagu
                //if(cActor.bDead) continue; //ale powinny "fruwac" PeH
                
                float fRange = vPos.Distance(cActor.GetPosition());
                if (fRange < fBlastRange)
                {
                    float fDamage = (fBlastRange - fRange) / fBlastRange;
                    float fPhDamage = fDamage;
                    fDamage *= fBlastDamage;
                    fPhDamage *= fPhSimDamage;                    
                    
                    // liczymy kierunek predkosci
                    vSpeed = cActor.GetPosition();
                    vSpeed.Sub(vPos);
                    vSpeed.Normalize();
                    vSpeed.Mul(100000.0f);
                    
                    // w odleglosci 30% zasiegu dzialania bulleta i tak zabiera energie
                    if(fRange > fBlastRange*0.1f)
                    {
                        // liczymy trace'a czy czasem cos nie zaslania obiektu
                        Vector vActorMin = new Vector();
                        Vector vActorMax = new Vector();
                        cActor.GetMeshExtents(vActorMin, vActorMax);
                    
                        vActorMin.fX = 0; vActorMin.fZ = 0;
                    
                        // inaczej sprawdzamy czy nie schowal sie ktos za przeszkoda
                        int nSamples = nBlastDamageSamples;
                        float fStep = (vActorMax.fY - vActorMin.fY) / ((float)nSamples);
                        float fDamageStep = fDamage/((float)nSamples);
                        float fPhDamageStep = fPhDamage/((float)nSamples);
                        for(int j=0; j<nSamples; ++j)
                        {
                         
                            if(bShowBeams)
                            {
                                CreateTestLaserBeams();
                                Vector vEnd = cActor.GetPosition();
                                vEnd.Add(vActorMin);
                                aBeams[j].PointFromTo(GetPositionVectorVolatile(), vEnd);
                            }
                            
                            if(!CheckTrace(cActor, new Vector(), vActorMin))
                            {
                                // nie ma trafienia, wiec odbieramy mniej energii
                                fDamage -= fDamageStep;
                                fPhDamage -= fPhDamageStep;
                            }
                            vActorMin.fY += fStep;
                        }
                    }
                    
                    Vector vHitPos = ((MeshObject)cActor).GetBoneJointPos(EBones._SPINE1);
                    Collision stColl = TraceObjectsExclude(vPos, vHitPos, this);
                    Vector vDir = vHitPos.Sub2(vPos).Normalize2();
                    //vDir.fY = 0;
                    //vDir.Normalize();
                    //if (stColl != null)
                    //    CrashLog("Kolizja ok");
                    
                    if (fDamage >= 1)
                        cActor.TakeDamage(fDamage, fPhDamage, this, vHitPos, vDir,EBones._NON_BONE , nDamageType );
                }
            }
        }
    }
    

    /** Zatrzesienie kamera kazdego z pobliskich graczy
     */
    public void ShakePlayersCamera(float fStrength, float fMinDistance, float fMaxDistance)
    {
        if (GetModule() instanceof ChromeModule)
        {
            Player player = ((ChromeModule)GetModule()).GetMainPlayer();
            if (player != null &&
                player.Camera != null)
            {
                float dist2 = GetDistance2To(player);
                float minDist2 = fMinDistance * fMinDistance,
                    maxDist2 = fMaxDistance * fMaxDistance;
                float shakeVal = fStrength * (1 - Tools.InterpLinear(minDist2, maxDist2, dist2));
                if (shakeVal > 1)
                    player.Camera.ShakeImpulse(shakeVal, shakeVal, shakeVal, CameraPlayer._SHAKE_EXPLOSION);
            }
        }
    }
    
    
    /////////////////////////////////////////////////////////////////////////////
    // Implementacja interfejsow 'parametrowych'
    
    public void ChangeConflictSide(int nNewConflictSide)
    {
        cConflictSide = new EnumConflictSide(nNewConflictSide);
    }
    
    public void ChangeHealth(float fHealth)
    {
        this.fHealth = fHealth;
    }
    
    public void ChangeDamageThreshold(float fDamageThreshold)
    {
        this.fDamageThreshold = fDamageThreshold;
    }

    //////////////////////////////////////////////////////
    //// Make noise zoptymalizowany do strzelania
    /////////////////////////////////////////////////////
	// minimalny czas pomiedzy wolaniami make noise
	float _MIN_TIME_BTW_MAKE_SHOT_NOISE = 1f;
	// czas, kiedy ostatnio byl zawolany make noise
    float m_fLastMakeNoiseTime = 0f;
	
    public void MakeNoise(float fLoudness, int nNoiseType)
	{
		if (nNoiseType == Pawn._NOISE_WEAPON_FIRING)
		{
			float fTime = GetTime();
			
			if (fTime - m_fLastMakeNoiseTime > _MIN_TIME_BTW_MAKE_SHOT_NOISE)
			{
				m_fLastMakeNoiseTime = fTime;
				super.MakeNoise(fLoudness, nNoiseType);
			}
		}
		else
			super.MakeNoise(fLoudness, nNoiseType);
	}
	
    public void MakeNoise(Vector vPos, float fLoudness, int nNoiseType)
	{
		if (nNoiseType == Pawn._NOISE_WEAPON_FIRING)
		{
			float fTime = GetTime();
			
			if (fTime - m_fLastMakeNoiseTime > _MIN_TIME_BTW_MAKE_SHOT_NOISE)
			{
				m_fLastMakeNoiseTime = fTime;
				super.MakeNoise(vPos, fLoudness, nNoiseType);
			}
		}
		else
			super.MakeNoise(vPos, fLoudness, nNoiseType);
	}
	
	public boolean IsDead()
	{
		return bDead;
	}
	
	public ChromeActor [] GetObjectsToDestroy()
	{
		if (!IsDead())
		{
			ChromeActor [] tab = new ChromeActor[1];
			tab[0] = this;
			return tab;
		}
		else
			return null;
	}
	
    //*********************************************************
    //                  Termowizja
    //*********************************************************
    
    int m_tTermovisionVisibleMeshElems[] = null;
    MeshObject m_tTermovisionMeshes[] = null;
    String m_tTermovisionMeshesNames[] = null;
    Vector m_tTermovisionMeshesScales[] = null;
    boolean m_bTermovisionParamsLoaded = false;
    
    void TermovisionOn()
    {
        SetUnImportanceThreshold(3500f);
        if (m_tTermovisionVisibleMeshElems == null)
        {
            InitTermovisionMeshes();
        }
        
        if (m_tTermovisionMeshes == null)
            SetupTermovisionMeshes();
            
    }
    
    void TermovisionOff()
    {
        SetUnImportanceThreshold(2000f);
        if (m_tTermovisionMeshes != null)
        {
            ReleaseTermovisionMeshes();
        }
    }
    
    boolean IsTermovisionOn()
    {
        return (m_tTermovisionMeshes != null);
    }
    
    void InitTermovisionMeshes()
    {
        if (m_tTermovisionVisibleMeshElems == null)
        {
            java.util.Vector vElems = new java.util.Vector();
            int nIdx = 1;
            while (true)
            {
                String sName = "Termo_";
                if (nIdx < 10)
                    sName += ("0" + nIdx);
                else
                    sName += String.valueOf(nIdx);
                    
                int nElement = GetElementID(sName);
                if (nElement != -1)
                    vElems.addElement(new TermoElement(nElement, new Vector(0.33f, 0.33f, 0.33f)));
                else
                    break;
                nIdx ++;    
            }
              
            nIdx = 1;
            while (true)
            {
                String sName = "TermoXL_";
                if (nIdx < 10)
                    sName += ("0" + nIdx);
                else
                    sName += String.valueOf(nIdx);
                    
                int nElement = GetElementID(sName);
                if (nElement != -1)
                    vElems.addElement(new TermoElement(nElement, new Vector(0.75f, 0.75f, 0.75f)));
                else
                    break;
                nIdx ++;
            }
               
            nIdx = 1;
            while (true)
            {
                String sName = "TermoXXL_";
                if (nIdx < 10)
                    sName += ("0" + nIdx);
                else
                    sName += String.valueOf(nIdx);
                    
                int nElement = GetElementID(sName);
                if (nElement != -1)
                    vElems.addElement(new TermoElement(nElement, new Vector(1, 1, 1)));
                else
                    break;
                nIdx ++;
            }
            //CrashLog("ELEMS " + vElems.size());
            
            int nElemCount = vElems.size();
            if (nElemCount == 0)
            {
                int nElement = GetElementID("Bip01 Head");
                if (nElement != -1)
                {
                    vElems.addElement(new TermoElement(nElement, new Vector(2, 3, 1)));
                    nElemCount = 1;
                }
            }
            
            m_tTermovisionVisibleMeshElems = new int[nElemCount];
            m_tTermovisionMeshesScales = new Vector[nElemCount];
        
            for (int i = 0; i < m_tTermovisionVisibleMeshElems.length; i++)
            {
                TermoElement cElem = (TermoElement)vElems.elementAt(i);
                m_tTermovisionVisibleMeshElems[i] = cElem.nElementID;
                m_tTermovisionMeshesScales[i] = cElem.vScale;
            }
        }
        
    }

    void SetupTermovisionMeshes()
    {
        if (m_tTermovisionVisibleMeshElems != null)
            m_tTermovisionMeshes = new MeshObject[m_tTermovisionVisibleMeshElems.length];
        for (int i = 0; i < m_tTermovisionVisibleMeshElems.length; i++)
        {
            if (m_tTermovisionVisibleMeshElems[i] != -1)
            {   
                MeshObject cMesh = ImplantTermovision.GetTermoMesh();
                m_tTermovisionMeshes[i] = cMesh;
                
                AttachChild(cMesh, m_tTermovisionVisibleMeshElems[i]);
                cMesh.ClearFlag(EFlags._LINKED_TO_PORTAL_OBJECT);
                cMesh.FromUpForwardPos(new Vector(Vector.Up),
                                       new Vector(Vector.Forward),
                                       GetElementPos(m_tTermovisionVisibleMeshElems[i]));
                if (m_tTermovisionMeshesScales != null && m_tTermovisionMeshesScales[i] != null)
                    cMesh.SetScale(m_tTermovisionMeshesScales[i]);
                
                cMesh.ActivateOnModule();
            }
            else
            {
                Tools.CrashLogError("No elem " + m_tTermovisionMeshesNames[i]);
            }
        }
    }

    void ReleaseTermovisionMeshes()
    {
        for (int i = 0; i < m_tTermovisionMeshes.length; i ++)
        {
            m_tTermovisionMeshes[i].DetachFromParent();
            ImplantTermovision.ReleaseTermoMesh(m_tTermovisionMeshes[i]);
            m_tTermovisionMeshes[i] = null;
        }
        m_tTermovisionMeshes = null;
    }
    
    void LoadTermovisionParams()
    {
        String sFile = MeshName + ".trm";
        ParseKeyword cRoot = GameObject.ParseFile( sFile );
        
        if (cRoot==null)
        {
            GameObject.LogR("Read or parse error in file " + sFile + " !\n");
        }
        else
        {
            TermoElement cElement = null;
            ParseKeyword cKey, cSubKey;
            java.util.Vector vElems = new java.util.Vector();
            
            for ( cKey=cRoot; cKey!=null; cKey=cKey.GetNext() )
            {
                if (cKey.Equals("TermoElement"))
                {
                    cElement = new TermoElement(cKey.GetSParam(0), 
                                                new Vector(cKey.GetFParam(1), cKey.GetFParam(2), cKey.GetFParam(3)));
                    vElems.addElement(cElement);
                }
            }
            m_tTermovisionMeshesNames = new String[vElems.size()];
            m_tTermovisionVisibleMeshElems = new int[vElems.size()];
            m_tTermovisionMeshesScales = new Vector[vElems.size()];
            m_tTermovisionMeshes = new MeshObject[vElems.size()];
            for (int i = 0; i < vElems.size(); i ++)
            {
                cElement = (TermoElement)vElems.elementAt(i);
                m_tTermovisionVisibleMeshElems[i] = GetElementID(cElement.sMeshElementName);
                m_tTermovisionMeshesScales[i] = cElement.vScale;
                m_tTermovisionMeshesNames[i] = cElement.sMeshElementName;
                //CrashLog("Element " + cElement.sMeshElementName + " " + GetElementID(cElement.sMeshElementName));
            }
            cRoot.delete();
            m_bTermovisionParamsLoaded = true;
        }
    }
    
	//*********************************************************
	//					SaveGame
    //*********************************************************
	
	boolean m_bDeadSG = false;
	boolean m_bCloakingSG = false;
    
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[ChromeActor.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_CHROMEACTOR)
        {	
			m_bDeadSG = bDead;
			
			super.SGLoadChunk(cFC);

			// konieczne zapisywanie, poniewa jak aktor jest  martwy to nie moe by widoczny w Binoculars
            bIsActiveTarget = cFC.LoadBool();
			// minimalny damage, ktory w jakikolwiek sposob moze zaszkodzic ChromeActorowi
            fDamageThreshold = cFC.LoadFloat();
			
			// identyfikacja stron konfliktu
			cConflictSide = new EnumConflictSide(cFC.LoadInt());
			// identyfikator teamu ChromeActora
			nTeamID = cFC.LoadInt();
			
			// parametry efektw specjalnych
			fFXSizeX = cFC.LoadFloat();
			fFXSizeY = cFC.LoadFloat();
			fFXZero = cFC.LoadFloat();
			fFXPosY = cFC.LoadFloat();		
						
			// obiekt, ktory si ukrywa
			cHideoutInvader = (ChromeActor)GameObject.cSG.LoadObjectRef(cFC);					
			
			// termowizja
			//if (cFC.LoadBool()) TermovisionOff(); else TermovisionOn();
					
			m_bCloakingSG = cFC.LoadBool();			
            
            m_fLastMakeNoiseTime = cFC.LoadFloat();			
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[ChromeActor.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_CHROMEACTOR, cFCParent);
		
		super.SGSaveChunk(cFC);

		// konieczne zapisywanie, poniewa jak aktor jest  martwy to nie moe by widoczny w Binoculars
        cFC.SaveBool( bIsActiveTarget ); 
		// minimalny damage, ktory w jakikolwiek sposob moze zaszkodzic ChromeActorowi
        cFC.SaveFloat( fDamageThreshold );
				
		// identyfikacja stron konfliktu
		cFC.SaveInt( cConflictSide.GetValue() );
		// identyfikator teamu ChromeActora
        cFC.SaveInt( nTeamID );		

		// parametry efektw specjalnych
		cFC.SaveFloat(fFXSizeX);
		cFC.SaveFloat(fFXSizeY);
		cFC.SaveFloat(fFXZero);
		cFC.SaveFloat(fFXPosY);		
		
		// obiekt, ktory si ukrywa
		GameObject.cSG.SaveObjectRef(cFC,cHideoutInvader);
		
		// termowizja
		//cFC.SaveBool(m_tTermovisionMeshes == null);
				
		cFC.SaveBool( m_bCloaking );
		        
        cFC.SaveFloat(m_fLastMakeNoiseTime);
		
        cFC.delete();
    }	
	
	public void SGUpdate()
	{
		FileChunk.Log("[ChromeActor.SGUpdate]\n");
		
		super.SGUpdate();
						
		// nie wywouj Die(), bo mi chodzi tylko o ukrycie i usunicie z poziomu
		if (bDead && !m_bDeadSG)
		{
			bIsActiveTarget = false;
            TermovisionOff();
			SetCloaking(false);
			// Powiadomienie levelu
			if (GetModule() instanceof ChromeModule)
			    ((ChromeModule)GetModule()).ChromeActorDied(this);
		}
		else
		{			
			if (m_bCloakingSG && !m_bCloaking)
				SetCloaking(true);
			else if (!m_bCloakingSG && m_bCloaking)
			{
				if (sPrevMaterials != null)
					SetCloaking(false);
			}
		}
	}
    
    void ServerRespawn( SpawnInfo cSpawnInfo, float fDelayTime )
    {
        //uniewazniamy callmethody z ewentualnych wczesniejszych spawnow
        DisableAllRespawnMethods();
                
        //ustawiamy obiekt na pozycji wg spawninfo
        FromUpForwardPos( cSpawnInfo.m_vUp, cSpawnInfo.m_vDir, cSpawnInfo.m_vPos );
        //zabawe z interpolacja zaczynamy od nowa
        ResetInterpolatePosAndDir();
        //jesli zadalismy jakis delay, to schowajmy go na chwile...
        //jesli nie to konczymy zabawe, powiedzmy tylko o tym klientom
        if ( fDelayTime > 0.0f )
            HideUnderGround( fDelayTime );
        else
            MsgSendClientState();
        DeleteAttachedFXes();
    }
    
    /**
     * Metoda disabluje wszystkie metody, ktore mogly
     * byc zaplanowane w CallMethodIn podczas akcji spawnowania
     */
    void DisableAllRespawnMethods()
    {
        //metoda wyciagajaca obiekt spod ziemi na prawdziwe miejsce spawnu
        DisableCallMethod( METHOD_ShowFromUnderGround );
    }
       
    
    /**
     * Pozycja na ktorej obiekt ma sie wyspalnowac
     */
    Vector  m_vSpawnPos      = null;
    /**
     * Do testow, flaga mowiaca czy powyzsza zmienna jest
     * ustawiona i nie zostala jeszcze wykorzystana
     */
    boolean m_bSpawnPosValid = false;
    
    /**
     * Schowanie obiektu pod ziemie na pewien czas
     */
    void HideUnderGround(float fShowDelayTime)
    {
        if ( /*Game.CheatsEnabled() && */(m_bSpawnPosValid) )
        {
            //LogR(" Proba ponownego schowania obiektu "+this+" do spawnu!\n");
            //Thread.dumpStack();
        }
        //zapamietujemy prawdziwa pozycje do wyspalnowania
        m_vSpawnPos      = GetPosition();
        m_bSpawnPosValid = true;
        
        //bierzemy ta pozycje zeby ja zmodyfikowac
        Vector vPos = new Vector( m_vSpawnPos );
        //teraz przesuwamy obiekt troszke w dol
        vPos.fY -= 1000.0f;
        //i tu go na jakis czas zostawiamy
        SetPositionWorld( vPos );
        //i zresetujmy interpolacje zeby nam sie nie interpolowal z poprzedniej pozycji 
        ResetInterpolatePosAndDir();
        
        //wysylamy stan obiektu klientom
        MsgSendClientState();
        
        //i zaplanujmy wyciagniecie z pod ziemi
        CallMethodIn( METHOD_ShowFromUnderGround, fShowDelayTime );
    }
    
    /**
     * Przywrocenie pozycji obiektu na faktyczna pozycje spawnu
     * czyli do stanu z przed schowania pod ziemie
     */
    void ShowFromUnderGround()
    {
        if ( /*Game.CheatsEnabled() &&*/ (!m_bSpawnPosValid) )
        {
            LogR(" Proba ponownego wyciagniecia obiektu "+this+" do spawnu!\n");
            Thread.dumpStack();
        }
        
        //przywracamy obiektowi prawdziwa pozycje do spawnu
        if ( m_vSpawnPos != null )
        {
            SetPositionWorld( m_vSpawnPos );
            //zawsze przy takiej zmianie pozycji zabawe z interpolacja 
            //zaczynamy od nowa
            ResetInterpolatePosAndDir();    
        }
        else
            CrashLog(" Brak zapamietanej pozycji do wyspawnowania "+this+"\n");
        m_bSpawnPosValid = false;
        
        //pochwalmy sie o naszym stanie klientom
        MsgSendClientState();
    }
    
    /**
     * Callback wolany z levelu tuz po wyspawnowaniu 
     * obiektu na levelu
     */
    void RespawnedOnLevel(){}
        
    
    /** Odlacza wszystkie dzieci obiektu, przydatne np. przy kasowaniu
     */
    void DeactivateChildren()
    {
        GameObject [] children = GetChildren();
        if (children == null) return;
        for (int i = 0; i < children.length; i++)
        {
            // zwykle obiekty odlaczamy i chowamy
            ((MeshObject)children[i]).DetachFromParent();
            ((MeshObject)children[i]).EnableRendering(false);
            ((MeshObject)children[i]).DeactivateOnModuleAlways();
            ((MeshObject)children[i]).DeleteAttachedFXes();
        }
    }
    
    void Execute( Actor executor )
    {
        
        //CrashLog( "[TriggerObject.Execute] Executed by " + executor + "\n" );
        Module cModule = GetModule();
        if (cModule != null)
            cModule.TriggerExecuted(this, executor);
        
        
        InvokeActionChain(aOnExecute);
    }
    
    /**
     * SIEC
     * Do interpolowania pozycji i kierunku 
     * Wykorzystywane dla graczy i pojazdow
     * Membery wykorzystywane do tego celu sa wspolne
     * dla gracza i pojazdu, zatem zostana zdefiniowane tu
     * Metody beda (w wiekszosci przypadkow) zaimplementowane
     * w klasach potomnych (PlayerNetBaseMover i Vehicle)
     */

    /** czas poprzedniego renderu, < 0.0f oznacza ze w poprzedniej 
     * ramce obiekt nie byl renderowany i sia zeczy nie bedzie zadnej 
     * interpolacji w ramce nastepnej. 
     * Uwaga. Fakt ten mozna wykorzystac w sytuacjach kiedy chcemy 
     * przestawic nagle obiekt, wtedy podajac mu nowa pozycje i 
     * ustawiajac m_fRenderTime na wartosc < 0.0f zagwarantujemy sobie
     * chwilowe wylaczenie interpolacji */
    private
    float   m_fRenderTime = -1.0f;
    float   GetRenderTime(){ return m_fRenderTime; }
    void    SetRenderTime(float fTime){ m_fRenderTime = fTime; }
    /** Metoda ustawia czas ostatniego renderu na -1.0f
     * Mozna to wykorzystac w sytuacjach kiedy chcemy 
     * przestawic nagle obiekt, wtedy podajac mu nowa pozycje i 
     * ta metode zagwarantujemy sobie chwilowe wylaczenie interpolacji */
    void    ResetInterpolatePosAndDir(){ m_fRenderTime = -1.0f; }
    
    float   m_fMaxInterpolateDistance2 = (1000.0f * 1000.0f);
    void    SetMaxInterpolateDistance(float fMax){m_fMaxInterpolateDistance2=fMax*fMax;}
    
    /** Czy mamy w ogole cokolwiek robic z pozycja na OnRender */
    boolean m_bCorrectPosOnRender   = false;
    
    /** faktyczna pozycja aktora (wg fizyki) */
    Vector  m_vActuallyPosition     = new Vector();
    /** pozycja wyliczona tylko do renderu*/
    Vector  m_vRenderPosition       = new Vector();
    /** czy aktualnie aktor jest ustawiony na pozycji specjalnie tylko do renderu?*/
    boolean m_bOnRenderPosition     = false;
    /** czy aktualnie interpolujemy pozycje pomiedzy poszczegolnymi renderami?*/
    boolean m_bInterpolateRenderPos = false;

    /** predkosc interpolowania pozycji podczas renderu gdy gracz stoi
     *  powinny byc wyliczane dla kazdego obiektu z osobna
     *  dla gracza jest to konstruktorze na podstawie predkosci gracza 
     *  dla pojazdu na razie jest ro co ponizej
     */
    float m_fInterpolateSpeedOnStop = 300.0f;
    /**
     * Wspolczynnik predkosci z jaka mozna interpolowac
     * aktora (w stosunku do aktualnej/biezacej) predkosci
     */
    float m_fInterpolateSpeedOnMoveMul = 1.2f;
    /** Pomocnicze statiki do liczenie render position */
    static Vector sm_vPos           = new Vector();
    static Vector sm_vSpeed         = new Vector();
    static Vector sm_vMove          = new Vector();

    void SetCorrectPosOnRender( boolean bVal )
    {
        m_bCorrectPosOnRender   = bVal;
        m_bInterpolateRenderPos = bVal;
        m_bInterpolateRenderDir = bVal;
        m_fRenderTime           = -1.0f;
    }
    
    /** czy obiekt dokonuje wyliczenia specjalnego kierunku na render?*/
    boolean m_bInterpolateRenderDir = false;
    /** faktycznY kierunek gracza (wg fizyki) */
    Vector  m_vActuallyDir          = new Vector();
    /** kierunek goscia wyliczona tylko do renderu*/
    Vector  m_vRenderDir            = new Vector();
    /** czy aktualnie  ma ustawiony kierunek specjalnie tylko do renderu?*/
    boolean m_bOnRenderDir          = false;
    /** Pomocniczy static do liczenie render dir*/
    static 
    Vector sm_vDir                  = new Vector();
    //Predkosc (stopnie/sekunde) z jaka interpolowany jest dir gracza/pojazdu
    float  sm_fAngSpeed             = 360.0f;

    
    
    void InterpolateRenderPos(Vector vPos)
    {
        //pozycje do renderowania mozna interpolowac
        if ( m_fRenderTime > 0.0f              && 
             (GetParent() == null)             &&      
             vPos.Distance2( m_vRenderPosition )< m_fMaxInterpolateDistance2 )
        {
            //obiekt moze sie poruszac z jakas zalozona predkoscia
            //i jesli ja przekraczyl (przestawienie) to jest "zwalniany"
                
            float fFrameTime     = GetTime() - m_fRenderTime;
            //czy od poprzedniej ramki minal jakis czas?
            if ( fFrameTime > 0.0f )
            {
                //na jaka maksymalna odleglosc mogl sie
                //poruszyc pojazd od poprzedniej ramki?
                float fSpeed         = GetInterpolateSpeed();
                if ( fSpeed <= m_fInterpolateSpeedOnStop )
                     fSpeed  = m_fInterpolateSpeedOnStop;
                float fMaxDistance   = fSpeed * fFrameTime;
                //policzmy o ile sie faktycznie poruszyl
                sm_vMove.Set( vPos );
                sm_vMove.Sub( m_vRenderPosition );
                // czy nie czasem wiecej niz mogl?
                if ( sm_vMove.Len() > fMaxDistance )
                {
                    //jesli tak, to pozwalamy mu tylko tyle 
                    //ile sobie zalozylismy
                    sm_vMove.Normalize();
                    sm_vMove.Mul( fMaxDistance );
                }
                
                //no i zwrocmy koncowa pozycje
                vPos.Set( m_vRenderPosition );
                vPos.Add( sm_vMove );
            }
        }
    }

    /**
     * Interpolacja kierunku na potrzeby renderingu
     * Wykorzystywana w graczach i pojazdach (sieciowycj)
     * Metoda jest identyczna w obu przypadkach
     */
    void InterpolateRenderDir( Vector vDir,Vector vDestDir, Vector vRenderDir )
    {
        if ( m_bInterpolateRenderDir &&
            (m_fRenderTime > 0.0f)   &&
            (GetParent()  == null)   &&
            ( ( vDir.fX * vDestDir.fX + vDir.fZ * vDestDir.fZ < 0.0f ) 
               ||
              ( Math.abs(vDir.fX * vDestDir.fZ - vDir.fZ * vDestDir.fX) > 0.001f ) 
            )  
           )
        {
            float fDirAngle     = (float)Math.atan2( vDir.fZ,     vDir.fX     );
            float fDestDirAngle = (float)Math.atan2( vDestDir.fZ, vDestDir.fX );
            
            float fInterval     = GetTime() - m_fRenderTime;
            float fTurn         = sm_fAngSpeed * ((float)Math.PI / 180.0f) * fInterval;
                        

            float fAngle = fDestDirAngle - fDirAngle;
            if (fAngle >  Math.PI)
                fAngle -= 2*Math.PI;
            else
            if (fAngle <  -Math.PI)
                fAngle += 2*Math.PI;

            if ( Math.abs( fAngle ) > fTurn )
            {
                // obracamy
                if ( fAngle < 0.0f )
                    fTurn = -fTurn;
                
                float fSin = (float)Math.sin( fTurn );
                float fCos = (float)Math.cos( fTurn );

                vRenderDir.fX = vDir.fX * fCos - vDir.fZ * fSin;
                vRenderDir.fZ = vDir.fX * fSin + vDir.fZ * fCos;
                //TODO: dorobic interpolacje po Y
                vRenderDir.fY = vDestDir.fY;
            }
            else
            {
                // koniec interpolowania
                vRenderDir.Set( vDestDir );
            }
        }
        else
        {
            //nie trzeba interpolowac
            vRenderDir.Set( vDestDir );
        }
    }
    
    /**
     * Domyslnie interpolujemy z predkoscia taka
     * jakby gracz stal
     */
    float GetInterpolateSpeed()
    {
        return m_fInterpolateSpeedOnStop;
    }

    /** Ustawnienie pozycji na potrzeby renderingu
     */
    void OnRender()
    {
    }
    
    /**
     * Przywrocenie pozycji z przed renderu (jesli trzeba)
     * wspolne dla gracza i pojazdu, wiec zostawiem tu
     */
    void OnPostRender()
    {
        /**
         * Po renderze przywracamy stan obiektu
         * ktory zastalismy w OnRender         * 
         */
        
        //jezeli jestesmy w pozycji specjalnej tylko do renderu
        if ( m_bOnRenderPosition )
        {
            // to przypracamy faktyczna pozycje
            SetPositionWorld( m_vActuallyPosition );
            m_bOnRenderPosition = false;
        }
        
        //jezeli jestesmy w pozycji specjalnej tylko do renderu
        if ( m_bOnRenderDir )
        {
            // to przypracamy faktyczna pozycje
            SetDir( m_vActuallyDir );
            m_bOnRenderDir = false;
        }
    }

 
    String GetDemoInfoText()
    {
        String  sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO&";
        
        if (this instanceof WeaponPistolHeavy)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_PISTOL_HEAVY&";
        if (this instanceof WeaponRifleHeavy)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_RIFLE_HEAVY&";
        if (this instanceof WeaponRifleShortSilenced)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_RIFLE_SHORT_SILENCED&";
        if (this instanceof WeaponRifleSniper)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_RIFLE_SNIPER&";
        if (this instanceof WeaponRifleSniperHeavy)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_RIFLE_SNIPER_HEAVY&";
        if (this instanceof WeaponRifleMachineGun)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_RIFLE_MACHINEGUN&";
        if (this instanceof WeaponHeavyRocketLauncher)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_ROCKET_LAUNCHER&";
        if (this instanceof WeaponHeavyElectro)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_ELECTRO&";
        if (this instanceof Speeder)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_SPEEDER&";
        if (this instanceof CloakingDevice)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_CLOAKING_DEVICE&";
        if (this instanceof WeaponGrenadeCluster)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_GRENADE_CLUSTER&";
        if (this instanceof WalkerLight)
            sDemoVersionInfo = "&UNAVAILABLE_IN_DEMO_WALKER&";
        return sDemoVersionInfo;
    }
    
    
    boolean IsAvailableInDemo()
    {
        if (!(GetModule() instanceof ModuleNet) &&
            ChromeModuleSingle._bEnglishDemo1 &&
                (
                this instanceof WeaponPistolHeavy ||
                this instanceof WeaponRifleHeavy ||
                this instanceof WeaponRifleShortSilenced ||
                this instanceof WeaponRifleSniper ||
                this instanceof WeaponRifleSniperHeavy ||
                this instanceof WeaponRifleMachineGun ||
                this instanceof WeaponHeavyRocketLauncher ||
                this instanceof WeaponHeavyElectro ||
                this instanceof Speeder ||
                this instanceof CloakingDevice ||
                this instanceof WeaponGrenadeCluster ||
                this instanceof WalkerLight)
                )
            
            return false;
        else
            return true;
    }

}
    

class TermoElement
{
    String sMeshElementName = null;
    Vector vScale = null;
    TermoElement(String sMeshElementName, Vector vScale)
    {
        this.vScale = vScale;
        this.sMeshElementName = new String(sMeshElementName);
    }
    
    int nElementID = -1;
    String sFXName = "";
    TermoElement(int nId, Vector vScale)
    {
        nElementID = nId;
        this.vScale = new Vector(vScale);
    }
    
}