public class ForceFieldFence extends MeshObject implements iact_EnableDisable
{
    /** Znacznik, ze siatka ma byc wlaczona na starcie */
    public boolean bIsEnabled = true;//#FORCEFIELDFENCE;Initial state of fence

    /** Jaki damage dostaje player
     */
    public float fPlayerDamage = 300;//#FORCEFIELDFENCE

    /** Jaka ma byc sila odskoczenia gracza od fence'a w gore
     */
    public float fPlayerJumpUp = 1300;//#FORCEFIELDFENCE
    
    /** Jaka ma byc sila odskoczenia gracza od fence'a w tyl
     */
    public float fPlayerJumpHoriz = 400;//#FORCEFIELDFENCE

    /** FX odpalany przy zetknieciu czegos z plotem
     */
    public String sFXTouched = "FXBlowFence";//#FX
    
    /** Dzwiek odpalany przy zetknieciu czegos z plotem */
    public String sTouchedSound = "Data/Sounds/RedBarriersShock.wav";//#FX
    
    /** Dzwiek odpalany ciagle */
    public String sAmbientSound = "Data/Sounds/RedBarriersLoop.wav";//#FX
    
    int nAmbientLoop = 0;
	
	boolean bCurrEnabled = false;
    
    void OnCreate()
    {
        super.OnCreate();
		bSG = true;
        SetFlags(EFlags._BLOCK_MOVE | EFlags._NO_ANIMATION);
        SetPhysicsActive(false);
        SetCollType(ECollFlags.OBJCOL_TYPE_PERFACEOBJECT);
        SetCollHandler(ECollFlags.OBJCOL_HANDLER_FULLINFO); //full, bo moze kiedys bedzie sie tez obiekt odsuwalo
        if (bIsEnabled)
            CallMethodIn("Enable", 0f);
    }
    
    void AttachAmbient()
    {
        // podpinamy dzwiek
        if (nAmbientLoop == 0)
            nAmbientLoop = AttachSound(sAmbientSound, 800.0f, 1200.0f);
    }
    
    void DetachAmbient()
    {
        if (nAmbientLoop != 0)
            DetachSound(nAmbientLoop);
        nAmbientLoop = 0;
    }
    
    void OnDestroy()
    {
        DetachAmbient();
    }
    
    
    public void OnCollisionFullInfo(ControlObject Col,Vector vPoint,Vector vNormal)
    {
        if (Col instanceof Actor)
        {
            Actor cActor = (Actor)Col;
            Vector vFXPoint = new Vector(vPoint);
            if (cActor.fHealth > 0)
            {
                if (cActor instanceof Vehicle)
                {
                    Vehicle vehicle = (Vehicle) cActor;
                    vehicle.TakeDamage(fPlayerDamage, EnumDamageType._BULLET);	
                }
                else
                if (cActor instanceof Player)
                {
                    Player cPlayer = (Player)cActor;
                        
                    cPlayer.TakeDamage(fPlayerDamage, fPlayerDamage, this, new Vector(Vector.Up), vNormal,
									   EBones._NON_BONE, EnumDamageType._ELECTRIC);
                        
                    //cPlayer.SetMaxFallSpeed(new Vector(0, 0, 0));
                    cPlayer.SetFallSpeed(new Vector(0, 0, 0));
                        
                    //LogD("SetLocalSpeed");
                    cPlayer.SetLocalSpeed(new Vector(), false);
                        
                    Vector vJump = new Vector(vNormal);
                    vJump.fY = fPlayerJumpUp;
                    vJump.fX *= fPlayerJumpHoriz;
                    vJump.fZ *= fPlayerJumpHoriz;
                    cPlayer.Jump(vJump);
                    vFXPoint.fY += 50; //zeby lepiej widzial...
                }
                else
                if (cActor instanceof InvObject)
                {
                    if (GetTime() - ((InvObject)cActor).fLastForceFieldCollision > 1f)
                    {
                        cActor.TakeDamage(cActor.fHealth * 10, cActor.fHealth * 10,  this, new Vector(Vector.Up), vNormal,
									  EBones._NON_BONE, EnumDamageType._ELECTRIC);
                        ((InvObject)cActor).fLastForceFieldCollision = GetTime();
                    }
                    else 
                        return;
                    
                }
                else
                    cActor.TakeDamage(cActor.fHealth * 10, cActor.fHealth * 10,  this, new Vector(Vector.Up), vNormal,
									  EBones._NON_BONE, EnumDamageType._ELECTRIC);
                
                ExecuteFX(sFXTouched, vFXPoint, Vector.Up);
                ExecuteParticleSystem(sFXTouched, vFXPoint, Vector.Up);
                PlaySound(sTouchedSound);
            }
        }
        else
            CrashLog("WARNING: ForceFieldFence - collision with NON-Actor object\n");
    }
    

    
    public void EnableForceField(boolean bEnable)
    {
		bCurrEnabled = bEnable;
        EnableRendering(bEnable);
        if (bEnable)
        {
            SetFlag(EFlags._COLLISION);
            ActivateOnModule();
            AttachAmbient();
        }
        else
        {
            ClearFlag(EFlags._COLLISION);
            DetachAmbient();
            DeactivateOnModuleAlways();
        }
    }
    
    public void Enable()
    {
        DisableCallMethod("Enable");        
        DisableCallMethod("Disable");        
        EnableForceField(true);
    }
    
    public void Disable()
    {
        DisableCallMethod("Enable");        
        DisableCallMethod("Disable");        
        EnableForceField(false);
    }
    
    public boolean IsEnabled()
    {
        return bIsEnabled;
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
	boolean bCurrEnabledSG = false;
    
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[ForceFieldFence.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunksChrome._CHK_FORCE_FIELD_FENCE)
        {
			super.SGLoadChunk(cFC);
			
			// Znacznik, ze siatka ma byc wlaczona na starcie 
			bCurrEnabledSG = cFC.LoadBool();	
        }
		
		cFC.delete();
    }

    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[ForceFieldFence.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_FORCE_FIELD_FENCE, cFCParent);
        
        super.SGSaveChunk(cFC);

        // Znacznik, ze siatka ma byc wlaczona na starcie 
        cFC.SaveBool(bCurrEnabled);		
        
        cFC.delete();
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[ForceFieldFence.SGUpdate]\n");
		super.SGUpdate();

		DetachAmbient();
        if (bCurrEnabledSG)
            CallMethodIn("Enable", 0f);
		else
			CallMethodIn("Disable", 0f);
        
    }
    
    
    public void OnAudioReinitialize()
    {
        
        if (nAmbientLoop != 0)
        {
            nAmbientLoop = 0;
            AttachAmbient();
        }

    }
}
