/** klasa obslugujaca wyswitlanie ramek aktywnego triggera
 */
public class ActiveTriggerFrames extends GameObject
{
	private static float _CORNER_SPRITE_SIZE = 13f;
	
    float m_fFramesMovementSpeed = 15f;
    
    /** kolor tekstu z tipsem */
	static Vector _TIP_COL = new Vector(1f, 1f, 1f);
	
    /** czas pomiedzy update predkosci pulsowania ramek */
	static float _ANIM_SPEED = 0.05f;
	
    /** na jaka odleglosc (wzgldem wyjsciowych rozmiarow) maja rozjezdzac sie ramki - w pixelach*/
	static float _FRAMES_MOVEMENT = 6.0f;
	
	static Vector _TIP_SIZE = new Vector(50f, 15f, 0);
	
    /** sprity otaczajace aktywnego triggera UL, UR, DR, DL */
    //Sprite m_TriggerSprites[] = new Sprite[4];

    UIWindow m_cTriggerWindow = null;
    
    /** sprite z tipsem - co mozna zrobic z aktywnym triggerem */
	Sprite m_TriggerTip = null;
    
    Sprite m_AdditionalTip = null;
    
    /** ekstenty aktywnego triggera */
    Vector m_vTriggerLU = new Vector(0, 0, 0);
    Vector m_vTriggerRD = new Vector(0, 0, 0);
    
    /** aktualne przesuniecie ramek */
    float m_fActualFramesMovement = 0f;
    
    /** aktualny kierunek zmiany rozmiaru okna (1 - rosnie, -1 - maleje) */
    float m_fActualMovementDirection = 1f;
    
    /** referencja do aktywnego trigger objecta */
    TriggerObject m_TriggerObject = null;
    
    /** referencja do kamery */
    Camera m_Camera = null;

	float m_fScreenWidth = 0;
	float m_fScreenHeight = 0;
	String m_TipText = "";
    String m_AdditionalTipText = "";
	
    static int GetCornerSpriteSize()
    {
        return ((int)((float)_CORNER_SPRITE_SIZE * Game.GetScreenWidth()/800f));
    }
    
    public void OnCreate()
    {
        m_cTriggerWindow = (UIWindow)CreateObject("UIWindow");
        AddSprite(m_cTriggerWindow);
        UITexture cTexture = new UITexture("data/hud/ch_p_hud.png", 1024, 512, 486, 236, 514, 264);
        m_cTriggerWindow.m_vFrameCornerSize = new Vector (_CORNER_SPRITE_SIZE, _CORNER_SPRITE_SIZE, 0);
        m_cTriggerWindow.LoadTexture(cTexture);
        m_cTriggerWindow.SetNormalTexture(cTexture);
        m_cTriggerWindow.m_sId = "TRIGGER_FRAME";
        m_cTriggerWindow.m_bInited = true;
        m_cTriggerWindow.OnTextureChange();
		
		m_TriggerTip = (Sprite) CreateObject("Sprite");
		m_TriggerTip.SetNoTexture();
		m_TriggerTip.ProcessMouse(false);
		m_TriggerTip.MoveTop();
		m_TriggerTip.SetTextColor(_TIP_COL.fX, _TIP_COL.fY, _TIP_COL.fZ, 1.0f);
		m_TriggerTip.SetSize(new Vector(50f, 50f, 0f));
		AddSprite(m_TriggerTip);
        
        m_AdditionalTip = (Sprite) CreateObject("Sprite");
		m_AdditionalTip.SetNoTexture();
		m_AdditionalTip.ProcessMouse(false);
		m_AdditionalTip.MoveTop();
        m_AdditionalTip.SetTextColor(_TIP_COL.fX, _TIP_COL.fY, _TIP_COL.fZ, 0.75f);
		m_AdditionalTip.SetSize(new Vector(50f, 50f, 0f));
        m_AdditionalTip.SetTextPosFlags(273);
		AddSprite(m_AdditionalTip);
    }
	
    /** metoda wyswietla ramki dla podanego trigger objecta 
     */
    public void DisplayTriggerFrames(TriggerObject trigger, Camera camera)
    {
        m_fActualFramesMovement = 0.0f;
        m_fActualMovementDirection = 1.0f;
        
        m_TriggerObject = trigger;
        m_Camera = camera;
        m_TriggerObject.GetTriggerExtents(camera, m_vTriggerLU, m_vTriggerRD);

		m_fScreenWidth = GetScreenWidth();
		m_fScreenHeight = GetScreenHeight();

		FitTriggersToScreen();
		
        m_TriggerTip.SetFont(GetModule().fonts.GetProperFont(ChromeFonts.FONT_TRIGGER_TIP));
        m_TipText = m_TriggerObject.GetTip();
       
        m_AdditionalTip.SetFont(GetModule().fonts.GetProperFont(ChromeFonts.FONT_TRIGGER_TIP));
        m_AdditionalTipText = m_TriggerObject.GetAdditionalTip();
        
        
		UpdateFramesPosition();
        
        ActivateOnModule();//
        //CallMethodEvery("AnimateFrames", _ANIM_SPEED);   
    }

	/** metoda dbajaca, by ramki nie wyjechaly za ekran
	 */
	void FitTriggersToScreen()
	{
		FitExtent(m_vTriggerLU);
		FitExtent(m_vTriggerRD);
	}
	
	/** metoda dbajaca, by ramki nie wyjechaly za ekran
	 */
	void FitExtent(Vector vScreenPos)
	{
		if (vScreenPos.fX < m_fScreenWidth / 4)
			vScreenPos.fX = m_fScreenWidth / 4;
        if (vScreenPos.fX > m_fScreenWidth - m_fScreenWidth / 4)
			vScreenPos.fX = m_fScreenWidth - m_fScreenWidth / 4;
		if (vScreenPos.fY < m_fScreenHeight / 4)
			vScreenPos.fY = m_fScreenHeight / 4;
        if (vScreenPos.fY > m_fScreenHeight - m_fScreenHeight / 4)
			vScreenPos.fY = m_fScreenHeight - m_fScreenHeight / 4;
	}
	
    /** chowanie ramek aktywnego triggera
     */
    public void HideTriggerFrames()
    {
        DisableCallMethod("AnimateFrames");
		/*for (int i = 0; i < 4; i++)
			m_TriggerSprites[i].Hide();		*/
        m_cTriggerWindow.Hide();
        m_TriggerTip.Hide();
        m_AdditionalTip.Hide();
        DeactivateOnModule();
    }
	
    public void OnPostUpdate()
    {
        AnimateFrames();
        //UpdateFramesPosition();
    }
    
    float m_fPrevTime = 0;
    
    /** pulsowanie ramek otaczajacych aktywnego triggera
     */
    void AnimateFrames()
    {
        float fTime = GetTime();
        float fTimeDelta = fTime - m_fPrevTime;
        m_fPrevTime = fTime;
        m_fActualFramesMovement += fTimeDelta * m_fFramesMovementSpeed * m_fActualMovementDirection;
        
        if (m_fActualFramesMovement > _FRAMES_MOVEMENT)
        {
            m_fActualFramesMovement = _FRAMES_MOVEMENT;
            m_fActualMovementDirection *= -1f;
        }
		else
		{
            if (m_fActualFramesMovement < 0f)
            {
                m_fActualFramesMovement = 0f;
                m_fActualMovementDirection *= -1f;
            }
		}
		
        m_TriggerObject.GetTriggerExtents(m_Camera, m_vTriggerLU, m_vTriggerRD);
        
		FitTriggersToScreen();
		
        m_vTriggerLU = new Vector(m_vTriggerLU.fX - m_fActualFramesMovement,
								  m_vTriggerLU.fY - m_fActualFramesMovement,
								  0f);
        m_vTriggerRD = new Vector(m_vTriggerRD.fX + m_fActualFramesMovement,
								  m_vTriggerRD.fY + m_fActualFramesMovement,
								  0f);
        UpdateFramesPosition();
    }
	
    void UpdateFramesPosition()
    {
        // pozycja tipsa
        m_TriggerTip.SetPos(new Vector(m_vTriggerLU.fX,
									   m_vTriggerRD.fY,
									   0f));
        
                // pozycje i ramek
		Vector vPos = new Vector();
		
		vPos.Set(m_vTriggerLU);
		//vPos.fX -= GetCornerSpriteSize();
		//vPos.fY -= GetCornerSpriteSize();
        
        m_cTriggerWindow.SetPos(vPos);
        Vector vSize = m_vTriggerRD.Sub2(m_vTriggerLU);
        
        if (vSize.fX < 2 * GetCornerSpriteSize() + 1)
            vSize.fX = 2 * GetCornerSpriteSize() + 1;
        if (vSize.fY < 2 * GetCornerSpriteSize() + 1)
            vSize.fY = 2 * GetCornerSpriteSize() + 1;
        
        m_AdditionalTip.SetPos(new Vector(m_vTriggerLU.fX + 10, m_vTriggerLU.fY + 5, 0));
        m_AdditionalTip.SetSize(vSize);
        
        m_cTriggerWindow.UpdateLogicSizeAndPos(vSize, vPos);
        m_cTriggerWindow.OnSizeAndPos();
				
        m_TipText = m_TriggerObject.GetTip();
        m_AdditionalTipText = m_TriggerObject.GetAdditionalTip();
        
        // FIXIT, tymczasowo do testow
        Module cModule = m_TriggerObject.GetModule();
        if (cModule instanceof ModuleNet )
        {
            m_TipText = m_TipText +
					    ((ModuleNet) cModule).GetSpecialTip(m_TriggerObject);
        }
        m_TriggerTip.DisplayText(m_TipText);
		m_AdditionalTip.DisplayText(m_AdditionalTipText);
		/*for (int i = 0; i < 4; i++)
			m_TriggerSprites[i].Show();		*/
        
        m_cTriggerWindow.Show();
        m_TriggerTip.Show();
        m_AdditionalTip.Show();
    }
	
    public void OnDestroy()
    {
        m_cTriggerWindow.Remove();
        RemoveSprite(m_cTriggerWindow);
        m_cTriggerWindow.delete();
        m_cTriggerWindow = null;
		
		RemoveSprite(m_TriggerTip);
		m_TriggerTip.delete();
		m_TriggerTip = null;
		
        RemoveSprite(m_AdditionalTip);
        m_AdditionalTip.delete();
        m_AdditionalTip = null;
        
	}	
}
