/** klasa zarzadzajaca systemem waypointow
 */
public class HUDWaypointsManager
{
    /** lista z glownymi waypointami */
    protected java.util.Vector m_cWaypoints = null;
    
    /////////////////////////////
    // METODY
    
    /** dodawanie glownego waypointa 
     * @param sName nazwa
     * @param vPos pozycja
     * @param vActive czy ma byc widoczny na mapie
     */
    int AddWaypoint( String sName, Vector vPos, boolean bActive, DefLocation cLocation )
    {
        if( m_cWaypoints == null )
        {
            m_cWaypoints = new java.util.Vector();
        }
        return AddWaypoint( sName, vPos, bActive, cLocation, m_cWaypoints );
    }
    /** dodawanie waypointa
     * @param sName nazwa waypointa
     * @param vPos pozycja
     * @param bActive czy ma byc aktywny - pokazywany na mapie
     * @param cList lista do ktorej ma byc dodany
     */
    protected int AddWaypoint( String sName, Vector vPos, boolean bActive, DefLocation cLocation, java.util.Vector cList )
    {
        if( cList == null )
        {
            cList = new java.util.Vector();
        }
        
        HUDWaypoint cElem = null;
        
        if (cLocation != null)
            cElem = new HUDWaypoint(cLocation);
        else
            cElem = new HUDWaypoint( sName, vPos, bActive );
        
        cList.addElement( cElem );
        
        return cList.indexOf( cElem );
    }
    /** pobieranie glownego waypointa o zadanym indeksie
     * @param nIndex indeks waypointa
     */
    protected HUDWaypoint GetWaypoint( int nIndex )
    {
        return GetElem( nIndex, m_cWaypoints );
    }
    
    HUDWaypoint FindWaypoint(DefLocation cLocation)
    {
        if (m_cWaypoints == null)
            return null;
        
        HUDWaypoint cPoint = null;
        for (int i = 0; i < m_cWaypoints.size(); i++)
        {
            if (((HUDWaypoint)m_cWaypoints.elementAt(i)).GetLocation() == cLocation)
            {
                cPoint = (HUDWaypoint)m_cWaypoints.elementAt(i);
                break;
            }
        }        
        return cPoint;
    }
    
    void SetWaypointRotorIdx(int nWaypoint, int nRotor)
    {
        ((HUDWaypoint)m_cWaypoints.elementAt(nWaypoint)).SetRotorIdx(nRotor);
    }
    
    void RemoveWaypoint(DefLocation cLocation)
    {
        HUDWaypoint cPoint = FindWaypoint(cLocation);
        if (cPoint != null)
            m_cWaypoints.removeElement(cPoint);
    }
    /** pobieranie waypointa o zadanym indeksie
     * @param nIndex indeks waypointa
     * @param cList lista z ktorej ma byc pobrany
     */
    protected HUDWaypoint GetElem( int nIndex, java.util.Vector cList )
    {
        if( cList != null )
        {
            if( nIndex >= 0 && nIndex < cList.size() )
            {
                return (HUDWaypoint)cList.elementAt( nIndex );
            }
            GameObject.CrashLog("HUDWaypointsManager::GetElem - zly indeks");
            return null;
        }
        GameObject.CrashLog("HUDWaypointsManager::GetElem - lista jest null");
        return null;
    }
    /** pobieranie nazwy glownego waypointa o zadanym indeksie */
    String GetWaypointName( int nIndex )
    {
        return GetElemName( nIndex, m_cWaypoints );
    }
    /** pobieranie nazwy waypointa o zadanym indeksie */
    protected String GetElemName( int nIndex, java.util.Vector cList )
    {
        return GetElem( nIndex, cList ).GetElemName();
    }
    /** pobieranie pozycji glownego waypointa o zadanym indeksie */
    Vector GetWaypointPos( int nIndex )
    {
        return GetElemPos( nIndex, m_cWaypoints );
    }
    /** pobieranie pozycji waypointa o zadanym indeksie */
    protected Vector GetElemPos( int nIndex, java.util.Vector cList )
    {
        return GetElem( nIndex, cList ).GetElemPos();
    }
    /** zmiana pozycji waypointa o zadanym indeksie */
    void SetWaypointPos( int nIndex, Vector vPos )
    {
        SetElemPos( nIndex, m_cWaypoints, vPos );
    }
    protected void SetElemPos( int nIndex, java.util.Vector cList, Vector vPos )
    {
        GetElem( nIndex, cList ).SetElemPos( vPos );
    }
    /** czy glowny waypoint o zadanym indeksie jest aktywny */
    boolean IsWaypointActive( int nIndex )
    {
        return IsElemActive( nIndex, m_cWaypoints );
    }
    /** czy waypoint o zadanym indeksie jest aktywny */
    protected boolean IsElemActive( int nIndex, java.util.Vector cList )
    {
        return GetElem( nIndex, cList ).IsElemActive();
    }
    
    
    /////////////////////////////
    // SPRZATANIE
    /** sprzatanie */
    void delete()
    {
        // usuniecie listy glownych waypointow
        if( m_cWaypoints != null )
        {
            if( m_cWaypoints.size() > 0 )
            {
                for( int i = 0; i < m_cWaypoints.size(); i++ )
                {
                    ((HUDWaypoint)m_cWaypoints.elementAt( i )).delete();
                }
            }
        }
    }
    
    /*void ShowLocation(DefLocation cLocation)
    {
        HUDWaypoint cPoint = null;
        for (int i = 0; i < m_cWaypoints.size(); i++)
        {
            if (((HUDWaypoint)m_cWaypoints.elementAt(i)).GetLocation() == cLocation)
            {
                cPoint = (HUDWaypoint)m_cWaypoints.elementAt(i);
                break;
            }
        }
        if (cPoint == null)
            cPoint = new HUDWaypoint(cLocation);
        
        m_cWaypoints.addElement(cPoint);
        
        cPoint.SetElemActive(true);
        
    }*/
    
    void HideLocation(DefLocation cLocation)
    {
        
        HUDWaypoint cPoint = FindWaypoint(cLocation);
        if (cPoint == null)
            return;
        
        cPoint.SetElemActive(false);
    }
    
    /////////////////////////////
    // POJEDYNCZY WAYPOINT
    
    public class HUDWaypoint
    {
        /** nazwa */
        protected String m_sName = null;
        /** pozycja */
        protected Vector m_vPos = null;
        /** flaga aktywnosci */
        protected boolean m_bActive = false;
        
        /**numerek rotora w kompasie, potrzebne do usuwania*/
        protected int m_nRotorIdx = -1;
        
        /**Jesli waypoint jest lokacja edytorowa to tu jest do niej referencja*/
        protected DefLocation m_cLocation = null;
        //////////////////////
        // METODY
        
        /** konstruktor */
        HUDWaypoint()
        {
        }
        
        HUDWaypoint( String sName, Vector vPos, boolean bActive )
        {
            m_sName = new String( sName );
            m_vPos = new Vector(vPos );
            m_bActive = bActive;
        }
        
        HUDWaypoint(DefLocation cLocation)
        {
            m_sName = new String( cLocation.m_sText);
            m_vPos = cLocation.m_vWorldPos;
            m_bActive = cLocation.IsEnabled();
            m_cLocation = cLocation;
        }
        
        /** nadanie nazwy */
        void SetName( String sName )
        {
            m_sName = new String( sName );
        }
        /** ustawienie pozycji */
        void SetElemPos( Vector vPos )
        {
            m_vPos = new Vector( vPos );
        }
        /** ustawianie flagi aktywnosci */
        void SetElemActive( boolean bActive )
        {
            m_bActive = bActive;
        }
        /** pobranie nazwy */
        String GetElemName()
        {
            return m_sName;
        }
        /** pobranie pozycji */
        Vector GetElemPos()
        {
            return m_vPos;
        }
        /** pobranie flagi aktywnosci */
        boolean IsElemActive()
        {
            return m_bActive;
        }
        
        int GetRotorIdx()
        {
            return m_nRotorIdx;
        }
        
        void SetRotorIdx(int nRotor)
        {
            m_nRotorIdx = nRotor;
        }
        
        DefLocation GetLocation()
        {
            return m_cLocation;
        }
        //////////////////////
        // SPRZATANIE
        
        /** sprzatanie */
        void delete()
        {
            if( m_sName != null)
                m_sName = null;
            if( m_vPos != null)
                m_vPos = null;
        }
    }
}
