/** klasa reprezentujaca kompas na HUD'zie
 */
public class HUDCompass extends HUDComponent
{
    /////////////////////////////
    // USTAWIENIA DLA PIRAMIDKI
    /** liczba elementow w piramidzie */
    protected final static int    _NUMBER_OF_ELEMENTS = 3;
    /** identyfikator elementu */
    protected final static String _ELEM_ID            = "ID_PYR_";
    //
    /////////////////////////////
    /** wektor punktu docelowego */
	protected Vector DestPosition = new Vector();
	/** textury map dla leveli */
    protected final static String     _MAP_L1         = "Data/HUD/map_l00.png";
    /** identyfikator elemenut mapy */
    protected final static int        _RING           = 1,
                                    _MAP            = 2;
    /** indeksy leveli */
    protected final static int        _LEVEL_0        = 0,
                                    _LEVEL_1        = 1,
                                    _LEVEL_2        = 2,
                                    _LEVEL_3        = 3;
    /** ustawienia zooma */
    protected final static float      _ZOOM_MIN       = 0.2f,
                                    _ZOOM_NORMAL    = 0.4f,
                                    _ZOOM_MAX       = 0.7f;
    
    /** rozmiary tekstur map dla leveli */
    protected final static float      _L0_MAP_WIDTH   = 256,
                                    _L0_MAP_HEIGHT  = 256;
    
    /** promien wewnetrzny mapy */
    final static float              _MAP_IN_RADIUS  = 126f / 2f;
    
    /** liczba cyfr na ilu ma byc wyswietlany dystans do wybranego waypointa */
    protected final static int        _DISTANCE_CHR   = 4;
    
    protected final static float      _DISTANCE_GRAIN = 999f;
    
    static float _MARGIN = 0;
    
    /** skok zoom'a */
    protected float m_fZoomCoef = 0.1f;
    /** aktualny level */
    protected int m_nCurrentLevel = -1;
    /** id tekstury mapy */
    protected int m_nTexId = -1;
    /** aktualny zoom */
    protected float m_fCurrentZoom = _ZOOM_NORMAL;
    /** teksty dla waypointow */
    protected UIRotorText m_cRotorText = null;
    /** menedzer waypointow */
    protected HUDWaypointsManager m_cWaypointsManager = null;
    /** dystans do wybranego waypointa */
    protected UIStatic m_cDistance = null;
    /** aktywny waypoint */
    //protected int m_nCurrentWaypoint = -1;
    /** czy mapa zostala zainicjowana */
    protected boolean m_bMapInited = false;
    /** odleglosc do waypointa w terenie */
    protected float m_fDistance = -1f;
    /** dystans pomiedzy graczem a waypointem */
    protected Vector m_vDistance = null;
    /** wektor w prawo playera */
    protected Vector m_vRight = null;
    /** kat polozenia mapy */
    protected float m_fMapAngle = 0.0f;
    /** flaga updateow waypointow */
    boolean m_bWaypointsAreUpdated = false;
    
    
    /////////////////////////
    // wektory pozycji, kierunku gracza i waypontow
    
    /** pozycja gracza */
    protected Vector m_vPlayerPos = null;
    /** kierunek gracza */
    protected Vector m_vPlayerForward = null;
    /** pozycja punktu docelowego - kolejny waypont */
    protected Vector m_vDestPos = null;
    /** wektor polnocy */
    protected Vector m_vNorth = null;
    /** rozmiar terenu */
    protected Vector m_vLevelSize = null;
    /** koordynaty mapy */
    protected Rect m_cMapCoords = new Rect();
    /** rozmiar tekstury mapy */
    protected Vector m_vMapTexSize = new Vector();
    /** odlegosc przy jakiej waypoint ma sie znalezc wewnatrz mapki */
    protected float m_fMapRadius = -1f;
    /** id waypointa ustawianego przez gracza */
    protected int m_nPlayerWaypointID = -1;
    
    /**Skala mapy wynikajaca z dodatkowego marginesu*/
    float m_fMarginMapScale = 1;
    
    String m_sScriptName = "data/hud/HUDCompass.scr";
    
    /** inicjowanie kompasu */
    public void OnInit()
    {
        super.OnInit();
        LoadElements( _NUMBER_OF_ELEMENTS, _ELEM_ID);

        GetPosAndSizes();
        
        RegisterMouseAxisRequest();
        m_cRotorText = (UIRotorText)GetElem("ID_ROTOR_TEXT");
        m_cDistance = (UIStatic)GetElem("ID_DISTANCE");
        m_cWaypointsManager = CreateWaypointsManager();
        
        LoadLevelMap();
        UpdateMapRadiusLength();
        CalculateMapCoords();
        
        
        m_nPlayerWaypointID = AddWaypoint( "P", DestPosition, true );
        
        RotateMap( true );
        
        m_bMapInited = true;
        
        // Update pozycji playera na mapie, oparte na stalym ziarnie
        // zostalo to skonsultowane z Peh'em. Wiec jak cos, to jego mlaszczyc :o)
        //                                             4-03-2003 sloopek
        CallMethodEvery("SetPlayerPos", 0.084f);
        
    }
    
    HUDWaypointsManager CreateWaypointsManager()
    {
        return new HUDWaypointsManager();
    }
    
    
    public void SetDisortion(boolean bOn)
    {
        if (bOn)
            GetElem("ID_PYR_2").SetSpriteMaterial("mapMask.mat");
        else
            GetElem("ID_PYR_2").SetSpriteMaterial("Mask.mat");
    }
    
    float fAnglePlus = 0;//180f;
    /** obrocenie mapy */
    void RotateMap( boolean bUpdate )
    {
        GetPlayerAngle();
        
        GetPyramidElem( _MAP ).SetRotation( m_fMapAngle + fAnglePlus );
        GetPyramidElem( _RING ).SetRotation( m_fMapAngle + 180);
        
        if( bUpdate )
            UpdateWaypointPos();
    }
    /** zoom in */
    void MapZoomIn()
    {
        m_fCurrentZoom -= m_fZoomCoef;
        if( m_fCurrentZoom < _ZOOM_MIN)
            m_fCurrentZoom = _ZOOM_MIN;
        UpdateMapCoords();
        UpdateMapRadiusLength();
        if( !m_bWaypointsAreUpdated )
        {
            m_bWaypointsAreUpdated = true;
            UpdateWaypointPos();
            
        }
        
        m_bWaypointsAreUpdated = false;
        
    }
    
    /** zoom out */
    void MapZoomOut()
    {
        m_fCurrentZoom += m_fZoomCoef;
        if( m_fCurrentZoom > _ZOOM_MAX)
            m_fCurrentZoom = _ZOOM_MAX;
        UpdateMapCoords();
        UpdateMapRadiusLength();
        if( !m_bWaypointsAreUpdated )
        {
            m_bWaypointsAreUpdated = true;
            UpdateWaypointPos();
            
        }
        
        m_bWaypointsAreUpdated = false;
    }
    /** pobieranie pozycji gracza */
    protected void GetPosAndSizes()
    {
        if( cPlayer == null )
        {
			cPlayer = ((ChromeModuleSingle)GetModule()).MainPlayer;
            //CrashLog("HUDCompass.cPlayer jest null - nie moge pobrac rozmiarow terenu\n");
            //return;
        }
        // wektor polnocy
        m_vNorth = new Vector( 0, 0, 1f );
        // pozycja gracza
        m_vPlayerForward = new Vector( 0, 0, 1f );
        
        // pobieranie rozmiaru terenu
        m_vLevelSize = new Vector( cPlayer.GetTerrainSize() );
        // pobieranie pozycji gracza
        m_vPlayerPos = cPlayer.GetPosition();
        
        // ustalenie pozycji waypointa ustawianego przez gracza
        m_vDestPos = new Vector();
        
        GetPlayerAngle();
    }
    
    int AddWaypoint( String sName, Vector vPos, boolean bActive)
    {
        return AddWaypoint( sName, vPos, bActive , null);
    }
    
    int AddWaypoint(DefLocation cLocation)
    {
        return AddWaypoint( null, null, false , cLocation);
    }
    
    /** dodanie waypointa */
    int AddWaypoint( String sName, Vector vPos, boolean bActive , DefLocation cLocation)
    {
        int nIndex = -1;
        
        if( m_cRotorText == null )
        {
            CrashLog("UICompassAndMapIndicator::AddWaypoint() - nie jest zainicjowany system opisow waypointow; m_cRotorText == null");
            return -1;
        }
        
        if( m_cWaypointsManager == null )
        {
            m_cWaypointsManager = new HUDWaypointsManager();
        }

        if (m_cWaypointsManager.FindWaypoint(cLocation) != null)
            return -1;
        
        nIndex = m_cWaypointsManager.AddWaypoint( sName, vPos, bActive, cLocation );
        
        int nRotorIdx = m_cRotorText.AddElem( GetWaypointName( nIndex ), CalculateWaypointAngle( GetWaypointPos( nIndex ) ), cLocation);
        UIRotorText.RotorTextElement cElem = m_cRotorText.GetElem(nRotorIdx);
        m_cWaypointsManager.SetWaypointRotorIdx(nIndex, cElem.m_nIndex);
        
        if( m_nPlayerWaypointID < 0 )
        {
            m_nPlayerWaypointID = nIndex;
        }
        SetCurrentWaypoint();
        
        UpdateWaypointPos();
        
        return nIndex;
    }
    /** zwraca nazwe waypointa */
    protected String GetWaypointName( int nIndex )
    {
        if( m_cWaypointsManager != null )
            return m_cWaypointsManager.GetWaypointName( nIndex );
        else
        {
            CrashLog("HUDCompass::GetWaypointName - m_cWaypointsManager == null; nie moge pobrac nazwy\n");
            return null;
        }
    }
    /** wybranie aktualnego waypointa */
    protected void SetCurrentWaypoint()
    {
        for( int i = 0; i < m_cRotorText.GetElementsSize(); i++ )
        {
            if( i == m_nPlayerWaypointID )
            {
                m_cRotorText.MoveTopElement(i);
                m_cRotorText.SetElementTextColor( i, 1f, 1f, 1f, 1f );
            }
            else
            {
                m_cRotorText.SetElementTextColor( i, 0.8f, 0.8f, 0.8f, 0.8f );
            }
        }
    }
    /** wyliczenie poczatkowego kata */
    protected void GetPlayerAngle()
    {
        GetPlayerForward();
        
        if( m_vPlayerForward.fX < 0 )
            m_fMapAngle = 360f - (float)Tools.acos( m_vPlayerForward.fZ ) * 180f / (float)Math.PI;
        else
            m_fMapAngle = (float)Tools.acos( m_vPlayerForward.fZ ) * 180f / (float)Math.PI;
    }
    /** zaladowanie tekstury mapy */
    protected void LoadLevelMap()
    {
        String sMap = GetCurrentMap();
        Vector vSize = null;
        m_nTexId = GetPyramidElem( _MAP ).AddTexture( sMap );
        m_vMapTexSize.fX = GetPyramidElem( _MAP ).GetTextureWidth(m_nTexId);
        m_vMapTexSize.fZ = GetPyramidElem( _MAP ).GetTextureHeight(m_nTexId);
        _MARGIN = ((ChromeModule)GetModule()).nMapMargin;
        m_fMarginMapScale = m_vMapTexSize.fX/(m_vMapTexSize.fX - 2 * _MARGIN);
            
        if( sMap == null )
        {
            CrashLog("Brak tekstury");
            return;
        }
        
        GetPyramidElem( _MAP ).SetTexture( m_nTexId, m_cMapCoords );
    }
    /** wyliczanie kierunku gracza */
    protected void GetPlayerForward()
    {
        if( cPlayer == null )
        {
            m_vPlayerForward = new Vector();
        }
        else
        {
            //czy gracz jest w pojezdzie
            if (cPlayer.cSeatInfo != null && cPlayer.cSeatInfo.m_ObjectToOccupy != null)
            {
                //dla gunnera trzeba wyciagnac kierunek z cannona
                if (( cPlayer.cSeatInfo.m_ObjectToOccupy instanceof VehicleAttacker )&&
                    ( cPlayer.cSeatInfo.GetRole() == EPassengerRole._VEHICLE_PASSENGER))
                {
                    //gunner
                    VehicleAttacker cAttacker = (VehicleAttacker)cPlayer.cSeatInfo.m_ObjectToOccupy;

                    if ( cAttacker.cCannon != null )
                        m_vPlayerForward.Set( cAttacker.cCannon.GetCannonDir() );
                    else
                        m_vPlayerForward.Set( cPlayer.cSeatInfo.m_ObjectToOccupy.GetForwardVectorVolatile() );                            
                }
                else
                if (( cPlayer.cSeatInfo.m_ObjectToOccupy instanceof VehicleAttacker )&&
                    ( cPlayer.cSeatInfo.GetRole() == EPassengerRole._VEHICLE_PASSENGER_IDLE))
                {
                    //pasazer w APC
                    cPlayer.GetCameraForwardVector( m_vPlayerForward );
                }
                else
                {
                    //reszta graczy w pojazdach
                    m_vPlayerForward.Set(cPlayer.cSeatInfo.m_ObjectToOccupy.GetForwardVectorVolatile() );
                }
            }
            else 
            {
                //tu trzeba zawolac CameraForwardVector, bo w grze sieciowej
                //faktyczny kierunek gracza odswieza sie z jakims ziarnem
                //dlatego zeby uzyskac plynny obrot kompasu, to
                //trzeba pobrac wektor patrzenia kamery
                cPlayer.GetCameraForwardVector( m_vPlayerForward );
            }
        }

        //tak trzeba, zeby inne obiekty nie "tanczyly" na kompasie
        //jak gracz jezdzi pojazdem po nierownosciach...
        m_vPlayerForward.fY = 0;
        
        m_vPlayerForward.Normalize();
    }
    /** update pozycji waypontow */
    void UpdateWaypointPos()
    {
        for( int i = 0; i < m_cRotorText.GetElementsSize(); i++ )
        {
            m_cRotorText.UpdateElemPos( i, CalculateWaypointAngle( GetWaypointPos( i ) ), m_fMapRadius, m_fDistance, GetArmLength() );
            if( i == m_nPlayerWaypointID )
            {
                SetDistanceCounter( m_fDistance );
            }
        }
    }
        
    /** mapa level */
    protected String GetCurrentMap()
    {
        String sMap = "data/hud/map_l00.png";
        sMap = ((ChromeModule)GetModule()).GetInGameMapName();
        
        return sMap;
    }
    /** wyliczenie promienia mapy */
    protected void UpdateMapRadiusLength()
    {
        float fArmLen = GetArmLength();
        Vector vSize = GetSize();
        m_fMapRadius = m_fCurrentZoom * m_vLevelSize.fX * fArmLen / (( vSize.fX + vSize.fY ) / 2f);
    }
    
    /** wyliczanie koordynat mapy */
    protected void CalculateMapCoords()
    {
        m_cMapCoords.fLeft = _MARGIN/(m_vMapTexSize.fX) + (1f - m_vPlayerPos.fX/m_vLevelSize.fX   - (m_fCurrentZoom / 2f))/m_fMarginMapScale;
        m_cMapCoords.fTop =  _MARGIN/(m_vMapTexSize.fZ) + (1f - m_vPlayerPos.fZ/m_vLevelSize.fZ  - ( m_fCurrentZoom / 2f ))/m_fMarginMapScale;
        m_cMapCoords.fRight = m_cMapCoords.fLeft  + m_fCurrentZoom / m_fMarginMapScale;
        m_cMapCoords.fBottom = m_cMapCoords.fTop  + m_fCurrentZoom / m_fMarginMapScale ;
        
    }
    
    protected float CalculateWaypointAngle( Vector vPos )
    {
        double kat = 0.0;
        // dystans
        if (m_vPlayerPos == null)
            return 0;

        if( m_vDistance == null )
        {
            m_vDistance = new Vector();
        }
        if( m_vRight == null )
        {
            m_vRight = new  Vector();
        }
        
        m_vDistance.fX = vPos.fX - m_vPlayerPos.fX;
        m_vDistance.fZ = vPos.fZ - m_vPlayerPos.fZ;
        
        m_fDistance = m_vDistance.Len();
        
        m_vDistance.Normalize();
        // wektor w prawo gracza
        m_vRight.fX = -m_vPlayerForward.fZ;
        m_vRight.fZ = m_vPlayerForward.fX;
        
        float fCosAlpha = m_vDistance.Dot( m_vPlayerForward );
        float fCosBeta = m_vDistance.Dot( m_vRight );
        
        double kat_alfa = (double)Tools.acos(fCosAlpha);
        double kat_degree = (kat_alfa*180.0)/java.lang.Math.PI;
        
        if (fCosBeta>=0)
			kat = kat_degree;
		else
			kat = 180.0f+(180.0f-kat_degree);
        
        return ((float)kat-90);
    }
    /** zwraca pozycje waypointa w terenie */
    protected Vector GetWaypointPos( int nIndex )
    {
        return m_cWaypointsManager.GetWaypointPos( nIndex );
    }
    
    boolean IsWaypointActive(int nIndex)
    {
        return m_cWaypointsManager.IsWaypointActive(nIndex);
    }
    
    /** zmienia pozycje waypointa w terenie */
    protected void SetWaypointPos( int nIndex, Vector vPos )
    {
        m_cWaypointsManager.SetWaypointPos( nIndex, vPos );
    }
    /** wyliczanie dystansu i wyswietlenie na hudzie */
    protected void SetDistanceCounter( float fVal )
    {
        if( fVal < 0 )
            fVal = 0;
        
        fVal /= 100f;
        int nVal = (int)fVal;
        String sCounterText = new String();
        int nMaxCounter = -1;
        
        if( Math.pow( 10, _DISTANCE_CHR ) < _DISTANCE_GRAIN )
        {
            int fTemp = (int)_DISTANCE_GRAIN;
            nMaxCounter = 0;
            while( fTemp > 0 )
            {
                fTemp /= 10;
                nMaxCounter++;
            }
        }
        else
        {
            nMaxCounter = _DISTANCE_CHR;
        }
        
        for( int i = 0; i < nMaxCounter; i++ )
        {
            if( Math.pow( 10, i ) > nVal )
            {
                sCounterText += "0";
            }
        }
        
        if( nVal != 0 )
            sCounterText += "" + nVal;
        
        m_cDistance.SetText( sCounterText );
    }
    /** pobeiranie aktualnego levelu - indeks */
    protected int GetCurrentLevelIndex()
    {
        //TODO: pobieranie aktualnego levelu
        return 0;
    }
    /** wyliczanie wewnetrznego promienia hud'a */
    static float GetArmLength()
    {
        float fScreenWidth = GetScreenWidth();
        float fScreenHeight = GetScreenHeight();

        return _MAP_IN_RADIUS * fScreenWidth/800f;
    }
        
    /** ustawianie pozycji gracza */
    void SetPlayerPos()
    {
        if( m_vPlayerPos == null )
            m_vPlayerPos = new Vector();
        
        GetPlayerForward();
        
        m_vPlayerPos.Set(cPlayer.GetPositionVectorVolatile());
        
        //zabezpieczenie zeby nie wyjsc na clampowany margines mapy
        if( m_vPlayerPos.fX < m_vLevelSize.fX / m_vMapTexSize.fX )
            m_vPlayerPos.fX = m_vLevelSize.fX / m_vMapTexSize.fX;
        if( m_vPlayerPos.fX > m_vLevelSize.fX - m_vLevelSize.fX / m_vMapTexSize.fX)
            m_vPlayerPos.fX = m_vLevelSize.fX - m_vLevelSize.fX / m_vMapTexSize.fX;
        
        if( m_vPlayerPos.fZ < m_vLevelSize.fZ / m_vMapTexSize.fZ )
            m_vPlayerPos.fZ = m_vLevelSize.fZ / m_vMapTexSize.fZ;
        if( m_vPlayerPos.fZ > m_vLevelSize.fZ - m_vLevelSize.fZ / m_vMapTexSize.fZ)
            m_vPlayerPos.fZ = m_vLevelSize.fZ - m_vLevelSize.fZ / m_vMapTexSize.fZ;
        
        UpdateMapCoords();
        
        if( !m_bWaypointsAreUpdated )
        {
            m_bWaypointsAreUpdated = true;
            UpdateWaypointPos();
            RotateMap(false);
        }
        m_bWaypointsAreUpdated = false;
        
    }
    /** zmiana koordynat dla tekstury mapy */
    protected void UpdateMapCoords()
    {
        if( m_nTexId != -1 )
        {
            CalculateMapCoords();
            GetPyramidElem( _MAP ).SetTexture( m_nTexId, m_cMapCoords );
        }
    }
    
    void SetDestPosition( float fX, float fY, float fZ )
    {
        DestPosition.Set( fX, fY, fZ );
        if(m_nPlayerWaypointID > -1)
            SetWaypointPos( m_nPlayerWaypointID, DestPosition );
        UpdateWaypointPos();
    }
    
	public void OnCreate()
	{
        //Log("[HUDCompass.OnCreate]: Creating HUDCompass \n");
		super.OnCreate();
		
        if (!cHUDManager.m_bDisableInNetGame)
            Load(GetModule(), m_sScriptName, true);
        
	}

    public void OnDestroy()
	{
		super.OnDestroy();	
		
        //CrashLog("[HUDCompass.OnDestroy]: Destroing DONE. \n");
	}
	//////////////////////////////////////////////
    //  METODY ZE STAREGO KOMPASU
    //
	/** metoda aktywujaca kompas
	 */
	public void Activate()
	{
		if (bActiveFlag)
			return;
		if (!cHUDManager.m_bActivateAllowed)
			return;
		
		super.Activate();

		Draw();
		
	}

	public void ActivateSlow()
	{
		if (bActiveFlag)
			return;
		if (!cHUDManager.m_bActivateAllowed)
			return;
		
        super.Activate();   
		
        Draw();

	}

	public void ComponentMoved()
	{
		if (!bActiveFlag)
			HideAll();
	}
	
	/** metoda deaktywujaca kompas
	  */
	public void Deactivate()
	{
		if (!bActiveFlag)
			return;
		
		super.Deactivate();
		
		HideAll();
		
	 }

	public void DeactivateSlow()
	{
		if (!bActiveFlag)
			return;
		
        super.Deactivate();   

	}
	
    void OnInputMouseAxis(int nAxis, int nDelta)
	{
		if( !m_bWaypointsAreUpdated )
        {
            m_bWaypointsAreUpdated = true;
            
        }
        
        RotateMap( m_bWaypointsAreUpdated );
        
        m_bWaypointsAreUpdated = false;
	}
    
    public Vector GetDestPosition()
	{
		return DestPosition;
	}
    
    public void MoveTop()
    {
        super.MoveTop();
    }
    
    void ShowLocation(DefLocation cLocation)
    {
        AddWaypoint(cLocation);
    }
    
    void HideLocation(DefLocation cLocation)
    {
        LogR("HideLocation " + cLocation + " " + cLocation.m_sText);
        HUDWaypointsManager.HUDWaypoint cPoint = m_cWaypointsManager.FindWaypoint(cLocation);
        if (cPoint != null)
        {
            LogR("removing " + cPoint + " " + cLocation.m_sText);
            m_cRotorText.RemoveElem(cPoint.GetRotorIdx());
            m_cWaypointsManager.RemoveWaypoint(cLocation);
        }
    }
	
	//*********************************************************
	//					SaveGame
    //*********************************************************
    
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[HUDCompass.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_HUD_COMPASS)
        {
			super.SGLoadChunk(cFC);
			
			// wektor punktu docelowego
			DestPosition = cFC.LoadVector();
            SetDestPosition( DestPosition.fX, DestPosition.fY, DestPosition.fZ );
			// pozycja punktu docelowego - kolejny waypont
			m_vDestPos = cFC.LoadVector();
        }
		
		cFC.delete();
	}
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[HUDCompass.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_HUD_COMPASS, cFCParent);
		
		super.SGSaveChunk(cFC);
			
		// wektor punktu docelowego
		cFC.SaveVector(DestPosition);
		// pozycja punktu docelowego - kolejny waypont
		cFC.SaveVector(m_vDestPos);			
					        
		cFC.delete(); 
	}				
    
}
