public class HUDVoiceMenu extends HUDComponent
{
    UIMenuButton m_tOrders[] = null;
    UIMenuButton m_tRequests[] = null;
    UIMenuButton m_tAnswers[] = null;
    
    UIMenu m_tMenus[] = new UIMenu[3];                       
    UIMenu m_tMenuStart = null;
    
    int m_nCurMenu = -1;
    
    int m_nOrdersCount = 0,
        m_nRequestsCount = 0,
        m_nAnswersCount = 0;

    
    UIMenuButton[] GetButtons(int nType)
    {
        switch (nType)
        {
        case NetVoice._TYPE_ORDER:
            return m_tOrders;
        case NetVoice._TYPE_REQUEST:
            return m_tRequests;
        case NetVoice._TYPE_ANSWER:
            return m_tAnswers;
        }
        return null;
    }
    
    int GetCount(int nType)
    {
        switch (nType)
        {
        case NetVoice._TYPE_ORDER:
            return m_nOrdersCount;
        case NetVoice._TYPE_REQUEST:
            return m_nRequestsCount;
        case NetVoice._TYPE_ANSWER:
            return m_nAnswersCount;
        } 
        return 0;
    }
    
    void IncCount(int nType)
    {
        switch (nType)
        {
        case NetVoice._TYPE_ORDER:
            m_nOrdersCount++;
            break;
        case NetVoice._TYPE_REQUEST:
            m_nRequestsCount++;
            break;
        case NetVoice._TYPE_ANSWER:
            m_nAnswersCount++;
            break;
        } 
    }
    
    void AddVoice(NetVoice cVoice)
    {
        int nType = cVoice.GetType();
        if (nType == NetVoice._TYPE_UNKNOWN)
            return;
        UIMenuButton cBtn = GetButton(nType, GetCount(nType));
        if(cBtn == null) return;
        cBtn.SetActive(true);
        cBtn.SetText(" " + (GetCount(nType)+1) + ". " + cVoice.GetText());
        cBtn.m_cData = cVoice;
        cBtn.Show();
        IncCount(nType);
    }
    
    UIMenuButton GetButton(int nType, int nIdx)
    {
        if (nIdx < 0 || 
            nIdx >= GetButtons(nType).length)
            return null;
        
        return (GetButtons(nType))[nIdx];
    }
    
    NetVoice GetVoice(int nType, int nIdx)
    {
        UIMenuButton cBtn = GetButton(nType, nIdx);
        if (cBtn.m_bActive)
            return (NetVoice)cBtn.m_cData;
        else
            return null;
        
    }
    
    int GetActiveMenu()
    {
        for (int i = 0; i < m_tMenus.length; i ++)
        {
            if (m_tMenus[i].m_bFocus)
                return i;
        }
        return -1;
    }
    
    
    
    public void OnInit()
    {
        super.OnInit();
        m_tMenus[NetVoice._TYPE_ORDER] = (UIMenu)GetElem("ID_MENU_ORDERS");
        m_tOrders = FindButtons(m_tMenus[NetVoice._TYPE_ORDER]);
        m_tMenus[NetVoice._TYPE_REQUEST] = (UIMenu)GetElem("ID_MENU_REQUESTS");
        m_tRequests = FindButtons(m_tMenus[NetVoice._TYPE_REQUEST]);
        m_tMenus[NetVoice._TYPE_ANSWER] = (UIMenu)GetElem("ID_MENU_ANSWERS");
        m_tAnswers = FindButtons(m_tMenus[NetVoice._TYPE_ANSWER]);
        
        m_tMenuStart = (UIMenu)GetElem("ID_MENU_START");
        
        java.util.Vector vVoices = GetVoices();
        if (vVoices == null)
            return;
        
        NetVoice cVoice = null;
        for (int i = 0; i < vVoices.size(); i++)
        {
            cVoice = (NetVoice)vVoices.elementAt(i);
            AddVoice(cVoice);
        }
        
        CallMethodIn("RemoveAdditionalButtons",0);
        RegisterKeyRequest();
    }
    
    void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
    {
        if (bKeyState && bActiveFlag)
        {
            int nOption = -1;
            switch(nKeyCode)
            {
            case EKey._1:
                nOption = 0;
                break;
            case EKey._2:
                nOption = 1;
                break;
            case EKey._3:
                nOption = 2;
                break;
            case EKey._4:
                nOption = 3;
                break;
            case EKey._5:
                nOption = 4;
                break;
            case EKey._6:
                nOption = 5;
                break;
            case EKey._7:
                nOption = 6;
                break;
            case EKey._8:
                nOption = 7;
                break;
            case EKey._9:
                nOption = 8;
                break;
            case EKey._0:
                nOption = 9;
                break;
                                    
            }
            
            if (nOption != -1)
            {
                if (m_nCurMenu == -1)
                {
                    if (nOption < 3)
                        ShowMenu(nOption);
                }
                else
                {
                    UIButton cBtn = GetButton(m_nCurMenu, nOption);
                    if (cBtn != null)
                    {
                        cBtn.SetFocus(true);
                        OnUICommand(cBtn, EUIConst.CM_CLICK);
                    }
                }
            }
        }
    }
    
    void RemoveAdditionalButtons()
    {
        for (int i = 0; i < m_tMenus.length; i++)
        {
            for (int j = 0; j < GetButtons(i).length; j++)
            {
                UIMenuButton cBtn = GetButton(i, j);
                if (!cBtn.m_bActive)//nie ma zadnego voice'a
                {
                    m_tMenus[i].RemoveChild(cBtn);
                    cBtn.Remove();
                    cBtn.delete();
                    cBtn = null;
                    GetButtons(i)[j] = null;
                }
            }
        }
    }
    
    void OnUICommand(UIWindow cSender, int nCommand)
	{
		super.OnUICommand(cSender,nCommand);
        
        if (nCommand == EUIConst.CM_CLICK ||
            nCommand == EUIConst.CM_DBCLICK)
        {
            if (cSender instanceof UIMenuButton)
            {
                if (m_nCurMenu == -1)
                {
                    if (cSender.m_sId.equals("ID_BTN_ORDERS"))
                        ShowMenu(NetVoice._TYPE_ORDER);
                    else
                        if (cSender.m_sId.equals("ID_BTN_REQUESTS"))
                            ShowMenu(NetVoice._TYPE_REQUEST);
                        else
                            if(cSender.m_sId.equals("ID_BTN_ANSWERS"))
                                ShowMenu(NetVoice._TYPE_ANSWER);
                }
                else
                {
                    GetGameInfo().LocalEventVoice(((NetVoice)cSender.m_cData).GetId());
                    CallMethodIn("Deactivate", 0);
                }
            }
        }
    }
    
    
    UIMenuButton[] FindButtons(UIMenu cMenu)
    {
        java.util.Vector vBtns = new java.util.Vector();
        for(int i = 0; i < cMenu.m_vMembers.size(); i++)
        {
            UIWindow cWnd = (UIWindow)cMenu.m_vMembers.elementAt(i);
            if (cWnd instanceof UIMenuButton)
            {
                vBtns.addElement(cWnd);
                ((UIMenuButton)cWnd).SetFocus(false);
                ((UIMenuButton)cWnd).SetActive(false);
            }
        }
        UIMenuButton cBtns[] = new UIMenuButton[vBtns.size()];
        for (int i = 0; i < vBtns.size(); i++) 
            cBtns[i] = (UIMenuButton)vBtns.elementAt(i);
        return cBtns;
    }
    
    
    public void OnCreate()
	{					
		super.OnCreate();
		
        if (cHUDManager.m_bDisableInNetGame) 
            return;

		ActivateOnModule();
		
        Load(GetModule(), "data/hud/HUDVoiceMenu.scr");
	}
    
    public void Activate()
    {
		if (bActiveFlag || GetVoices() == null)
			return;
		if (!cHUDManager.CanActivate(HUDManager._HUD_VOICE_MENU))
			return;
		
        cPlayer.UnregisterKeyRequest();
        
        ShowMenu(-1);
        
        super.Activate();   
		
        ProcessMouse(true);
		Draw();
		
        cHUDManager.HudMenuActivated(HUDManager._HUD_VOICE_MENU);
        
    }

    void ShowMenu(int nMenu)
    {
        m_nCurMenu = nMenu;
        
        if (nMenu == -1)
        {
            m_tMenuStart.Show();
            m_tMenuStart.SetFocus(true);
        }
        else
        {
            m_tMenuStart.Hide();
        }
        
        for (int i = 0; i < m_tMenus.length; i++)
        {
            if (nMenu == i)
            {
                m_tMenus[i].Show();
                m_tMenus[i].SetFocus(true);
                GetButton(i,0).SetFocus(true);
            }
            else
            {
                m_tMenus[i].Hide();
            }
        }
    }
        
    
    public void Deactivate()
    {
		if (!bActiveFlag)
			return;
		
        super.Deactivate();   
        
        cPlayer.RegisterKeyRequest();
        
        ProcessMouse(false);
        cHUDManager.HudMenuDeactivated(HUDManager._HUD_VOICE_MENU);
    }
    
    public void SetParams(PlayerHUD player,HUDManager manager)
    {
        super.SetParams(player, manager);
            
    }
        
    private java.util.Vector GetVoices()
    {
        if ((GetGameInfo()                   != null)&&
            (GetGameInfo().GetVoiceManager() != null))
            return GetGameInfo().GetVoiceManager().GetList();
        
        return null;
    }
    
    private NetGameInfo GetGameInfo()
    {
        return ((PlayerNet)cPlayer).GetGameInfo();
    }
    
}
