import java.lang.Math;

public class Rocket extends Bullet 
{

    /** Odleglosc z jakiej bedzie slyszany halas wybuchu
     */
    float fExplosionNoiseDistance = 3000.f;

    /** Czas co ktory beda odpalane kolejne FXy
     */
    float fFXUpdateTime = 0.1f;
    float fFXUpdateDist = 100f;
    Vector vLastUpdatePos = new Vector();

    /** zasieg rakiety */
    float fRange = 10000f;
    /** Kwadrat minimalnej odleglosci jaka musi przeleciec rakieta, zeby wlaczyc kolizje z obiektami */
    //public float fMinDistance2ToArm = 70.f * 70.f;
    /** Czas co jaki rakieta bedzie sie updejtowala */
    float fSpeedUpdateTimer = 0.1f;
    /** pozycja startu rakiety */
    protected Vector m_vStartPos = null;
    
    
    
    Rocket()
    {
        MeshName= "Data/Meshes/Equipment/rakieta.3DA";
        fHealth = 1.0f;
        fStartSpeed = 7000.0f;
        fDamage = 100.0f;
        fPhysicsSimDamage = fDamage;
        fBlastDamage = 1250.0f;
        fDirectHitDamage = fBlastDamage;
        fPhysicsSimBlastDamage = fBlastDamage;
        fBlastRadius = 800.0f;
        //TODO zmienic na prawidlowy
        sFXHitName = "FXBlowRocket";
        //TODO zmienic na prawidlowy
        sSoundExplodeName = "data/sounds/weapon/GrenadeExplosion.wav";
    }

    /**Maksymalna predkosc rakiety*/
    public float fMaxSpeed   = 10000.0f;
    
    /**Przyspieszenie rakiety */
    public float fAccelerate = 3400.0f;
    
    /**Nazwa FX dymu z rakiety*/
    public String sFXSmokeName = "FXRocketSmoke";
    
    /**Nazwa dzwieku odpalanego dla lecacej rakiety*/
    public String sSoundFlyName = "Data/Sounds/Weapon/RocketFlyFF.wav";
    
    public void OnCreate()
    {
        super.OnCreate();
//        fLightScaleExplosion = fBlastRadius;
        SetPhysicsType(EPhysics._NEWFLY);
        SetPosition(GetPosition());
        //SetLightColor(0.5f, 0.5f, 0.5f);
        SetMass(0.0f);
        
        
    }
    
    /**Ustawia kolor dynamicznego swiatla podpietego do rakiety*/
    void SetLightColor(float fR, float fG, float fB)
    {
        if (cDynLight != null)
        {
            cDynLight.SetColor(fR, fG, fB);            
        }
    }

    
    void Start(float fForce, Vector vPos, Vector vDir, float fTime)
    {
        // bullet leci, wiec musi sie zapisac do Save'a
        bSG = true;
        //juz leci, wiec nie bedziemy musieli kasowac razem z bronia
        SetStarted();
        // od teraz bullet jest sobie niezalezny od broni
        
        //ale ownera sobie zapamietajmy, zeby wiadomo kto to 
        //strzelal (gra sieciowa)
        SetOwner();
        
        if ( m_bSetPositionWorld )
            SetPositionWorld( vPos );
        else            
            SetPosition( vPos );
        SetDir( vDir );
        Vector vPlayerSpeed = new Vector();
        if (Owner != null)
		{
			vPlayerSpeed = new Vector(Owner.GetAbsoluteSpeed());
		}
                
        Vector vSpeed = new Vector( vDir );
        vSpeed.Normalize();
        vSpeed.Mul( fStartSpeed );
        //TODO jesli cos sie bedzie pierdolic to trzeba przywrocic
        //vSpeed.Add( vPlayerSpeed );
        
        SetDir( vSpeed );
        SetSpeed( vSpeed );
        SetDestSpeedValue( fMaxSpeed );
        
        SetPhysicsActive( fTime );
        
        ActivateOnModule();
        SetCorrectPosOnRender(true);

        SetState("Flying");
    }
	
    //-----------------------------------------------------------------------------------
    /** Pocisk przechodzi w ten stan gdy zostaje wystrzelony*/
    
    public class Flying extends Bullet.Flying 
    {   
        public void OnEnter()
        {
            fTimeToLive = fRange / fStartSpeed; //chcemy miec zasieg fRange
        
            super.OnEnter();
            
            //CallMethodIn("SpeedUpdate",fSpeedUpdateTime);
            CallMethodIn("FXUpdate",0);
            AttachSound(sSoundFlyName);
            vLastUpdatePos = GetPosition();//GetForwardVector();
            //vLastUpdatePos.Mul(fFXUpdateDist);
            //vLastUpdatePos.Add(GetPosition());
            
            EnableRendering(false);
            /*if (fSpeedUpdateTimer > 0f)
            {
                CallMethodIn("SpeedUpdate", fSpeedUpdateTimer);
                m_vStartPos = GetPosition();
                SetCollFlags(ECollFlags.OBJCOL_FLAG_NOOBJECTS);
            }*/
        }
        
        void SpeedUpdate()
        {
            /*if (m_vStartPos != null && 
                GetDistance2To(m_vStartPos) > fMinDistance2ToArm)
            {
                ClearCollFlags(ECollFlags.OBJCOL_FLAG_NOOBJECTS);
                return;
            }*/
            CallMethodIn("SpeedUpdate", fSpeedUpdateTimer);
        }

        float fLastFxTime = 0;
        /**Odpalany jest FX  dymu z tylu rakiety*/
        void FXUpdate()
        {
            float fTime = GetTime();

            if (fLastFxTime == fTime)
                return; //nie odpalamy wiecej niz jednego fxa w ramce
            
            fLastFxTime = fTime;

            Vector vPos = GetPosition();
            Vector vDir = new Vector(vPos);
            vDir.Sub(vLastUpdatePos);
            float fLen = vDir.Len();
            vDir.Normalize();
            vDir.Mul(fFXUpdateDist);
            while (fLen > fFXUpdateDist)
            {
                EnableRendering(true);
                ExecuteFX(sFXSmokeName, vLastUpdatePos);
                vLastUpdatePos.Add(vDir);
                fLen -= fFXUpdateDist;
            }
            
            if (!bExploded)
            {
                CallMethodIn("FXUpdate",fFXUpdateTime);
            }
        }

        
        /**Gdy ktos nas trafi to wybuchamy*/
/*        public void OnTakeDamage(float fDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nDamageType)
		{
            super.OnTakeDamage(fDamage,Assassin,vHitLocation,vSpeed,nDamageType);
        }
*/		
		// poprawione OnTakeDamage //////////////////////////////////////////////////////////////
		public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation,
								 Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
		{
            super.OnTakeDamage(fDamage, fPhSimDamage, Assassin, vHitLocation, vSpeed, nMeshElementorBoneID, nDamageType);
		}
        
        /**Gdy czegos dodkniemy wybuchamy*/
        public void OnCollisionFullInfo(ControlObject Col,Vector v1,Vector v2)
        {
            //LogR("Rocket collision " + Col + " owner " + Owner);
            
            if (Col != null &&
                (Col == Owner || 
                Col == cWeaponOwner ||
                Col instanceof RocketCannon ||
                Col instanceof WeaponHeavyRocketLauncher))
            {
                return;
            }
            //bezposrednie oberwanie jest pomijane
            //super.OnCollisionFullInfo(Col,v1,v2);
			m_DirectVictim = Col;
            CallOnFrameEnd("Explode");
        }
        
                
                
        
 		//*********************************************************
		//					SaveGame
		//*********************************************************
				
		public void SGLoad(FileChunk cParentFC)
		{
			FileChunk.Log("[Rocket$Flying] : SGLoad\n");
			
			FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
			if (cFC.GetID() == ESGChunksChrome._CHK_ROCKET_FLYING_STATE)
			{
				super.SGLoad(cFC);
		
				fLastFxTime = cFC.LoadFloat();	
                vLastUpdatePos = cFC.LoadVector();
			}
			
			cFC.delete();
		}
	
		public void SGSave(FileChunk cParentFC)
		{
			FileChunk.Log("[Rocket$Flying] : SGSave\n");
			
			FileChunk cFC = NewChunk(ESGChunksChrome._CHK_ROCKET_FLYING_STATE, cParentFC);
			
			super.SGSave(cFC);
			
			cFC.SaveFloat(fLastFxTime);
			cFC.SaveVector(vLastUpdatePos);
            
			cFC.delete();
		}
		
		public void SGUpdate()
		{
			FileChunk.Log("[Rocket$Flying] : SGUpdate\n");
			
			DetachAllSounds();
			
            AttachSound(sSoundFlyName);
            
			super.SGUpdate();
		}
    }
    
    void RemoveForMovie()
    {
        super.RemoveForMovie();
        DetachAllSounds();
    }
    
    /**Wybuchamy rakiete i odbieramy wszystkim w promieniu odpowiednia ilosc energi*/
    void Explode()
    {
        if (!bExploded)
        {
            MakeNoise(fExplosionNoiseDistance, Pawn._NOISE_BULLET_HIT);
            
            //SetLightColor(0.75f, 0.5f, 0.5f);
            
            DetachAllSounds();
            super.Explode();
        }
    }
    
    public void delete()
    {
        if(!bNoLight)
        {
            //cDynLight.delete();
        }
        super.delete();
    }	
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[Rocket.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_BULLET)
        {
			super.SGLoadChunk(cFC);
			
			m_vStartPos = cFC.LoadVector();			
        }
		
		cFC.delete();
	}
	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Rocket.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_BULLET, cFCParent);
		
		super.SGSaveChunk(cFC);

		cFC.SaveVector(m_vStartPos);
        
		cFC.delete(); 
    }		
    
	public void SGUpdate()
	{
		FileChunk.Log("[Rocket.SGUpdate]\n");
        
		DetachAllSounds();
        
		super.SGUpdate();
	}
}

