/** Rakieta automatycznie naprowadzajaca sie na podany cel (Fire&Forget)
 */
public class RocketFF extends Rocket
{
    /** Zapamietany cel rakiety (jesli null - leci prosto) */
    MeshObject cTarget = null;
    
    /** Zapamietany indeks LookAt pointa celu (jesli -1 leci w kierunku srodka celu)
     *  (dziala dla Pawna i wyzej - GetPointToLookAt, dla innych musi byc -1) */
    int nTargetPointIdx = -1;
    
    /** Maksymalna predkosc skretu rakiety (wplywa na jakosc naprowadzania) [deg/s] */
    float fMaxAngSpeed = 200f;
    
    /** Czas zycia rakiety - po uplynieciu - wybucha [s] */
    float fRocketLifeTimer = 10f;
    
    /**Co ile ma sie zwiekszac predkosc*/
    public float fSpeedUpdateTime = 0.1f;    
    
    RocketFF()
    {
        MeshName= "Data/Meshes/Equipment/rakieta.3DA";

        fStartSpeed = 4000f;
        fMaxSpeed   = 8000f;
        fAccelerate = 2000f;

        fDamage = 0f;
        fPhysicsSimDamage = fDamage;
        fBlastDamage = 1150;
        fPhysicsSimBlastDamage = fBlastDamage;
        fBlastRadius = 100f;
        
        fFXUpdateTime = 0.05f;
        sFXHitName = "FXBlowRocketFF";
        sFXSmokeName = "FXRocketFFSmoke";
        
        sSoundFlyName = "Data/Sounds/Weapon/RocketFlyFF";
        sSoundExplodeName = "Data/Sounds/Explosion/RocketExplosionFF";
        
        fLightScaleFlight = 250f;
        fLightScaleExplosion = 1500;
    }
	
    public void OnCreate()
    {
        super.OnCreate();
		
        SetPhysicsType(EPhysics._NEWFLY);
        SetMaxAngSpeed(fMaxAngSpeed);
        SetMass(0f);
    }
	
    /** Ustawienie celu rakiety i jego LookAt pointa */
    void SetTarget(MeshObject cTgt, int nTgtPointIdx)
    {
        cTarget = cTgt;
        nTargetPointIdx = nTgtPointIdx;
    }
    
    /** Ustawienie celu rakiety. Jesli jest to Pawn - rakieta leci na GetPointToLookAt(0),
     *  in. na GetPosition() */
    void SetTarget(MeshObject cTgt)
    {
        if (cTgt instanceof Pawn)
            SetTarget(cTgt, 0);
        else
            SetTarget(cTgt, -1);
    }
    
    

    void Start(float fForce, Vector vPos, Vector vDir, float fTime)
    {
        // bullet leci, wiec musi sie zapisac do Save'a
        bSG = true;
        
        CrashLog("[RocketFF.Start] bSG=true\n");
        // od teraz bullet jest sobie niezalezny od broni
        
        ActivateOnModuleGridIndependent();
        SetUnImportanceFactor(0);
		
        if ( m_bSetPositionWorld )
            SetPositionWorld( vPos );
        else            
            SetPosition(vPos);
        SetDir( vDir );
        
        Vector vParentSpeed = new Vector();
        if (Owner != null)
			vParentSpeed = new Vector(Owner.GetAbsoluteSpeed());
        
        Vector vStartSpeed = new Vector(vDir);
        vStartSpeed.Normalize();
        vStartSpeed.Mul(fStartSpeed);
        vStartSpeed.Add(vParentSpeed);

		SetDir(vStartSpeed);
        SetSpeed(vStartSpeed);
		SetDestSpeedDir(vDir);
        SetDestSpeedValue( fMaxSpeed );
        
        SetPhysicsActive(fTime);
        SetCorrectPosOnRender(true);

        SetState("Flying");
    }
    
    public class Flying extends Rocket.Flying 
    {   
        public void OnEnter()
        {
            super.OnEnter();

			SetMaxAcceleration(fAccelerate);
			SetDestSpeedValue(fMaxSpeed);
			
            CallMethodIn("LifeTimer", fRocketLifeTimer);
        }
        
        /** Wolana, gdy minie czas zycia rakiety */
        void LifeTimer()
        {
            if (!bExploded)
                Explode();
        }

        /** Update predkosci (i kierunku rakiety) */
        void SpeedUpdate()
        {
            /*if (m_vStartPos != null && 
                GetDistance2To(m_vStartPos) > fMinDistance2ToArm)
            {
                ClearCollFlags(ECollFlags.OBJCOL_FLAG_NOOBJECTS);
            }*/
            if (cTarget != null)
            {
                Vector vDir;
                if (nTargetPointIdx > -1)
                {
                    vDir = ((Pawn) cTarget).GetPointToLookAt(nTargetPointIdx);
                    if (vDir == null)
                        vDir = ((Pawn)cTarget).GetPointToLookAt(0); 
                }
                else
                    vDir = cTarget.GetPosition();
				
                if (vDir != null)
                {
                    vDir.Sub(GetPosition());
					vDir.Normalize();
					SetDir(vDir);
                    SetDestSpeedDir(vDir);
                }
            }

            CallMethodIn("SpeedUpdate", fSpeedUpdateTime);
        }
		
 		//*********************************************************
		//					SaveGame
		//*********************************************************
						
		public void SGUpdate()
		{
			FileChunk.Log("[RocketFF$Flying] : SGUpdate\n");
			
			super.SGUpdate();
			
			SetMaxAcceleration(fAccelerate);
			SetDestSpeedValue(fMaxSpeed);			
		}		
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[Bullet.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_BULLET)
        {
			super.SGLoadChunk(cFC);
			
			// Zapamietany cel rakiety (jesli null - leci prosto)
			cTarget = (MeshObject)cFC.LoadObjectRef();		
			    
			// Zapamietany indeks LookAt pointa celu (jesli -1 leci w kierunku srodka celu)
			nTargetPointIdx = cFC.LoadInt();
        }
		
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Bullet.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_BULLET, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		// Zapamietany cel rakiety (jesli null - leci prosto)
		cFC.SaveObjectRef(cTarget);		
		    
		// Zapamietany indeks LookAt pointa celu (jesli -1 leci w kierunku srodka celu)
		cFC.SaveInt(nTargetPointIdx);
        
		cFC.delete(); 
    }		
}
