/** Strzelajace pociskami
 */
public class WeaponBullet extends Weapon
{
    public static float SHOT_THRU_FENCE_CHANCE = 0.5f;
        
    // Mesh uzywany do liczenia kolizji, traktowany jako wlasciciel.
    // Wprowadzony, aby CannonController mogl wolac metode FireBullet
    // bezposrednio z WeaponBullet
    ChromeActor cFBMesh = null;
    
    ChromeActor cHitOwner = null;
    
    static Vector l_vStart = new Vector();
    static Vector l_vEnd = new Vector();
    static Vector l_vEndOriginal = new Vector();
    static Vector l_vFireDir = new Vector();
    
    float FireBullet(Vector vBarrelPos, Vector vDir)
    {
        // nie ustawienie mesha przelacza go na normlany tryb
        if(cFBMesh == null) cFBMesh = this;
        
        float fDist = 1e10f;

        l_vStart.Set(vBarrelPos);
        
        // wyliczamy, pobieramy, ustalamy kierunek w ktory strzelamy
        l_vEnd.Set(vDir);
        l_vEnd.Mul(fRange);
        l_vEnd.Add(vBarrelPos);

        l_vFireDir.Set(vDir);
        l_vFireDir.Normalize();
        l_vFireDir.Mul(0.1f);
                   
        l_vEndOriginal.Set(l_vEnd);
        
        // zmienne trzymajace punkt kolizji
        Vector vNormal = null;
        ControlObject cObstacle = null;
        int nObstacleElement = 0;
        int nMaterial = -1;
        int nMaterialFlags = 0;

        boolean bCollision = false;
        boolean bWaterCollision = false;
        
        int nCollisions = 0;
        
        do {
            nMaterial = -1;
            nMaterialFlags = 0;
            
            Collision stCollTerrain=cFBMesh.TraceTerrain(l_vStart, l_vEnd);
            if(stCollTerrain!=null)
            {
                // trafienie w ziemie
                fDist = stCollTerrain.fDistance;
            }

            Collision stCollTrees=cFBMesh.TraceTrees(l_vStart, l_vEnd);
            if(stCollTrees!=null)
            {
                // trafienie w drzewo
                if (stCollTrees.fDistance < fDist)
                    fDist = stCollTrees.fDistance;
            }

            Collision stCollObjects = new Collision();
            int nFaceSkipFlags = EFaceFlags.NO_BLOCK_RAY;
            if (RandomFloat(1) < SHOT_THRU_FENCE_CHANCE)
                nFaceSkipFlags |= EFaceFlags.SHOOT_THROUGH;
            
            if(TraceObjectsWithFaceFlags(stCollObjects, l_vStart, l_vEnd, GetObjectDamageExclude(), 0, 
                                         nFaceSkipFlags))
            {
                if (stCollObjects.fDistance < fDist) fDist = stCollObjects.fDistance;

                //zapamietujemy ostatnio trafiony controlObject
                oLastHit = stCollObjects.Obstacle;
            }
            else
            {
                stCollObjects = null;
                oLastHit = null;
            }
            
            // fake do strzelania w poblize kamery gracza
            Player player = ((ChromeModule)GetModule()).GetMainPlayer();
            if(player != null)
            {
                if (oLastHit != player) // jesli trafienie nie bylo w playera
                {
                    Collision camColl = player.CheckBulletTrajectory(this, l_vStart, l_vEnd);
                    if (camColl != null) // jesli fake hit w kamere - potraktuj to jako obiekt
                    {
                        stCollObjects = camColl;   
                        if (stCollObjects.fDistance < fDist) fDist = stCollObjects.fDistance;
                        oLastHit = stCollObjects.Obstacle;
                    }
                }
            }

            if(sFXHitWater!=null)
            {
                // sprawdzamy czy nie zostala trafiona tafla wody
                float fWaterLevel = GetModule().GetWaterLevel();

                if(fWaterLevel != -1.0f)
                    if((vBarrelPos.fY > fWaterLevel) && (l_vEnd.fY <= fWaterLevel))
                    {
                        // kula przeszla przez wode!
                        float fDistWater = (vBarrelPos.fY - fWaterLevel)/(vBarrelPos.fY - l_vEnd.fY);

                        Vector vWaterHit = new Vector(l_vEnd);
                        vWaterHit.Sub(vBarrelPos);

                        vWaterHit.Mul(fDistWater);
                        vWaterHit.Add(vBarrelPos);

                        // odpalamy efekt!
                        cFBMesh.ExecuteFX(sFXHitWater,vWaterHit,Vector.Up);
                        cFBMesh.ExecuteParticleSystem(sFXHitWater,vWaterHit,Vector.Up);
                        //ExecuteFXTrace(sFXHitWater,vWaterHit,Vector.Up);
                        
                        // odpalamy dzwiek
                        cFBMesh.PlaySound(sSoundHitWater, vWaterHit);
                        
                        bWaterCollision = true;
                    }
            }

            // gdy byla jakakolwiek kolizja to dodajemy efekt
            if( (stCollTerrain!=null) || (stCollObjects!=null) || (stCollTrees!=null))
            {
                bCollision = true;
                if(stCollObjects!=null)
                {
                    cObstacle = stCollObjects.Obstacle;
                    nObstacleElement = stCollObjects.ObstacleElement;
                    
                    vNormal = stCollObjects.vNormal;
                    nMaterial = stCollObjects.nMaterial;
                    nMaterialFlags = stCollObjects.nMaterialFlags;
                }
                else
                {
                    // nie bylo kolizji z obiektem dynamicznym
                    vNormal = (stCollTrees!=null) ? stCollTrees.vNormal : stCollTerrain.vNormal;
                    nMaterial = (stCollTrees!=null) ? stCollTrees.nMaterial : stCollTerrain.nMaterial;
                    nMaterialFlags = (stCollTrees!=null) ? stCollTrees.nMaterialFlags : stCollTerrain.nMaterialFlags;
                }
            }
            else
                bCollision = false;
            
            // trafienie w szybe
            if(nMaterial == EMaterialType._GLASS)
            {
                StaticExecBreakGlass(l_vEnd, vNormal, vDir, cObstacle, nObstacleElement);
                
                // jest szyba to dalej liczymy kolizje
                l_vStart.Set(l_vEnd);
                // dodajemy lekkie przesuniecie
                l_vStart.Add(l_vFireDir);
                l_vEnd.Set(l_vEndOriginal);
            }

            nCollisions++;
            // dodatkowe ograniczenie na ilosc trace'ow
        } while(nMaterial == EMaterialType._GLASS && nCollisions<100);

        if(bCollision)
        {
            cFBMesh.MakeNoiseOnPosition(l_vEnd, fNoiseHit, Pawn._NOISE_BULLET_HIT);
                
            Hit(fDist, cObstacle, nObstacleElement, l_vEnd, vDir, vNormal, nMaterial, nMaterialFlags, bWaterCollision);
        }
        
        return fDist;
    }

    /** Wykonuje wszystko co zwiazane z trafieniem w miejsce strzalu.
     * @param fDistance odleglosc trafienia od punktu wystrzalu
     * @param cObstacele trafiony obiekt
     * @param nObstacleElement trafiony meshelement obiektu
     * @param vHitPoint punkt trafienia 
     * @param vDir kierunek wystrzalu
     * @param vNormal normalna w punkcie trafienia
     * @param nMaterial numer materialu gdzie nastpilo trafinie
     * @param bWaterHit okresla czy byla trafiona woda
     */
    void Hit(float fDistance, ControlObject cObstacle, int nObstacleElement, Vector vHitPoint, Vector vDir, Vector vNormal, int nMaterial, int nMaterialFlags, boolean bWaterHit)
    {
        cHitOwner = Owner;
        
        Vector vSpeed=new Vector(vDir);
        vSpeed.Mul(fBulletSpeed);

        // kiedy jest jakis cel to odbieramy energie
        if(cObstacle != null)
        {
            Vector vSpeedTmp = new Vector(vSpeed);//bo cObstacle.TakeDamage() normalizuje wektor 
            
            int iBoneId = ((MeshObject)cObstacle).GetBoneIDFromMeshElem(nObstacleElement);
            cObstacle.TakeDamage(fDamage * fDamageMult, fPhysicsSimDamage, cHitOwner, vHitPoint, vSpeed, iBoneId, EnumDamageType._BULLET);
            
            CutBodyPart((MeshObject)cObstacle, nObstacleElement, vSpeedTmp);//tak ma byc (nie wstawiac pzed TakeDamage)
        }
        
        // podczas trafienia w wode nie ma efektow bezposredniego trafienia
        if(bWaterHit) return;
        
        if(nMaterial == -1) nMaterial = EMaterialType._GROUND;
        
        //CrashLog("Hit on material <"+nMaterialToUse+">\n");
        // nie odpalamy trace na szybie
        boolean bNoTrace = (nMaterial == EMaterialType._GLASS) ||
                           ((nMaterialFlags & EFaceFlags.SHOOT_THROUGH) != 0);
        
        // odpalamy efekt trafienia
        FxAndSoundManager(cHitOwner,
                          nMaterial,
                          vHitPoint, vNormal, vSpeed,
                          cObstacle, nObstacleElement,
                          eBulletType, sFXHitName, bNoTrace );
    }
    
    boolean bNoShootSound = false;
    
    /** Strzelanie pociskami
     */
    void Fire(Vector vBarrelPos, Vector vDir)
    {
        //CrashLog(" vBarrelPos "+vBarrel+" vDir "+vDir+"\n");
        super.Fire(vBarrelPos, vDir);
        

        float fDist = FireBullet(vBarrelPos, vDir);

        if (fDist > fRange)
            fDist = fRange;
    
        if (_bShowTestBeam)
        {
            if (_cTestLaser == null)
                CreateTestBeamLaser();
            UpdateTestBeamParams();
            _cTestLaser.SetRange(fDist);
            _cTestLaser.PointFromTowards(vBarrelPos, vDir);
        }

        //wczesniej nalezy ustawic odowiednie pola FXa
        fFXSizeY = fDist;
        fFXPosY = fFXSizeY * .5f;
        
        if(sFXCombName!=null)
            OwnerExecuteFXBarrel(sFXCombName);
        
        if (Owner != null)
        {
            if (!bNoShootSound)
            {
                if(nSoundShoot == 1)
                    Owner.AttachOneShotSound(sSoundShoot, fMinShootSound, fMaxShootSound);//, vBarrelPos);
                else
                {
                    Owner.AttachOneShotSound(sSoundShoot + "0"+ String.valueOf(RandomInt(nSoundShoot)+1), fMinShootSound, fMaxShootSound);//, vBarrelPos);
                }
            }
        
            Owner.MakeNoise(fNoiseShot, Pawn._NOISE_WEAPON_FIRING);
        }
        else
            LogR("WeaponBullet.Fire Owner == null\n");
    }

    /** Do pokrycia w broni dla wlasciciela posiadajacego kilka gunow
     */
    void OwnerExecuteFXBarrel(String sFXCombName)
    {
        if ( m_cWalker != null )
            WalkerExecuteFXBarrel( sFXCombName );
        else
        {
            if ( Owner != null )
                 Owner.ExecuteFXBarrel( sFXCombName );
        }
    }
    
    
}
