public class LiftPlatformFloor extends MeshObject
{
    public Object cDoors = null;//#FLOOR
    
    DoorsObject cControledDoors = null;
    LiftPlatformButton cButton = null;
    
    /**Czy winda jest na danym pietrze*/
    boolean bLiftOnFloor = false;
    
    // okresla czy drzwi zostaly zamkniete nie przez winde tylko w misjach
    boolean bDoorsMissionLocked = false;

    // flaga okresla czy pietro ma zamykac automatycznie drzwi
    boolean bNoAutoLock = false;
    
    // definiujemy rysowanie kresek miedzy kolejnymi Waypointami
    /*
    public static void DefineVisualisation()
    {
        DefineDebugLine("cDoors", new Vector(1,1,1));
        DefineDebugLine("cControledDoors", "cButton", new Vector(1,1,1));
    }
    */
    
    void OnCreate()
    {
		bSG = true;
		
        super.OnCreate();
        
        if(cDoors!=null && (cDoors instanceof DoorsObject))
        {
            cControledDoors = (DoorsObject)cDoors;
        
            // drzwi na dziendobry zamkniete, czyli w edytorze ktos je zamknal
            bDoorsMissionLocked = cControledDoors.IsLocked();
            
            // poczatkowo drzwi sa zamkniete
            cControledDoors.Lock();
        }
        else
            CrashLog("No doors connected with floor <"+GetName()+"> found.");
        
        //hak, zeby przy pierwszym update guzika sie wlaczylo
        bMissionLockedPrev = !bDoorsMissionLocked;
        CallMethodEvery("UpdateButton", 0.5f);
    }
    
    boolean bMissionLockedPrev = false;
    
    public void UpdateButton()
    {
        UpdateLockStatusFull();
        if (bMissionLockedPrev != bDoorsMissionLocked && cButton != null)
        {
            if (bDoorsMissionLocked)
            {
                cButton.DoorsLockedByMission(bLiftOnFloor);
            }
            else
            {
                cButton.DoorsUnlockedByMission(bLiftOnFloor);
                //winda jest aktywowana przez otwarcie drzwi, jesli jest ona poza pietrem to zamykamy drzwi, 
                //ale bedzie sie dalo winde wezwac
                if (!bLiftOnFloor && cControledDoors != null && bNoAutoLock)
                    cControledDoors.Lock();
            }
        }
        bMissionLockedPrev = bDoorsMissionLocked;
    }
    
    /** Metoda wolana przez winde kiedy jedzie w kierunku pietra
     */
    void LiftLiftRunningToFloor(LiftPlatform cLift)
    {
        bLiftOnFloor = false;
        UpdateLockStatus();
        if ( cButton != null ) 
            cButton.LiftRunningToFloor();
    }
    
    /** Metoda wolana przez winde kiedy wjedzie na pietro
     */
    void _LiftEntersFloor(LiftPlatform cLift, boolean bSG)
    {
        bLiftOnFloor = true;
        UpdateLockStatus();
        
        if(cControledDoors != null &&
           !bDoorsMissionLocked) 
		{
			if (bSG == true)
				cControledDoors.UpdateIndicators();  
			else
				cControledDoors.Unlock();
		}
        
        if(cButton!=null) 
            cButton.LiftOnFloor(bDoorsMissionLocked);
    }
	
	void LiftEntersFloor(LiftPlatform cLift)
	{
		_LiftEntersFloor(cLift, false);
	}
	
    void SGLiftEntersFloor(LiftPlatform cLift)
    {
		_LiftEntersFloor(cLift, true);
    }
    
    /** Metoda wolana przez winde kiedy wyjedzie z pietro
     */
    void LiftLeavesFloor(LiftPlatform cLift)
    {
        bLiftOnFloor = false;
        
        UpdateLockStatus();
        
        if(cControledDoors != null &&
           !bDoorsMissionLocked) 
                cControledDoors.Lock();
        if(cButton!=null) 
            cButton.NoLiftOnFloor(bDoorsMissionLocked);
    }
	
    void UpdateLockStatus()
    {
        if(cControledDoors!=null)
        {
            if(bDoorsMissionLocked)
            {
                // drzwi zamkniete z misji, wiec sprawdzamy czy dalej zamkniete
                bDoorsMissionLocked = cControledDoors.IsLocked() && !cControledDoors.bDoorsOpened;
            }
        }
    }
    
    void UpdateLockStatusFull()
    {
        if(cControledDoors!=null)
        {
            //wiemy, ze winda ma juz ustalony stan wiec mozemy sprawdzic czy drzwi nie
            //zostaly zamkniete w trakcie misji (winda na pietrze a mimo to drzwi zamkniete
            if(bDoorsMissionLocked || (bLiftOnFloor && cControledDoors.IsLocked()))
            {
                // drzwi zamkniete z misji, wiec sprawdzamy czy dalej zamkniete
                bDoorsMissionLocked = cControledDoors.IsLocked() && !cControledDoors.bDoorsOpened;
            }
        }
    }
                           
    
    boolean IsLeftOnFloor()
    {
        return bLiftOnFloor;
    }

    /** Metoda wolana na filmach */
    void EnableAutoLock()
    {
        bNoAutoLock = false;
    }
    
    /** Metoda wolana na filmach */
    void DisableAutoLock()
    {
        bNoAutoLock = true;
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[LiftPlatformFloor.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_LIFT_PLATFORM_FLOOR)
        {
			super.SGLoadChunk(cFC);
									
			// okresla czy drzwi zostaly zamkniete nie przez winde tylko w misjach
			bDoorsMissionLocked = cFC.LoadBool();			
			// czy winda jest na danym pietrze ?
			bLiftOnFloor = cFC.LoadBool();			
			// Czy winda jest na danym pietrze ?
			bLiftOnFloor = cFC.LoadBool();			
        }
		
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[LiftPlatformFloor.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_LIFT_PLATFORM_FLOOR, cFCParent);
		
		super.SGSaveChunk(cFC);
		    
		// okresla czy drzwi zostaly zamkniete nie przez winde tylko w misjach
		cFC.SaveBool(bDoorsMissionLocked);
		cFC.SaveBool(bMissionLockedPrev);
		// Czy winda jest na danym pietrze ?
		cFC.SaveBool(bLiftOnFloor);
			
		cFC.delete(); 
	}	
	
	public void SGUpdate()
	{
		UpdateButton();
	}
}
