public class ChromeInvButton extends Button
{
    ChromeInventoryMenu m_cParent = null;
    
    void SetParent(ChromeInventoryMenu cParent)
    {
        m_cParent = cParent;
    }
    
    void Init(UIElement data, Module module)
    {
        SetModule(module);
        nID = data.nID;
        nIDExtra = data.nIDExtra;
        nBtnType = data.nFlags & EButtonFlags.BTN_TYPE_MASK;
        if ( (data.nFlags & EButtonFlags.BTN_STATE_MASK) != 0 )
        {
            nBtnState = data.nFlags & EButtonFlags.BTN_STATE_MASK;
        }
        if (data.sUncheckedTex.length() > 0) 
        {
            nUncheckedTex = AddTexture(data.sUncheckedTex, true, true);
            rUncheckedTex = data.rUncheckedTex;
        }
        if (data.sCheckedTex.length() > 0) 
        {
            nCheckedTex = AddTexture(data.sCheckedTex, true, true);
            rCheckedTex = data.rCheckedTex; 
        }
        if (data.sHighlightedTex.length() > 0)
			nHighlightedTex = AddTexture(data.sHighlightedTex, true, true);
        rHighlightedTex = data.rHighlightedTex;
        if (data.sDisabledTex.length() > 0)
			nDisabledTex = AddTexture(data.sDisabledTex, true, true);
        rDisabledTex = data.rDisabledTex;
        
        if (nBtnState == EButtonFlags.BTN_STATE_UNCHECKED)
        {
            if (nUncheckedTex != 0)
				SetTexture(nUncheckedTex, rUncheckedTex);
        }
        else
        {
            if (nCheckedTex != 0)
				SetTexture(nCheckedTex, rCheckedTex);
        }
        
        sText = data.sText;
        SetSize(data.vSize);
        SetRelativePos(data.vPos);
        ProcessMouse(true);
        Show();
        SetState("Idle");
    }
	
    /**Ustawia stan zaznaczenia/wcisniecia przycisku
     * @param bCheck jesli true to przycisk wcisniety/zaznaczony
     */
    void SetCheck(boolean bCheck)
    {
        if (nBtnState == EButtonFlags.BTN_STATE_DISABLED)
			return;
        if (bCheck)
        {
            if (nCheckedTex != 0)
				SetTexture(nCheckedTex, rCheckedTex);
            nBtnState = EButtonFlags.BTN_STATE_CHECKED;
        }
        else
        {
            if (nUncheckedTex != 0)
				SetTexture(nUncheckedTex, rUncheckedTex);
            nBtnState = EButtonFlags.BTN_STATE_UNCHECKED;
        }
    }
    
    boolean GetCheck()
    {
        return ((nBtnState & EButtonFlags.BTN_STATE_CHECKED) == EButtonFlags.BTN_STATE_CHECKED);
    }
        

    void OnRBUp(float fMouseX, float fMouseY)
    {
    }
    
    /**Ustala stan podswietlenia przycisku
     * @param bHighlight jesli true to podswietlamy przycisk
     * @param bNotify ustala, czy powiadomic rodzica o podswietlaniu
     * @see Window#OnButtonHighlight(Button btnSender)
     */
    void SetHighlight(boolean bHighlight, boolean bNotify)
    {
        if ((nBtnType == EButtonFlags.BTN_TYPE_LISTBOX_ITEM) ||
			(nBtnType == EButtonFlags.BTN_TYPE_LISTBOX)) 
            return;
		
        if (bHighlight)
        {
            if ((nBtnState & EButtonFlags.BTN_STATE_HIGHLIGHTED ) == 0)
            {
                nBtnState = nBtnState | EButtonFlags.BTN_STATE_HIGHLIGHTED;
                if (nHighlightedTex != 0) 
                {
                    SetTexture(nHighlightedTex, rHighlightedTex);
                    if (bNotify)
						ButtonHighlighted(this);
                }
            }
        }
        else
        {
            if ((nBtnState & EButtonFlags.BTN_STATE_HIGHLIGHTED ) == EButtonFlags.BTN_STATE_HIGHLIGHTED)
            {
                nBtnState = nBtnState & (~EButtonFlags.BTN_STATE_HIGHLIGHTED);
                SetCheck((nBtnState & EButtonFlags.BTN_STATE_CHECKED) ==
						 EButtonFlags.BTN_STATE_CHECKED);
            }
        }
    }
    
    void OnMouseEnter(float fMouseX, float fMouseY)
    {
	    m_cParent.ShowDesc(this);
		
        bMouseInside = true;
		
        SetHighlight(true, true);
    }
    
	void OnMouseLeave(float fMouseX, float fMouseY)
	{
        m_cParent.ClearDes();
		
        bMouseInside = false;
		
        if ((nBtnState & EButtonFlags.BTN_STATE_DISABLED) == EButtonFlags.BTN_STATE_DISABLED)
            return;
        if ((nBtnState & EButtonFlags.BTN_STATE_PUSHED) == EButtonFlags.BTN_STATE_PUSHED)
        {
            SetHighlight(false, true);
            nBtnState = nBtnState & (~EButtonFlags.BTN_STATE_PUSHED);   
            if ((nBtnType == EButtonFlags.BTN_TYPE_PUSHBUTTON) ||
                (nBtnType == EButtonFlags.BTN_TYPE_PRESSBUTTON))
                SetCheck(false);
        }
        else 
            SetHighlight(false, true);
	}

    void OnMouseMove(float fMouseX, float fMouseY, int iButtonsFlags)
    {
	    m_cParent.ShowDesc(this);
		
        bMouseInside = true;
        SetHighlight(true, true);
    }
    
    /**Metoda wywolywana po nacisnieciu przycisku lub checkboxa. 
     * Wywoluje handlera ktory odpala {@link Window#OnButton(Button btnSender)} lub 
     * {@link Window#OnButtonDoubleClick(Button btnSender)}
     *@param btnSender wcisniety przycisk potomny
     */
    void ButtonPushed(Button btnSender)
    {
        float fDblClckTime = 0.3f;
        float fPrevTime = fClickTime;
        fClickTime = GetTime();

        if (btnSender == null) 
            btnSender = this;
        
        if (fClickTime - fPrevTime < fDblClckTime)
        {
            bDoubleClick = true;
            m_cParent.DoubleClick(this);
            fClickTime = 0.0f;
        }
        else
        {
            m_cParent.OnButton(this);
        }
    }
    
    /**Metoda wywolywana po podswietleniu przycisku lub checkboxa. 
     * Wywoluje handlera ktory odpala {@link Window#OnButtonHighlight(Button btnSender)}
     *@param btnSender wcisniety przycisk potomny
     * @see #SetHighlight(boolean bHighlight, boolean bNotify)
     */
    void ButtonHighlighted(Button btnSender)
    {
        if (btnSender == null) 
            btnSender = this;
        CallOnButtonHighlight(btnSender);
    }
    
    /**Metoda wywolywana po wcisnieciu przycisku lub checkboxa prawym
     * klawiszem myszki.
     * Wywoluje handlera ktory odpala {@link Window#OnButtonRB(Button btnSender)}
     *@param btnSender wcisniety przycisk potomny
     */
    void ButtonPushedRB(Button btnSender)
    {
        if (btnSender == null) 
            btnSender = this;
        CallOnButtonRB(btnSender);
    }
    
    /**Pozwala aktywowac lub deaktywowac przycisk. Jesli przy 
     * inicjacji podalismy teksture wyszarzonego przycisku to 
     * przy deaktywacji zostanie ona wyswietlona.
     * @param bEnable jesli true to przycisk aktywny jesli false 
     * to nieaktywny*/
    void Enable(boolean bEnable)
    {
        if (bEnable)
        {            
            nBtnState = nBtnState & (~EButtonFlags.BTN_STATE_DISABLED);
            SetCheck(((nBtnState & EButtonFlags.BTN_STATE_CHECKED) == EButtonFlags.BTN_STATE_CHECKED));
            SetState("Idle");
        }
        else
        {
            nBtnState = nBtnState | EButtonFlags.BTN_STATE_DISABLED;
            if (nDisabledTex != 0)
            {
                SetTexture(nDisabledTex, rDisabledTex);
            }
            SetState("Disabled");            
        }
    }
   
    public class RMBDown extends State
    {
        public void OnEnter()
        {
        }
		
        void OnRBUp(float fMouseX, float fMouseY)
        {
            ButtonPushedRB(ChromeInvButton.this);
            SetState("Idle");
        }
        
        void OnRBDown(float fMouseX, float fMouseY)
        {
            SetState("Idle");
        }
        
        void OnMouseEnter(float fMouseX, float fMouseY)
        {
		    m_cParent.ShowDesc(ChromeInvButton.this);

	        bMouseInside = true;
			
            SetHighlight(true, true);
            SetState("Idle");
        }
        
        void OnMouseLeave(float fMouseX, float fMouseY)
        {
			m_cParent.ClearDes();
		
	        bMouseInside = false;
			
            SetHighlight(false, true);
            SetState("Idle");
        }
    }
    
    public class Idle extends State 
    {
        public void OnEnter()
        {
            if (nBtnType == EButtonFlags.BTN_TYPE_PRESSBUTTON)
            {
                if (bCallingPressButton)
					DisableCallMethod("PressButton");
                bCallingPressButton = false;
            }
        }
     
        void OnLBDown(float fMouseX, float fMouseY)
	    {
            if (!CanProcessMouseBtns())
				return;
			
            ButtonPushed(ChromeInvButton.this);
            if ((nBtnType == EButtonFlags.BTN_TYPE_PUSHBUTTON) ||
                (nBtnType == EButtonFlags.BTN_TYPE_PRESSBUTTON))
                SetCheck(true);

			nBtnState = nBtnState | EButtonFlags.BTN_STATE_PUSHED;
            SetState("LMBDown");
	    }
        
        void OnRBDown(float fMouseX, float fMouseY)
        {
            if (!CanProcessMouseBtns())
				return;
			
            SetState("RMBDown");
        }
    }
    
    public class LMBDown extends State 
    {
        //zmienna potrzebna gdy przycisk jest pressbuttonem
        float fLastOnButtonTime;
        float fPeriod = 0.1f;
		
        public void OnEnter()
        {
            if (nBtnType == EButtonFlags.BTN_TYPE_PRESSBUTTON)
            {
                CallMethodEvery("PressButton", 0.1f);
                bCallingPressButton = true;
            }
        }
		
        void OnLBUp(float fMouseX, float fMouseY)
	    {
            if (!CanProcessMouseBtns())
				return;
			
            if ((nBtnType == EButtonFlags.BTN_TYPE_CHECKBUTTON) ||
                (nBtnType == EButtonFlags.BTN_TYPE_LISTBOX_ITEM))
            {
                if ( (nBtnState & EButtonFlags.BTN_STATE_CHECKED) == EButtonFlags.BTN_STATE_CHECKED)
                    SetCheck(false);
                else
                    SetCheck(true);
            }
            else
                SetCheck(false);
			
            SetHighlight(true, false);
            SetState("Idle");
	    }   
        
        void OnRBDown(float fMouseX, float fMouseY)
        {
            SetState("Idle");
        }
        
        void OnMouseEnter(float fMouseX, float fMouseY)
        {
			LogLine("LMBDown:OnMouseEnter");
					
			bMouseInside = true;
		
            SetHighlight(true, true);
            SetState("Idle");
        }
        
        void OnMouseLeave(float fMouseX, float fMouseY)
        {
	        bMouseInside = false;
			
            SetHighlight(false, true);
            SetCheck(false);
            SetState("Idle");
        }
    }
    
    public class Disabled extends State
    {
        void OnLBDown(float fMouseX, float fMouseY){}
        void OnRBDown(float fMouseX, float fMouseY){}
        void OnMouseEnter(float fMouseX, float fMouseY){}
        void OnMouseLeave(float fMouseX, float fMouseY){}
        void OnMouseMove(float fMouseX, float fMouseY, int iButtonsFlags){}
    }
    
    void PressButton()
    {
        if (bDoubleClick)
            bDoubleClick = false;
        else
            CallOnButton(this);
    }
    
    /** metoda wywolywana gdy jest doubleclick
     */
    void OnButtonDoubleClick(Button btnSender)
    {
		m_cParent.DoubleClick(this);
    }
}
