import java.util.*;
public class SecuritySystem extends GameObject 
    implements iact_EnableDisable, ipar_ConflictSide
{
    public boolean bEnabledOnStart = true;//#\SECUR_SYSTEM;Initial state (Enabled/Disabled) of security system components
    
    public EnumConflictSide cConflictSide = new EnumConflictSide(EnumConflictSide._COUNTER_PLAYER);//#\SECUR_SYSTEM;Initial conflict side of security system components
    
    /////////////////////////////////////////////////////////////////////////////
    
    protected HashSet aComponents = new HashSet();


    boolean bEnabled = false;
	
	public void OnCreate()
	{
		super.OnCreate();
		bSG = true;
	}
    
    /////////////////////////////////////////////////////////////////////////////

    public void OnPostCreate()
    {
        if (bEnabledOnStart)
            Enable();
        else
            Disable();
        ChangeConflictSide(cConflictSide.GetValue());
    }
    
    public void RegisterComponent(ISecuritySystemComponent cComponent)
    {
        aComponents.add(cComponent);
    }
    
    
    
    /////////////////////////////////////////////////////////////////////////////
    
    public void ChangeConflictSide(int nNewConflictSide)
    {
        Iterator iter = aComponents.iterator();
        while (iter.hasNext())
        {
            ipar_ConflictSide cComponent = (ipar_ConflictSide)iter.next();
            cComponent.ChangeConflictSide(nNewConflictSide);
        }
    }
    
    public void Enable()
    {
        bEnabled = true;
            
        Iterator iter = aComponents.iterator();
        while (iter.hasNext())
        {
            iact_EnableDisable cComponent = (iact_EnableDisable)iter.next();
            cComponent.Enable();
        }
    }
    
    public void Disable()
    {
        bEnabled = false;
        
        Iterator iter = aComponents.iterator();
        while (iter.hasNext())
        {
            iact_EnableDisable cComponent = (iact_EnableDisable)iter.next();
            cComponent.Disable();
        }
    }
    
    public boolean IsEnabled()
    {
        return bEnabled;
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[SecuritySystem.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_SECURITY_SYSTEM)
        {	
			super.SGLoadChunk(cFC);

			cConflictSide = new EnumConflictSide(cFC.LoadInt());
			
			int nSize = cFC.LoadInt();
			
			aComponents.clear();
				
			for (int i=0; i < nSize; i++)
				aComponents.add(cFC.LoadObjectRef());
		
			bEnabled = cFC.LoadBool();
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[SecuritySystem.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_SECURITY_SYSTEM, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		cFC.SaveInt(cConflictSide.GetValue());
		
        Iterator iter = aComponents.iterator();
		
		cFC.SaveInt(aComponents.size());
		
        while (iter.hasNext())
        {
            GameObject cComponent = (GameObject)iter.next();
            cFC.SaveObjectRef(cComponent);
        }
		
		cFC.SaveBool(bEnabled);
	
        cFC.delete();
    }	    
	
	public void SGUpdate()
	{
		FileChunk.Log("[SecuritySystem.SGUpdate]\n");
		
		super.SGUpdate();
		
		//! Usu jak wszystko bdzie si zapisywao
		if (bEnabled)
			Enable();
		else
			Disable();
	}
}
