public class sensor_Track extends Sensor
{
    /** Strona konfliktu na jaka reaguje sensor */
    public EnumSides cSide = new EnumSides(EnumSides._PLAYER);//#\SENSOR
    
    /** Obiekt, ktorego dotyczy ten sensor */
    public Object cObject = null;//#\SENSOR
    
    /** Grupa, ktorej dotyczy ten sensor */
    public IGroup cGroup = null;//#\SENSOR
    
    /** Akcja wolana gdy obiekt wchodzi w sensor */
    public Action[] cOnEnter = null;//#\SENSOR
    
    /** Akcja wolana gdy obiekt wychodzi z sensora */
    public Action[] cOnLeave = null;//#\SENSOR
    
    /** Kwadrat odleglosci od obiektu lub grupy */
    float fLastDistance2 = Float.MAX_VALUE;//trzeba zainicjowac duza wartoscia, bo inaczej na poczatku bedzie leave
    
    // ile razy ma dzialac onEnter
    public int nOnEnterTimes = -1;//#\SENSOR;
    
    // ile razy ma dzialac onLeave
    public int nOnLeaveTimes = -1;//#\SENSOR;
    
    final static int SENS_NONE   = -1;
    final static int SENS_PLAYER = 0;
    final static int SENS_OBJECT = 1;
    final static int SENS_GROUP  = 2;
    final static int SENS_SIDE   = 3;

    /** Typ sensora ustalony na podstawie  */
    int nSensorType = -1;

    /** Modul singleplayer do pobierania gracza */
    ChromeModuleSingle cModule = null;
    
    public void OnCreate()
    {
        super.OnCreate();
        
        Module mod = GetModule();
        if(!(mod instanceof ChromeModuleSingle)) return;
        
        cModule = (ChromeModuleSingle)mod;
     
        if(cObject != null)
        {
            nSensorType = SENS_OBJECT;
        }
        else
        {
            if( cGroup != null)
            {
                nSensorType = SENS_GROUP;
            }
            else
            {
                if(cSide != null)
                {
                    // ustalamy rodzaj sensoru
                    if(cSide.GetValue() == EnumSides._PLAYER )
                    {
                        nSensorType = SENS_PLAYER;
                    }
                    else
                    {
                        nSensorType = SENS_SIDE;
                    }
                }
            }
        }
    }
    
    public void OnDestroy()
    {
    }
    
    void Check()
    {
        switch(nSensorType)
        {
        case SENS_NONE:
            DisableCallMethod("Check");
            break;
            
        case SENS_PLAYER:
        case SENS_OBJECT:
            CheckObject();
            break;
            
        case SENS_GROUP:
            CheckGroup();
            break;
            
        case SENS_SIDE:
            CheckSide();
            break;
        }
    }
    
    void CheckSide()
    {
        //TODO
    }
    
    void CheckGroup()
    {
        //TODO
    }
    
    void CheckObject()
    {
        Vector vPos = GetPosition();

        // wybieramy pozycje z obiektu
        Vector vObjectPos = null;
        if(nSensorType == SENS_PLAYER)
            vObjectPos = cModule.MainPlayer.GetPosition();
        else
            vObjectPos = ((ControlObject)cObject).GetPosition();

        if(!bEnabled) return;

        //TODO splaszczamy obliczenie odleglosci
        //vPos.fY = 0.0f;
        //vObjectPos.fY = 0.0f;
        
        // wyliczamy odleglosc gracza od hotspota
        float fDistance2 = vPos.Distance2(vObjectPos);

        //CrashLog("[sensor_Track.Check] Checking fRadius2="+fRadius2+" fDistance2="+fDistance2+" bDisabled="+bEnabled+"\n");
        
        if ((fLastDistance2 >  fRadius2) &&
            (fDistance2     <= fRadius2))
        {
            if(nOnEnterTimes != 0)
            {
                bEnteredOnce = true;
            
                // wywolujemy akcje wejscia w sensor
                if(cOnEnter != null)
                {
                    InvokeActionChain(cOnEnter);
                    if(nOnEnterTimes>0) nOnEnterTimes--;
                }
            }
        }

        // sprawdzanie czy sensor nie jest wylaczony
        if(!bEnabled) return;

        if ((fLastDistance2 <= fRadius2) &&
            (fDistance2     >  fRadius2))
        {
            if(nOnLeaveTimes != 0)
            {
                bLeftOnce = true;
                
                // wywolujemy akcje wyjscia z sensora
                if(cOnLeave != null)
                {
                    InvokeActionChain(cOnLeave);
                    if(nOnLeaveTimes>0) nOnLeaveTimes--;
                }
            }
        }

        // przepisujemy odleglosc
        fLastDistance2 = fDistance2;
        
        // sprawdzamy czy nie skasowac Sensora, ale wylaczamy jego dzialanie
        // tylko, bo ktos moze sie do niego odwolywac za pomoca akcji i bedzie
        // wtedy exception
        if(nOnEnterTimes==0 && nOnLeaveTimes==0)
        {
            DisableCallMethod("Check");
        }
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[sensor_Track.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunksChrome._CHK_SENSOR_TRACK)
        {
			super.SGLoadChunk(cFC);

			// Kwadrat odleglosci od obiektu lub grupy
			fLastDistance2 = cFC.LoadFloat();
			// ile razy ma dzialac onEnter	
			nOnEnterTimes = cFC.LoadInt();
			// ile razy ma dzialac onLeave
			nOnLeaveTimes = cFC.LoadInt();
        }
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[sensor_Track.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_SENSOR_TRACK, cFCParent);
		
		super.SGSaveChunk(cFC);

		// Kwadrat odleglosci od obiektu lub grupy
		cFC.SaveFloat(fLastDistance2);
		// ile razy ma dzialac onEnter	
		cFC.SaveInt(nOnEnterTimes);
		// ile razy ma dzialac onLeave
		cFC.SaveInt(nOnLeaveTimes);
			
		cFC.delete();
	}			
}
