
/** Manager komend wpisywanych z konsoli
 * Aby mozna bylo uzyc komendy nalezy w OnCreate stworzyc jej obiekt
 * oraz dodac go do list m_cCommandsList 
 */
public class ConsoleCommandsManager extends GameObject
{
    ConsoleCommandsList m_cCommandsList = null;
    
    /** NetGameInfo w ktorym umieszczony jest ten manager
     */
    protected NetGameInfoHud m_cGameInfo   = null;
    void SetGameInfo(NetGameInfoHud cGameInfo) 
    { 
        m_cGameInfo = cGameInfo; 
        m_cCommandsList.SetGameInfo( cGameInfo );
    }
    
    static int _COMMAND_SERVER        = -1,
               _COMMAND_UNKNOWN       = -1;
    

    
    void OnCreate()
    {
        m_cCommandsList = new ConsoleCommandsList();
        //te ponizsze 2 komendy specjalne musza byc zawsze!
        _COMMAND_SERVER  = m_cCommandsList.AddCommand( new ConsoleCommandServer() );
        _COMMAND_UNKNOWN = m_cCommandsList.AddCommand( new ConsoleCommandUnknown() );
        //tu juz "komendy" zwykle
        m_cCommandsList.AddCommand( new CommandMsgTeam() );
        m_cCommandsList.AddCommand( new CommandChangeTeam() );
        m_cCommandsList.AddCommand( new CommandNick() );
        m_cCommandsList.AddCommand( new CommandKillMe() );
        m_cCommandsList.AddCommand( new CommandTimeLeft() );
        m_cCommandsList.AddCommand( new CommandPlayersList() );
        m_cCommandsList.AddCommand( new CommandAdminLogin() );
        m_cCommandsList.AddCommand( new CommandAdminLogout() );
        m_cCommandsList.AddCommand( new CommandAdminSay() );
        m_cCommandsList.AddCommand( new CommandRun() );
        m_cCommandsList.AddCommand( new CommandKick() );
        m_cCommandsList.AddCommand( new CommandKill() );
        m_cCommandsList.AddCommand( new CommandMap() );
        m_cCommandsList.AddCommand( new CommandMapNext() );
        m_cCommandsList.AddCommand( new CommandMapRemove() );
        m_cCommandsList.AddCommand( new CommandMapSetParam() );
        m_cCommandsList.AddCommand( new CommandMapSetDefaultParams() );
        m_cCommandsList.AddCommand( new CommandMapSetCustomParams() );
        m_cCommandsList.AddCommand( new CommandMapSaveParams() );
        m_cCommandsList.AddCommand( new CommandServerShutdown() );
        m_cCommandsList.AddCommand( new CommandExit() );
        //m_cCommandsList.AddCommand( new CommandServerRestart() );
        //TODO: tu ewentualnie dodac nowe komendy
    }
    
    //to testow metody znanej tylko serwerowi
    void AddCommands()
    {
        //m_cCommandsList.AddCommand( new ConsoleCommandServerTest() );
    }
    
    void OnDestroy()
    {
        m_cCommandsList.Destroy();
        m_cCommandsList = null;
    }

    /**
     * Metoda sprawdza czy wpisany tekst nie jest czasem
     * jakas komenda. Komendy rozpoczynaja sie od znaku "/".
     * Jesli podany tekst nie rozpoczyna sie od tego znaku
     * to metoda zwraca null.
     * Jesli podany tekst rozpoczyna sie od takiego znaku
     * to szukana jest komenda o podanej nazwie i jesli zostanie
     * znaleziona to zostaje zwrocony obiekt tej komendy (z listy)
     * Jesli nie zostanie znaleziona to zostaje zwrocony null,
     * chyba ze bForceCommand == true, wtedy zwrocony zostanie obiekt 
     * komendy specjalnej (ConsoleCommandUnknown)
     */
    ConsoleCommand CheckCommand( String sTypedText, boolean bForceCommand )
    {
        String sCommandName = ParseCommandName( sTypedText );
        
        if ( sCommandName != null )
        {
            ConsoleCommand cCommand = m_cCommandsList.GetCommandByName( sCommandName );
            if ( cCommand == null )
            {
                String sAlias = "";
                                                      
                if ( bForceCommand )
                    cCommand = FindCommand( _COMMAND_UNKNOWN );
                else
                    return null;
            }  
            cCommand.Clean();
            cCommand.InitStreamParams( sTypedText );
            return cCommand;
        }
        return null;
    }
    ConsoleCommand CheckCommand( String sTypedText )
    {
        return CheckCommand( sTypedText, true );
    }
    
    
    
    void ServerTypedCommand( ConsoleCommand cCommand )
    {
        //jestesmy serwerem, wiec no problem
        //wykonujemy komende lokalnie
        cCommand.SetLocalPerform( true );
        PerformCommand( cCommand );
            
        //czy udalo sie dobrze wykonac
        if ( ! cCommand.PerformedSuccessfully() )
        {
            //niestety nie, pokazmy bledy
            cCommand.ShowErrors();
            //i nic tu po nas
            return;
        }
            
        //czy maja byc jakies wyniki?
        if ( cCommand.AreResultsExpected() )
        {
            //jesli wyniki maja byc znane wszystkim to 
            //wysylamy je do klientow
            if ( cCommand.AreResultsToAll() )
                m_cGameInfo.MsgSendClientCommandResults( cCommand );
                
            //u nas trzeba oczywiscie tez to pokazac
            cCommand.ShowResults();
        }
        
        //sprawdzmy czy komenda ma cos jeszcze do zrobienia na koniec
        cCommand.PostPerform();
            
        //dobrym zwyczajem po zabawie czyscimy komende
        cCommand.Clean();
    }
    
    void ClientTypedCommand( ConsoleCommand cCommand )
    {
        //jestesmy klientem, wiec sprawdzmy
        //czy to lokalna komenda, w przeciwnym wypadku spadamy
        if ( cCommand.ShouldExecuteOnlyOnServer() )
        {
            NetLog.ConsoleCommand(""+GetNetInfo()+" is not local -> skip on client"); 
            return;
        }
        
        //wykonujemy komende lokalnie
        cCommand.SetLocalPerform( true );
        PerformCommand( cCommand );
            
        //czy udalo sie dobrze wykonac
        if ( ! cCommand.PerformedSuccessfully() )
        {
            //niestety nie, pokazmy bledy
            cCommand.ShowErrors();
            //i nic tu po nas
            return;
        }
            
        //czy maja byc jakies wyniki?
        if ( cCommand.AreResultsExpected() )
        {
            //TODO: w sumie to komendy lokalne nie powinny
            //chyba rozpropagowywac wynikow, ale jesli kiedys
            //zajdzie taka potrzeba, to tu trzeba to uwzgladnic
            
            //pokazujemy je
            cCommand.ShowResults();
        }
        
        //sprawdzmy czy komenda ma cos jeszcze do zrobienia na koniec
        cCommand.PostPerform();
        
        //dobrym zwyczajem po zabawie czyscimy komende
        cCommand.Clean();
    }
    
    void ServerReceivedCommand( ConsoleCommand cReceivedCommand )
    {
        //klient prosi o wykonanie komendy, 
        
        //tu bedzie komenda ktora bedzie wykonywana
        ConsoleCommand cCommand = null;
        
        //wiec sprawdzmy czy to jest jakas znana jemu komenda
        if ( cReceivedCommand.GetId() != _COMMAND_UNKNOWN )
        {
            //klient jak najbardziej wiedzial o co mu chodzi,
            //zatem idzie ona bezposrednio do wykonania.
            cCommand = cReceivedCommand;
        }
        else            
        {
            //klient nie rozpoznal tej komendy, 
            //wiec sprawdzmy czy to nie jest jakas znana tylko na
            //serwerze (np poprzez mutatory)
            
            //pobieramy jej nazwe z komendy ktorej przyszla
            String sNameWithPrefix = sm_sCommandPrefix + cReceivedCommand.GetName();
            //i szukamy takiej komendy
            ConsoleCommand cServerCommand = CheckCommand( sNameWithPrefix, false );
            //znalazl jakas?
            if ( cServerCommand != null )
            {
                //znalazl, wiec
                
                //to bedzie nasza metode do wykonania
                cCommand = cServerCommand;
                //dajmy jej parametry z tego co przyslal klient
                cCommand.Init( cReceivedCommand );
            }
            else
            {
                //nie znalazl zadnej takiej komendy, wolamy ta 
                //co przyszla z klienta 
                cCommand = cReceivedCommand;
                
                //nie musimy robic nic bo Perform() dla metody Unknown 
                //powinien zwrocic blad _ERROR_COMMAND_UNKNOWN
            }
        }
        
        //wywolujemy komende, ale nie jest to komenda nasza
        cCommand.SetLocalPerform( false );
        PerformCommand( cCommand );
        
        //czy wszystko sie udalo
        if ( cCommand.PerformedSuccessfully() )
        {
            //tak, 
            
            //miala byc odpowiedz?
            if ( cCommand.AreResultsExpected() )
            {
                //no to wysylamy (albo jemu, albo wszystkim)
                m_cGameInfo.MsgSendClientCommandResults( cCommand );
                
                //jak to odpowiedz do wszystkich to sobie tez wyswietlimy
                if ( cCommand.AreResultsToAll() ) 
                     cCommand.ShowResults();
            }
            
            //sprawdzmy czy komenda ma cos jeszcze do zrobienia na koniec
            cCommand.PostPerform();
        }
        else
        {
            //niestety nie, wyslijmy bledy klientowi
            m_cGameInfo.MsgSendClientCommandErrors( cCommand );
        }
        
        //dobrym zwyczajem po zabawie czyscimy komende
        cCommand.Clean();
    }
    
    ConsoleCommand FindCommand( int nCommandId )
    {
        ConsoleCommand cCommand = m_cCommandsList.GetCommandById( nCommandId );
        if ( cCommand != null )
             cCommand.Clean();
        return cCommand;
    }
           
    void PerformCommand( ConsoleCommand cCommand )
    {
        if ( cCommand == null )
            return;
        if ( cCommand.IsForAdminsOnly() ) 
        {
            int nHostId = cCommand.GetHost();
            
            int nRights = m_cGameInfo.GetAdminsManager().GetAdminRights( nHostId );
            
            if ( nRights == EAdminRights._NONE )
            {
                cCommand.SetErrorId( EConsoleCommandError._YOU_ARE_NOT_ADMIN );
                return;
            }
            
            if ( nRights < cCommand.GetRequiredAdminRights() )
            {
                cCommand.SetErrorId( EConsoleCommandError._YOU_ARE_NOT_AUTHORIZED );
                return;
            }
        }
        cCommand.InitPerform();
        if ( cCommand.PerformFailed() )
            return;
        
        cCommand.Perform();
    }
   
    
    final static String sm_sCommandPrefix = "/";
    
    /**
     * Zwraca nazwe komendy ze stringa,
     * string musi sie zaczynac od sm_sCommandPrefix
     */
    static String ParseCommandName(String sTypedText)
    {
        return NetGameTools.GetTextWithoutPrefix( sTypedText, sm_sCommandPrefix );
    }
    
    

}
