public abstract class ConsoleCommandParams extends ConsoleCommand
{
    ConsoleCommandParams()
    {
        super();
    }

    /** Parametry komendy odczytane z command prompta, 
     * (wszystko to co zostalo wpisane po nazwie komendy (i spacji) */
    private String m_sStreamParams = null;
    String GetStreamParams(){ return m_sStreamParams; }
    

    /**
     * Array of parsed parameters
     */
    private 
    String[] m_aParams       = null;
    String[] GetParams(){ return m_aParams; }
    void     SetParams( String[] aParams ){ m_aParams = aParams; }
    int      GetParamsCnt() { return (m_aParams != null)? m_aParams.length : 0; }
    
    String GetParam(int nParam)
    { 
        if ( nParam < GetParamsCnt() )
            return m_aParams[ nParam ];
        NetLog.Error("Can't get "+nParam+" param "+GetNetInfo());
        return null;
    }
    
    void SetParam( int nParam, String sParam )
    {
        if ( nParam < GetParamsCnt() )
            m_aParams[ nParam ] = sParam; 
        else
            NetLog.Error("Can't set "+nParam+" param "+GetNetInfo());
    }
    
    /**
     * Array of parsed parameters from parameter
     * to end of stream.
     */
    private 
    String[] m_aParamsToEnd  = null;
    String[] GetParamsToEnd(){ return m_aParamsToEnd; }
    void     SetParamsToEnd(String[] aParams){ m_aParamsToEnd = aParams; }
    int      GetParamsToEndCnt() { return (m_aParamsToEnd != null)? m_aParamsToEnd.length : 0; }    
    
    String GetParamsToEnd(int nParam)
    { 
        if ( nParam < GetParamsToEndCnt() )
            return m_aParamsToEnd[ nParam ];
        NetLog.Error("Can't get "+nParam+" params to end "+GetNetInfo());
        return null;
    }
    
    
    
    /**
     * Ile minimum parametrow mnie interesuje
     * jak -1 to znaczy ze nie mam zadnych wymagan
     */
    int m_nMinParams  = -1;
    /**
     * Ile maksimum parametrow mnie interesuje
     * jak -1 to znaczy ze nie mam zadnych wymagan i parsowane beda wszystkie
     */
    int m_nMaxParams = -1;
    
    
    private boolean m_bParamsParsed    = false;
    
    void Clean()
    {
        super.Clean();
        m_sStreamParams    = null;        
        m_aParams          = null;
        m_aParamsToEnd     = null;
        m_sWorkParams      = null;
        m_bParamsParsed    = false;
    }
    
    void InitStreamParams(String sText)
    {
        //ogolnie odczyt parametrow to wyciecie z tekstu nazwy komendy
        String sNameWithPrefix = ConsoleCommandsManager.sm_sCommandPrefix + GetName();
        m_sStreamParams = NetGameTools.RemoveWord( sText, sNameWithPrefix );
    }
    
    
    void Init( ConsoleCommandParams cCommand )
    {
        super.Init( cCommand );
        
        m_sStreamParams = cCommand.GetStreamParams();
    }
    
    void Pack(GameObject cMessager)
    {
        super.Pack( cMessager );
        PackParams( cMessager );
    }
    
    
    void UnPack(GameObject cMessager)
    {
        super.UnPack( cMessager );
        UnPackParams( cMessager );
    }
    
    /** Metoda pakujaca parametry komendy
     * Bedzie nadpisywana jesli przydatne bedzie sparsowanie
     * parametrow (i ew. konwersja ich na liczby) juz po stronie
     * klienckiej.
     **/
    void PackParams(GameObject cMessager)
    {
        cMessager.MessagePackStringBool( m_sStreamParams );
    }
    
    /** Metoda odpakowujaca parametry komendy
     **/
    void UnPackParams(GameObject cMessager)
    {
        m_sStreamParams = cMessager.MessageUnPackStringBool( null );
    }
        
    void InitPerform()
    {
        super.InitPerform();
        
        ParseParams();
        
        if ( ( m_nMinParams > 0 ) && ( GetParamsCnt() < m_nMinParams ) )
        {
            SetErrorId( EConsoleCommandError._WRONG_PARAMS_CNT );
        }
    }
    
    /**
     * Metody do parsowania parametrow podczas
     * rozbijania parametrow komendy
     */
    
    /** Pobiera slowo ze stringa z parametrami i 
     * jednoczesnie usuwa go ze stringa */
    private String GetAndRemoveNextParam()
    {
        String sWord = NetGameTools.GetFirstWord( m_sWorkParams );
        m_sWorkParams = NetGameTools.RemoveWord( m_sWorkParams, sWord );    
        return sWord;
    }
    
    /** Pobiera pozostalosc ze stringa (czyli to co nie
     * zostalo pobrane za pomoca GetParam() )
     */
    private String GetFromNextParamToEnd() { return m_sWorkParams; }

    /** Robocze parametry komendy, w metodzie InitPerform beda
     * tu przekopiowane parametry z m_sParams a nastepnie podczas
     * wykonywanai komendy stad beda pobierane (i usuwane) kolejne
     * slowa (parametry) */
    private String m_sWorkParams = null;
    
    private void ParseParams()
    {
        if ( m_bParamsParsed )
            return;
        
        m_sWorkParams = GetStreamParams();
        
        ObjectsList cList      = new ObjectsList();
        int         nParamsCnt = 0;
        do 
        {
            String sToEnd    = GetFromNextParamToEnd();
            String sCurrent  = GetAndRemoveNextParam();
            if ( ! NetGameTools.IsStringValid( sCurrent ) )
                break;
            cList.Add( sCurrent );
            cList.Add( sToEnd );
            
            nParamsCnt++;
            if ( (m_nMaxParams > 0) && (nParamsCnt >= m_nMaxParams) )
                break;
        } while ( true );
        
        m_aParams      = new String[ cList.GetSize()/2 ];
        m_aParamsToEnd = new String[ cList.GetSize()/2 ];
        for ( int i=0; i<m_aParams.length; ++i )
        {
            m_aParams       [ i ] = (String)cList.GetObjectAt( 2*i );
            m_aParamsToEnd  [ i ] = (String)cList.GetObjectAt( 2*i+1 );
            if ( NetLog.IsEnabled( NetLog._TYPE_CONSOLE_COMMAND ) )
            {
                NetLog.ConsoleCommand("Param "+i+" : "+m_aParams[i]+" "+m_aParamsToEnd[i]);
            }
        }
        cList.Destroy();
        m_bParamsParsed = true;
    }
    
    String GetSParam(int nParam)
    {
        //obslugujemy tylko do wystapienia pierwszego bledu
        if ( PerformFailed() )
            return null;

        String sValue = GetParam( nParam );
        NetLog.ConsoleCommand("GetSParam("+nParam+") "+sValue);
        return sValue; 
    }
    
    int GetIParam(int nParam)
    {
        //obslugujemy tylko do wystapienia pierwszego bledu
        if ( PerformFailed() )
            return 0;

        String sValue = GetParam( nParam );
        int    nValue = 0;
        try 
        {
		    nValue = Integer.parseInt( sValue );
        }
        catch(Exception e)
        {
            SetErrorId( EConsoleCommandError._WRONG_PARAM_TYPE ); 
        }
        NetLog.ConsoleCommand("GetIParam("+nParam+") '"+sValue+"' -> "+nValue);
        return nValue;
    }
    
    float GetFParam(int nParam)
    {
        //obslugujemy tylko do wystapienia pierwszego bledu
        if ( PerformFailed() )
            return 0.0f;

        String sValue = GetParam( nParam );
        float  fValue = 0.0f;
        try 
        {
		    fValue = Float.parseFloat( sValue );
        }
        catch(Exception e)
        {
            SetErrorId( EConsoleCommandError._WRONG_PARAM_TYPE ); 
        }
        NetLog.ConsoleCommand("GetFParam("+nParam+") '"+sValue+"' -> "+fValue);
        return fValue;
    }

    boolean CheckIParam( int nParam )
    {
        boolean bOk = true;
        String sValue = GetParam( nParam );
        try 
        {
		    Integer.parseInt( sValue );
        }
        catch(Exception e)
        {
            bOk = false;
        }
        return bOk;
    }
    boolean CheckFParam( int nParam )
    {
        boolean bOk = true;
        String sValue = GetParam( nParam );
        try 
        { 
            Float.parseFloat( sValue ); 
        }
        catch(Exception e) 
        {
            bOk = false;
        }
        return bOk;
    }
    
    String GetNetInfo(){ return super.GetNetInfo()+" "+GetStreamParams(); }
}

