public class NetPlayerInfo extends NetCompetitorInfo
{

    void TryFindPlayerByNetId()
    {
        PlayerNet cOldPlayer = m_cPlayer;
        if(m_nPlayerId > -1)
        {
            m_cPlayer = GetPlayerNetByNetId(m_nPlayerId);
            if(m_cPlayer == null)
            {
                CallMethodIn(METHOD_TryFindPlayerByNetId, 0.5F);
                return;
            }
            m_cPlayer.SetPlayerInfo(this);
            m_cPlayer.TryToSetYourName();
        }
        DisableCallMethod(METHOD_TryFindPlayerByNetId);
        if(cOldPlayer != null && cOldPlayer != m_cPlayer)
            cOldPlayer.SetPlayerInfo(null);
    }

    boolean IsCTFFlagOwner()
    {
        return m_bCTFFlagOwner;
    }

    void MsgRecvClientNextRespawnTime()
    {
        float fNextRespawnTime = MessageUnPackFloat();
        MessageEnd();
        if(super.Role < 4)
        {
            m_fNextRespawnTime = fNextRespawnTime;
            RefreshStatistics();
        }
    }

    void MsgSendClientCTFFlagOwner()
    {
        MessageNew(14);
        MessagePackBoolean(m_bCTFFlagOwner);
        MessageSend();
    }

    void SetCTFFlagOwner(boolean bFlagOwner)
    {
        m_bCTFFlagOwner = bFlagOwner;
        if(super.Role == 4)
            MsgSendClientCTFFlagOwner();
    }

    boolean IsAlive()
    {
        return !((Actor) (m_cPlayer)).bDead && !((PlayerNetBaseMortal) (m_cPlayer)).m_bJustDied;
    }

    void OnDestroy()
    {
        RemoveFromGameInfo();
    }

    void SetName(String sName)
    {
        super.SetName(sName);
        m_sNickClamped = null;
        if(m_cPlayer != null)
            m_cPlayer.SetName(sName);
    }

    static void CleanGlobalChangeNickCounter()
    {
        sm_cGlobalChangeNickCounter.Clean();
    }

    void ScheduleKickOutFromTheGame()
    {
        m_bKickedOutFromTheGame = true;
        CallMethodIn(METHOD_KickOutYourFromTheGame, 0.5F);
    }

    void KickOutYourFromTheGame()
    {
        ForceKickClient(GetClientOwnerId());
    }

    void SetPlayer(PlayerNet player)
    {
        m_cPlayer = player;
        if(player != null)
            m_nPlayerId = player.GetNetId();
        else
            m_nPlayerId = -1;
        if(super.Role == 4)
            MsgSendClientUpdatePlayerId();
    }

    void MsgSendClientUpdateName()
    {
        MessageNew(0);
        MessagePackInt(GetNextChangeNickGlobalCounter());
        MessagePackString(GetNick());
        MessageSend();
    }

    void SetPlayerPacketLoss(int nLoss)
    {
        m_nPacketLoss = nLoss;
    }

    String GetLatencyAndLoss()
    {
        return Integer.toString(m_nPing);
    }

    int GetClientOwnerId()
    {
        return m_nClientOwnerId;
    }

    void IncTeamKill()
    {
        m_nTeamKill++;
    }

    int GetTeamKill()
    {
        return m_nTeamKill;
    }

    void MsgSendClientUpdatePlayerId()
    {
        MessageNew(11);
        MessagePackInt(m_nPlayerId);
        MessagePackInt(m_nClientOwnerId);
        MessageSend();
    }

    int UpdateAndGetPacketLoss()
    {
        int nHostID = GetClientOwnerId();
        float fPacketLoss = GetPacketLoss(nHostID);
        m_nPacketLoss = (int)(fPacketLoss * 100F);
        return m_nPacketLoss;
    }

    void ServerForceKillMe()
    {
        if(super.Role == 4 && GetPlayer() != null)
            GetPlayer().ForceDieByYourSelf();
    }

    public void OnReplicateServer()
    {
        super.OnReplicateServer();
        if(super.bNetInit)
            MsgSendClientUpdatePlayerId();
    }

    boolean ShouldPerformVoiceOnFriendlyFire(NetPlayerInfo cAssassinInfo)
    {
        if(!m_bEnableAutoVoiceOnFriendlyFire)
        {
            return false;
        } else
        {
            m_bEnableAutoVoiceOnFriendlyFire = false;
            CallMethodIn(METHOD_EnableAutoVoiceOnFriendlyFire, 1.5F);
            return true;
        }
    }

    int GetPing()
    {
        return m_nPing;
    }

    void MsgSendUpdatePing()
    {
        MessageNewUnreliable(15);
        MessagePackInt(m_nPing);
        MessageSend();
    }

    void SendClientUpdateData()
    {
        super.SendClientUpdateData();
        MsgSendClientCTFFlagOwner();
    }

    void SendClientOwnerUpdateData()
    {
        super.SendClientOwnerUpdateData();
        MsgSendClientNextRespawnTime();
        MsgSendClientRespawnCnt();
    }

    boolean CheckFailed()
    {
        if(super.m_cGameInfo == null)
            return true;
        else
            return !IsAlive() && !RespawnLimitAvailable();
    }

    void MsgRecvUpdatePing()
    {
        m_nPing = MessageUnPackInt();
        MessageEnd();
    }

    void MsgSendClientRespawnCnt()
    {
        MessageNew(13);
        MessagePackInt(m_nRespawnCnt);
        MessageSend();
    }

    void SetReceivedPlayerState(byte nPlayerState, float fTime)
    {
        m_nPlayerState = nPlayerState;
        m_fStateTime = fTime;
    }

    void MsgRecvClientCTFFlagOwner()
    {
        boolean bCTFFlagOwner = MessageUnPackBoolean();
        MessageEnd();
        if(super.Role < 4)
        {
            m_bCTFFlagOwner = bCTFFlagOwner;
            RefreshStatistics();
        }
    }

    static int GetNextChangeNickGlobalCounter()
    {
        return sm_cGlobalChangeNickCounter.GetNext();
    }

    boolean IsBot()
    {
        return m_bBot;
    }

    void MsgRecvClientUpdateName()
    {
        int nCounter = MessageUnPackInt();
        String sNick = MessageUnPackString();
        MessageEnd();
        if(super.Role < 4 && VerifyChangeNickCounter(nCounter))
        {
            SetName(sNick);
            RefreshStatistics();
        }
    }

    PlayerNet GetPlayer()
    {
        return m_cPlayer;
    }

    boolean VerifyChangeNickCounter(int nCounter)
    {
        return m_cChangeNickCounter.Verify(nCounter);
    }

    void MsgRecvClientUpdatePlayerId()
    {
        int nPlayerId = MessageUnPackInt();
        int nClientOwnerId = MessageUnPackInt();
        MessageEnd();
        if(super.Role < 4)
        {
            m_nPlayerId = nPlayerId;
            TryFindPlayerByNetId();
        }
        SetClientOwnerId(nClientOwnerId);
    }

    void ServerInit(PlayerNet cPlayer, int nId, int nGameVersion)
    {
        SetPlayer(cPlayer);
        SetName(cPlayer.GetName());
        SetClientOwnerId(nId);
        SetClientGameVersion(nGameVersion);
        SetNextRespawnTime(GetTime());
        m_bAutoPlayer = cPlayer.IsAutoPlayer();
        m_bBot = cPlayer.IsBot();
    }

    int GetPlayerPacketLoss()
    {
        return m_nPacketLoss;
    }

    void SetClientGameVersion(int nGameVersion)
    {
        m_nGameVersion = nGameVersion;
    }

    void SetTeamId(int nId)
    {
        super.SetTeamId(nId);
        if(super.m_cGameInfo != null)
            super.m_cGameInfo.OnTeamChange();
    }

    int GetStatus()
    {
        return !IsAlive() ? 0 : 1;
    }

    boolean WasPressedFireToRespawn()
    {
        return GetPlayer() != null && GetPlayer().WasPressedFireToRespawn();
    }

    void SetVisibleOnHudNet(boolean bVisible)
    {
        m_bVisibleOnHudNet = bVisible;
    }

    boolean IsVisibleOnHudNet()
    {
        return m_bVisibleOnHudNet;
    }

    public void ReplicationInit()
    {
        super.ReplicationInit();
        SetNetPriority(0.1F);
    }

    void UpdatePing()
    {
        if(((ModuleNetBase) (super.m_cModule)).bBotsForTest && m_nClientOwnerId >= 0x186a0)
            return;
        if(m_nClientOwnerId > 0)
            m_nPing = GetClientLatency(m_nClientOwnerId);
    }

    int GetRespawnCnt()
    {
        return m_nRespawnCnt;
    }

    String GetNickClamped(UIStatic cTextField)
    {
        if(!IsNickValid())
            return GetNick();
        if(m_sNickClamped == null)
        {
            float fField = cTextField.GetSize().fX;
            if(fField == 0.0F)
                return GetNick();
            String sNick = new String(GetNick());
            boolean bClamped;
            for(bClamped = false; cTextField.GetTextExtent(sNick).fX > fField; bClamped = true)
                sNick = sNick.substring(0, sNick.length() - 1);

            if(bClamped)
            {
                sNick = sNick.substring(0, sNick.length() - 3);
                sNick += "...";
            }
            m_sNickClamped = sNick;
            return sNick;
        } else
        {
            return m_sNickClamped;
        }
    }

    void MsgRecvClientRespawnCnt()
    {
        int nRespawnCnt = MessageUnPackInt();
        MessageEnd();
        if(super.Role < 4)
        {
            m_nRespawnCnt = nRespawnCnt;
            RefreshStatistics();
        }
    }

    void IncRespawnCnt()
    {
        m_nRespawnCnt++;
        if(super.Role == 4)
            MsgSendClientRespawnCnt();
    }

    void ClearRespawnCnt()
    {
        m_nRespawnCnt = 0;
        if(ModuleNet.IsGameGoing() && super.Role == 4)
            MsgSendClientRespawnCnt();
    }

    void UpdateRespawnLimitCnt()
    {
        if(super.m_cGameInfo == null)
            return;
        int nLimit = super.m_cGameInfo.GetGameSettings().GetRespawnCntLimit();
        if(nLimit > 0)
        {
            nLimit -= GetRespawnCnt();
            SetRespawnLimitCnt(nLimit);
        }
        if(GetTeamId() != 0)
            super.m_cGameInfo.UpdateTeamRespawnLimitCnt(GetTeamId());
    }

    byte GetReceivedPlayerState()
    {
        return m_nPlayerState;
    }

    boolean IsKickedOutFromTheGame()
    {
        return m_bKickedOutFromTheGame;
    }

    void MsgSendClientNextRespawnTime()
    {
        MessageNew(12);
        MessagePackFloat(m_fNextRespawnTime);
        MessageSend();
    }

    public void OnMessage()
    {
        switch(MessageGetId())
        {
        case 15: // '\017'
            MsgRecvUpdatePing();
            break;

        case 11: // '\013'
            MsgRecvClientUpdatePlayerId();
            break;

        case 12: // '\f'
            MsgRecvClientNextRespawnTime();
            break;

        case 13: // '\r'
            MsgRecvClientRespawnCnt();
            break;

        case 14: // '\016'
            MsgRecvClientCTFFlagOwner();
            break;

        default:
            super.OnMessage();
            break;
        }
    }

    void SetNextRespawnTime(float fTime)
    {
        m_fNextRespawnTime = fTime;
        if(super.Role == 4)
            MsgSendClientNextRespawnTime();
    }

    float GetReceivedPlayerStateTime()
    {
        return m_fStateTime;
    }

    boolean IsAutoPlayer()
    {
        return m_bAutoPlayer;
    }

    boolean ShouldPerformVoiceOnTeamKill(NetPlayerInfo cAssassinInfo)
    {
        cAssassinInfo.IncTeamKill();
        cAssassinInfo.CallMethodIn("CheckShouldKickOutOnTeamKill", 0.0F);
        return cAssassinInfo.GetTeamKill() > 0;
    }

    void CheckShouldKickOutOnTeamKill()
    {
        DisableCallMethod("CheckShouldKickOutOnTeamKill");
        if(super.m_cGameInfo == null || super.m_cGameInfo.GetGameSettings() == null)
            return;
        int nLimit = super.m_cGameInfo.GetGameSettings().GetTeamKillLimitToKickOut();
        if(nLimit < 0 || nLimit >= GetTeamKill())
        {
            return;
        } else
        {
            super.m_cGameInfo.EventPlayerToKick(GetClientOwnerId(), 2);
            return;
        }
    }

    int GetClientGameVersion()
    {
        return m_nGameVersion;
    }

    public NetPlayerInfo()
    {
        m_nPing = 0;
        m_nPacketLoss = 0;
        m_fNextRespawnTime = 0.0F;
        m_nRespawnCnt = 0;
        m_bCTFFlagOwner = false;
        m_cPlayer = null;
        m_nPlayerId = -1;
        m_nClientOwnerId = -1;
        m_nTeamKill = 0;
        m_nGameVersion = 0;
        m_bKickedOutFromTheGame = false;
        m_sNickClamped = null;
        m_bVisibleOnHudNet = true;
        METHOD_AddToGameInfo = 0;
        METHOD_RemoveFromGameInfo = 0;
        METHOD_TryFindPlayerByNetId = 0;
        METHOD_ServerForceKillMe = 0;
        METHOD_KickOutYourFromTheGame = 0;
        METHOD_EnableAutoVoiceOnFriendlyFire = 0;
        m_bAutoPlayer = false;
        m_bBot = false;
        m_nPlayerState = 0;
        m_fStateTime = 0.0F;
        m_bEnableAutoVoiceOnFriendlyFire = true;
        m_cChangeNickCounter = new ChangeStateCounter(getClass().getName());
    }

    void AssignTeam(int nNewTeamId)
    {
        if(super.m_cGameInfo == null)
        {
            GameObject.CrashLog("AssignTeam failed\n");
            return;
        }
        NetTeamInfo cTeam = null;
        if(GetTeamId() != 0)
        {
            cTeam = super.m_cGameInfo.GetTeamInfoById(GetTeamId());
            if(cTeam != null)
                cTeam.DecNumPlayers();
        }
        if(nNewTeamId != 0)
        {
            cTeam = super.m_cGameInfo.GetTeamInfoById(nNewTeamId);
            if(cTeam != null);
            cTeam.IncNumPlayers();
        }
        SetTeamId(nNewTeamId);
    }

    void AddToGameInfo()
    {
        GameObject.LogD(" " + this + " ");
        if(super.m_cModule != null && super.m_cModule.GetGameInfo() != null)
        {
            GameObject.Log("ok\n");
            super.m_cModule.GetGameInfo().AddPlayerInfo(this);
            return;
        } else
        {
            GameObject.Log("m_cModule.cGameInfo object is null, later\n");
            CallMethodIn(METHOD_AddToGameInfo, 0.1F);
            return;
        }
    }

    void RemoveFromGameInfo()
    {
        if(super.m_cGameInfo != null)
        {
            AssignTeam(0);
            if(GetPlayer() != null)
            {
                PlayerNet cOldPlayer = GetPlayer();
                SetPlayer(null);
                cOldPlayer.SetPlayerInfo(null);
            }
            super.m_cGameInfo.RemovePlayerInfo(this);
            return;
        } else
        {
            return;
        }
    }

    boolean RespawnLimitAvailable()
    {
        if(super.m_cGameInfo == null)
            return false;
        if(super.m_cGameInfo.GetGameSettings().GetRespawnCntLimit() < 0)
            return true;
        return super.m_cGameInfo.GetGameSettings().GetRespawnCntLimit() > GetRespawnCnt();
    }

    void SetClientOwnerId(int nId)
    {
        m_nClientOwnerId = nId;
    }

    void EnableAutoVoiceOnFriendlyFire()
    {
        m_bEnableAutoVoiceOnFriendlyFire = true;
    }

    void CleanResults()
    {
        super.CleanResults();
        SetNextRespawnTime(0.0F);
        ClearRespawnCnt();
        m_nTeamKill = 0;
        EnableAutoVoiceOnFriendlyFire();
    }

    void SetExcluded(boolean bExcluded)
    {
        boolean bLastExcluded = IsExcluded();
        super.SetExcluded(bExcluded);
        if(!bLastExcluded && IsExcluded())
            if(super.m_cGameInfo instanceof NetGameInfo)
                ((NetGameInfo)super.m_cGameInfo).EventPlayerExcluded(this);
            else
                GameObject.CrashLog("SetExcluded " + this + " m_cGameInfo == " + super.m_cGameInfo + "\n");
    }

    void SetPing(int nPing)
    {
        m_nPing = nPing;
    }

    int UpdateAndGetPing()
    {
        UpdatePing();
        return m_nPing;
    }

    int GetPID()
    {
        return GetClientOwnerId();
    }

    String GetNick()
    {
        return GetName();
    }

    void OnCreate()
    {
        METHOD_AddToGameInfo = RegisterCallMethod("AddToGameInfo");
        METHOD_RemoveFromGameInfo = RegisterCallMethod("RemoveFromGameInfo");
        METHOD_TryFindPlayerByNetId = RegisterCallMethod("TryFindPlayerByNetId");
        METHOD_ServerForceKillMe = RegisterCallMethod("ServerForceKillMe");
        METHOD_KickOutYourFromTheGame = RegisterCallMethod("KickOutYourFromTheGame");
        METHOD_EnableAutoVoiceOnFriendlyFire = RegisterCallMethod("EnableAutoVoiceOnFriendlyFire");
        super.OnCreate();
        AddToGameInfo();
    }

    boolean IsNickValid()
    {
        return IsNameValid();
    }

    float GetNextRespawnTime()
    {
        return m_fNextRespawnTime;
    }

    static final int _MSG_UPDATE_NICK = 10;
    static final int _MSG_UPDATE_PLAYERID = 11;
    static final int _MSG_NEXT_RESPAWN_TIME = 12;
    static final int _MSG_RESPAWN_CNT = 13;
    static final int _MSG_CTF_FLAG_OWNER = 14;
    static final int _MSG_UPDATE_PING = 15;
    private int m_nPing;
    private int m_nPacketLoss;
    private float m_fNextRespawnTime;
    private int m_nRespawnCnt;
    private boolean m_bCTFFlagOwner;
    private PlayerNet m_cPlayer;
    private int m_nPlayerId;
    private int m_nClientOwnerId;
    private int m_nTeamKill;
    private int m_nGameVersion;
    private boolean m_bKickedOutFromTheGame;
    String m_sNickClamped;
    static final int _STATUS_SPECTATOR = -1;
    static final int _STATUS_DEAD = 0;
    static final int _STATUS_ALIVE = 1;
    private boolean m_bVisibleOnHudNet;
    int METHOD_AddToGameInfo;
    int METHOD_RemoveFromGameInfo;
    int METHOD_TryFindPlayerByNetId;
    int METHOD_ServerForceKillMe;
    int METHOD_KickOutYourFromTheGame;
    int METHOD_EnableAutoVoiceOnFriendlyFire;
    private boolean m_bAutoPlayer;
    private boolean m_bBot;
    private byte m_nPlayerState;
    private float m_fStateTime;
    private boolean m_bEnableAutoVoiceOnFriendlyFire;
    private static ChangeStateCounter sm_cGlobalChangeNickCounter = new ChangeStateCounter();
    private ChangeStateCounter m_cChangeNickCounter;

}
