public class NetGameInfoCTF extends NetGameInfoTeam
{
    //co jaki czas ma byc robiony update pozycji flag
    final static float _UPDATE_FLAGS_POS_TIME  = 0.5f;
    
    /** Bazy znalezione na levelu */
    protected GameObject        []m_aBases          = null;
    /** Flagi do nich przyporzadkowane */
    protected java.util.Vector  m_aFlags            = new java.util.Vector();
    
    int METHOD_UpdateFlagsPos = 0;
                   
    NetGameInfoCTF()
    {
        super();
        
        SetGameType( ENetGameType.CTF );
    }
    
    void OnCreate()
    {
        METHOD_UpdateFlagsPos = RegisterCallMethod("UpdateFlagsPos");
        
        super.OnCreate();
    }
    
    public void OnDestroy()
    {
        super.OnDestroy();
        m_cGameSettingsCTF = null;
        m_cGameStatusMsgMngrCTF = null;
    }
        
    NetGameSettingsCTF m_cGameSettingsCTF = null;
    NetGameSettingsCTF GetGameSettingsCTF(){ return m_cGameSettingsCTF; }
    void SetGameSettings( NetGameSettings cGameSettings )
    {
        super.SetGameSettings( cGameSettings );
        if ( cGameSettings instanceof NetGameSettingsCTF )
             m_cGameSettingsCTF = (NetGameSettingsCTF)GetGameSettings(); 
        else
            CrashLog(" Niepoprawny obiekt GameSettings "+cGameSettings+
                     " dla NetGameInfoCTF "+this+"\n");
    }
    
    GameStatusMsgMngrCTF m_cGameStatusMsgMngrCTF = null;
    void SetGameStatusMsgManager(GameStatusMsgManager cManager)
    {
        super.SetGameStatusMsgManager( cManager );
        
        if ( cManager instanceof GameStatusMsgMngrCTF )
            m_cGameStatusMsgMngrCTF = (GameStatusMsgMngrCTF)cManager;
        else
            CrashLog(" Niepoprawny obiekt GameStatusMsgManager "+cManager+
                     " dla NetGameInfoCTF "+this+"\n");
    }

    
    void ServerInitGame() 
    {
        super.ServerInitGame();
        
        m_aBases = GetObjectsOfClass("CTFBase");
        if ( m_aBases == null )
        {
            CrashLog("Brak baz dla flag\n");
            return;
        }
        for (int i=0; i<m_aBases.length; i++)
        {
            //wyspawnowac flage dla druzyny
            CTFBase cBase  = (CTFBase)m_aBases[ i ];
            int nTeamId = cBase.GetTeamId();
            String sFlagClassName = ( nTeamId == NetTeamInfo.BLUE ) ? 
                                      "CTFFlagBlue" : "CTFFlagRed";
            CTFFlag cFlag = (CTFFlag)CreateObject( sFlagClassName );
            cFlag.ServerInit( cBase, this );
            //LogD(" Stworzona flaga "+cFlag+" z meshem "+cFlag.MeshName+"\n");
            m_aFlags.addElement( cFlag );
        }
        
        if ( GetGameSettingsCTF().GetFlagsVisibleOnHudMap() )
        {
            UpdateFlagsPos();    
            CallMethodEvery( METHOD_UpdateFlagsPos,_UPDATE_FLAGS_POS_TIME );
        }
    }
     
    
    void AddTeamInfo(NetTeamInfo cTeamInfo)
    {
        super.AddTeamInfo( cTeamInfo );

        if ( Role < ERole._AUTHORITY )
            return;
        
        //przypisac druzynie flage
        for (int i=0; i<m_aBases.length; i++)
        {
            CTFBase cBase = (CTFBase)m_aBases[ i ];
                        
            if ( cBase.GetTeamId() == cTeamInfo.GetTeamId() )
                cTeamInfo.SetCTFFlag( CTFFlag.OWN,   cBase.GetFlag() );
            else
                cTeamInfo.SetCTFFlag( CTFFlag.ENEMY, cBase.GetFlag() );
        }
    }
    
    final static int _MSG_FLAG_CAPTURED            = 100;
    final static int _MSG_FLAG_DROPPED             = 101;
    final static int _MSG_FLAG_RETURNED            = 102;
    final static int _MSG_FLAG_DELIVERED           = 103;
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case _MSG_FLAG_CAPTURED:
            LogD("_MSG_FLAG_CAPTURED\n");
            MsgRecvClientFlagCaptured();
            break;
            
        case _MSG_FLAG_DROPPED:
            LogD("_MSG_FLAG_DROPPED\n");
            MsgRecvClientFlagDropped();
            break;
            
        case _MSG_FLAG_RETURNED:
            LogD("_MSG_FLAG_RETURNED\n");
            MsgRecvClientFlagReturned();
            break;
            
        case _MSG_FLAG_DELIVERED:
            LogD("_MSG_FLAG_DELIVERED\n");
            MsgRecvClientFlagDelivered();
            break;
        
        default:
            super.OnMessage();
        }
    }
    
    void MsgSendClientFlagCaptured( int nPlayerInfoId, int nFlagColor )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_FLAG_CAPTURED);
        MessagePackInt( nPlayerInfoId );
        MessagePackInt( nFlagColor );
        MessageSend();
    }
    
    void MsgSendClientFlagDropped( int nPlayerInfoId, int nFlagColor )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_FLAG_DROPPED);
        MessagePackInt( nPlayerInfoId );
        MessagePackInt( nFlagColor );
        MessageSend();
    }
    
    void MsgSendClientFlagReturned( int nPlayerInfoId, int nFlagColor )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_FLAG_RETURNED);
        MessagePackInt( nPlayerInfoId );
        MessagePackInt( nFlagColor );
        MessageSend();
    }
    
    void MsgSendClientFlagDelivered( int nTeamInfoId, int nPlayerInfoId, int nFlagColor )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_FLAG_DELIVERED);
        MessagePackInt( nTeamInfoId );
        MessagePackInt( nPlayerInfoId );
        MessagePackInt( nFlagColor );
        MessageSend();
    }
    
    void MsgRecvClientFlagCaptured()
    {
        int nPlayerInfoId = MessageUnPackInt();
        int nFlagColor    = MessageUnPackInt();
        MessageEnd();

        AddIncomingMessage( NetIncomingMsg._CTF_FLAG_CAPTURED, 
                            nPlayerInfoId,
                            -1,
                            -1,
                            nFlagColor );
    }
    
    void MsgRecvClientFlagDropped()
    {
        int nPlayerInfoId = MessageUnPackInt();
        int nFlagColor    = MessageUnPackInt();
        MessageEnd();

        AddIncomingMessage( NetIncomingMsg._CTF_FLAG_DROPPED, 
                            nPlayerInfoId,
                            -1,
                            -1,
                            nFlagColor );
    }
    
    void MsgRecvClientFlagReturned()
    {
        int nPlayerInfoId = MessageUnPackInt();
        int nFlagColor    = MessageUnPackInt();
        MessageEnd();
        
        AddIncomingMessage( NetIncomingMsg._CTF_FLAG_RETURNED, 
                            nPlayerInfoId,
                            -1,
                            -1,
                            nFlagColor );
    }
    
    void MsgRecvClientFlagDelivered()
    {
        int nTeamInfoId   = MessageUnPackInt();
        int nPlayerInfoId = MessageUnPackInt();
        int nFlagColor    = MessageUnPackInt();
        MessageEnd();
        
        AddIncomingMessage( NetIncomingMsg._CTF_FLAG_DELIVERED, 
                            nPlayerInfoId,
                            -1,
                            nTeamInfoId,
                            nFlagColor);
    }
    
    
    void EventPlayerReachedFlag( CTFFlag cFlag, PlayerNet cPlayer )
    {
        NetPlayerInfo cPlayerInfo   = null;
        int           nPlayerInfoId = -1;
        if ( cPlayer != null )
        {
            cPlayerInfo = cPlayer.GetPlayerInfo();
            if (cPlayerInfo != null)
                nPlayerInfoId = cPlayerInfo.GetNetId();
            else
            {
                CrashLog("ERROR: "+this+" PlayerReachedFlag "+cPlayer+" cNetPlayerInfo == null\n");
                return;
            }
        }
        else
        {
            CrashLog("ERROR: "+this+" PlayerReachedFlag "+cPlayer+"\n");
            return;
        }
        
        
        if ( cFlag.m_nTeamId != cPlayerInfo.GetTeamId() )
        {
            //TODO: sprawdzic, czy flaga jest jeszcze na ziemi
            //wbiegl we flage wroga, pobierzmy ja, 
            cFlag.ActionExecute( cPlayer );
            //system za chwile zglosi EventFlagCapturedByPlayer
        }
        else
        {
            //wbiegl w swoja flage
            //jezeli jest przesunieta od swojej domyslnej (bazowej) pozycji
            if ( ! cFlag.IsBaseOn() )
            {
                //to ja trzeba przestawic
                LogD("RespawnInBasePos()\n");
                cFlag.ServerRespawn(); 
                
                int nFlagColor = cFlag.m_nTeamId;
                m_cGameStatusMsgMngrCTF.FlagReturned( nFlagColor );
                //i oglosic to wszystkim, ze flaga wrocila
                if ( ! MessagesDisabled() )
                {
                    AddIncomingMessage( NetIncomingMsg._CTF_FLAG_RETURNED, 
                                        nPlayerInfoId,
                                        -1,
                                        -1,
                                        nFlagColor );
                    MsgSendClientFlagReturned( nPlayerInfoId, nFlagColor );
                }
            }
            else
            {
                //flaga jest na swoim miejscu, 
                //to zobaczmy co sie dzieje z flaga wroga
                NetTeamInfo cTeamInfo = GetTeamInfoById( cPlayerInfo.GetTeamId() );
                
                if ( cTeamInfo != null )
                {
                    CTFFlag cEnemyFlag = cTeamInfo.GetCTFFlag( CTFFlag.ENEMY );
                
                    //jezeli flaga wroga tez jest w posiadaniu gracza
                    if ( cEnemyFlag != null &&
                         cEnemyFlag.Owner == cPlayer )
                    {
                        //to ja wyrzucamy automatycznie.
                        //system to wykryje i zglosi EventFlagDeliveredByTeam
                        //cEnemyFlag.ThrowFlag();
                        //moze automatucznie zglosic to tu?
                        EventFlagDelivered( cEnemyFlag, cPlayer, cTeamInfo );
                    }
                }
            }
        }
    }
    
    void EventFlagCapturedByPlayer( CTFFlag cFlag, PlayerNet cPlayer )
    {
        //TODO
        int           nPlayerInfoId = -1;
        NetPlayerInfo cPlayerInfo   = null;
        if ( cPlayer != null )
        {
            cPlayerInfo = cPlayer.GetPlayerInfo();
            if (cPlayerInfo != null)
                nPlayerInfoId = cPlayerInfo.GetNetId();
            else
                CrashLog("ERROR: "+this+" FlagCapturedByPlayer "+cPlayer+" cNetPlayerInfo == null\n");
        }
        else
        {
            CrashLog("ERROR: "+this+" FlagCapturedByPlayer "+cPlayer+"\n");
            //return;
        }
        
        
        if ( cPlayerInfo.GetTeamId() != cFlag.m_nTeamId )
        {
            cFlag.m_cPlayerOwner.GetPlayerInfo().SetCTFFlagOwner( true );
            
            int nFlagColor = cFlag.m_nTeamId;
            m_cGameStatusMsgMngrCTF.FlagCaptured( nFlagColor, nPlayerInfoId );
            AddIncomingMessage( NetIncomingMsg._CTF_FLAG_CAPTURED, 
                                nPlayerInfoId,
                                -1,
                                -1,
                                nFlagColor );
            MsgSendClientFlagCaptured( nPlayerInfoId, nFlagColor );
        }
        else
        {
            CrashLog("ERROR: Gracz nie powinien miec swojej flagi!\n");
        }
    }
    
    void EventFlagRemovedOnGround( CTFFlag cFlag, PlayerNet cPlayer )
    {
        //wylaczmy graczowi flage        
        cPlayer.GetPlayerInfo().SetCTFFlagOwner( false );
        
        //sprawdzmy czy flaga zostala zrespawnowana?
        if ( !cFlag.IsBaseOn() )  
        {
            //nie, to jakies wyrzucenie
            int           nPlayerInfoId = -1;
            NetPlayerInfo cPlayerInfo   = null;
            if ( cPlayer != null )
            {
                cPlayerInfo = cPlayer.GetPlayerInfo();
                if (cPlayerInfo != null)
                    nPlayerInfoId = cPlayerInfo.GetNetId();
                else
                    CrashLog("ERROR: "+this+" FlagDroppedByPlayer "+cPlayer+" cNetPlayerInfo == null\n");
            }
            else
            {
                CrashLog("ERROR: "+this+" FlagCapturedByPlayer "+cPlayer+"\n");
                //return;
            }   
            
            int nFlagColor = cFlag.m_nTeamId;
            m_cGameStatusMsgMngrCTF.FlagDropped( nFlagColor );
            if ( ! MessagesDisabled() )
            {
                AddIncomingMessage( NetIncomingMsg._CTF_FLAG_DROPPED, 
                                    nPlayerInfoId,
                                    -1,
                                    -1,
                                    nFlagColor );
                MsgSendClientFlagDropped( nPlayerInfoId, nFlagColor );
            }
            
            ScheduleBackToBase( cFlag );
        }
    }
    
    void EventFlagDelivered( CTFFlag cFlag, PlayerNet cPlayer, NetTeamInfo cTeamInfo )
    {
        if ( cTeamInfo == null )
        {
            CrashLog("ERROR: EventFlagDeliveredByTeam "+cTeamInfo+"\n");
            return;
        }
        
        int nTeamInfoId   = -1; 
        int nPlayerInfoId = -1; 
        
        //dajemy punty druzynie
        cTeamInfo.IncPoints();
        nTeamInfoId = cTeamInfo.GetNetId();
        
        //dodac punkty gostkowi ktory ta flage doniosl
                
        if ( cPlayer                 != null &&
             cPlayer.GetPlayerInfo() != null )
        {
            cPlayer.GetPlayerInfo().IncPoints();
            
            nPlayerInfoId = cPlayer.GetPlayerInfo().GetNetId();
        }
        else
        {
            LogD("Nie wiadomo kto doniosl flage!!!\n");
        }
                
        //przywrocic flage wroga na miejsce
        cFlag.ServerRespawn();
        
        int nFlagColor = cFlag.m_nTeamId;
        
        m_cGameStatusMsgMngrCTF.FlagDelivered( nFlagColor );
        AddIncomingMessage( NetIncomingMsg._CTF_FLAG_DELIVERED, 
                            nPlayerInfoId, 
                            -1, 
                            nTeamInfoId,
                            nFlagColor );
        
        MsgSendClientFlagDelivered( nTeamInfoId, nPlayerInfoId, nFlagColor );
    }
    
    void EventScheduledBackToBase( CTFFlag cFlag )
    {
        if ( cFlag != null &&  ! cFlag.IsBaseOn() )
        {
            LogD("RespawnInBasePos()\n");
            cFlag.ServerRespawn(); 
                
            int nFlagColor = cFlag.m_nTeamId;
            //i oglosic to wszystkim, ze flaga wrocila
                
            if ( ! MessagesDisabled() )
            {
                m_cGameStatusMsgMngrCTF.FlagReturned( nFlagColor );
                AddIncomingMessage( NetIncomingMsg._CTF_FLAG_RETURNED, 
                                    -1,
                                    -1,
                                    -1,
                                    nFlagColor );
                MsgSendClientFlagReturned( -1, nFlagColor );
            }
        }
    }
    
    void ServerStopGame()
    {
        super.ServerStopGame();
        
        for ( int i=0; i<m_aFlags.size(); i++ )
        {
            if ( ! (m_aFlags.elementAt( i ) instanceof CTFFlag) )
                continue;
            CTFFlag cFlag = (CTFFlag)m_aFlags.elementAt( i );
            cFlag.ServerRespawn();
        }
    }
        
    int     m_nBlueFlagPosX     = -1;
    int     m_nBlueFlagPosZ     = -1;
    boolean m_bBlueFlagHasOwner = false;
    int     m_nRedFlagPosX      = -1;
    int     m_nRedFlagPosZ      = -1;
    boolean m_bRedFlagHasOwner  = false;
        
    CTFFlagInfo GetCTFFlagInfo(int nTeamId)
    {
        if ( !GetGameSettingsCTF().GetFlagsVisibleOnHudMap() )
            return null;
        if ( nTeamId == NetTeamInfo.BLUE )
        {
            if ( m_nBlueFlagPosX < 0 || m_nBlueFlagPosZ < 0 )
                return null;
            return new CTFFlagInfo( m_nBlueFlagPosX, m_nBlueFlagPosZ, m_bBlueFlagHasOwner );
        }
        else
        {
            if ( m_nRedFlagPosX < 0 || m_nRedFlagPosZ < 0 )
                return null;
            return new CTFFlagInfo( m_nRedFlagPosX,  m_nRedFlagPosZ,  m_bRedFlagHasOwner  );
        }
    }
    
    CTFBaseInfo GetCTFBaseInfo(int nTeamId)
    {
        LogD(" "+m_aFlags+"\n");
        for ( int i=0; i<m_aFlags.size(); i++ )
        {
            if ( ! (m_aFlags.elementAt( i ) instanceof CTFFlag) )
                continue;
            CTFFlag cFlag = (CTFFlag)m_aFlags.elementAt( i );
                        
            if ( cFlag.m_nTeamId == nTeamId )
            {
                return new CTFBaseInfo( cFlag.GetBasePos(), cFlag.IsBaseOnOff() );
            }
        }
        return null;
    }
    
    
    void UpdateFlagsPos()
    {
        if ( Role < ERole._AUTHORITY )
            return;
        if ( !GetGameSettingsCTF().GetFlagsVisibleOnHudMap() )
            return;
        
        for ( int i=0; i<m_aFlags.size(); i++ )
        {
            if ( ! (m_aFlags.elementAt( i ) instanceof CTFFlag) )
                continue;
            CTFFlag cFlag = (CTFFlag)m_aFlags.elementAt( i );
            
            boolean bOwner = cFlag.Owner != null;
            Vector vPos = bOwner ? cFlag.Owner.GetPosition() : cFlag.GetPosition();
            if ( cFlag.m_nTeamId == NetTeamInfo.BLUE )
            {
                m_bBlueFlagHasOwner = bOwner;
                m_nBlueFlagPosX     = (int)vPos.fX;
                m_nBlueFlagPosZ     = (int)vPos.fZ;
            }
            else
            {
                m_bRedFlagHasOwner  = bOwner;
                m_nRedFlagPosX      = (int)vPos.fX;
                m_nRedFlagPosZ      = (int)vPos.fZ;
            }
        }
    }
    
    
    static int sm_nSetFlagsPos = 0;
    
    public void ReplicationInit()
    {
        super.ReplicationInit();
        
        sm_nSetFlagsPos = NewReplicationSet("m_nBlueFlagPosX,m_nBlueFlagPosZ,m_bBlueFlagHasOwner,m_nRedFlagPosX,m_nRedFlagPosZ,m_bRedFlagHasOwner");
    }
  
    /** Replikacja odpowiednich zestawow dla odpowiednich warunkow
     */
    public void OnReplicateServer()
    {
        super.OnReplicateServer();
        
        if ( GetGameSettingsCTF().GetFlagsVisibleOnHudMap() )
            Replicate( sm_nSetFlagsPos );
    }
    
    void ScheduleBackToBase( CTFFlag cFlag )
    {
        float fTime = (GetGameSettingsCTF() != null)?
                      GetGameSettingsCTF().GetFlagAutoReturnTime(): -1.0f ;
        if ( fTime > 0.0f )
            cFlag.ScheduleBackToBase( fTime );
    }
    
    void OnGetActiveTrigger()
    {
        //gracz szuka aktywnych triggerow, umozliwmy
        //mu odnalezienie obiektw CTFFlag
        StoreFlagsAndEnableGetActiveTriggerForCTFObjs();
    }
    
    void OnPostGetActiveTrigger()
    {
        //gracz skonczyl szukac aktywnych triggerow, 
        //przywrocmy obiektom CTFFlag zapamietane wczesniej flagi
        RestoreFlagsForCTFObjs();
    }
    
    
    /**
     * Odnajduje wszystkie obiekty CTFFlag na levelu,
     * i kazdemu z nich wymusza flagi do umozliwienia
     * traceowania tych obiektow. Oryginalne flagi sa zapamietywane
     * celem pozniejszego ich przywrocenia
     */
    void StoreFlagsAndEnableGetActiveTriggerForCTFObjs()
    {
        for ( int i=0; i<m_aFlags.size(); i++ )
        {
            if ( ! (m_aFlags.elementAt( i ) instanceof CTFFlag) )
                continue;
            CTFFlag cFlag = (CTFFlag)m_aFlags.elementAt( i );
            cFlag.StoreFlagsAndEnableGetActiveTrigger();
        }
    }
    
    /**
     * Przywrocenie wszystkim obiektom CTFFlag na levelu
     * flag zapamietanych podczas wywolania metody
     * StoreFlagsAndEnableGetActiveTriggerForCTFObjs()
     */
    void RestoreFlagsForCTFObjs()
    {
        for ( int i=0; i<m_aFlags.size(); i++ )
        {
            if ( ! (m_aFlags.elementAt( i ) instanceof CTFFlag) )
                continue;
            CTFFlag cFlag = (CTFFlag)m_aFlags.elementAt( i );
            cFlag.RestoreFlags();
        }
    }    
}

