public class NetGameInfoTeam extends NetGameInfoCommon
{
    int METHOD_SetLocalChangeTeamModeOff        = 0;
    int METHOD_UnregisterKeyRequestExclusive    = 0;

    
    
    void OnCreate()
    {
        METHOD_SetLocalChangeTeamModeOff     = RegisterCallMethod("SetLocalChangeTeamModeOff");
        METHOD_UnregisterKeyRequestExclusive = RegisterCallMethod("UnregisterKeyRequestExclusive");
        
        super.OnCreate();
        
        //przechwytujemy klawisze do zmiany druzyny
        RegisterKeyRequest();
        RegisterMouseButtonRequest();
        
        if ( Role == ERole._AUTHORITY )
            CreateTeamInfo();
    }
    
    public void OnDestroy()
    {
        super.OnDestroy();
        
        UnregisterKeyRequest();
        m_cGameSettingsTeam = null;
    }
    
    GameStatusMsgManager CreateGameStatusMsgManager()
    {
        return(GameStatusMsgManager)CreateObject("GameStatusMsgMngrTeam");
    }
    
    /**
     * Tworzenie obiektow informacji o teamach
     */
    void CreateTeamInfo()
    {
        /**
         * Jako ze kiedys NetTeamInfo tworzylo sie dopiero po
         * dolaczeniu pierwszego gracza, zatem zostalo uzbrojona
         * w mechanizm dodawania sie do NetGameInfo
         * Nie trzeba zatem tego robic, wystarczy stworzyc obiekty
         */
        for ( int nTeamId = NetTeamInfo._FIRST ; nTeamId < NetTeamInfo.TEAMS_CNT; nTeamId++ )
        {
            NetTeamInfo cTeam = (NetTeamInfo)CreateObjectNoInit("NetTeamInfo");
            cTeam.SetTeamId( nTeamId );
            cTeam.InitObject();
        }
    }
    
    
    NetGameSettingsTeam m_cGameSettingsTeam = null;
    NetGameSettingsTeam GetGameSettingsTeam(){ return m_cGameSettingsTeam; }
    void SetGameSettings( NetGameSettings cGameSettings )
    {
        super.SetGameSettings( cGameSettings );
        if ( cGameSettings instanceof NetGameSettingsTeam )
             m_cGameSettingsTeam = (NetGameSettingsTeam)GetGameSettings(); 
        else
            CrashLog(" Niepoprawny obiekt GameSettings: "+cGameSettings+
                     " dla GameInfoTeam: "+this+"\n");
    }
    

    final static int _MSG_PLAYER_JOINED_TEAM       = 81;
    final static int _MSG_TEAM_CHANGE_REQUEST      = 82;
    final static int _MSG_PLAYER_CHANGED_TEAM      = 83;
    final static int _MSG_TEAM_EXCLUDED            = 84;
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case _MSG_PLAYER_JOINED_TEAM:
            LogD("_MSG_PLAYER_JOINED_TEAM\n");
            MsgRecvClientPlayerJoinedTeam();
            break;
            
        case _MSG_TEAM_CHANGE_REQUEST:
            MsgRecvServerTeamChangeRequest();
            break;
            
        case _MSG_PLAYER_CHANGED_TEAM:
            LogD("_MSG_PLAYER_CHANGED_TEAM\n");
            MsgRecvClientPlayerChangedTeam();
            break;

        case _MSG_TEAM_EXCLUDED:
            LogD("_MSG_TEAM_EXCLUDED\n");
            MsgRecvClientTeamExcluded();
            break;
            
        default:
            super.OnMessage();
        }
    }

    void MsgSendClientPlayerJoinedTeam( int nJoinedInfoId, int nTeamInfoId, int nSkin, int nChangeSkinCounter )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
        
        MessageNew(_MSG_PLAYER_JOINED_TEAM);
        MessagePackInt( nJoinedInfoId );
        MessagePackInt( nTeamInfoId );
        MessagePackInt( nSkin );
        MessagePackInt( nChangeSkinCounter );
        MessageSend();
    }
    
    void MsgRecvClientPlayerJoinedTeam()
    {
        int     nJoinedInfoId       = MessageUnPackInt();
        int     nTeamInfoId         = MessageUnPackInt();
        int     nSkin               = MessageUnPackInt();
        int     nChangeSkinCounter  = MessageUnPackInt();
        MessageEnd();
        
        AddIncomingMessage( NetIncomingMsg._PLAYER_JOINED_TEAM, nJoinedInfoId, -1, nTeamInfoId ); 
        ChangePlayerTeamSkins( nJoinedInfoId, nSkin, nChangeSkinCounter );
        OnTeamChange();
    }
    
    
    void MsgSendServerTeamChangeRequest(int nNewTeamId)
    {
        MessageNew( _MSG_TEAM_CHANGE_REQUEST );
        MessagePackInt( nNewTeamId );
        MessageSend();
    }
    
    void MsgRecvServerTeamChangeRequest()
    {
        int nHost      = MessageGetSenderId();
        int nNewTeamId = MessageUnPackInt();
        MessageEnd();
        
        int nInfoId = -1;
        NetPlayerInfo cInfo = GetPlayerInfoList().GetPlayerByHostId( nHost );
        if ( cInfo != null )
        {
            EventPlayerRequestedChangeTeam( cInfo, nNewTeamId );    
        }
    }
    
    
    void MsgSendClientPlayerChangedTeam(int nPlayerInfoId,int nTeamInfoId, int nSkin,int nChangeSkinCounter,boolean bFirstTime )
    {
        MessageNew( _MSG_PLAYER_CHANGED_TEAM );
        MessagePackInt( nPlayerInfoId );
        MessagePackInt( nTeamInfoId );
        MessagePackInt( nSkin );
        MessagePackInt( nChangeSkinCounter );
        MessagePackBoolean( bFirstTime );
        MessageSend();
    }
    
    void MsgRecvClientPlayerChangedTeam()
    {
        int nPlayerInfoId       = MessageUnPackInt();
        int nTeamInfoId         = MessageUnPackInt();
        int nSkin               = MessageUnPackInt();
        int nChangeSkinCounter  = MessageUnPackInt();
        boolean bFirstTime      = MessageUnPackBoolean();
        MessageEnd();
               
        int nMessageId  = bFirstTime ?
                          NetIncomingMsg._PLAYER_ASSIGNED_TEAM:
                          NetIncomingMsg._PLAYER_CHANGED_TEAM ;
        
        AddIncomingMessage( nMessageId, 
                            nPlayerInfoId, 
                            -1,
                            nTeamInfoId );
        
        ChangePlayerTeamSkins( nPlayerInfoId, nSkin, nChangeSkinCounter );
        UpdateEnableObjectivesIfOwner( nPlayerInfoId );
    }
    
    void MsgSendClientTeamExcluded(int nTeamInfoId)
    {
        MessageNew( _MSG_TEAM_EXCLUDED );
        MessagePackInt( nTeamInfoId );
        MessageSend();
    }
    
    void MsgRecvClientTeamExcluded()
    {
        int nTeamInfoId   = MessageUnPackInt();
        MessageEnd();
        
        AddIncomingMessage( NetIncomingMsg._TEAM_EXCLUDED, 
                            -1, -1, nTeamInfoId ); 
    }
    
       
    void MessagePackPlayersTeamSkins()
    {
        GetPlayerInfoList().MessagePackPlayersTeam();
    }
    void MessageUnPackPlayersTeamSkins()
    {
        GetPlayerInfoList().MessageUnPackPlayersTeam();
    }
    
    void OnSetGameSettings()
    {
        super.OnSetGameSettings();
        
        CallMethodEvery( "VerifyPlayersTeam", 0.5f );
    }
    
    void VerifyPlayersTeam()
    {
        GetPlayerInfoList().VerifyPlayersTeam();
    }
    
    void PlayersTeamVerified()
    {
        DisableCallMethod( "VerifyPlayersTeam" );
    }
    
    boolean IsTeamAllowedInTargetInfo( int nObserverTeam, int nTargetTeam )
    {
        return ((nObserverTeam == nTargetTeam)||
                super.IsTeamAllowedInTargetInfo( nObserverTeam, nTargetTeam ));
    }
    
    void SetGameOver()
    {
        super.SetGameOver();
        
        if ( m_bChangeTeamMode )
            SetLocalChangeTeamModeOff();
    }
    
    public void OnInputMouseButton(int nButton, int nState)
    {
        OnInputKey( Player.ConvertMouseButtonToKey(nButton), (nState!=0) , '0');
    }
    
    void OnInputMouseAxis(int nAxis, int nDelta)
    {
        HUDManager.ProcessMouseWheel(this, nAxis, nDelta);
    }
    
    private 
    boolean m_bChangeTeamMode      = false;
    boolean IsChangeTeamMode(){ return m_bChangeTeamMode; }    
    
    /** Czy lokalny gracz jest przypisany do jakiejs druzyny?*/
    boolean IsLocalPlayerAssignedToTeam(){ return IsLocalPlayerPresent() && (GetPlayerInfoList().GetMyPlayer().GetTeamId() != NetTeamInfo.NONE); }
    
    
    int     m_nLastStatisticsState = _STATISTICS_STATE_HIDDEN;
    
    void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
    {
        super.OnInputKey( nKeyCode, bKeyState, nKeyAscii );
        
        if ( IsGameOver() || !IsLocalPlayer() ) 
            return;
        
        //jesli ma zakaz spawnu i nie wynika on z faktu, ze gra
        //jeszcze nie wystartowala, to nie mozna zmieniac teamow
        if ( ( m_nStatisticsState     == _STATISTICS_STATE_RESPAWN_REJECT )&&
             ( m_nRespawnRejectReason != ERespawnRequestResults._DISABLED_GAME_NOTSTARTED ) )
            return;
        
        if ( bKeyState )
        {
            if ( m_bChangeTeamMode )
            {
                if ( nKeyCode == EKey._1 )
                    LocalEventRequestChangeTeam( NetTeamInfo.BLUE );
                else
                if ( nKeyCode == EKey._2 )
                    LocalEventRequestChangeTeam( NetTeamInfo.RED );
                else
                if ( nKeyCode == EKey._ESCAPE )
                {
                    if ( IsLocalPlayerAssignedToTeam() )
                        SetLocalChangeTeamModeOff();
                    else
                        LocalEventRequestChangeTeam();
                }
                /*else
                if ( nKeyCode == EGameControl.FIRE && IsLocalPlayerAlive() )
                    LocalEventRequestChangeTeam();
                */
            }
            else
            {
                if (nKeyCode == EGameControl.NET_CHANGE_TEAM)
                    SetLocalChangeTeamMode( true );
            }
        }
    }
    
    void SetLocalChangeTeamMode( boolean bOn )
    {
        DisableCallMethod( METHOD_SetLocalChangeTeamModeOff );
        if (m_bChangeTeamMode == bOn)
            return;
        m_bChangeTeamMode = bOn;
        if ( m_bChangeTeamMode )
        {
            m_nLastStatisticsState = m_nStatisticsState;
            LogD(" m_nLastStatisticsState "+m_nLastStatisticsState+"\n");
            ActivateStatisticsOnChooseTeam();
            NetPlayerInfo cPlayerInfo = GetPlayerInfoList().GetMyPlayer();
            boolean bTeamChoosen = ( cPlayerInfo != null && cPlayerInfo.GetTeamId() != NetTeamInfo.NONE );    
            if ( bTeamChoosen )
                CallMethodIn( METHOD_SetLocalChangeTeamModeOff, 5.0f );
            DisableCallMethod( METHOD_UnregisterKeyRequestExclusive );
            RegisterKeyRequestExclusive();
        }
        else
        {
            LogD(" m_nLastStatisticsState "+m_nLastStatisticsState+"\n");
            //podczas zmiany druzyny (jesli taka wlasnie nastapila)
            //gracz bedzie "usmiercony" i zawola sie tam na CallMethod
            //wlasnie pojawienie sie statystyk OnDie
            //my w tym przypadku tego nie chcemy wiec wolamy disable
            DisableCallMethod( METHOD_ActivateStatisticsOnDeath );
            switch ( m_nLastStatisticsState )
            {
            case _STATISTICS_STATE_HIDDEN:
                DeactivateStatistics();
                break;
            case _STATISTICS_STATE_RESPAWN_REJECT:
                ActivateStatisticsOnRespawnReject();
                break;    
            default:
                ActivateStatisticsOnDeath();
            }
            CallMethodIn( METHOD_UnregisterKeyRequestExclusive, 0.0f );
        }
    }
    
    void ForceSetLocalChangeTeamMode()
    {
        SetLocalChangeTeamMode( false );
        SetLocalChangeTeamMode( true );
    }
    
    void SetLocalChangeTeamModeOff()
    {
        SetLocalChangeTeamMode( false );
    }
    

    
    void LocalEventRequestChangeTeam()
    {
        //NetTeamInfo.NONE zagwarantuje, za 
        //- jezeli gracz jest juz w jakiejs druzynie to przepisze go do drugiej
        //- jesli nie, to zostanie wybrany automatycznie (by team balalce)
        LocalEventRequestChangeTeam( NetTeamInfo.NONE );
    }
    void LocalEventRequestChangeTeam(int nNewTeamId)
    {
        if ( Role == ERole._AUTHORITY )
            EventPlayerRequestedChangeTeam( GetPlayerInfoList().GetMyPlayer(), nNewTeamId );
        else
            MsgSendServerTeamChangeRequest( nNewTeamId );
        //kiedys zawolanie tego bylo na poczatku tej metody
        //ale przenioslem na koniec, bowiem zalezy nam na 
        //tym aby zmiana teamu (na serwerze) odbyla sie wczesniej.
        SetLocalChangeTeamModeOff();
    }

    
    void EventPlayerDied( PlayerNetBaseMortal victim, PlayerNetBaseMortal assassin, int nMode )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;

        //Klasa bazowa wyswietli odpowiednie komunikaty        
        super.EventPlayerDied( victim, assassin, nMode );
        
        //kazmy przeliczyc punkty, dla gry druzynowej
        CountTeamFragsOnDie( victim, assassin, nMode );
    }
    
    void EventMessageOnPlayerDied( int nVictimInfoId, int nAssassinInfoId, boolean bTeamKill )
    {
        if ( bTeamKill )
            AddIncomingMessage( NetIncomingMsg._PLAYER_DIED_TEAM_KILL, nVictimInfoId, nAssassinInfoId );
        else
            AddIncomingMessage( NetIncomingMsg._PLAYER_DIED, nVictimInfoId, nAssassinInfoId );
        MsgSendClientPlayerDied( nVictimInfoId, nAssassinInfoId, bTeamKill );            
    }
    
    //fragi dla gry teamowej
    void CountTeamFragsOnDie( PlayerNetBaseMortal victim, PlayerNetBaseMortal assassin, int nMode )
    {
        if ( nMode == EPlayerAction.DIE_MODE_GAMEOVER ||
             nMode == EPlayerAction.DIE_MODE_GAMERESTART )
            return;
            
        NetPlayerInfo cVictimInfo   = victim != null   ? victim.GetPlayerInfo()   : null;
        NetPlayerInfo cAssassinInfo = assassin != null ? assassin.GetPlayerInfo() : null;
                
        // dodajemy graczowi jedna smierc
        if ( cVictimInfo != null )
        {
            //
            cVictimInfo.IncDeaths();
             
            // jego druzyna dostaje jedna smierc
            NetTeamInfo cVictimTeam = GetTeamInfoById( cVictimInfo.GetTeamId() );
            if ( cVictimTeam != null )
                cVictimTeam.IncDeaths();
        }
        else
            CrashLog("ERROR: "+this+" PlayerDied "+victim+" ( cVictimInfo == null )\n");
        
        if (cAssassinInfo != null)
        {
			// ktos go zabil, tylko kto?
            
            //jego druzyna dostaje jednego fraga
            NetTeamInfo cAssassinTeam = GetTeamInfoById( cAssassinInfo.GetTeamId() );
			
            if (cAssassinInfo               != cVictimInfo && 
                cAssassinInfo.GetTeamId()   != cVictimInfo.GetTeamId() ) 
            {
				// juz wiemy kto
                
				// zabojca dostaje jednego fraga
                cAssassinInfo.IncFrags();
                    
                //jego druzyna dostaje jednego fraga
                if ( cAssassinTeam != null )
                    cAssassinTeam.IncFrags();
            }
			else 
            {
				// gracz zabil sam siebie
				// zabieramy mu jednego fraga
                cAssassinInfo.DecFrags();
                if ( cAssassinTeam != null )
                    cAssassinTeam.DecFrags();
            }
        }
        else
        {
			// gracz nie zyje, ale nie wiadomo nic wiecej na ten temat
        }
    }
    
    void EventTeamExcluded(NetTeamInfo cTeamInfo)
    {
        //jakas druzyna wykluczona
        int nTeamInfoId = cTeamInfo != null ? cTeamInfo.GetNetId() : -1;
        //obwiesmy to wszem i wobec
        AddIncomingMessage( NetIncomingMsg._TEAM_EXCLUDED, -1, -1, nTeamInfoId );
        MsgSendClientTeamExcluded( nTeamInfoId );
    }
    
    void EventPlayerLeft(PlayerNet player)
    {
        super.EventPlayerLeft( player );
        
        UpdateTeamsRespawnLimitCnt();
    }
    
    void EventPlayerTryToRespawn(PlayerNetBase cPlayer)
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return;
                
        if ( cPlayer == null )
            return;
        
        NetPlayerInfo cPlayerInfo = cPlayer.GetPlayerInfo();
        if (cPlayerInfo == null)
        {
            CrashLog("ERROR: "+this+" EventPlayerTryToRespawn cPlayer "+cPlayer+" cNetPlayerInfo == null\n");
            return;
        }
        
        if ( cPlayerInfo.GetTeamId() == NetTeamInfo.NONE )
        {
            //nie znalazl dobie zadnej druzyny, to robimy balance
            int nNewTeamId = GetTeamInfoList().FindTeamIdByTeamBalance();
            EventPlayerRequestedChangeTeam( cPlayerInfo, nNewTeamId );
        }
    }
    /** Serwer, 
     * zmiana druzyny przez gracza
     */
    void EventPlayerRequestedChangeTeam(NetPlayerInfo cPlayerInfo,int nNewTeamId )
    {
        if ( cPlayerInfo == null )
            return;
        
        int nOldTeamId = cPlayerInfo.GetTeamId();

        //nie podano druzyny docelowej
        if ( nNewTeamId == NetTeamInfo.NONE )
        {
            //jesli jeszcze nie jest w zadnej to balans,
            //jesli jest to zmiana
            if ( nOldTeamId == NetTeamInfo.NONE )
                nNewTeamId = GetTeamInfoList().FindTeamIdByTeamBalance();
            else
                nNewTeamId = nOldTeamId      == NetTeamInfo.BLUE ?
                             NetTeamInfo.RED :  NetTeamInfo.BLUE;
        }
        
        if ( nNewTeamId == nOldTeamId )
            return;
        
        if ( TryToChangeTeam( cPlayerInfo, nNewTeamId ) )
        {
            DisableMessages();
            //wymuszamy zeby umarl "po cichu"
            cPlayerInfo.GetPlayer().ForceDieOnChangeTeam();
            EnableMessages();
          
            EventPlayerChangedTeam( cPlayerInfo, nOldTeamId );
        }
    }
    
    void EventPlayerChangedTeam( NetPlayerInfo cPlayerInfo, int nOldTeamId )
    {
        int nNewTeamId = cPlayerInfo.GetTeamId();
            
        NetTeamInfo cTeamInfo = GetTeamInfoById( nNewTeamId ); 
        int nTeamInfoId = -1;
        int nSkin       = NetTeamInfo.NONE;
        if ( cTeamInfo != null )
        {
            nTeamInfoId = cTeamInfo.GetNetId();
            nSkin       = cTeamInfo.GetTeamId();
        }
        int     nChangeSkinCounter  = PlayerNetBase.GetNextGlobalChangeSkinCounter();
        
        int nPlayerInfoId = cPlayerInfo.GetNetId();
        
        boolean bFirstTime  = (nOldTeamId == NetTeamInfo.NONE);
        int     nMessageId  = bFirstTime ?
                              NetIncomingMsg._PLAYER_ASSIGNED_TEAM:
                              NetIncomingMsg._PLAYER_CHANGED_TEAM ;
        MsgSendClientPlayerChangedTeam( nPlayerInfoId, nTeamInfoId, nSkin, nChangeSkinCounter, bFirstTime );
        AddIncomingMessage( nMessageId, nPlayerInfoId, -1, nTeamInfoId );

        ChangePlayerTeamSkins( nPlayerInfoId, nSkin, nChangeSkinCounter );
        UpdateEnableObjectivesIfOwner( nPlayerInfoId );
        
        UpdateTeamRespawnLimitCnt( nOldTeamId );
        UpdateTeamRespawnLimitCnt( nNewTeamId );
            
        if ( ! IsGameStarted() )
        {
            StartGameIfNeeded();
        }
    }
        
    
    float GetDamageMultiplier( PlayerNetBaseMortal victim, PlayerNetBaseMortal assassin )
    {
        float fDamageMul = super.GetDamageMultiplier( victim, assassin );
        
        // jesli FriendlyFire jest wlaczone, to analizujemy
        // kto do kogo strzela
        if ( GetGameSettingsTeam().GetFriendlyFire() )
        {
            NetPlayerInfo cVictimInfo   = victim != null   ? victim.GetPlayerInfo()   : null;
            NetPlayerInfo cAssassinInfo = assassin != null ? assassin.GetPlayerInfo() : null;
                    
            // kto w kogo strzela?
            if ( cVictimInfo    != null         && 
                 cAssassinInfo  != null         &&
                 cAssassinInfo  != cVictimInfo )
            {
		    	// zarobil od kogos
                // pomnozymy przez m_fFriendlyFireMul jesli z tej samej druzyny
                if ( cAssassinInfo.GetTeamId() == cVictimInfo.GetTeamId() )
                    fDamageMul *= GetGameSettingsTeam().GetFriendlyFireMul(); 
            }
        }
        
        return fDamageMul;
    }

    int FindTeamIdForNewPlayer(NetPlayerInfo cInfo)
    {
        //jak mammy team balanz to znadujemy druzyne
        LogD(" "+GetGameSettingsTeam().GetTeamBalance()+"\n");
        if ( GetGameSettingsTeam().GetTeamBalance() ||
             cInfo.IsBot() || cInfo.IsAutoPlayer() )
            return GetTeamInfoList().FindTeamIdByTeamBalance();
        return NetTeamInfo.NONE;
    }
    
    boolean TryToChangeTeam( NetPlayerInfo cInfo, int nNewTeam )
    {
        if ( cInfo == null )
            return false;
        
        cInfo.AssignTeam( nNewTeam );
        
        return cInfo.GetTeamId() == nNewTeam;
    }
 
    boolean VerifySpawnPoint( NetSpawnPoint cSpawnPoint, NetPlayerInfo cPlayerInfo )
    {
        if ( cSpawnPoint == null )
        {
            LogD(" failed\n");
            return false;
        }
        return cSpawnPoint.AllowTeam( cPlayerInfo.GetTeamId() );
    }
    
    
    
    /** Sprawdzenie czy gra powinna wystartowac
     */
    boolean CheckGameShouldStarted()
    {
        return ( GetTeamInfoList().CalculateNotEmptyTeamsCnt() >= 
                 GetGameSettingsTeam().GetMinTeamsToStartGame() );
    }
    
    /**
     * 
     * Sprawdzenie warunkow zakonczenia gry
     * 
     */
    /** Sprawdzenie czy ktoras druzyna wygrala juz gre */
    boolean CheckGameWon() { return GetTeamInfoList().CheckTeamWon(); }
    /** Sprawdzenie czy na placu boju zostala jeszcze odpowiednia ilosc druzyn*/
    boolean CheckGameFailed() { return GetTeamInfoList().CheckTeamFailed( GetGameSettings().GetMinNumberActivePlayersInGame() ); }
    
    /**
     * 
     * Informacje wykorzystywane w hudzie statusu gry
     * 
     */
    /** Zwraca ilosc punktow druzyny prowadzacej*/
    int   GetLeaderPoints() { return GetTeamInfoList().GetLeaderPoints(); }
    /** Zwraca ilosc fragow druzyny prowadzacej*/
    int   GetLeaderFrags()     { return GetTeamInfoList().GetLeaderFrags();  }
    /** Zwraca tablice z informacjami o statusie gry*/    
    int[] GetGameScoreStatus() { return GetTeamInfoList().GetGameScoreStatus(); }
    
        
    //TODO: odermowac to co ponizej i przetestowac
    boolean ActivateStatisticsOnJoin()
    {
        if ( super.ActivateStatisticsOnJoin() )
        {
            if ( (!GetGameSettingsTeam().GetTeamBalance()) && 
                   IsLocalPlayer() &&
                  (!IsLocalPlayerAssignedToTeam()) )
            SetLocalChangeTeamMode( true );
            return true;
        }
        return false;
    }
    
    void ActivateStatisticsOnGameRestart()
    {
        super.ActivateStatisticsOnGameRestart();
                
        if ( m_bChangeTeamMode )
        {
            ForceSetLocalChangeTeamMode();
        }
    }
    
    void ActivateStatisticsOnDisconnect()
    {
        SetLocalChangeTeamMode( false );
        super.ActivateStatisticsOnDisconnect();
    }
    
    void ActivateStatisticsOnRespawnReject(int nReason)
    {
        //jesli mamy wyswietlic statystyki ze wzgledu na to ze gra jeszcze
        //nie wystartowala (i nie mozna sie wyspawnowac) a my chcemy
        //wlasnie zmienic team to ignorujemy zawolanie statystyk.
        if ( nReason == ERespawnRequestResults._DISABLED_GAME_NOTSTARTED &&
             m_bChangeTeamMode )
            return;
        
        super.ActivateStatisticsOnRespawnReject( nReason );
    }

    
    
    void UpdateEnableObjectives(NetPlayerInfo cPlayerInfo)
    {
        m_cModule.m_ObjectivesManager.SetEnableNet( true, cPlayerInfo.GetTeamId() );
    }
    
    void OnScoreChange() // Fallout : Updated Scrore Board..
    {
        GetTeamInfoList().Sort();
        GetPlayerInfoList().Sort();
        super.OnScoreChange();
    }
    
    int GetWinnerId()
    {
        return GetTeamInfoList().GetLeaderInfoId();
    }
    
    boolean IsPlayerWinner(NetPlayerInfo cPlayerInfo,int nWinnerId)
    {
        if ( cPlayerInfo == null )
            return false;
        NetTeamInfo cWinnerInfo = GetTeamInfoByNetId( nWinnerId );
        if ( cWinnerInfo == null )
            return false;
        return cPlayerInfo.GetTeamId() == cWinnerInfo.GetTeamId();
    }
    
    
   
}
