/** Klasa przechowuje informacje o parametrach uruchamianej
 * gry sieciowej.
 * Obiekt tej klasy nie replikuje sie z serwera na klientow,
 * Na serwerze obiekt zawiera wszystkie dane (m.in. liste gier)
 * Na kliencie tylko najistotniejsze dane (np aktualna gre)
 */
public class NetGameInitInfo extends GameInitInfo
{
    private NetServerInitInfo m_cServerInfo = new NetServerInitInfo();
    NetServerInitInfo GetServerInfo(){ return m_cServerInfo; }
        
    // typ gry ENetGameType
    public int nGameType = ENetGameType.DEFAULT;
    
    NetGameInitInfo()
    {
        // wczytanie listy leveli
        //cLevels.InitNet("Data/LevelsNet.scr" );
        cLevels.InitNet( NetPlayerProfile.GetCurrentFavoritesMapsFileName() );
    }
        
    /** Wybranie nastepnego levelu.
     */
    void SetNextLevel()
    {
        cLevels.NextLevel();
        if (cLevels.GetCurrent() == null && m_cServerInfo.GetLoopLevels() )
            cLevels.Restart();
        
        if (cLevels.GetCurrent() != null)
            nGameType = cLevels.GetCurrent().GetNetGameType();
    }

    /**
     */
    
    String GetCurrentLevelName()
    {
        if (cLevels.GetCurrent() != null)
            return cLevels.GetCurrent().sLevelName;
        return null;
    }    

    /**
     */
    LevelContent GetNextLevel()
    {
        int          nNextLevel = cLevels.GetCurrentLevelPos() + 1;
        LevelContent cNextLevel = cLevels.GetLevel( nNextLevel );
        
        if ( cNextLevel == null && m_cServerInfo.GetLoopLevels() )
            cNextLevel = cLevels.GetLevel( 0 );    
                
        return cNextLevel;
    }    
    
    /**
     */
    Module LoadCurrentLevel(GameObject cObject)
    {
        LevelContent cCurLevel = cLevels.GetCurrent();
        
        cCurLevel.AddTexturePaths();
        //GameObject.CrashLog("[NetGameInitInfo.LoadCurrentLevel] Loading level <"+ cCurLevel.sLevelName +">\n");
        
        Module cSomeModule = null;
        //zeby tylko na serwerze byla tu aktualizacja
        if ( this.nGameType == ENetGameType.UNKNOWN )
        {
            //TODO: tu przepisac ewentualnie jeszcze jakies inne parametry do GameInifInfo
            this.nGameType = cCurLevel.GetNetGameType();
        }
                
        if( cCurLevel.nLevelType == LevelContent._MAP)
        {
            cSomeModule = cObject.LoadModule( cCurLevel.sLevelName );
        }
        
        if( cCurLevel.nLevelType == LevelContent._MODULE)
        {
            cSomeModule = (Module)cObject.CreateObject( cCurLevel.sLevelName );
        }
        
        GameObject.CrashLog("NetGameInitInfo.LoadCurrentLevel] Level loaded <"+ cCurLevel.sLevelName +">\n");
        
        cCurLevel.RemoveTexturePaths();
        
        return cSomeModule;
    }

    /** Metoda wczytuje dane */
    public void PreLoad(FileChunk cFC, Module cModule)
    {
    }
    
    /** Metoda zapisuje dane */
    public void PreSave(FileChunk cFC)
    {
    }
    
    
    static NetGameInitInfo GetOnServer()
    {
        //tworzymy obiekt GameInfo
        NetGameInitInfo cGameInitInfo = new NetGameInitInfo();
        //ServerInfo odczytujemy ze skryptu
        cGameInitInfo.GetServerInfo().ParseScriptFile();
        //po to, zeby na serwerze te dane odczytaly sie pozniej
        cGameInitInfo.nGameType = ENetGameType.UNKNOWN;
        return cGameInitInfo;        
    }
    
    
    /**
     * Actions for console commands
     */

    /**
     * CommandMap
     */
    int ForceNetLevel( NetMapInfo[] aList )
    {
        int nRetOK = EConsoleCommandError._NONE;
        
        NetMapInfo cMapInfo   = aList[ 0 ];
        boolean    bForceType = aList.length == 1;
        int nCurrent = cLevels.GetCurrentLevelPos();
        int nLevel   = nCurrent;
        do 
        {
            nLevel ++;
            LevelContent cLevel = cLevels.GetLevel( nLevel );
            if ( cLevel == null )
            {
                nLevel = 0;
                cLevel = cLevels.GetLevel( nLevel );                
                if ( cLevel == null )
                    return EConsoleCommandError._WRONG_MAP_NAME;
            }
                
            if ( cLevel.IsEqual( cMapInfo, bForceType ) )
            {
                if ( cLevels.ForceNetLevel( nLevel ) )
                    return nRetOK;
                return EConsoleCommandError._UNKNOWN;
            }
            
        } while ( nLevel != nCurrent );
        
        LevelContent cLevel = new NetLevelContent( cMapInfo.GetFileName(), 
                                                   cMapInfo.GetGameType() );
        if ( cLevels.ForceNetLevel( nLevel, cLevel ) )
            return nRetOK;
        
        return EConsoleCommandError._UNKNOWN;
    }
    
    /**
     * CommandMapRemove
     */
    int RemoveNetLevel( NetMapInfo[] aList )
    {
        int nRetOK = EConsoleCommandError._NONE;
        
        NetMapInfo cMapInfo   = aList[ 0 ];
        boolean    bForceType = aList.length == 1;
        int nCurrent = cLevels.GetCurrentLevelPos();
        int nLevel   = nCurrent;
        do 
        {
            nLevel ++;
            LevelContent cLevel = cLevels.GetLevel( nLevel );
            if ( cLevel == null )
            {
                nLevel = 0;
                cLevel = cLevels.GetLevel( nLevel );                
                if ( cLevel == null )
                    return EConsoleCommandError._WRONG_MAP_NAME;
            }
                
            if ( cLevel.IsEqual( cMapInfo, bForceType ) )
            {
                if ( cLevels.RemoveNetLevel( nLevel ))
                    return nRetOK;
                return EConsoleCommandError._CANNOT_REMOVE_THIS_MAP_FROM_THE_MAP_LIST;
            }
            
        } while ( nLevel != nCurrent );
        
        return EConsoleCommandError._WRONG_MAP_NAME;
    }
}

