public class NetInfo extends GameObject
{
    /** Identyfikatory messagy wysylanych przez ta klase.
     */
	final static int _MSG_UPDATE_NAME                   = 0;
    final static int _MSG_UPDATE_TEAM_ID                = 1;
    final static int _MSG_UPDATE_FRAGS                  = 2;
	final static int _MSG_UPDATE_DEATHS                 = 3;
    final static int _MSG_UPDATE_POINTS                 = 4;
    final static int _MSG_UPDATE_EXCLUDED               = 5;
    final static int _MSG_UPDATE_RESPAWN_LIMIT_CNT      = 6;
	final static int _MSG_UPDATE_DOMINATION_BASE_OWNER  = 7;
   
    
    // aktualny modul
    protected ModuleNet     m_cModule = null;
    
    protected NetGameInfoBase m_cGameInfo = null;
    
    void SetGameInfo(NetGameInfoBase cGameInfo)
    { 
        m_cGameInfo = cGameInfo; 
    }
    boolean IsAddedToGameInfo()             { return m_cGameInfo != null;   }
    
    // numer druzyny (albo do ktorej nalezy gracz, albo aktualnej)
    private int     m_nTeamId       = NetTeamInfo.NONE;
    // nazwa (nick , lub nazwa druzyny )
    private String  m_sName         = null;
    // flaga mowiaca o tym, czy nazwa jest ustawiona
    private boolean m_bNameIsValid  = false;
    
    //wyniki
    
    // ilosc fragow
    private int m_nFrags = 0;
    // ilosc smierci
    private int m_nDeaths = 0;
    //punkty, zliczane w roznych rodzajach gier
    private int m_nPoints = 0;
    //czy gracz/druzyna zostala zdyskwalifikowana
    private boolean m_bExcluded = false;
    //wyniki-end, uwaga:po dopisaniu nowych dodac do CleanResults!
    
    //ilosc respawnow jakie zostalo graczowi, wykorzystywane w LMS i LTS
    private int m_nRespawnLimitCnt = -1;
    //czy gracz/druzyna jest wlascicielem bazy
    private int m_nDominationBaseOwner = 0;
    
    int     GetTeamId()  { return m_nTeamId;  }
    int     GetFrags()   { return m_nFrags;   }
    int     GetDeaths()  { return m_nDeaths;  }
    int     GetPoints()  { return m_nPoints;  }
    boolean IsExcluded() { return m_bExcluded;}
    
    String  GetName()    { if (m_bNameIsValid) return m_sName; else return ""; }
    boolean IsNameValid(){ return m_bNameIsValid; };
        
    int     GetRespawnLimitCnt()        { return m_nRespawnLimitCnt; }
       
    boolean IsDominationBaseOwner()               { return m_nDominationBaseOwner!=0;   }
    int DominationBaseOwnerCnt() { return NetGameTools.CntBits(m_nDominationBaseOwner);   }
    
    
    
    
    NetInfo()
    {
        Role=ERole._PROXY;
    }
    
    
    void OnCreate()
    {
        if ( GetModule() instanceof ModuleNet )
             m_cModule = (ModuleNet)GetModule();
        
        CleanResults();
    }
    
    void CleanResults()
    {
        m_nFrags    = 0;
        m_nDeaths   = 0;
        m_nPoints   = 0;
        m_bExcluded = false;
        
        if ( ModuleNet.IsGameGoing() )
        {
            if ( Role == ERole._AUTHORITY )
                SendClientUpdateData();
            
            if ( m_cGameInfo != null )
            {
                //to wolamy z opoznieniem, zeby sie odbylo tylko 
                //raz po WSZYSTKICH CleanResults
                m_cGameInfo.DisableCallMethod("OnScoreChange");
                m_cGameInfo.CallMethodIn("OnScoreChange",0.0f);
            }
        }
    }
    
    public void ReplicationInit()
    {
    }
    
    /** Replikacja odpowiednich zestawow dla odpowiednich warunkow
     */
    public void OnReplicateServer()
    {
		if ( bNetInit )
		{
            MsgSendClientUpdateName();
            MsgSendClientUpdateTeamId();
            
			SendClientUpdateData();
            if ( bNetIsOwner )
                SendClientOwnerUpdateData();
		}
        
        //Replicate( sm_nNetInfoDataSet );
    }
    
    void SendClientUpdateData()
    {
        MsgSendClientUpdateFrags();
	    MsgSendClientUpdateDeaths();
        MsgSendClientUpdatePoints();
        MsgSendClientUpdateExcluded();
        MsgSendClientUpdateDominationBaseOwner();
    }
    
    void SendClientOwnerUpdateData()
    {
        MsgSendClientUpdateRespawnLimitCnt();
    }
    
    void DeleteInFewSeconds()
    {
        CallMethodIn("delete",10.0f);
    }
	
    void IncFrags()
	{
		if ( m_cGameInfo == null )
		{
			CrashLog( "IncFrags " + this + " m_cGameInfo == null ");
			return;
		}
		
		if ( m_cGameInfo.IsScoringEnabled() )
        {
		    ++m_nFrags;
            m_cGameInfo.OnScoreChange();
		    if ( Role == ERole._AUTHORITY )
		    	MsgSendClientUpdateFrags();
        }
	}
    
	
	void DecFrags()
	{
		if ( m_cGameInfo == null )
		{
			CrashLog( "DecFrags " + this + " m_cGameInfo == null ");
			return;
		}
		
		if ( m_cGameInfo.IsScoringEnabled() )
        {
		    --m_nFrags;
            m_cGameInfo.OnScoreChange();
		    if ( Role == ERole._AUTHORITY )
		    	MsgSendClientUpdateFrags();
        }
	}
	    
    void IncDeaths()
	{
		if ( m_cGameInfo == null )
		{
			CrashLog( "IncDeaths " + this + " m_cGameInfo == null ");
			return;
		}
		
		if ( m_cGameInfo.IsScoringEnabled() )
        {
		    ++m_nDeaths;
            //m_cGameInfo.OnScoreChange();//to chyba nie bedzie istotne
		    if ( Role == ERole._AUTHORITY )
		    	MsgSendClientUpdateDeaths();
        }
	}
    
    void IncPoints()
	{
		if ( m_cGameInfo == null )
		{
			CrashLog( "IncPoints " + this + " m_cGameInfo == null ");
			return;
		}
		
		if ( m_cGameInfo.IsScoringEnabled() )
        {
		    ++m_nPoints;
            m_cGameInfo.OnScoreChange();
		    if ( Role == ERole._AUTHORITY )
		    	MsgSendClientUpdatePoints();
        }
	}
    
    void SetExcluded(boolean bExcluded)
	{
        if ( m_cGameInfo == null )
		{
			CrashLog( "SetExcluded " + this + " m_cGameInfo == null ");
			return;
		}
        
        if ( m_cGameInfo.IsGameGoing() )
        {
		    m_bExcluded = bExcluded;
		
		    if ( Role == ERole._AUTHORITY )
			    MsgSendClientUpdateExcluded();
        }
	}
    
    void CleanDominationBaseOwner( int nBase )
    {
        SetDominationBaseOwner( nBase, false );
    }
    
    void SetDominationBaseOwner( int nBase )
    {
        SetDominationBaseOwner( nBase, true );
    }
    
    void SetDominationBaseOwner(int nBase, boolean bOwner)
    { 
        if ( m_cGameInfo == null )
		{
			CrashLog( "SetDominationBaseOwner " + this + " m_cGameInfo == null ");
			return;
		}
        
        if ( m_cGameInfo.IsGameGoing() )
        {
            int nBits = NetGameTools.Int2Bit( nBase );
            if ( bOwner )
                m_nDominationBaseOwner |= nBits; 
            else
                m_nDominationBaseOwner &= NetGameTools.NegateBits( nBits ); 
        
            if ( Role == ERole._AUTHORITY )
		    	MsgSendClientUpdateDominationBaseOwner();
        }
    }
    
    void SetRespawnLimitCnt(int nCnt)
    { 
        if ( m_cGameInfo == null )
		{
			CrashLog( "SetRespawnLimitCnt " + this + " m_cGameInfo == null ");
			return;
		}
        
        if ( m_cGameInfo.IsGameGoing() )
        {
            m_nRespawnLimitCnt = nCnt; 
        
            if ( Role == ERole._AUTHORITY )
		    	MsgSendClientUpdateRespawnLimitCnt();
        }
    }
    
    void SetName( String sName )
	{
		m_sName         = new String( sName );
        m_bNameIsValid  = true;
		
		if ( Role == ERole._AUTHORITY )
			MsgSendClientUpdateName();
	}
    
    void MsgSendClientUpdateName()
	{
		MessageNew( _MSG_UPDATE_NAME );
		MessagePackString( m_sName );
		MessageSend();
	}
    
    void MsgRecvClientUpdateName()
	{
		String sName = MessageUnPackString();
		MessageEnd();
			
        if ( Role < ERole._AUTHORITY )
        {
		    SetName( sName );
		    RefreshStatistics();
        }
	}
        
    void SetTeamId( int nId )
    {
        m_nTeamId = nId ;
        if ( Role == ERole._AUTHORITY )
            MsgSendClientUpdateTeamId();
    }
	    
    void MsgSendClientUpdateTeamId()
	{
		MessageNew( _MSG_UPDATE_TEAM_ID );
		MessagePackInt( m_nTeamId );
		MessageSend();
	}
	
	void MsgRecvClientUpdateTeamId()
	{
		int nTeamId = MessageUnPackInt();
		MessageEnd();
        
        if ( Role < ERole._AUTHORITY )
        {
            SetTeamId( nTeamId );
		    RefreshStatistics();
        }
	}
    
    void MsgSendClientUpdateFrags()
	{
		MessageNew( _MSG_UPDATE_FRAGS );
		MessagePackInt( m_nFrags );
		MessageSend();
	}
	
	void MsgRecvClientUpdateFrags()
	{
		int nFrags = MessageUnPackInt();
		MessageEnd();
		
		if ( Role < ERole._AUTHORITY )
        {
            m_nFrags = nFrags;
            RefreshStatistics();    
            if ( m_cGameInfo != null )
                 m_cGameInfo.OnScoreChange();
        }
	}
    
    void MsgSendClientUpdatePoints()
	{
		MessageNew( _MSG_UPDATE_POINTS );
		MessagePackInt( m_nPoints );
		MessageSend();
	}
	
	void MsgRecvClientUpdatePoints()
	{
		int nPoints = MessageUnPackInt();
		MessageEnd();
		
		if ( Role < ERole._AUTHORITY )
        {
            m_nPoints = nPoints;
            RefreshStatistics();    
            if ( m_cGameInfo != null )
                 m_cGameInfo.OnScoreChange();
        }
	}
    
    void MsgSendClientUpdateDeaths()
	{
		MessageNew( _MSG_UPDATE_DEATHS );
		MessagePackInt( m_nDeaths );
		MessageSend();
	}
	
	void MsgRecvClientUpdateDeaths()
	{
		int nDeaths = MessageUnPackInt();
		MessageEnd();
		
		if ( Role < ERole._AUTHORITY )
        {
            m_nDeaths = nDeaths;
            RefreshStatistics();    
            /*if ( m_cGameInfo != null )//to chyba nie bedzie potrzebne
                 m_cGameInfo.OnScoreChange();*/
        }
	}
	
	
    void MsgSendClientUpdateExcluded()
	{
		MessageNew( _MSG_UPDATE_EXCLUDED );
		MessagePackBoolean( m_bExcluded );
		MessageSend();
	}
	
	void MsgRecvClientUpdateExcluded()
	{
		boolean bExcluded = MessageUnPackBoolean();
		MessageEnd();
		
		if ( Role < ERole._AUTHORITY )
        {
            m_bExcluded = bExcluded;
            RefreshStatistics();    
        }
	}
    
    
    void MsgSendClientUpdateRespawnLimitCnt()
    {
        MessageNew( _MSG_UPDATE_RESPAWN_LIMIT_CNT );
		MessagePackInt( m_nRespawnLimitCnt );
		MessageSend();
    }
     
    void MsgRecvClientUpdateRespawnLimitCnt()
	{
		int nRespawnLimitCnt = MessageUnPackInt();
		MessageEnd();
		
		if ( Role < ERole._AUTHORITY )
        {
            m_nRespawnLimitCnt = nRespawnLimitCnt;
            RefreshStatistics();    
        }
	}
    
    void MsgSendClientUpdateDominationBaseOwner()
    {
        MessageNew( _MSG_UPDATE_DOMINATION_BASE_OWNER );
		MessagePackInt( m_nDominationBaseOwner );
		MessageSend();
    }
    
    void MsgRecvClientUpdateDominationBaseOwner()
	{
		int nDominationBaseOwner = MessageUnPackInt();
		MessageEnd();
		
        if ( Role < ERole._AUTHORITY )
        {
            m_nDominationBaseOwner = nDominationBaseOwner;
            RefreshStatistics();    
        }
	}

    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case _MSG_UPDATE_NAME:
			MsgRecvClientUpdateName();
			break;
            
		case _MSG_UPDATE_TEAM_ID:
			MsgRecvClientUpdateTeamId();
			break;
            
        case _MSG_UPDATE_FRAGS:
			MsgRecvClientUpdateFrags();
			break;
            
		case _MSG_UPDATE_DEATHS:
			MsgRecvClientUpdateDeaths();
			break;
            
        case _MSG_UPDATE_POINTS:
			MsgRecvClientUpdatePoints();
			break;
            
        case _MSG_UPDATE_EXCLUDED:
			MsgRecvClientUpdateExcluded();
			break;
            
        case _MSG_UPDATE_RESPAWN_LIMIT_CNT:
			MsgRecvClientUpdateRespawnLimitCnt();
			break;
            
        case _MSG_UPDATE_DOMINATION_BASE_OWNER:
			MsgRecvClientUpdateDominationBaseOwner();
			break;
            
        default:
            MessageCancel();
		}
	}
    
    void RefreshStatistics()
	{
        if ( m_cGameInfo != null )
		    m_cGameInfo.RefreshStatistics();
        else
            LogD(" RefreshStatistics "+this+" failed\n");
	}
    
    
    
    PlayerNet GetPlayerNetByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            PlayerNet player = (PlayerNet) object;
            return player;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from PlayerNet\n");
        }
        
        return null;
    }
    
    
    /** Sprawdza czy gracz (druzyna) osiagnela koniec gry
     * Porownuje zdobyte punkty z maksymalnymi,
     */
    boolean CheckWon()
    {
        if ( m_cGameInfo == null )
            return false;
        return 
        (
         ( (m_cGameInfo.GetGameSettings().GetPointsLimit() > -1) && 
           (m_cGameInfo.GetGameSettings().GetPointsLimit() <= GetPoints()) )
         ||
         ( (m_cGameInfo.GetGameSettings().GetFragsLimit() > -1) && 
           (m_cGameInfo.GetGameSettings().GetFragsLimit() <= GetFrags()) )
        );
    }

    /**
     * Pobranie wyniku (punktow lub fragow w zaleznosci od trybu gry)
     */
    int GetScore()
    {
        if ( m_cGameInfo == null )
            return 0;
        return m_cGameInfo.GetPointsFromInfoToCompare( this, 0 );    
    }
            

}
