public class NetPlayerInfo extends NetInfo
{
    /** Identyfikatory messagy wysylanych przez ta klase.
     */
    final static int _MSG_UPDATE_NICK         = 10;
    final static int _MSG_UPDATE_PLAYERID     = 11;
    final static int _MSG_NEXT_RESPAWN_TIME   = 12;
    final static int _MSG_RESPAWN_CNT         = 13;
    final static int _MSG_CTF_FLAG_OWNER      = 14;
    final static int _MSG_UPDATE_PING         = 15;
	
    /////////////////////////////////////////////////////
    // atrybuty zawierajace informacje o graczu w grze //
    /////////////////////////////////////////////////////
 
    // ping gracza
    private int m_nPing = 0;
    void SetPing(int nPing){ m_nPing = nPing; };
    
    //wyniki-
    //czas gry, w ktorym gracz sie moze nastepny raz wyspawnowac
    private float m_fNextRespawnTime = 0.0f;
    //ilosc zliczonych spawnow
    private int   m_nRespawnCnt      = 0;
    //wyniki-end, uwaga: po dodaniu zmiennych dopisac je do CleanData
    
    //czy niesie gracz glage?    
    private boolean m_bCTFFlagOwner = false;
        

    // gracz ktorego dotyczy info
    private PlayerNet m_cPlayer = null;
    private int m_nPlayerId = -1;
    
    // id klienta, ktory jest posiadaczem playera
    private int m_nClientOwnerId = -1;

    
        
    PlayerNet   GetPlayer()         { return m_cPlayer;         }
    int         GetClientOwnerId()  { return m_nClientOwnerId;  }
    int         GetPing()           { return m_nPing;           }
    
    int         GetRespawnCnt()     { return m_nRespawnCnt;     }
    float       GetNextRespawnTime(){ return m_fNextRespawnTime;}    
    String      GetNick()           { return GetName();         }//zmienic to
    boolean     IsNickValid()       { return IsNameValid();     }//zmienic to   
    
    boolean     IsCTFFlagOwner()    { return m_bCTFFlagOwner;   }
    
    private
    boolean m_bKickedOutFromTheGame = false;
    boolean IsKickedOutFromTheGame(){ return m_bKickedOutFromTheGame; };

    String m_sNickClamped = null;
    String GetNickClamped(UIStatic cTextField)
    {
        //jak nick jeszcze nie jest ustalony, to go nie obrabiamy
        if ( ! IsNickValid() )
            return GetNick();
        
        if (m_sNickClamped == null)
        {
            float fField = cTextField.GetSize().fX;
            if (fField == 0)
                return GetNick();
            String sNick = new String(GetNick());
            boolean bClamped = false;
            while (cTextField.GetTextExtent(sNick).fX > fField)
            {
                sNick = sNick.substring(0, sNick.length() - 1);
                bClamped = true;
            }
            if (bClamped)
            {
                sNick = sNick.substring(0, sNick.length() - 3);
                sNick += "...";
            }
            m_sNickClamped = sNick;
            return sNick;
        }
        else
        {
            return m_sNickClamped;
        }
    }
    
    
    /** Identyfikatory statusu gracza.
     * Przydatne miedzy innymi do wysylania komunikatow 
     * Nie wolno wyslac komunikatu do innego gracza o wyzszym statusie
     */
    final static int _STATUS_SPECTATOR  = -1;
    final static int _STATUS_DEAD       =  0;
    final static int _STATUS_ALIVE      =  1;
        
    private boolean m_bVisibleOnHudNet = true;
    void SetVisibleOnHudNet(boolean bVisible){m_bVisibleOnHudNet=bVisible;}
    boolean IsVisibleOnHudNet(){ return m_bVisibleOnHudNet; }
    
    int GetStatus()
    {
        if ( IsAlive() )
            return _STATUS_ALIVE;
        //TODO: uwzglednic stan Spectatora
        return _STATUS_DEAD;
    }
    
    ///////////////////////////////////////////////////////
    // inicjalizacja obiektu tak aby replikowal atrybuty //
    ///////////////////////////////////////////////////////
    
    int METHOD_AddToGameInfo            = 0;
    int METHOD_RemoveFromGameInfo       = 0;
    int METHOD_TryFindPlayerByNetId     = 0;
    int METHOD_ServerForceKillMe        = 0;
    int METHOD_KickOutYourFromTheGame   = 0;
    
    void OnCreate()
    {
        super.OnCreate();
        
        METHOD_AddToGameInfo            = RegisterCallMethod("AddToGameInfo");
        METHOD_RemoveFromGameInfo       = RegisterCallMethod("RemoveFromGameInfo");
        METHOD_TryFindPlayerByNetId     = RegisterCallMethod("TryFindPlayerByNetId");
        METHOD_ServerForceKillMe        = RegisterCallMethod("ServerForceKillMe");    
        METHOD_KickOutYourFromTheGame   = RegisterCallMethod("KickOutYourFromTheGame");

        // zapisanie na liscie informacji o graczu
		AddToGameInfo();
    }
    
    void OnDestroy()
    {
        // obiekt musi sie skasowac z listy
        RemoveFromGameInfo();
    }
    
    void AddToGameInfo()
	{
        LogD(" "+this+" ");
        
        if (m_cModule != null && m_cModule.GetGameInfo() != null)
        {
            Log("ok\n");
            m_cModule.GetGameInfo().AddPlayerInfo( this );
            return;
        }
                    
        Log("m_cModule.cGameInfo object is null, later\n");
		CallMethodIn( METHOD_AddToGameInfo, 0.1f );
	}
    
    
    void RemoveFromGameInfo()
	{
        if (m_cGameInfo != null)
        {
            AssignTeam( NetTeamInfo.NONE );
            
            if ( GetPlayer() != null )
            {
                PlayerNet cOldPlayer = GetPlayer();
                SetPlayer( null );
                cOldPlayer.SetPlayerInfo( null );
            }
            
            m_cGameInfo.RemovePlayerInfo( this );
            return;
        }
	}
	
    void TryFindPlayerByNetId()
    {
        PlayerNet cOldPlayer = m_cPlayer;
        
        if ( m_nPlayerId > -1 )
        {
            m_cPlayer = GetPlayerNetByNetId(m_nPlayerId);
            
            if ( m_cPlayer == null )
            {
                CallMethodIn(METHOD_TryFindPlayerByNetId,0.5f);
                return;
            }
            
            // polaczanie do znalezionego playera
            m_cPlayer.SetPlayerInfo( this );
            m_cPlayer.TryToSetYourName();
        }
        
        DisableCallMethod( METHOD_TryFindPlayerByNetId );
        
        if ((cOldPlayer != null) && (cOldPlayer != m_cPlayer))
        {
            // odrejestrowanie sie "poprzedniemu" playerowi
            cOldPlayer.SetPlayerInfo( null );
        }
    }
    
    private boolean m_bAutoPlayer = false;
    private boolean m_bBot        = false;
    
    boolean IsBot(){ return m_bBot; }
    boolean IsAutoPlayer(){ return m_bAutoPlayer; }
    
    /**SERVER inicjalizacja player info i powiazanie
     * z obiektem playera
     */
    void ServerInit( PlayerNet cPlayer, int nId )
    {
        SetPlayer( cPlayer );
        SetName( cPlayer.GetName() );
        SetClientOwnerId( nId );
        SetNextRespawnTime( GetTime() );
        m_bAutoPlayer = cPlayer.IsAutoPlayer();
        m_bBot        = cPlayer.IsBot();
    }    
    
    void CleanResults()
    {
        super.CleanResults();
        
        SetNextRespawnTime( 0.0f );
        ClearRespawnCnt();                
    }
    
    /**SERVER replikacja wszystkich informacji o grze
     */
    //static int sm_nNetPlayerInfoDataSet;
    
    public void ReplicationInit()
    {
        super.ReplicationInit();
		//sm_nNetPlayerInfoDataSet = NewReplicationSet("m_nPing");
        SetNetPriority( 0.1f );
    }
	
	  
    /** Replikacja odpowiednich zestawow dla odpowiednich warunkow
     */
    public void OnReplicateServer()
    {
        super.OnReplicateServer();
        
		if ( bNetInit )
		{
			// na poczatek warto wyslac aktualne nicki (jak zrobi sie replikacje
			// stringow to mozna to pozniej puscic normalnym kanalem replikacji)
            MsgSendClientUpdatePlayerId();
		}
        
        // repliujemy informacje o grze do kazdego, gdzie by nie byl
        //Replicate( sm_nNetPlayerInfoDataSet );
    }
    
    void SendClientUpdateData()
    {
        super.SendClientUpdateData();
        MsgSendClientCTFFlagOwner();
    }
    
    void SendClientOwnerUpdateData()
    {
        super.SendClientOwnerUpdateData();
        MsgSendClientNextRespawnTime();
        MsgSendClientRespawnCnt();
    }
	
    
    void SetClientOwnerId(int nId)
    {
        m_nClientOwnerId = nId ;
    }
	
    void MsgSendClientUpdatePlayerId()
	{
		MessageNew( _MSG_UPDATE_PLAYERID );
		MessagePackInt( m_nPlayerId );
		MessageSend();
	}
		
	void MsgRecvClientUpdatePlayerId()
	{
		int nPlayerId = MessageUnPackInt();
		MessageEnd();
		
        if ( Role < ERole._AUTHORITY )
        {
            m_nPlayerId = nPlayerId;
            TryFindPlayerByNetId();
        }
	}
    
    
    void MsgSendClientNextRespawnTime()
	{
		MessageNew( _MSG_NEXT_RESPAWN_TIME );
		MessagePackFloat( m_fNextRespawnTime );
		MessageSend( );
	}
    		
	void MsgRecvClientNextRespawnTime()
	{
		float fNextRespawnTime = MessageUnPackFloat();
		MessageEnd();
        
        if ( Role < ERole._AUTHORITY )
        {
            m_fNextRespawnTime = fNextRespawnTime;
            RefreshStatistics();    
        }
	}
    
    void MsgSendClientRespawnCnt()
	{
		MessageNew( _MSG_RESPAWN_CNT );
		MessagePackInt( m_nRespawnCnt );
		MessageSend( );
	}
		
	void MsgRecvClientRespawnCnt()
	{
		int nRespawnCnt = MessageUnPackInt();
		MessageEnd();
        
        if ( Role < ERole._AUTHORITY )
        {
            m_nRespawnCnt = nRespawnCnt;
            RefreshStatistics();    
        }
	}
    
    
    void MsgSendClientCTFFlagOwner()
	{
		MessageNew( _MSG_CTF_FLAG_OWNER );
		MessagePackBoolean( m_bCTFFlagOwner );
		MessageSend( );
	}
    
		
	void MsgRecvClientCTFFlagOwner()
	{
		boolean bCTFFlagOwner = MessageUnPackBoolean();
		MessageEnd();
        
        if ( Role < ERole._AUTHORITY )
        {
            m_bCTFFlagOwner = bCTFFlagOwner;
            RefreshStatistics();    
        }
	}
		
    void MsgSendUpdatePing()
    {
        MessageNewUnreliable( _MSG_UPDATE_PING );
        MessagePackInt( m_nPing );
        MessageSend();
    }

    void MsgRecvUpdatePing()
    {
        m_nPing = MessageUnPackInt();
        MessageEnd();
    }
    


	public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case _MSG_UPDATE_PING:
            MsgRecvUpdatePing();
            break;
                           
        case _MSG_UPDATE_PLAYERID:
            MsgRecvClientUpdatePlayerId();
            break;
        
        case _MSG_NEXT_RESPAWN_TIME:
            MsgRecvClientNextRespawnTime();
            break;
            
        case _MSG_RESPAWN_CNT:
            MsgRecvClientRespawnCnt();
            break;
            
        case _MSG_CTF_FLAG_OWNER:
            MsgRecvClientCTFFlagOwner();
            break;
        
        default:
            super.OnMessage();
        }
	}
        
	
    void UpdatePing()
    {
        if (m_cModule.bBotsForTest && m_nClientOwnerId >= m_cModule.sm_nBotsIdMin)
            return;
                        
        //TODO: czy na pewno serwer ma id 0
        if ( m_nClientOwnerId > 0 )
            m_nPing = GetClientPing( m_nClientOwnerId );
        //MsgSendUpdatePing();
    }
    
    int UpdateAndGetPing()
    {
        UpdatePing();
        return m_nPing;
    }
    
        
    void SetPlayer(PlayerNet player)
    {
        m_cPlayer     = player;
        if ( player != null )
            m_nPlayerId   = player.GetNetId();
        else
            m_nPlayerId   = -1;

        if ( Role == ERole._AUTHORITY )
            MsgSendClientUpdatePlayerId();
    }
    
    void SetExcluded(boolean bExcluded)
    {
        boolean bLastExcluded = IsExcluded();
        
        super.SetExcluded( bExcluded );
        
        if ( !bLastExcluded && IsExcluded() )
        {
            if ( m_cGameInfo instanceof NetGameInfo )
                ((NetGameInfo)m_cGameInfo).EventPlayerExcluded( this );
            else
			    CrashLog( "SetExcluded " + this + " m_cGameInfo == "+m_cGameInfo+"\n");
        }
    }
    
    
    void AssignTeam( int nNewTeamId )
    {
        if ( m_cGameInfo == null )
        {
            CrashLog("AssignTeam failed\n");
            return;
        }
        
        NetTeamInfo cTeam = null;
        
        //najpierw wypiszmy ze starej druzyny
        if ( GetTeamId() != NetTeamInfo.NONE )
        {
            cTeam = m_cGameInfo.GetTeamInfoById( GetTeamId() );   
            
            if ( cTeam != null )
            {
                cTeam.DecNumPlayers();
                
                //TODO: a moze usunac cale TeamInfo jak juz nie ma zadnego gracza?
            }
        }
        
        // teraz wpiszmy ewentualnie do nowej druzyny
        if ( nNewTeamId != NetTeamInfo.NONE )
        {
            cTeam = m_cGameInfo.GetTeamInfoById( nNewTeamId );   
            if ( cTeam == null )
            {
                //trzeba stworzyc nowe info dla druzyny
                //Tworzenie teamow przeniosione do NetGameInfoTeam.OnCreate w 
                /*
                LogD(" Tworze nowa druzyne, bedzie miala teamid "+nNewTeamId+"\n");
                cTeam = (NetTeamInfo)CreateObjectNoInit("NetTeamInfo");
                cTeam.SetTeamId( nNewTeamId );
                cTeam.InitObject();*/
            }    
            cTeam.IncNumPlayers();
        }
        
        SetTeamId( nNewTeamId );
    }
    
    void SetCTFFlagOwner(boolean bFlagOwner)
    {
        m_bCTFFlagOwner = bFlagOwner;
        
        if ( Role == ERole._AUTHORITY )
            MsgSendClientCTFFlagOwner();
    }
    
    void SetNextRespawnTime(float fTime)
    {
        m_fNextRespawnTime = fTime;
        
        if ( Role == ERole._AUTHORITY )
            MsgSendClientNextRespawnTime();
    }
    
    void IncRespawnCnt()
    {
        m_nRespawnCnt++;
        
        if ( Role == ERole._AUTHORITY )
            MsgSendClientRespawnCnt();
    }
    
    void ClearRespawnCnt()
    {
        m_nRespawnCnt = 0;

        if ( ModuleNet.IsGameGoing() && (Role == ERole._AUTHORITY) )
            MsgSendClientRespawnCnt();
    }
    
    boolean IsAlive()  {   return ( m_cPlayer.bDead == false );  }    

    
    void UpdateRespawnLimitCnt()
    {
        if ( m_cGameInfo == null )
            return;
        int nLimit = m_cGameInfo.GetGameSettings().GetRespawnCntLimit();
        if ( nLimit > 0 )
        {
            nLimit -= GetRespawnCnt();
            
            SetRespawnLimitCnt( nLimit );
        }
        
        if ( GetTeamId() != NetTeamInfo.NONE )
        {
            m_cGameInfo.UpdateTeamRespawnLimitCnt( GetTeamId() );
        }
    }
    
    boolean RespawnLimitAvailable()
    {
        if ( m_cGameInfo == null )
            return false;
        //jesli nie ma zadnych ograniczen, to nie ma co sprawdzac
        if ( m_cGameInfo.GetGameSettings().GetRespawnCntLimit() < 0 )
            return true;
        return ( m_cGameInfo.GetGameSettings().GetRespawnCntLimit() > GetRespawnCnt() );
    }
    
    /** Sprawdza czy gracz juz nie ma szans na zakonczenie gry
     */
    boolean CheckFailed()
    {
        if ( m_cGameInfo == null )
            return true;
        return ( ! (IsAlive() || RespawnLimitAvailable()) );
    }
    
    void SetTeamId( int nId )
    {
        super.SetTeamId( nId );
        
        if ( m_cGameInfo != null )
             m_cGameInfo.OnTeamChange();
    }
    
    
    void ServerForceKillMe()
    {
        if ( Role == ERole._AUTHORITY && GetPlayer() != null )
            GetPlayer().ForceDieByYourSelf();
    }
    
    /**
     * Opis stanu gracza przysylany co jakis czas z serwera
     */
    private
    byte    m_nPlayerState = 0;
    byte    GetReceivedPlayerState() { return m_nPlayerState; }
    
    private
    float   m_fStateTime   = 0.0f; 
    float   GetReceivedPlayerStateTime() { return m_fStateTime; }
                          
    /**
     * Zapamietanie stanu gracza odebranego z serwera
     */
    void SetReceivedPlayerState( byte nPlayerState, float fTime )
    {
        m_nPlayerState = nPlayerState;
        m_fStateTime   = fTime; 
    }
    
    void ScheduleKickOutFromTheGame()
    {
        m_bKickedOutFromTheGame = true;
        CallMethodIn( METHOD_KickOutYourFromTheGame, 0.5f );
    }
    
    void KickOutYourFromTheGame()
    {
        ForceKickClient( GetClientOwnerId() );
    }
}
