public class NetPlayerInfoList extends NetInfoList
{
    /**
     * Flaga blokujaca na pewien czas mozliwosc sortowania
     * listy graczy.
     * Robione jest to po to, aby przy kolejnych
     * iteracjach listy graczy miec pewnosc ze nie
     * zmieni sie kolejnosc jej elementow (co mogloby 
     * skutkowac pominieciem ktoregos z nich)
     */
    boolean m_bSortingLocked = false;
                      
    NetPlayerInfoList(NetGameInfoBase cGameInfo)
    {
        super( cGameInfo );
    }

    /** Pobranie NetPlayerInfo z NetInfo
     */
    private NetPlayerInfo GetPlayerInfo( NetInfo cInfo )
    {
        return cInfo instanceof NetPlayerInfo ? (NetPlayerInfo)cInfo : null;
    }
    
    /** Pobranie NetPlayerInfo po indeksie
     */
    NetPlayerInfo GetPlayerByIndex(int nIndex)
    {
        NetInfo cInfo = GetInfoByIndex( nIndex );
        return GetPlayerInfo( cInfo );
    }
       
    /** Pobranie NetPlayerInfo po nicku
     */
    NetPlayerInfo GetPlayerByNick(String sNick)
    {
        NetInfo cInfo = GetInfoByName( sNick );
        return GetPlayerInfo( cInfo );
    }
    
    /** Pobranie NetPlayerInfo ktory ma bNetIsOwner = true,
     * czyli jego wlascicielem jest lokalny gracz
     */
    NetPlayerInfo GetMyPlayer()
    {
        NetInfo cInfo = GetMyInfo();
        return GetPlayerInfo( cInfo );
    }
    
    /** Pobranie NetPlayerInfo po identyfikatorze hosta wlasciciela
     */
    NetPlayerInfo GetPlayerByHostId(int nHostId)
    {
        for ( int i=0; i<GetSize(); i++ )
        {
            NetPlayerInfo cInfo = GetPlayerByIndex( i );
            
            if ( cInfo == null )
                continue;
            
            if ( cInfo.GetClientOwnerId() == nHostId )
			{
				return cInfo;
			}
        }
        return null;
    }
    
    /** Pobranie NetPlayerInfo po identyfikatorze sieciowym obiekty
     */
    NetPlayerInfo GetPlayerByNetId(int nNetId)
    {
        NetInfo cInfo = GetInfoByNetId( nNetId );
        return GetPlayerInfo( cInfo );
    }
        
    /** Ustawienie wszystkim gracza stanu GameOver
     */    
    void ForcePlayersDieOnGameOver()
    {
        boolean bSortingLocked = m_bSortingLocked;
        m_bSortingLocked = true;
        
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo info = GetPlayerByIndex( i );
            
            if ( info == null )
                continue;
            
			if (info.GetPlayer()==null)
				continue;
            
            info.GetPlayer().ForceDieOnGameOver();
        }
        
        m_bSortingLocked = bSortingLocked;
    }
    
    /** Ustawienie wszystkim gracza stanu GameRestart
     */    
    void ForcePlayersDieOnGameRestart()
    {
        boolean bSortingLocked = m_bSortingLocked;
        m_bSortingLocked = true;
        
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo info = GetPlayerByIndex( i );

            if ( info == null )
                continue;
            
			if (info.GetPlayer()==null)
				continue;
                        
            info.GetPlayer().ForceDieOnGameRestart();
        }
        
        m_bSortingLocked = bSortingLocked;
    }
    
    /**
     * Zdeaktywowanie wszystkich trupkow
     */
    void ForceDeactivateDeadBodyOnGameRestart()
    {
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo info = GetPlayerByIndex( i );
            
            if ( info == null )
                continue;
            
			if (info.GetPlayer()==null)
				continue;
                        
            DeadBody cDeadBody = info.GetPlayer().GetDeadBody();
            if ( cDeadBody != null )
                cDeadBody.ServerDeactivate();
        }
    }
    
    /** Sprawdzenie czy nie ma juz na liscie graczy o
     * podanym nicku, jesli jest, to zostaje on zmodyfikowany
     * Zwracany jest zaakceptowany(zmodyfikowany) nick
     */
    String VerifyPlayerNick( String sRequestedNick )
    {
        //najpierw oczyscmy nicka ze spacji
        int nSafety = 100;
        do {
            int nBegin = sRequestedNick.indexOf( " " );
        
            if ( nBegin < 0 || nSafety-- < 0 )
                break;
            sRequestedNick = sRequestedNick.substring( 0, nBegin ) + sRequestedNick.substring( nBegin+1 );
        } while (true);
            
        String sNick = null;
        int i = 0;
        do {
            sNick = sRequestedNick;
            // przy pierwszym przejsciu nic nie dopiszemy, przy kolejnym
            // numerki 2, 3, 4...
            if ( ++i > 1 )
                sNick = sNick + String.valueOf( i );    
        } while ( GetPlayerByNick( sNick ) != null );
        
        return sNick;
    }
    
    /** Sprawdzenie czy ktos z graczy juz skonczyl gre
     */
    boolean CheckPlayerWon()
    {
        //domyslnie sprawdzamy czy jakis gracz skonczyl...
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
            
			if (cPlayerInfo == null)
				continue;
			
			if ( cPlayerInfo.CheckWon() )
			{
				// The winner is... :-)
				return true;
			}
		}
        return false;
    }
    
    //zwraca ilosc punktow lidera
    int GetLeaderPoints() 
    { 
        int nPoints = Integer.MIN_VALUE;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if (cPlayerInfo == null)
				continue;
			if ( nPoints < cPlayerInfo.GetPoints() )
                nPoints = cPlayerInfo.GetPoints();
		}
        return nPoints;
    }
    //zwraca ilosc fragow lidera
    int GetLeaderFrags() 
    { 
        int nFrags = Integer.MIN_VALUE;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if (cPlayerInfo == null)
				continue;
			if ( nFrags < cPlayerInfo.GetFrags() )
                nFrags = cPlayerInfo.GetFrags();
		}
        return nFrags;
    }
    
    /**
     * Procka zwraca trzy liczby, ktore obrazuja aktualny wynik gry
     * [0] to punkty gracza lokalnego
     * [1] to :
     * - +1 jesli jestesmy liderem i bedziemy podawali nasza 
     * przewage nad drugim zawodnikiem
     * - -1 jesli ktos inny jest liderem i bedziemy podawali nasza
     * strate do niego
     * [2] to _wartosc bezwzgledna_:
     * - przewaga nad vicelideram (jesli my prowadzimy)
     * - strata do lidera (jesli gracz lokalny nie jest liderem)
     * Liczby te beda interpretowane i wyswietlone w hudzie Gamestatus
     */
    int[] GetGameScoreStatus()
    {
        Sort();
        
        NetPlayerInfo cMyPlayer = GetMyPlayer();
        if ( cMyPlayer == null )
            return null;
        
        
        int nMyScore = cMyPlayer.GetScore();
        
        int[] aStatus = new int[ 3 ];
        
        //wlasne punkty ida na poczatek
        aStatus [ 0 ] = nMyScore;
        
        if ( IndexOf( cMyPlayer ) == 0 )
        {
            //jestesmy liderem, zatem
            //zaznaczamy ze ma byc plus przed kolejna wartoscia
            aStatus [ 1 ] = +1;
            
            //znajdzmy kto jest po nas i wpiszmy roznice punktow
            NetPlayerInfo cPlayer = GetPlayerByIndex( 1 );
            if ( cPlayer != null )
                 aStatus [ 2 ] = nMyScore - cPlayer.GetScore();
         }
        else
        {
            //nie jestesmy liderem, wiec
            //zaznaczamy ze ma byc minus przed kolejna wartoscia
            aStatus [ 1 ] = -1;
            
            //pobiezmy lidera i sprawdzmy ile nam brakuje do niego
            NetPlayerInfo cPlayer = GetPlayerByIndex( 0 );
            if ( cPlayer != null )
                aStatus [ 2 ] = cPlayer.GetScore() - nMyScore;
        }
        return aStatus;
    }

    /** Sprawdzenie czy gracze z podanej druzyny juz przegrali
     * TODO: cos mi tu nie gra, sprawdzic to
     */
    boolean CheckPlayersTeamFailed(int nTeamId, float fMinPlayersInGame)
    {
        //domyslnie liczymy graczy, w grach teamowych
        //wezmiemy pod uwage druzyny poszczegolne druzyny
        int nPlayersExcluded = 0;
        int nPlayersInGame   = 0;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if (cPlayerInfo == null)
				continue;
            
            if ( nTeamId != NetTeamInfo.NONE && 
                 nTeamId != cPlayerInfo.GetTeamId() )
                continue;
            
            //ten juz wczssniej odpadl
            if (cPlayerInfo.IsExcluded())
            {
                nPlayersExcluded ++;
                continue;
            }
			
			if ( cPlayerInfo.CheckFailed() )
			{
                //ten juz sie wykluczyl
                cPlayerInfo.SetExcluded( true );
                nPlayersExcluded ++;
			}
            else
            {
                nPlayersInGame++;
            }
		}
        if ( nTeamId == NetTeamInfo.NONE )
		    return (nPlayersExcluded > 0 && nPlayersInGame < fMinPlayersInGame );
        else
            return (nPlayersExcluded > 0 && nPlayersInGame < 1 );
    }
    
    /** Obliczenie aktualne limitu respawnow dla druzyny
     */
    int CalculateTeamRespawnLimitCnt( NetTeamInfo cTeamInfo )
    {
        int nLimit = 0;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if ( cPlayerInfo == null )
				continue;
                
            if ( cPlayerInfo.GetTeamId() != cTeamInfo.GetTeamId() )
                continue;
                
            nLimit += cPlayerInfo.GetRespawnLimitCnt();
        }
        return nLimit;
    }
    
    int m_nLastSize = 0 ;
    boolean Sort()
    {
        //sa momenty w ktorych nie mozna sortowac
        if ( m_bSortingLocked )
            return false;
        
        boolean bOrderChanged = super.Sort();
        if ( m_nLastSize != GetSize() || bOrderChanged )
        {
            m_nLastSize = GetSize();
            //TODO: "pozaznaczac" ktorych ewentualnie nie wyswietlac na HUDzie
            //najpierw zakladamy ze wszyscy widoczni
            for ( int i=0; i<GetSize(); i++ )
	        {
	        	NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
	        	if ( cPlayerInfo == null )
	        		continue;
                cPlayerInfo.SetVisibleOnHudNet( true );
            }
            //ustalic czy to gra teamowa czy indywidualna
            //m_cGameInfo
        }
        return bOrderChanged;
    }
    
    /** Czy nalezy zmienic kolejnosc na liscie?
     */
    boolean WrongOrder(NetInfo cCurrent,NetInfo cNext)
    {
        //najpierw sortujemy wg druzyn (malejaca zeby NONE wypadly na koniec)
        if ( cCurrent.GetTeamId() < cNext.GetTeamId() )
            return true;
        
        return super.WrongOrder( cCurrent, cNext );
    }
    

    /**
     * Metoda pakuje do komunikatu wysylanego przez cGameInfo
     * stan wszystkich graczy na serwerze
     */
    void MessagePackPlayersState(NetGameInfo cGameInfo,float fTime)
    {
        cGameInfo.MessagePackFloat( fTime );
        cGameInfo.MessagePackInt( GetSize() );
        for ( int i = 0; i < GetSize(); i++ )
        {
            NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
            PlayerNet     cPlayer     = cPlayerInfo.GetPlayer();
            
            cGameInfo.MessagePackInt( cPlayerInfo.GetNetId() );
            cGameInfo.MessagePackByte( cPlayer.GetStateFromServer() );
        }
    }

    /**
     * Metoda odpakowuje z komunikatu odebranego przez cGameInfo
     * stan wszystkich graczy z serwera
     * Zapamietuje ten stan w NetlayerInfo aby po pewnym
     * czasie zweryfikowac ten stan
     */
    void MessageUnPackPlayersState(NetGameInfo cGameInfo)
    {
        float fTime = cGameInfo.MessageUnPackFloat();
        int   nSize = cGameInfo.MessageUnPackInt();
        for ( int i = 0; i < nSize; i++ )
        {
            int  nPlayerInfoId = cGameInfo.MessageUnPackInt();
            byte nPlayerState  = cGameInfo.MessageUnPackByte();
            
            NetPlayerInfo cPlayerInfo = GetPlayerByNetId( nPlayerInfoId );
            if ( cPlayerInfo != null )
            {
                cPlayerInfo.SetReceivedPlayerState( nPlayerState, fTime );
            }
        }
    }

    
    /**
     * Metoda pakuje do komunikatu wysylanego przez cGameInfo
     * pingi wszystkich graczy na serwerze
     */
    void MessagePackPlayersPing(NetGameInfo cGameInfo)
    {
        cGameInfo.MessagePackByte( (byte)GetSize() );
        for ( int i = 0; i < GetSize(); i++ )
        {
            NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
            
            cGameInfo.MessagePackInt( cPlayerInfo.GetNetId() );
            cGameInfo.MessagePackInt( cPlayerInfo.UpdateAndGetPing() );
        }
    }

    /**
     * Metoda odpakowuje z komunikatu odebranego przez cGameInfo
     * pingi wszystkich graczy z serwera
     */
    void MessageUnPackPlayersPing(NetGameInfo cGameInfo)
    {
        int   nSize = cGameInfo.MessageUnPackByte();
        for ( int i = 0; i < nSize; i++ )
        {
            int nPlayerInfoId = cGameInfo.MessageUnPackInt();
            int nPing         = cGameInfo.MessageUnPackInt();
            
            NetPlayerInfo cPlayerInfo = GetPlayerByNetId( nPlayerInfoId );
            if ( cPlayerInfo != null )
            {
                cPlayerInfo.SetPing( nPing );
            }
        }
    }
    
    /**
     * Weryfikacja ostatanio odebranego stanu poszczegolnych
     * graczy.
     * Kazdemu z Playerow "pokazywany" jest jego aktualny stan
     * na serwerze i on sam jesli stwierdzi ze cos sie "rozjechalo"
     * to prosi serwer o szczegolowe dane na swoj temat
     */
    void VerifyPlayersState()
    {
        for ( int i = 0; i < GetSize(); i++ )
        {
            NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
            if ( cPlayerInfo == null )
                continue;
            PlayerNet     cPlayer     = cPlayerInfo.GetPlayer();
            if ( cPlayer == null )
                continue;
                     
            cPlayer.CheckStateFromServer( cPlayerInfo.GetReceivedPlayerState(),
                                          cPlayerInfo.GetReceivedPlayerStateTime() );
        }
    }
    
    /**
     * TODO: tymczasowo
     */    
    void Cheat()
    {
        NetPlayerInfo cMyPlayer = GetMyPlayer();
        if ( cMyPlayer == null )
            return;
        
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if (cPlayerInfo == null || cPlayerInfo == cMyPlayer)
				continue;
            cPlayerInfo.IncFrags();
        }
    }

    


}
