/**
 * Manager informacji o aktualnym stanie (statusie)
 * gry, Niektore przydatne informacje beda pojawialy
 * sie na hudzie
 * Sklada sie z 
 * -informacji o stanie gry 
 *      - ile czasu do konca
 *      - ile punktow/fragow do konca
 *      - kto ma jakie flagi (CTF)
 *      - kto kontroluje ile baz (Domination)
 * -informacji o biezacych wynikach
 */
public class GameStatusMsgManager extends GameObject
{
    NetGameInfoHud  m_cGameInfo     = null; 
    NetGameSettings m_cGameSettings = null;
        
    java.util.Vector m_aInfoMsg = new java.util.Vector();
    
    GameInfoMsgTime    m_cTime   = null;
    GameInfoMsgPoints  m_cPoints = null;
    GameInfoMsgFrags   m_cFrags  = null;
    
    GameInfoMsgData    m_cData   = null; 
    GameInfoMsgData    GetData(){ return m_cData; }
        
    GameScoreMsg m_cScoreMsg = null;
    
    
    /**
     * Aktualnie wyswietlany na hudzie komunikat
     */
    GameInfoMsg m_cInfoMsg        = null;
    /**
     * Czas w ktorym ten komunikat sie pojawil
     */
    float          m_fInfoMsgUseTime = 0.0f;
    
    void OnCreate()
    {
        m_cTime   = new GameInfoMsgTime();
        m_cPoints = new GameInfoMsgPoints();
        m_cFrags  = new GameInfoMsgFrags();
        
        AddInfoMsg( m_cTime );
        AddInfoMsg( m_cPoints );
        AddInfoMsg( m_cFrags );
        
        m_cScoreMsg = new GameScoreMsg();
    }
    
    void OnDestroy()
    {
        m_cGameInfo      = null;
        m_cGameSettings  = null;
        for (int i=0; i < m_aInfoMsg.size(); i++ )
        {
            GameInfoMsg cMsg = (GameInfoMsg)m_aInfoMsg.get(i);
            cMsg.Destroy();
        }
        m_aInfoMsg.removeAllElements();
        m_cTime   = null;
        m_cPoints = null;
        m_cFrags  = null;
        if ( m_cScoreMsg != null )
        {
            m_cScoreMsg.Destroy();
            m_cScoreMsg = null;
        }
    }
    
    void SetGameInfo(NetGameInfoHud cGameInfo)
    { 
        m_cGameInfo = cGameInfo; 
        if ( m_cScoreMsg != null )
            m_cScoreMsg.SetParams( this, cGameInfo );
    }
    void Init( boolean bGameIsGoing, NetGameSettings cSettings )
    { 
        m_bGameIsGoing  = bGameIsGoing;
        m_cGameSettings = cSettings; 
        if ( m_cScoreMsg != null )
            m_cScoreMsg.SetGameSettings( cSettings );
        ForceUpdate(); 
    }
       
    boolean m_bGameIsGoing = false;
    boolean IsGameGoing(){ return m_bGameIsGoing; }
    
    /**
     * Callback przychodzacy z GameInfo przy starcie gry
     */
    void OnGameRestart()
    {
        m_bGameIsGoing = true;
        ForceUpdate();
    }
    /**
     * Callback przychodzacy z GameInfo przy game over
     */
    void OnGameOver()
    {
        m_bGameIsGoing = false;
        ForceUpdate();
    }
    /**
     * Callback przychodzacy z GameInfo przy zmianie druzyny przez gracza
     */
    void OnTeamChange()
    {
        //UpdateInfo();
        //dlaczego tu bylo tylko to powyzsze?
        ForceUpdate();
    }
    /**
     * Callback przychodzacy z GameInfo przy zmianie punktacji
     */
    void OnGameScoreChange()
    {
        if ( IsActive() )
        {
            UpdateInfoScore();
            m_cScoreMsg.Update();
        }
    }
        
    /**
     * Czy manager jest aktywny, tzn czy mamy pozwolenie 
     * od managera glownego oraz czy znamy juz game settings
     */
    boolean IsActive(){ return IsGameGoing() && m_cGameSettings != null; }
    
    void ForceUpdate()
    {
        ForceUpdateInfo();
        ForceUpdateScore();
    }
    
    
    /**
     * INFO
     */
    
    /**
     * Rejestracja message'a w systemie
     */
    void AddInfoMsg(GameInfoMsg cMsg)
    {
        if ( cMsg instanceof GameInfoMsg )
        {
            m_aInfoMsg.addElement( cMsg );
            cMsg.SetManager( this );
        }
    }
        
    
    /**
     * Wymuszenie odswiezenia systemu komunikatow
     */
    final void ForceUpdateInfo()
    {
        //wylaczamy updateowanie czasu
        DisableCallMethod("UpdateInfoTime");
        
        if ( IsActive() )
        {
            //jestesmy aktywni
            //dokonujemy updatu poszczegolnych komunikatow
            UpdateInfo();
            
            //jesli gra ma ograniczony czas to wlaczamy updateowanie czasu
            if ( m_cGameSettings != null && 
                 m_cGameSettings.GetGameTimeLimit() >  -1.0f )
            {
                CallMethodEvery( "UpdateInfoTime", 1.0f );
            }
            
        }
        else
        {
            //jestesmy nieaktywni, trzeba schowac to co bylo wczesniej
            if ( m_cData != null )
                m_cData.Clean();
            SetInfoMsg( null );
        }
    }
    
    /**
     * Update poszczegolnych elementow
     * Ta metode nalezy poszerzyc w potomnych klasach o 
     * dodatkowe pola
     */
    void UpdateInfo()
    {
        UpdateInfoTime();
        UpdateInfoScore();    
    }
    
    void UpdateInfoTime()
    {
        //jezeli nie jestesmu aktywni to nic tu po nas
        if ( ! IsActive() )
            return;
        //jesli gra ograniczona jest czasowo,
        //to przekaz do odpowiedniego message'a pozostaly czas
        if ( m_cGameSettings.GetGameTimeLimit() >  -1.0f )
        {
            float fTimeLeft =  m_cGameSettings.GetGameTimeLimit() 
                             - m_cGameInfo.GetGameTime();
            m_cTime.Update( fTimeLeft );
        }
    }
    void UpdateInfoScore()
    {
        //jezeli nie jestesmu aktywni to nic tu po nas
        if ( ! IsActive() )
            return;

        //jesli gra ograniczona jest maksymalna iloscia punktow,
        //to przekaz do odpowiedniego message'a pozostaly limit punktow
        if ( m_cGameSettings.GetPointsLimit() > -1.0f )
        {
            int nPointsLeft =  m_cGameSettings.GetPointsLimit() 
                             - m_cGameInfo.GetLeaderPoints();
            //(float) po to, zeby
            m_cPoints.Update( (float)nPointsLeft );
        }
        //jesli gra ograniczona jest maksymalna iloscia fragow,
        //to przekaz do odpowiedniego message'a pozostaly limit fragow            
        if ( m_cGameSettings.GetFragsLimit() > -1.0f )
        {
            int nFragsLeft =  m_cGameSettings.GetFragsLimit() 
                            - m_cGameInfo.GetLeaderFrags();
            m_cFrags.Update( (float)nFragsLeft );
        }
    }
    
    /**
     * Callback przychodzacy z komunikatu po jego zmianie
     */
    void OnInfoMsgChange(GameInfoMsg cMsg)
    {
        if ( cMsg == null )
            return;
     
        if ( m_cInfoMsg == cMsg && !GameStatusMsg.IsValid( m_cInfoMsg ) )
        {
            //jesli zmienil sie aktualnie wyswietlany komunikat
            //i przestal on byc aktywnym, 
            
            //to przelaczamy na jakis inny
            //ScheduleSwitchInfoMsg();
            SwitchInfoMsg();
        }
        else
        if ( m_cInfoMsg               == null               ||
             m_cInfoMsg               == cMsg               ||
             m_cInfoMsg.GetPriority() <  cMsg.GetPriority() )
        {
            //jesli aktualnie nic nie wyswietlamy lub
            //wyswietlamy akurat ten komunikat co sie zmienil lub
            //wyswietlamy komunikat o nizszym priorytecie
            
            // to przelaczamy sie na ten komunikat co wlasnie
            // ulegl zmianie
            SetInfoMsg( cMsg );
        }
        else 
        if ( ! m_bSwitchInfoMsgScheduled )
        {
            //jesli nie mamy zaplanowanego update'u (czyli wczesniej
            //byl tylko jeden aktywny message) to zaplanujmy to, ale 
            //uwzglednijmy czas, przez jaki juz aktualny message jest wyswietlany
            //jesli aktualny komunikat jest juz na hudzie dluzej niz
            //ustawa przewiduje to zmiana nastapi natychmiastowo
            ScheduleSwitchInfoMsg( GetTime() - m_fInfoMsgUseTime );
        }
    }
    
    /**
     * Flaga mowiaca o tym czy mamy zaplanowany update komunikatu
     * Planowanie update'u ma miejsce wtedy gdy aktualnie jest
     * wiecej niz jeden aktywny komunikat i nalezy je czasem
     * przelaczac
     */
    boolean m_bSwitchInfoMsgScheduled = false;
    
    /**
     * zaplanowanie update'u komunikatow
     */
    void ScheduleSwitchInfoMsg()
    {
        ScheduleSwitchInfoMsg( 0.0f );
    }
    void ScheduleSwitchInfoMsg(float fDelta)
    {
        DisableCallMethod("SwitchInfoMsg");
        //czy mamy zaplanowac nastepne przelaczenie komunikatow?
        m_bSwitchInfoMsgScheduled = false; 
        //po pierwsze musimy byc aktywni
        if ( IsActive() )
        {
            //po drugie sprawdzmy ile jest aktywnych komunikatow
            int nCnt = 0;
            for (int i=0; i < m_aInfoMsg.size(); i++ )
            {
                GameInfoMsg cMsg = (GameInfoMsg)m_aInfoMsg.get(i);
                if ( GameStatusMsg.IsValid( cMsg ) )
                    nCnt ++;
            }    
            //jesli minimum 2 to ok
            m_bSwitchInfoMsgScheduled = nCnt > 1;
        }
        if (m_bSwitchInfoMsgScheduled)
            CallMethodIn( "SwitchInfoMsg", 10.0f/*_UPDATE_TIME*/ - fDelta );
    }
    
    
    /**
     * Przelaczenie sie na kolejnego aktywnego messagea
     */
    void SwitchInfoMsg()
    {
        SetInfoMsg( GetNextInfoMsg() );
    }
    
    /**
     * Zmiana aktualnie wyswietlanego komunikatu na hudzie
     */
    void SetInfoMsg(GameInfoMsg cMsg)
    {
        //nowy komunikat
        m_cInfoMsg          = cMsg;
        //czas jego pojawienia sie na hudzie
        m_fInfoMsgUseTime   = GetTime();
        //poinformujmy go ze zostal uzyty
        if ( m_cInfoMsg != null )
             m_cInfoMsg.Used();
        //zaktualizujmy huda
        UpdateInfoMsgHUD();
        //no i zaplanujmy sobie na przyszlosc update
        ScheduleSwitchInfoMsg();
    }
    
    /**
     * Aktualizacja huda
     */
    void UpdateInfoMsgHUD()
    {
        String sText  = "";
        Vector vColor = NetMsg._COLOR_DEFAULT;
        String sWave  = null;
        
        if ( IsActive() && GameStatusMsg.IsValid( m_cInfoMsg ) )
        {
             sText  = m_cInfoMsg.GetText();
             vColor = m_cInfoMsg.GetColor();
             sWave  = m_cInfoMsg.GetSound();
        }
                
        if ( m_cGameInfo            != null &&
             m_cGameInfo.GetHUDNet()!= null )
        {
            m_cGameInfo.GetHUDNet().SetTextGameInfo( sText, vColor );
        }    
        
        if ( sWave != null )
            PlaySound( sWave );
    }
    
    /**
     * Odnalezienie nastepnego messagea do wyswietlenia
     */
    private GameInfoMsg GetNextInfoMsg()
    {
        GameInfoMsg cNext = null;

        //jezeli jest aktywny jakis komunikat
        if ( GameStatusMsg.IsValid( m_cInfoMsg ) )
        {
            //sprawdzmy czy jest jeszcze jakis z tym samym priorytetem
            int nCurrentPriority = m_cInfoMsg.GetPriority();
            //oczywiscie szukamy nie od 0 tylko od aktualnego
            for (int i=m_aInfoMsg.indexOf(m_cInfoMsg)+1; i < m_aInfoMsg.size(); i++ )
            {
                GameInfoMsg cMsg = (GameInfoMsg)m_aInfoMsg.get(i);
                if ( GameStatusMsg.IsValid( cMsg ) && ( cMsg.GetPriority() == nCurrentPriority ))
                {
                    cNext = cMsg;
                    break;
                }
            }
            if ( cNext != null )
                return cNext;
            //jesli nie znalezlismy z tym samy priorytetem
            //to szukajmy jakis z mniejszym
            for (int i=0; i < m_aInfoMsg.size(); i++ )
            {
                GameInfoMsg cMsg = (GameInfoMsg)m_aInfoMsg.get(i);
                if ( GameStatusMsg.IsValid( cMsg ) && cMsg.GetPriority() < nCurrentPriority )
                {
                    if ( cNext == null || 
                         cNext.GetPriority() < cMsg.GetPriority() )
                        cNext = cMsg;
                }
            }
        }
        
        //jesli nadal nie znalazl to szukamy jakiegos z najwyzszym
        //priorytetem
        if ( cNext == null )
        {
            for (int i=0; i < m_aInfoMsg.size(); i++ )
            {
                GameInfoMsg cMsg = (GameInfoMsg)m_aInfoMsg.get(i);
                if ( GameStatusMsg.IsValid( cMsg ) )
                {
                    if ( cNext == null || 
                         cNext.GetPriority() < cMsg.GetPriority() )
                        cNext = cMsg;
                }
            }
        }
        return cNext;
    }

    
    
    void ForceUpdateScore()
    {
        //wylaczamy updateowanie czasu
        DisableCallMethod("UpdateScore");
        
        if ( IsActive() )
        {
            //jestesmy aktywni
            //dokonujemy updatu poszczegolnych komunikatow
            m_cScoreMsg.Update();
        }
        else
        {
            //jestesmy nieaktywni, trzeba schowac to co bylo wczesniej
            m_cScoreMsg.Clean();
            //zaktualizujmy huda
            UpdateScoreHUD();
        }        
    }
    
    /**
     * Callback przychodzacy z komunikatu po jego zmianie
     */
    void OnScoreMsgChange(GameScoreMsg cMsg)
    {
        if ( m_cScoreMsg != cMsg )
            return;
        //zaktualizujmy huda
        UpdateScoreHUD();
    }
    
    void UpdateScoreHUD()
    {
        String sText  = "";
        Vector vColor = NetMsg._COLOR_DEFAULT;
               
        if ( IsActive() && GameStatusMsg.IsValid( m_cScoreMsg ) )
            sText = m_cScoreMsg.GetText();;
        
        if ( m_cGameInfo            != null &&
             m_cGameInfo.GetHUDNet()!= null )
        {
            m_cGameInfo.GetHUDNet().SetTextGameScore( sText, vColor );
        }    
    }
    
    void StatusChanged()
    {
        if ( m_cGameInfo != null && m_cGameInfo.Role == ERole._AUTHORITY )
            m_cGameInfo.MsgSendClientGameStatusMsgUpdated( m_cData.ToDataContainer() );
    }
    
    /**
     * Callback
     */
    void StatusUpdated( NetDataContainer cContainer )
    {
        if ( m_cData != null )
            m_cData.FromDataContainer( cContainer );
        StatusChanged();
    }
    
    int GetPlayerTeamId()
    {
        if ( m_cGameInfo != null &&
             m_cGameInfo.GetPlayerInfoList() != null &&
             m_cGameInfo.GetPlayerInfoList().GetMyPlayer() != null )
        {
            return m_cGameInfo.GetPlayerInfoList().GetMyPlayer().GetTeamId();
        }
        return NetTeamInfo.NONE;
    }
}
