public class DominationBase extends TriggerObject implements INetGameObject
{
    /** czy baza ma byc aktywna na levelu, nawet jak nie gramy w SMD ani TD*/
    public boolean m_bAlwaysActive = false; //#Net
    
    /** GameInfo aktualnej gry */
    NetGameInfo m_cGameInfo = null;
    void SetGameInfo(NetGameInfo cGameInfo)  {  m_cGameInfo = cGameInfo;  }
    
    /** gracz kontrolujacy baze (SMD,TD)*/
    private NetPlayerInfo m_cPlayerOwner    = null;
    public  void          SetPlayerOwner(NetPlayerInfo cOwner){ m_cPlayerOwner = cOwner; }
    public  NetPlayerInfo GetPlayerOwner(){ return  m_cPlayerOwner; }
    
    /** team kontrolujacy baze (TD)*/
    private NetTeamInfo m_cTeamOwner    = null;
    public  void        SetTeamOwner(NetTeamInfo cOwner){ m_cTeamOwner = cOwner; }
    public  NetTeamInfo GetTeamOwner(){ return  m_cTeamOwner; }
    
    /** czas po ktorym wlasciciel bazy powinien dostac punkt*/
    private float m_fTimeToNextScore = -1.0f;
    public void   SetTimeToNextScore( float fTime ){ m_fTimeToNextScore = fTime; }
    public float  GetTimeToNextScore(){ return m_fTimeToNextScore; }
    public void   AddTimeToNextScore( float fTime ){ m_fTimeToNextScore += fTime; }
    
    /** Indeks w tablicy m_aDominationBases w NetGameInfo, sluzy do identyfikacji*/
    private int   m_nIndex = -1;
    private void  SetIndex(int nIndex){ m_nIndex = nIndex; }
    public  int   GetIndex(){ return m_nIndex; } 

    /**
     * Przelaczane materialy
     */
    /** Nazwa materialu ktory bedziemy przeladowywac */
    String m_sOriginalMaterialName = null;
    /** Identyfikator materialu ktory bedziemy przeladowywac */
    int m_nOriginalMaterialId = 0;
    /** Indeksy poszczegolnych materialow do przeladowywania*/
    final static int _MATERIAL_DEFAULT  = 0;
    final static int _MATERIAL_RED      = 1;
    final static int _MATERIAL_BLUE     = 2;
    final static int _MATERIAL_GREEN    = 3;
    final static int _MATERIAL_RED_ENEMY= 4;
    final static int _MATERIALS_CNT     = 5;
    /** Tablica z nazwami uzywanych materialow (wg powyzszych indeksow) */
    String[] m_aUsingMaterials          = new String[_MATERIALS_CNT];
    
    boolean m_bActiveOnLevel = false;
            
    public void OnCreate()
    {
        super.OnCreate();
        
        SetFlag(EFlags._PORTAL_INDEPENDENT);
        
        //pobieramy identyfikator materialu ktory bedziemy podmieniac
        if ( m_sOriginalMaterialName != null )
             m_nOriginalMaterialId = MaterialGetID( m_sOriginalMaterialName );
        
        if ( m_aUsingMaterials == null || m_aUsingMaterials.length < _MATERIALS_CNT )
            CrashLog(" Zle zdefiniowana tablica materialow dla obiektu "+this+"\n");
        
        for ( int i=0; i<m_aUsingMaterials.length; i++ )
            if ( m_aUsingMaterials[i] == null )
                CrashLog(" Nie zdefiniowany material "+i+" dla obiektu "+this+"\n");
                       
        //i domyslnie przeladowyjemy material na neutralny
        ReplaceOriginalMaterial( m_nOriginalMaterialId, 
                                 m_aUsingMaterials[ _MATERIAL_DEFAULT ] );
    }
    
    public void OnPostCreate()
    {
        CheckActive();
    }
        
    void Init(NetGameInfo cGameInfo,int nIndex)
    {
        SetGameInfo( cGameInfo );
        SetIndex( nIndex );
        CheckActive();
        UpdateState();
    }
    
    boolean CheckActive()
    {
        boolean bActive = m_bAlwaysActive || m_cGameInfo != null;
        if ( bActive )
            ActivateOnModule();
        else
            DeactivateOnModuleAlways();
        return bActive;
    }
    
    public void ServerDeleteIfNotActive()
    {
        if ( ! CheckActive() )
            CallMethodIn("delete",0.0f);
    }
    
    /** poszczegolne mozliwe stany bazy */
    final static int _STATE_UNKNOWN = -1;     
    final static int _STATE_FREE    = 0;
    final static int _STATE_RED     = 1;
    final static int _STATE_BLUE    = 2;
    final static int _STATE_OWNER   = 3;
    final static int _STATE_ENEMY   = 4;
    
    /**lokalny stan bazy, na kazdej z maszyn bedzie to inna wartosc*/
    private int m_nLocalState       = _STATE_UNKNOWN;
    int  GetLocalState() { return m_nLocalState; }
    void SetLocalState(int nState)   
    {   
        m_nLocalState = nState;
        
        int nNewMaterialIndex = _MATERIAL_DEFAULT;
        switch ( m_nLocalState )
        {
        case _STATE_RED:
            nNewMaterialIndex = _MATERIAL_RED;
            break;
        case _STATE_BLUE:
            nNewMaterialIndex = _MATERIAL_BLUE;
            break;
        case _STATE_OWNER:
            nNewMaterialIndex = _MATERIAL_GREEN;
            break;
        case _STATE_ENEMY:
            nNewMaterialIndex = _MATERIAL_RED_ENEMY;
            break;
        }
                
        ReplaceOriginalMaterial( m_nOriginalMaterialId, 
                                 m_aUsingMaterials[ nNewMaterialIndex ] );
        DeleteAttachedFXes();
    }
 
    
    void UpdateState()
    {
        if ( m_cGameInfo == null )
            return;
        
        if ( Role < ERole._AUTHORITY )
            return;
        
        for ( int i=0; i<m_cGameInfo.GetPlayerInfoList().GetSize(); i++ )
        {
            NetPlayerInfo cPlayerInfo = m_cGameInfo.GetPlayerInfoList().GetPlayerByIndex( i );
            
            if ( cPlayerInfo == null )
                continue;
            
            int nState = m_cGameInfo.GetDominationBaseLocalState( this, cPlayerInfo );
            
            if ( cPlayerInfo.bNetIsOwner )
            {
                if ( GetLocalState() != nState )
                    SetLocalState( nState );    
            }
            else
            {
                int nHostId = cPlayerInfo.GetClientOwnerId();
                
                MsgSendClientLocalState( nHostId, nState );
            }
        }
        
    }
    
    void MsgSendClientLocalState()
    {
        if ( m_cGameInfo == null )
            return;

        int           nHostId     = GetCurrentClientHostId();
        NetPlayerInfo cPlayerInfo = m_cGameInfo.GetPlayerInfoList().GetPlayerByHostId( nHostId );
        int           nState      = m_cGameInfo.GetDominationBaseLocalState( 
                                                this, cPlayerInfo );
        MsgSendClientLocalState( nHostId, nState );
    }
    
    public void OnReplicateServer()
    {
        super.OnReplicateServer();

        if ( bNetInit && m_cGameInfo != null )
        {
            MsgSendClientInit();
            MsgSendClientLocalState();
        }
    }
    
    static final int MSG_CLIENT_INIT         = 50; // netgameinfo, base pos, team id
    static final int MSG_CLIENT_LOCAL_STATE  = 51; 
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case MSG_CLIENT_INIT:
            LogD("MSG_CLIENT_INIT\n");
            MsgRecvClientInit();
            break;
            
        case MSG_CLIENT_LOCAL_STATE:
            LogD("MSG_CLIENT_LOCAL_STATE\n");
            MsgRecvClientLocalState();
            break;
            
        default:
            super.OnMessage();
        }
    }
    
    void MsgSendClientInit()
    {
        MessageNew( MSG_CLIENT_INIT );
        MessagePackInt( m_cGameInfo.GetNetId() );
        MessageSend();
    }
    
    void MsgRecvClientInit()
    {
        m_nGameInfoId   = MessageUnPackInt();
        MessageEnd();
        
        TryFindGameInfo();
    }
    
    
    void MsgSendClientLocalState( int nHostId, int nState )
    {
        MessageNew( MSG_CLIENT_LOCAL_STATE );
        MessagePackInt( nState );
        MessageSendHost( nHostId );
    }
    
    void MsgRecvClientLocalState()
    {
        int nState = MessageUnPackInt();
        MessageEnd();
    
        if ( GetLocalState() != nState )
            SetLocalState( nState );
    }
    
   
    int m_nGameInfoId = -1;
    
    void TryFindGameInfo()
    {
        GameObject     cObj = GetObjectByNetId( m_nGameInfoId );
        if ( cObj == null )
        {
            CallMethodIn("TryFindGameInfo",0.1f);
            return;
        }
        if ( cObj instanceof NetGameInfo )
        {
            m_cGameInfo = (NetGameInfo)cObj;
            
            LogD(" "+this+" "+m_cGameInfo+"\n");
            //dodajemy sie tam na liscie
            
            if ( m_cGameInfo instanceof NetGameInfoSMD )
                ((NetGameInfoSMD)m_cGameInfo).InitDominationBase( this );
            
            if ( m_cGameInfo instanceof NetGameInfoTD )
                ((NetGameInfoTD)m_cGameInfo).InitDominationBase( this );
        }
        else
        {
            CrashLog("ERROR: "+cObj+" nie jest z klasy NetGameInfo!!!!\n");
        }
    }
    
    boolean m_bIsActiveLocal = true;
    
    void ActiveOffForAWhile()
    {
        m_bIsActiveLocal = false;
        DisableCallMethod("ActiveOn");
        CallMethodIn("ActiveOn",2.0f);
    }
    
    void ActiveOn()
    {
        m_bIsActiveLocal = true;
    }
    
    public boolean IsActive(ControlObject cActivator)
    {
        return ( (m_cGameInfo != null) && m_bIsActiveLocal && super.IsActive( cActivator ) );
    }
    
    void Execute()
    {
    }
    
    /**
     * System wlaczania/wylaczania flag kolizji obiektu
     * Uzywany do tymczasowego wlaczania kolizji na
     * czas traceowania active trigerra
     * Istotne gdy obiekt jest typu hologramowego
     */
    /**
     * Zapamietanie stanu flag obiektu
     * a nastepnie wlaczenie flag kolizji
     */
    public void StoreFlagsAndEnableGetActiveTrigger()
    {
        StoreFlags();
        SetFlag( EFlags._BLOCK_TRACE );
        SetFlag( EFlags._COLLISION );
    }
    
    /** zapamietany ostatnio stanu flag obiektu */
    private int  m_nStoredFlags = 0;
    /** Zapamietanie stanu flag obiektu */
    private void StoreFlags()  { m_nStoredFlags = GetFlags(); }
    /** Przywrocenie zapamietanego stanu flag obiektu */
    public  void RestoreFlags(){ SetFlags( m_nStoredFlags );  }
}

