public class VehiclePod extends MeshObject
{

    
    
    /**
     * Pojazd ktory stoi na podzie.
     */
    Vehicle m_cVehicle = null;
    
    /**
     * Pozycja poda
     */
    Vector m_vPos    = new Vector();
    /**
     * Promien poda (jesli okragly)
     */
    float  m_fRadius = 0.0f;
    /**
     * Ekstenty poda (jesli prostakatny
     */
    Vector m_vMinExt = new Vector();
    Vector m_vMaxExt = new Vector();
    
    /**
     * Flaga mowiaca czy pod jest okrogly czy prostokatny
     * W zaleznosci o jej wartosci wybierany jest
     * sposob wyszukiwania gracza na podzie podczas spawnowania
     */
    public boolean m_bRound = true;//#Net
    
    /**
     * Czy szukajac gracza w okolicy bedziemy ignorowali jego 
     * pozycje w Y, potrzebne w przypadku, gdy bedziemy korzystali
     * z wirtualnych podow (schowanych pod ziemia)
     */
    public boolean m_bIgnorePosY = true;
    
    int METHOD_RespawnDo = 0;

    void OnCreate()
    {
        METHOD_RespawnDo = RegisterCallMethod( "RespawnDo" );
        
        GetPosition( m_vPos );
        if ( m_bIgnorePosY )
            m_vPos.fY = 0.0f;
        GetMeshExtents( m_vMinExt, m_vMaxExt );
        m_vMinExt.Add( m_vPos );
        m_vMaxExt.Add( m_vPos );
        m_fRadius = Math.abs( m_vMaxExt.fX - m_vMinExt.fX )/2.0f;
    }
    
    void OnDestroy()
    {
        super.OnDestroy();
        m_cVehicle = null;
    }
        
    
    void Respawn(boolean bOnRestartLevel, Vehicle cVehicle )
    {
        m_cVehicle = cVehicle;
        
        DisableCallMethod( METHOD_RespawnDo );
        
        if ( bOnRestartLevel )
            LightOff();
        else
        {
            //zapalamy ostrzegawczo poda
            LightOn();
            //i zaplanujmy zgaszenie na pozniej
            CallMethodIn( METHOD_RespawnDo, Vehicle.GetSpawnDelayTime() );
        }
    }
    
    void RespawnDo()
    {
        // przywrocic prawdziwy material
        LightOff();
        
        //na serwerze zabic graczy stojacych na podzie
        //pobierzmy PlayerManager'a
        PlayerNetManager cPlayerManager = GetModule() instanceof ModuleNet ?
                                          ((ModuleNet)GetModule()).GetPlayerManager():
                                          null;
        //jesli cPlayerManager == null to znaczy ze jestesmy na kliencie
        if ( cPlayerManager == null )
            return;
        for ( int i=0; i < cPlayerManager.GetMaxPlayers(); i++ )
        {
            PlayerNet cPlayer = cPlayerManager.GetPlayer( i );
            
            if ( cPlayer == null || cPlayer.bDead )
                continue;
            
            //zabijamy dziada
            if ( IsObjectArround( cPlayer ) )
                cPlayer.ForceDieByVehiclePod();
        }
        
        GameObject []aVehicles = GetObjectsFromClass( "Vehicle" );
        if ( aVehicles != null )
        {
            //jeden po drugim wskakuja  na liste
            for ( int i = 0 ; i < aVehicles.length; i++ )
            {
                Vehicle cVehicle = (Vehicle)aVehicles[i];
                
                //jezele pojazd kolo pod to go rozwal
                if ( IsObjectArround( cVehicle ) && 
                     (cVehicle != m_cVehicle)    && 
                     (cVehicle.bDead == false)
                   )
                {
                    cVehicle.Die();
                }
                
                
                if ( cVehicle.m_Wreck != null &&
                     cVehicle.m_bWreckVisible )
                {
                    //jezeli ma wreka i jest kolo poda to go zniknij
                    if ( IsObjectArround( cVehicle.m_Wreck ) )
                        cVehicle.ServerHideWreck();
                }
                
            }
        }
    }
    
    void LightOn()
    {
        SetSkin("Pulsowanie");
    }
    void LightOff()
    {
        SetSkin("Default");
    }
    
    
    Vector vObjPos = new Vector();
    
    boolean IsObjectArround( ControlObject cObj )
    {
        cObj.GetPosition( vObjPos );
        if ( m_bIgnorePosY )
            vObjPos.fY = 0.0f;
        if ( m_bRound )
            return vObjPos.Distance( m_vPos ) < m_fRadius;
        return ((vObjPos.fX > m_vMinExt.fX) && 
                (vObjPos.fX < m_vMaxExt.fX) &&
                (vObjPos.fZ > m_vMinExt.fZ) && 
                (vObjPos.fZ < m_vMaxExt.fZ) );
    }
}
