public class PlayerNetCTF extends PlayerNet
{
    NetGameInfoCTF m_cGameInfoCTF = null;

    /**
     * FX jaki bedziemy odpalali podczas niesienia flagi
     */
    String m_sFlagFX = null;
    /**
     * Obiekt do pozycjonowania tego FX'a
     */
    ControlObject m_cFlagFXPositioner = null;
    
    int METHOD_FlagFX = 0 ;
    
    public void OnCreate()
    {
        METHOD_FlagFX = RegisterCallMethod("FlagFX");

        super.OnCreate();
        
        m_cFlagFXPositioner = (ControlObject)CreateObject("ControlObject");
        m_cFlagFXPositioner.ActivateOnModule();
        AttachChild( m_cFlagFXPositioner );
        UpdateFlagFXPositioner();
    }
    
    public void OnDestroy()
    {
        if ( m_cFlagFXPositioner != null )
        {
            m_cFlagFXPositioner.DetachFromParent();
            m_cFlagFXPositioner.delete();
            m_cFlagFXPositioner = null;
        }
        super.OnDestroy();
    }
    
    void SetNetGameInfo( NetGameInfo cGameInfo )
    {
        super.SetNetGameInfo( cGameInfo );
        
        if ( cGameInfo instanceof NetGameInfoCTF )
            m_cGameInfoCTF = (NetGameInfoCTF)cGameInfo;
        else
            CrashLog(" Niepoprawny obiekt NetGameInfo "+cGameInfo+
                     " dla PlayerNetCTF "+this+"\n");
    }
    
    /**
     * Nadpisuje ta metode, aby ustawic (a pozniej przywrocic)
     * flagomCTF flage _BLOCK_TRACE _COLLISION
     */
    TriggerObject GetActiveTrigger(Vector vStart, Vector vDir, float fDistance)
    {
        //wlaczamy flagom CTF kolizje z traceami
        if ( m_cGameInfoCTF != null )
            m_cGameInfoCTF.OnGetActiveTrigger();
        
        TriggerObject cRet = super.GetActiveTrigger( vStart, vDir, fDistance );
        
        //wylaczamy flagom CTF kolizje z traceami
        if ( m_cGameInfoCTF != null )
            m_cGameInfoCTF.OnPostGetActiveTrigger();
                           
        return cRet;
    }
       
    public void SetActiveTrigger(TriggerObject trigger)
    {
        if ( trigger instanceof CTFFlag )
        {
            CTFFlag cFlag = (CTFFlag)trigger;
            
            if ( Role == ERole._AUTHORITY )
            {
                //serwer sam bedzie wiedzial co z nim zrobic
                m_cGameInfoCTF.EventPlayerReachedFlag( cFlag, this );
            }
            else
            {
                // klient poprosi serwera o wsparcie
                MsgSendServerFlagReached( cFlag );
                // i sprawmy zeby przez chwilke nie byl obiekt aktywny
                cFlag.ActiveOffForAWhile();
            }
        }
        else
            super.SetActiveTrigger( trigger );
    }
    
    static final int MSG_SERVER_FLAG_REACHED   = 50; // klient wpadl na flage:CTF
    
    /** Odebranie komunikatow przychodzacych z sieci
     */
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case MSG_SERVER_FLAG_REACHED:
            MsgRecvServerFlagReached();
            break;
        
        default:
            super.OnMessage();
        }
    }
    void MsgSendServerFlagReached( CTFFlag cFlag )
    {
        if ( Role == ERole._AUTHORITY || cFlag == null )
            return;
        
        MessageNew( MSG_SERVER_FLAG_REACHED );
        MessagePackInt( cFlag.GetNetId() );
        MessageSend();
    }
    
    void MsgRecvServerFlagReached()
    {
        int nFlagId = MessageUnPackInt();
        MessageEnd();

        //TODO: zrobic weryfikacje czy flaga nie jest juz 
        //czasem w czyims inwentory, oraz czy gracz jest 
        //faktycznie w poblizu flagi
        CTFFlag cFlag = GetCTFFlagByNetId( nFlagId );
        if ( cFlag != null && 
             ServerVerifyActionExecute( cFlag ) &&
             m_cGameInfoCTF != null )
        {
            m_cGameInfoCTF.EventPlayerReachedFlag( cFlag, this );
        }
    }
    
    CTFFlag GetCTFFlagByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        try 
        {
            CTFFlag flag = (CTFFlag) object;
            return flag;    
        } 
        catch( ClassCastException e) 
        {
            LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from CTFFlag\n");
        }
        
        return null;
    }
    
    public void CleanPlayer()
    {
        super.CleanPlayer();
        StopFlagFX();
    }

    
    void StartFlagFX(String sFX)
    {
        m_sFlagFX = sFX;
        CallMethodEvery( METHOD_FlagFX , (float)(FXFlagCTF._DURATION/1000.0) );
    }
    
    void StopFlagFX()
    {
        DisableCallMethod( METHOD_FlagFX );
    }
        
    Vector m_vFlagPos = new Vector(0,0,0);
    
    //TODO: tylko do testow
    float m_fFlagFXPosY = 0.0f;
    
    void FlagFX()
    {
        m_vFlagPos.fY = m_fFlagFXPosY;
        ExecuteFX( m_cFlagFXPositioner, 
                   m_sFlagFX , 
                   m_vFlagPos );
    }
    
    void CameraModeChanged(byte eMode)
    {
        super.CameraModeChanged( eMode );
        UpdateFlagFXPositioner();
    }

    void MotionModeChanged()
    {
        super.MotionModeChanged();
        UpdateFlagFXPositioner();
    }
    
    void UpdateFlagFXPositioner()
    {
        if ( m_cFlagFXPositioner != null )
             m_cFlagFXPositioner.SetPosition( new Vector( 0.0f, 180.0f, 0.0f ) );
    }
}
