/** Klasa obsluguje zarzadzanie graczami na serwerze w grze sieciowej
 */
public class PlayerNetManager
{
    private int         m_nMaxPlayers = 0;
    private PlayerNet[] m_aPlayers    = null;
    
    PlayerNetManager(int nMaxPlayers)
    {
        m_nMaxPlayers = nMaxPlayers;
        m_aPlayers    = new PlayerNet[ m_nMaxPlayers ];
        for ( int i=0; i<nMaxPlayers; i++ )
            m_aPlayers[i] = null;
    }
    
    /** Kasowanie listy graczy
     */
    void Destroy()
    {
        for ( int i=0; i<m_nMaxPlayers; i++ )
        {
            // TODO nie jestem przekonany, ze to tak ma byc !
            if ( m_aPlayers[i] != null )
            {
                m_aPlayers[i].delete();
                m_aPlayers[i] = null;
            }
        }
    }
    
    /** Dodanie nowego gracza, gdy ten sie przylaczy do serwera.
     * @param nHostID identyfikator hosta, ktory chce gracza przylaczyc
     * @param newPlayer gracz, ktorego dolaczamy
     */
    void AddPlayer(PlayerNet cPlayer,int nHostID)
    {
        for ( int i=0; i<m_nMaxPlayers; i++ )
            if ( m_aPlayers[i] == null )
            {
                cPlayer.nOwnerID = nHostID;
                m_aPlayers[i]    = cPlayer;
                break;
            }
    }

    /** Szuka gracza o podanym identyfikatorze hosta.
     * @param nHostID identyfikator hosta
     */
    PlayerNet FindPlayer(int nHostID)
    {
        for ( int i=0; i<m_nMaxPlayers; i++ )
            if ( m_aPlayers[i] != null &&
                 m_aPlayers[i].nOwnerID == nHostID )
            {
                return m_aPlayers[ i ];
            }
        return null;
    }
            
    void DeletePlayer(PlayerNet cPlayer)
    {
        if ( cPlayer == null )
            return;
        
        for ( int i=0; i<m_nMaxPlayers; i++ )
            if ( m_aPlayers[i] == cPlayer )
            {
                m_aPlayers[i].DeleteInventory();
                m_aPlayers[i].delete();
                m_aPlayers[i] = null;
                break;                    
            }
    }
    
    int GetMaxPlayers()
    {
        return m_nMaxPlayers;
    }

    PlayerNet GetPlayer(int i)
    {
        if ( i >= 0 && i < m_nMaxPlayers )
            return m_aPlayers[i];
        return null;
    }
    
    boolean CanAddPlayer()
    {
        return (m_nMaxPlayers > GetPlayersCnt());
    }
    
    /** Zwraca liczbe graczy aktulanie znajdujacych sie na levelu
     * @return liczba graczy znajdujacych sie na levelu
     */
    int GetPlayersCnt()
    {
        int nCount = 0;
        for (int i=0; i<m_nMaxPlayers; i++)
            if ( m_aPlayers[i] != null )
                nCount++;
        return nCount;
    }    
    
    void DisablePlayerCollision()
    {
        for (int i=0; i<m_nMaxPlayers; i++)
            if ( m_aPlayers[i] != null )
            {
                m_aPlayers[i].ClearFlag( EFlags._COLLISION );
                m_aPlayers[i].GetDeadBody().ClearFlag( EFlags._COLLISION );
            }
    }
    
    void EnablePlayerCollision()
    {
        for (int i=0; i<m_nMaxPlayers; i++)
            if ( m_aPlayers[i] != null )
            {
                m_aPlayers[i].SetFlag( EFlags._COLLISION );
                m_aPlayers[i].GetDeadBody().SetFlag( EFlags._COLLISION );
            }
    }
}
