/**
 * Kompletny opis stanu gracza
 * 
 */
public class PlayerNetState
{
    //identyfikator
    int            nId     = 0;

    
    // flagi mowiace czy dane zmienne nas interesuja
    static boolean bEllipsoid   = true;
    static boolean bGravity     = true;
    static boolean bEnvironment = true;
    static boolean bCollFlags   = false;
    static boolean bDestDir     = false;
    static boolean bDestPoint   = false;
    static boolean bSpeedDir    = false;
    
//    static boolean bODE         = true;
    
    // czas
    float fTimeStamp = -1;
    
    // liczba updatow ODE od ostatniego UpdatePhysics
    int nODEDoneUpdatesCount = 4;
    
    // rzeczy zwiazane z ruchem
    MotionState cMotion;
    MotionState cOldMotion;
    
    boolean bAmIAiming = false;
    boolean bWalking = false;
    boolean bDucking = false;
    boolean bCrawling = false;
    boolean bRecoiling;
    byte    nAdjustClimbing;
    byte    nHitReaction = 0;
    float   fSpeedMult = 1.f;
    boolean bCheckIsJumpStarted;
    boolean bDelayedCheckIsJumpStarted;
    
    // suchy stan gracza 
    Vector vPosition = new Vector();
    Vector vDir = new Vector();
    ControlObject cParent;
    int           nParentId;
    // aktywnosc portali - wykorzystywane na serwerze i kliencie w troche inny sposob
    boolean       bPortalSystemActive;
        
    // fizyka gracza
//    Vector  vSpeed;
    Vector  vLocalSpeed = new Vector();
    Vector  vDestSpeed = new Vector();
    Vector  vJavaSpeed = new Vector();
    Vector  vFallSpeed = new Vector();
    Vector  vMaxFallSpeed = new Vector();
    int     iSpeed;
    float   fGravity;
    int     iCollFlags;
    int     nEnvironment;
    boolean bJustFlying;
    //fizyka ODE
    Vector  vODEPos;
    Vector  vODELinVel;
    Vector  vODEForce;
    Vector  vODEVelAccu;
    Vector  vODEFDir1;
    boolean bODEBParam1;
    boolean bODEBParam2;
    float   fODEFParam1;
    float   fODEFParam2;
    float   fODEFParam3;
    float   fODEFParam4;
    float   fODEFParam5;
    
    PlayerNetState()
    {
        //
    }
    
    // kontruktory
    PlayerNetState(PlayerNetBaseMover player,float fTime)
    {
        Store(player,fTime);
    }
    
    PlayerNetState(PlayerNetBaseMover player)
    {
        Store( player );
    }

    
    //destruktor
    void Destroy()
    {
        fTimeStamp      = -1.f;
        cMotion         = null;
        cOldMotion      = null;
        vPosition       = null;
        vDir            = null;
        cParent         = null;
//        vSpeed          = null;
        vDestSpeed      = null;
        vJavaSpeed      = null;
        vFallSpeed      = null;
        vMaxFallSpeed   = null;
        vODEPos         = null;
        vODELinVel      = null;
        vODEForce       = null;
        vODEVelAccu     = null;
        vODEFDir1       = null;
    }

    
    
    ///////////////////////////////////////////////////////////////
    // SKLADOWANIE DANYCH
    //////////////////////////////////////////////////////////////
    
    ///////////////////////
    // skladowanie calosci
    
    public void Store(PlayerNetBaseMover player,float fTime)
    {
        fTimeStamp = player.RoundToPhysicsGranularity(fTime);
        if ( player.nGravityPhysics == EPhysics._ODE_GRAVITYWALK )
            nODEDoneUpdatesCount = player.GetODEGWDoneUpdates();

        Store( player );
    }
    
    public void Store(PlayerNetBaseMover player)
    {
        StoreMotion( player );
        StorePosAndDir( player );
        StorePhysics( player );
    }
    
    ////////////////////////////
    // skladowanie motion state'a
    
    public void StoreMotion(PlayerNetBaseMover player,float fTime)
    {
        fTimeStamp = player.RoundToPhysicsGranularity(fTime);
        if ( player.nGravityPhysics == EPhysics._ODE_GRAVITYWALK )
            nODEDoneUpdatesCount = player.GetODEGWDoneUpdates();

        StoreMotion( player );
    }
    public void StoreMotion(PlayerNetBaseMover player)
    {
        cMotion     = new MotionState( player.motion );
        cOldMotion  = new MotionState( player.oldMotion );
        
        bWalking    = player.bWalking;
        bDucking    = player.bDucking;
        bCrawling   = player.bCrawling;
        bRecoiling  = player.bRecoiling;
        nHitReaction= player.nHitReaction;
        fSpeedMult  = player.fSpeedMult;
        nAdjustClimbing = player.motion.nClimbing;
    }
    
    ///////////////////////////
    // skladowanie stanu (pozycji i kierunku)
    
    public void StorePosAndDir(PlayerNetBaseMover player,float fTime)
    {
        fTimeStamp = player.RoundToPhysicsGranularity(fTime);
        if ( player.nGravityPhysics == EPhysics._ODE_GRAVITYWALK )
            nODEDoneUpdatesCount = player.GetODEGWDoneUpdates();

        StorePosAndDir( player );
    }

    public void StorePosAndDir(PlayerNetBaseMover player)
    {
        StorePos( player );
        StoreDir( player );
    }
    

    // skladowanie pozycji
    
    public void StorePos(PlayerNetBaseMover player, float fTime)
    {
        fTimeStamp = player.RoundToPhysicsGranularity(fTime);
        if ( player.nGravityPhysics == EPhysics._ODE_GRAVITYWALK )
            nODEDoneUpdatesCount = player.GetODEGWDoneUpdates();

        StorePos( player );
    }
        
    public void StorePos(PlayerNetBaseMover player)
    {
        vPosition  = player.GetLocalPosition();    
        cParent    = player.GetParent();
        nParentId  = (cParent != null) ? cParent.GetNetId() : -1;
    }
    
    public void StorePortalSystemActiveFlag(PlayerNetBaseMover player)
    {
        bPortalSystemActive = player.GetPortalSystemActiveFlag();
    }
        
    // skladowanie kierunku
    
    public void StoreDir(PlayerNetBaseMover player, float fTime)
    {
        fTimeStamp = player.RoundToPhysicsGranularity(fTime);
        if ( player.nGravityPhysics == EPhysics._ODE_GRAVITYWALK )
            nODEDoneUpdatesCount = player.GetODEGWDoneUpdates();

        StoreDir( player );
    }
    
    public void StoreDir(PlayerNetBaseMover player)
    {
        vDir       = player.GetAimDirection();
    }
    
    
    /////////////////////////////
    // skladowanie fizyki
    
    public void StorePhysics(PlayerNetBaseMover player,float fTime)
    {
        fTimeStamp = player.RoundToPhysicsGranularity(fTime);
        if ( player.nGravityPhysics == EPhysics._ODE_GRAVITYWALK )
            nODEDoneUpdatesCount = player.GetODEGWDoneUpdates();

        StorePhysics( player );
    }
       
    public void StorePhysics(PlayerNetBaseMover player)
    {
        vLocalSpeed       = player.GetLocalSpeed();
        vDestSpeed   = player.GetWalkDestSpeed();
        vJavaSpeed   = player.vSpeed;
        
        if ( player.nGravityPhysics == EPhysics._ODE_GRAVITYWALK )
        {
            vFallSpeed  = player.GetJumpSpeed();
            vODEPos     = player.GetODEGWPos();
            vODELinVel  = player.GetODEGWLinVel();
            vODEForce   = player.GetODEGWForce();
            vODEVelAccu = player.GetODEGWVelocityAccumulator();
            vODEFDir1   = player.GetODEGWFDir1();
            bODEBParam1 = player.GetODEGWStateBParam1();
            bODEBParam2 = player.GetODEGWStateBParam2();
            fODEFParam1 = player.GetODEGWStateFParam1();
            fODEFParam2 = player.GetODEGWStateFParam2();
            fODEFParam3 = player.GetODEGWStateFParam3();
            fODEFParam4 = player.GetODEGWStateFParam4();
            fODEFParam5 = player.GetODEGWStateFParam5();
        }
        else
            vFallSpeed  = player.GetFallSpeed();

        if (bEnvironment)
        {
            nEnvironment = player.nEnvironment;
            bJustFlying  = player.AmIFlying();
            bCheckIsJumpStarted = player.m_bCheckIsJumpStarted;
            bDelayedCheckIsJumpStarted = player.m_bDelayedCheckIsJumpStarted;
        }
        if (bEllipsoid)
        {
            iSpeed      = player.m_iSpeed;
        }
        if (bGravity)
        {
            fGravity = player.GetMass();
        }
        if (bCollFlags)
        {
            iCollFlags= player.GetCollFlags();
        }
        if (bDestDir)
        {
        }
        if (bDestPoint)
        {
        }
        if (bSpeedDir)
        {
        }
    }
    
    ///////////////////////////////////////////////////////////////
    // PRZYWRACANIE DANYCH
    //////////////////////////////////////////////////////////////
    

    ///////////////////////
    // przywrocenie calego stanu
    
    public void Restore(PlayerNetBaseMover player)
    {
        RestorePhysicsAndPosAndDir( player );
        RestoreMotion( player );
    }
     
    
    ///////////////////////////
    // przywrocenie motion state'a
    
    public void RestoreMotion(PlayerNetBaseMover player)
    {
        player.motion    = new MotionState( cMotion );
        player.oldMotion = new MotionState( cOldMotion );
        
        player.bWalking     = bWalking;
        player.bDucking     = bDucking;
        player.bCrawling    = bCrawling;
        player.bRecoiling   = bRecoiling;
        player.motion.nClimbing = nAdjustClimbing;
        
        player.nHitReaction = nHitReaction;
        player.fSpeedMult   = fSpeedMult;
    }
    
    ///////////////////////////
    // zastosowanie nowego motion state'a (bez niepotrzebnych flag)
    
    public void ApplyMotionAndDir(PlayerNetBaseMover player)
    {
        MotionState cLastMotion      = player.motion;
        MotionState cLastOldMotion   = player.oldMotion;
        
        player.motion    = new MotionState( cMotion );
        player.oldMotion = new MotionState( cOldMotion );
    
        player.motion.Recover( cLastMotion );
        player.oldMotion.Recover( cLastOldMotion );

        player.bNetClientAiming = bAmIAiming;
        player.bWalking         = bWalking;
        player.bDucking         = bDucking;
        player.bCrawling        = bCrawling;
        player.bRecoiling       = bRecoiling;
        player.nHitReaction     = nHitReaction;
        player.fSpeedMult       = fSpeedMult;
        
        RestoreDir( player );
    }
    
    public void ApplyMotionAndDirAndLocalSpeed(PlayerNetBaseMover player)
    {
        ApplyMotionAndDir( player );
        player.SetLocalSpeed( vLocalSpeed );
        player.SetWalkDestSpeed( vDestSpeed );
        player.vSpeed = vJavaSpeed;
    }
    
    ////////////////////////////////
    // przywrocenie fizyki
    public void RestorePhysicsAndPosAndDir(PlayerNetBaseMover player)
    {
        //Taka kolejnosc jest wskazana, ze wzgledy na fakt, 
        //ze w fizyce odtwarzana jest takze eilpsoida
        //i to powoduje odsuniecie gracza od ziemi, 

        RestorePhysics( player );
        RestorePosAndDir( player );
        //zatem ustawianie pozycji powinno byc pozniej
        //RestorePosAndDir( player );
    }
    
    public void RestorePhysics(PlayerNetBaseMover player)
    {
//        if ( player.bDebug )
//            GameObject.LogD("vSpeed:"+vSpeed+" - vS:"+player.GetSpeed());
        player.SetLocalSpeed( vLocalSpeed );
        player.SetWalkDestSpeed( vDestSpeed );
        player.vSpeed = vJavaSpeed;
        if ( player.bDebug )
            GameObject.Log(" -> vS:"+player.GetSpeed()+" vLS:"+player.GetLocalSpeed()+"\n");
        
        if ( player.nGravityPhysics == EPhysics._ODE_GRAVITYWALK )
        {
            player.SetFallSpeed( vFallSpeed );
//            player.SetODEGWPos( vODEPos );
            player.SetODEGWForce( vODEForce );
            player.SetODEGWVelocityAccumulator( vODEVelAccu );
            player.SetODEGWFDir1(vODEFDir1);
            player.SetODEGWStateBParam1(bODEBParam1);
            player.SetODEGWStateBParam2(bODEBParam2);
            player.SetODEGWStateFParam1(fODEFParam1);
            player.SetODEGWStateFParam2(fODEFParam2);
            player.SetODEGWStateFParam3(fODEFParam3);
            player.SetODEGWStateFParam4(fODEFParam4);
            player.SetODEGWStateFParam5(fODEFParam5);
            player.SetODEGWLinVel(vODELinVel);
            player.SetODEGWDoneUpdates(nODEDoneUpdatesCount);
        }
        else
        {
            player.SetFallSpeed( vFallSpeed );
        }

        if (bEnvironment)
        {
            //player.nEnvironment = nEnvironment + 1;
            player.SetPhysicsEnv( nEnvironment );
            
            //@@
            //NetLog.PlayerMover("Ustawiam Env: " + nEnvironment + "\n");
            
            player.SetIAmFlying( bJustFlying );
            player.m_bCheckIsJumpStarted = bCheckIsJumpStarted;
            player.m_bDelayedCheckIsJumpStarted = bDelayedCheckIsJumpStarted;
        }
        if (bEllipsoid)
        {
            if (player.m_iSpeed != iSpeed)
            {
                Vector vPos = player.GetLocalPosition();
                player.SetEllipsoidAndMaxClimbAngle( iSpeed );
                player.SetPosition(vPos);
            }
        }
        if (bGravity)
        {
            player.SetMass( fGravity );
            //@@
            //player.LogR("Przywracam mass: " + fGravity + "\n");
            //player.motion.bClimbing = (fGravity == 0.0f);
        }
        if (bCollFlags)
        {
            player.ReplaceCollFlags( iCollFlags );           
        }
        if (bDestDir)
        {
            
        }
        if (bDestPoint)
        {
        }
        if (bSpeedDir)
        {
        }
    }
    
    /////////////////////////////
    // przywrocenie stanu (pozycji i kierunku)
    
    public void RestorePosAndDir(PlayerNetBaseMover player)
    {
        //GameObject.Log("RestorePosAndDir vS:"+player.GetSpeed()+" vLS:"+player.GetLocalSpeed()+"->");
        
        RestorePos( player );
        RestoreDir( player );
        
        if ( player.bDebug )
            GameObject.Log(" vS:"+player.GetSpeed()+" vLS:"+player.GetLocalSpeed()+"\n");
    }

    ///////////////////////
    // przywrocenie pozycji
    
    public void RestoreParent(PlayerNetBaseMover player)
    {
        player.SwitchParentTo( cParent );
    }
    public void RestorePos(PlayerNetBaseMover player)
    {
        RestoreParent( player );
        player.SetPosition( vPosition );
    }
    
    public void RestorePortalSystemActiveFlag(PlayerNetBaseMover player)
    {
        player.SetPortalSystemActiveFlag( bPortalSystemActive );
    }
    
    ////////////////////////////
    // przywrocenie kierunku
    
    public void RestoreDir(PlayerNetBaseMover player)
    {
        //GameObject.LogD("\n");
        Vector vOldDir = player.GetAimDirection();
        if ( vOldDir.equals( vDir ) == false )
        {
            player.SetAimDirection( vDir );
            player.BodyDeformAimAtVDo();
        }
    }
    
    ////////////////////////////////
    // przywrocenie motion state'a i kierunku
    
    public void RestoreMotionAndDir(PlayerNetBaseMover player)
    {
        RestoreMotion( player );
        RestoreDir( player );
    }
    
    public void RestoreMotionAndDirAndLocalSpeed(PlayerNetBaseMover player)
    {
        RestoreMotionAndDir( player );
        player.vSpeed = vJavaSpeed;
    }
    
    
    public void MessagePackAdjustState(GameObject cMessager,PlayerNetBaseMover cPlayer)
    {
        cMessager.MessagePackFloat( fTimeStamp );
        cMessager.MessagePackInt( nODEDoneUpdatesCount );
        cMessager.MessagePackVector( vPosition );
        cMessager.MessagePackVector( vLocalSpeed );
        cMessager.MessagePackVector( vDestSpeed );
        cMessager.MessagePackVector( vJavaSpeed );
        cMessager.MessagePackVector( vFallSpeed );
        cMessager.MessagePackVector( vODEPos );
        cMessager.MessagePackVector( vODELinVel );
        cMessager.MessagePackVector( vODEForce );
        cMessager.MessagePackVector( vODEVelAccu );
        cMessager.MessagePackVector( vODEFDir1 );
        cMessager.MessagePackFloat( fGravity );
        cMessager.MessagePackFloat( fODEFParam1 );
        cMessager.MessagePackFloat( fODEFParam2 );
        cMessager.MessagePackFloat( fODEFParam3 );
        cMessager.MessagePackFloat( fODEFParam4 );
        cMessager.MessagePackFloat( fODEFParam5 );
        cMessager.MessagePackInt( nParentId );
        cMessager.MessagePackByte( (byte)nEnvironment );
        cMessager.MessagePackByte( (byte)iSpeed );
        cMessager.MessagePackByte( nAdjustClimbing );
        
//        cMessager.MessagePackVector( vDir );
        
//        GameObject.LogR("PackAdjust nAdjustClimbing="+nAdjustClimbing+" vPos="+vPosition+"\n");
        
        byte nFlags = 0;
        nFlags = cMessager.PackBooleanToByte( nFlags, 0, bJustFlying );
        nFlags = cMessager.PackBooleanToByte( nFlags, 1, bCheckIsJumpStarted );
        nFlags = cMessager.PackBooleanToByte( nFlags, 2, bDelayedCheckIsJumpStarted );
        nFlags = cMessager.PackBooleanToByte( nFlags, 3, bODEBParam1 );
        nFlags = cMessager.PackBooleanToByte( nFlags, 4, bODEBParam2 );
        cMessager.MessagePackByte( nFlags );
                
        cMessager.MessagePackBoolean( cPlayer.GetPortalSystemActiveFlag() );
    }
    
    public void MessageUnPackAdjustState(GameObject cMessager)
    {
        fTimeStamp                  = cMessager.MessageUnPackFloat();
        nODEDoneUpdatesCount        = cMessager.MessageUnPackInt();
        vPosition                   = cMessager.MessageUnPackVector();
        vLocalSpeed                 = cMessager.MessageUnPackVector();
        vDestSpeed                  = cMessager.MessageUnPackVector();
        vJavaSpeed                  = cMessager.MessageUnPackVector();
        vFallSpeed                  = cMessager.MessageUnPackVector();
        vODEPos                     = cMessager.MessageUnPackVector();
        vODELinVel                  = cMessager.MessageUnPackVector();
        vODEForce                   = cMessager.MessageUnPackVector();
        vODEVelAccu                 = cMessager.MessageUnPackVector();
        vODEFDir1                   = cMessager.MessageUnPackVector();
        fGravity                    = cMessager.MessageUnPackFloat();
        fODEFParam1                 = cMessager.MessageUnPackFloat();
        fODEFParam2                 = cMessager.MessageUnPackFloat();
        fODEFParam3                 = cMessager.MessageUnPackFloat();
        fODEFParam4                 = cMessager.MessageUnPackFloat();
        fODEFParam5                 = cMessager.MessageUnPackFloat();
        nParentId                   = cMessager.MessageUnPackInt();
        nEnvironment                = cMessager.MessageUnPackByte();
        iSpeed                      = cMessager.MessageUnPackByte();
        nAdjustClimbing             = cMessager.MessageUnPackByte();
        
//        vDir                        = cMessager.MessageUnPackVector();
        
//        GameObject.LogR("UnPackAdjust nAdjustClimbing="+nAdjustClimbing+" vPos="+vPosition+"\n");
        
        byte nFlags                 = cMessager.MessageUnPackByte();
        bJustFlying                 = cMessager.UnpackBooleanFromByte( nFlags, 0 );
        bCheckIsJumpStarted         = cMessager.UnpackBooleanFromByte( nFlags, 1 );
        bDelayedCheckIsJumpStarted  = cMessager.UnpackBooleanFromByte( nFlags, 2 );
        bODEBParam1                 = cMessager.UnpackBooleanFromByte( nFlags, 3 );
        bODEBParam2                 = cMessager.UnpackBooleanFromByte( nFlags, 4 );
        
        bPortalSystemActive         = cMessager.MessageUnPackBoolean();
        
        //dodatkowo zamieniamy parentid na obiekt parenta
        if ( nParentId != - 1 )
        {
            GameObject cParent = cMessager.GetObjectByNetId( nParentId );
            if ( cParent instanceof ControlObject )
                cParent = (ControlObject) cParent ;
        }
        else
            cParent = null;
    }
    
    public void StoreAdjust( PlayerNetAdjustState cState )
    {
        fTimeStamp           = cState.fTimeStamp;
        nODEDoneUpdatesCount = cState.nODEDoneUpdatesCount;
        
        vPosition            = cState.vPosition;
        nParentId            = cState.nParentId;
    
        vLocalSpeed  = cState.vLocalSpeed;
        vDestSpeed   = cState.vDestSpeed;
        vJavaSpeed   = cState.vJavaSpeed;
        vFallSpeed   = cState.vFallSpeed;
        vODEPos      = cState.vODEPos;
        vODELinVel   = cState.vODELinVel;
        vODEForce    = cState.vODEForce;
        vODEVelAccu  = cState.vODEVelAccu;
        vODEFDir1    = cState.vODEFDir1;
        bODEBParam1  = cState.bODEBParam1;
        bODEBParam2  = cState.bODEBParam2;
        fODEFParam1  = cState.fODEFParam1;
        fODEFParam2  = cState.fODEFParam2;
        fODEFParam3  = cState.fODEFParam3;
        fODEFParam4  = cState.fODEFParam4;
        fODEFParam5  = cState.fODEFParam5;
        nEnvironment = cState.nEnvironment;
        bJustFlying  = cState.bJustFlying;
        bCheckIsJumpStarted = cState.bCheckIsJumpStarted;
        bDelayedCheckIsJumpStarted = cState.bDelayedCheckIsJumpStarted;
        iSpeed      = cState.iSpeed;
        fGravity    = cState.fGravity;
        cMotion.nClimbing = cState.nAdjustClimbing;

        bPortalSystemActive = cState.bPortalSystemActive;
    }
    
    /**
     * Porownanie dwoch stanow z wzgledu na czas, zwraca:
     *  1 jesli jestesmy nowsi od porownywanego stanu
     * -1 jesli starsi
     *  0 jesli czasy stanow sa identyczne
     */
    int CompareTime( PlayerNetState cState )
    {
        //jesli podany do porownania stan to null, to zakladamy
        //ze jestesmy nowsi
        if ( cState == null )
            return 1;
        
        
        if ( fTimeStamp > cState.fTimeStamp )
        {
            //jesli nasz czas jest wiekszy, to jestesmy nowsi
            return 1;
        }
        else if ( fTimeStamp == cState.fTimeStamp ) 
        {
            //jesli czasy sa taki same to porownujemy numerki updateow ODE
                        
            if ( nODEDoneUpdatesCount > cState.nODEDoneUpdatesCount )
            {
                //jesli nasz numerek wiekszy, to jestesmy nowsi
                return 1;
            }
            else if ( nODEDoneUpdatesCount == cState.nODEDoneUpdatesCount )
            {
                //jesli numerki sa takie same, to zwracamy remis
                return 0;
            }
        }
        //kazdy inny przypadek oznacza ze jesyesmy starsi
        return -1;                  
    }
}

