/** Komendy wpisywane z konsoli
 * Czesc z nich bedzie dotyczyla tylko trybu wysylania komunikatow
 * Czesc bedzie faktycznymi komendami do wykonania na _serwerze_.
 */
public class ConsoleCommand
{
    final static int _NONE          = -1;
    final static int _MODE_TEAM     = 0;
    final static int _TIME          = 1;
    final static int _CHANGE_TEAM   = 2;
    final static int _NEXT_MAP      = 3;
    final static int _KILL_ME       = 4;
    final static int _PLAYERSLIST   = 5;
    final static int _KICK          = 6;
    final static int _KICK_BY_ID    = 7;
    
    //pierwsza z "prawdziwych" komend 
    final static int _PURE_COMMAND  = _TIME;
            
    
    final static String[] sm_aCommand = { 
                                    "/TEAM",        //_MODE_TEAM
                                    "/TIME",        //_TIME
                                    "/CHANGETEAM",  //_CHANGE_TEAM
                                    "/NEXTMAP",      //_NEXT_MAP
                                    "/KILLME",       //_KILL_ME
                                    "/PLAYERSLIST",  //_PLAYERSLIST
                                    "/KICK",        //_KICK
                                    "/KICKBYID",    //_KICK_BY_ID
                                      };
        
    final static boolean[] sm_aRemove = {
                                    true,           //_MODE_TEAM
                                    false,          //_TIME
                                    false,          //_CHANGE_TEAM
                                    false,           //_NEXT_MAP
                                    false,          //_KILL_ME
                                    false,           //_PLAYERSLIST
                                    true,           //_KICK
                                    true           //_KICK_BY_ID
                                        };
    
    static FoundCommand Find( String sTypedText )
    {
        java.util.StringTokenizer cTokenizer = 
            new java.util.StringTokenizer( sTypedText, " ", true );
        
        if ( cTokenizer.hasMoreTokens() )
        {
            String sNextToken = cTokenizer.nextToken();
            
            for ( int i=0; i<sm_aCommand.length; i++ )
            {
                if ( CheckWord( sNextToken, sm_aCommand[i] ) )
                {
                    FoundCommand cFoundCommand = new FoundCommand( i );
                    
                    if ( i < sm_aRemove.length && sm_aRemove[i] )
                        cFoundCommand.SetParams( RemoveWord( sTypedText, sm_aCommand[i] ) );
                    return cFoundCommand;
                }     
            }
        }
        return null;
    }
    
    static String Perform( NetGameInfoBase cGameInfo,
                           int nCommand , String sText, int nHost )
    {
        if ( cGameInfo == null )
            return null;
        String sRet = null;
        switch ( nCommand )
        {
        case _TIME:
            int    nTime        = (int)cGameInfo.GetGameTime();
            String sTime        = NetGameTools.Time2String( nTime );
            int    nTimeLimit   = (int)cGameInfo.GetGameSettings().GetGameTimeLimit();
            String sTimeLimit   = NetGameTools.Time2String( nTimeLimit );
            sRet = "GameTime: "+sTime+", "+
                   "GameTimeLimit: "+sTimeLimit+"";
            break;
    
        case _CHANGE_TEAM:
            if ( cGameInfo instanceof NetGameInfoTeam )
            {
                NetPlayerInfo cWriterPlayerInfo = cGameInfo.GetPlayerInfoList().GetPlayerByHostId( nHost );
                if ( cWriterPlayerInfo != null )
                    ((NetGameInfoTeam)cGameInfo).EventPlayerRequestedChangeTeam( cWriterPlayerInfo, NetTeamInfo.NONE );
            }
            /*else
                sRet = "Komenda _CHANGE_TEAM dostepna tylko w grach teamowych";*/
            break;
            
        case _NEXT_MAP:
            if ( nHost == 0 )
            {
                if ( ! cGameInfo.IsGameOver() )
                {
                    cGameInfo.CallMethodIn( cGameInfo.METHOD_ServerForceGameOver,0.0f );
                    //sRet = "Przyjeto komende NEXT_MAP";
                }
                /*else
                    sRet = "Komenda NEXT_MAP zignorowana, gra juz skonczona";*/
            }
            /*else
                sRet = NetMsg.sCommandAvailableOnServer;*/
            break;
            
        case _KILL_ME:
            if ( ! cGameInfo.IsGameOver() )
            {
                NetPlayerInfo cPlayer = cGameInfo.GetPlayerInfoList().GetPlayerByHostId(nHost);
                if ( cPlayer != null && cPlayer.IsAlive() )
                {
                    cPlayer.CallMethodIn( cPlayer.METHOD_ServerForceKillMe,0.0f );
                    //sRet = "Smierc na wlasne zyczenie";
                }
            }
            break;            

        case _PLAYERSLIST:
            for ( int i=0; i<cGameInfo.GetPlayerInfoList().GetSize(); i++)
            {
                NetPlayerInfo cPlayer = cGameInfo.GetPlayerInfoList().GetPlayerByIndex( i );
                String sPlayer = cPlayer.GetNick()+" id:"+cPlayer.GetClientOwnerId()+"";
                if ( sRet != null )
                    sRet = sRet + ", ";
                else
                    sRet = "PlayersList: ";
                sRet = sRet + sPlayer;
            }
            break;

            
        case _KICK:
            if ( nHost == 0 )
            {
                String sNick = GetFirstWord( sText );
                NetPlayerInfo cPlayer = cGameInfo.GetPlayerInfoList().GetPlayerByNick( sNick );
                if ( cPlayer != null && 
                     cPlayer.GetClientOwnerId() > 0 && 
                     cPlayer.GetClientOwnerId() < ModuleNet.sm_nBotsIdMin )
                {
                    String sReason = RemoveWord( sText, sNick );
                    cGameInfo.EventPlayerKicked( cPlayer.GetClientOwnerId(), sReason );
                }
            }
            /*else
                sRet = NetMsg.sCommandAvailableOnServer;*/
            break;
            
        case _KICK_BY_ID:
            if ( nHost == 0 )
            {
                String sId = GetFirstWord( sText );
                NetPlayerInfo cPlayer = cGameInfo.GetPlayerInfoList().GetPlayerByHostId( Integer.parseInt( sId ) );
                if ( cPlayer != null && 
                     cPlayer.GetClientOwnerId() > 0 && 
                     cPlayer.GetClientOwnerId() < ModuleNet.sm_nBotsIdMin )
                {
                    String sReason = RemoveWord( sText, sId );
                    cGameInfo.EventPlayerKicked( cPlayer.GetClientOwnerId(), sReason );
                }
            }
            /*else
                sRet = NetMsg.sCommandAvailableOnServer;*/
            break;
            
        default:
            //sRet = "Komenda nie zostala obsluzona";
        }
        return sRet;
    }
    
    private static boolean CheckWord( String sText, String sWord )
    {
        return sText    != null && sWord != null && 
               GameObject.TextToUpperCase(sText).equals( GameObject.TextToUpperCase(sWord) );
    }
    
    private static String RemoveWord( String sText, String sWord )
    {
        int nBegin = GameObject.TextToUpperCase(sText).indexOf( GameObject.TextToUpperCase(sWord) );
        int nEnd   = nBegin + sWord.length();
        //wywalamy spacje na koncu slowa
        if ( sText.length() > nEnd )
            nEnd++;
        
        if ( nBegin < 0 )
        {
            GameObject.LogD(" substring "+sWord+" not found in "+sText+"\n");
            return null;
        }
        return sText.substring( 0, nBegin ) + sText.substring( nEnd );
    }
    
    private static String GetFirstWord( String sText )
    {
        //najpierw pozbywamy sie spacji na poczatku
        int nSafety = 100;
        do {
            int nBegin = sText.indexOf( " " );
        
            if ( nBegin != 0 || nSafety-- < 0 )
                break;
            sText = sText.substring( nBegin+1 );
        } while (true);
        
        int nSpacePos = sText.indexOf( " " );
        
        if ( nSpacePos > 0 )
            return sText.substring( 0, nSpacePos );
        return sText;
    }
}

class FoundCommand
{
    int    m_nId     = ConsoleCommand._NONE;
    String m_sParams = null;
    
    FoundCommand(int nId)
    {
        m_nId     = nId;
    }
    
    FoundCommand(int nId,String sParams)
    {
        this( nId );
        SetParams( sParams );
    }
    
    void SetParams(String sParams)
    {
        m_sParams = sParams;
    }
}