import java.io.*;
import java.util.StringTokenizer;

public class NetGameTools
{

    static String GetFirstWord(String sText)
    {
        if(sText == null)
            return null;
        int nSafety = 100;
        do
        {
            int nBegin = sText.indexOf(" ");
            if(nBegin != 0 || nSafety-- < 0)
                break;
            sText = sText.substring(nBegin + 1);
        } while(true);
        int nEndPos = -1;
        if(sText.startsWith("\""))
        {
            sText = sText.substring(1);
            nEndPos = sText.indexOf("\"");
        } else
        {
            nEndPos = sText.indexOf(" ");
        }
        if(nEndPos > 0)
            return sText.substring(0, nEndPos);
        else
            return sText;
    }

    static String Bool2OnOff(boolean bVal)
    {
        if(bVal)
            return "On";
        else
            return "Off";
    }

    static boolean CheckIfIsIPCorrect(String sIP)
    {
        boolean bCorrect = true;
        int nCount = 0;
        if(sIP != null)
        {
            StreamTokenizer st = new StreamTokenizer(new StringReader(sIP));
            st.whitespaceChars(0, 255);
            st.ordinaryChars(48, 57);
            st.wordChars(48, 57);
            try
            {
                st.nextToken();
                while(bCorrect && st.ttype != -1) 
                {
                    String s = st.sval;
                    try
                    {
                        int i = Integer.parseInt(s);
                        if(i < 0 || i > 255)
                            return false;
                    }
                    catch(NumberFormatException e)
                    {
                        bCorrect = false;
                    }
                    nCount++;
                    st.nextToken();
                }
            }
            catch(IOException e) { }
            return bCorrect && nCount == 4;
        } else
        {
            return false;
        }
    }

    static boolean IsLANAddress(String sIP)
    {
        NetLog.Test(" IsLANAddress " + sIP);
        int nIP = IPString2Int(sIP);
        int nIP1 = nIP >> 24 & 0xff;
        int nIP2 = nIP >> 16 & 0xff;
        boolean bRet = nIP1 == 10 || nIP1 == 127 || nIP1 == 172 && nIP2 >= 16 && nIP2 <= 31 || nIP1 == 192 && nIP2 >= 168;
        NetLog.Test(" " + nIP1 + " " + nIP2 + " " + bRet);
        return bRet;
    }

    static int Int2Bit(int nValue)
    {
        if(nValue < 0 || nValue > 31)
        {
            GameObject.LogD(" ERRROR: Parametr " + nValue + " poza zakresem\n");
            return 0;
        }
        int nBit;
        for(nBit = 1; nValue > 0; nBit *= 2)
            nValue--;

        return nBit;
    }

    public NetGameTools()
    {
    }

    static boolean IsStringValid(String sText)
    {
        return sText != null && sText.length() > 0;
    }

    static String GetTextWithoutPrefix(String sTypedText, String sPrefix)
    {
        StringTokenizer cTokenizer = new StringTokenizer(sTypedText, " ", true);
        if(cTokenizer.hasMoreTokens())
        {
            String sText = cTokenizer.nextToken();
            if(sText != null && sText.startsWith(sPrefix))
            {
                int nBegin = sText.indexOf(sPrefix);
                int nEnd = nBegin + sPrefix.length();
                sText = sText.substring(nEnd);
                if(sText != null && sText.length() > 0)
                    return sText;
            }
        }
        return null;
    }

    static String MixString(String sMain, String sName)
    {
        String sNameSubstring = sm_sNameSubstring;
        int nBegin = sMain.indexOf(sNameSubstring);
        if(nBegin < 0)
        {
            for(int i = 0; i < 9; i++)
            {
                sNameSubstring = sm_sNameSubstring.substring(0, 1) + i + sm_sNameSubstring.substring(1);
                nBegin = sMain.indexOf(sNameSubstring);
                if(nBegin >= 0)
                    break;
            }

        }
        if(nBegin < 0)
        {
            GameObject.LogD(" substring not found in " + sMain + "\n");
            return sMain;
        } else
        {
            int nEnd = nBegin + sNameSubstring.length();
            return sMain.substring(0, nBegin) + sName + sMain.substring(nEnd);
        }
    }

    static String MixString(String sMain, String sName1, String sName2)
    {
        String sTmp = MixString(sMain, sName1);
        return MixString(sTmp, sName2);
    }

    static String MixString(String sMain, String sName1, String sName2, String sName3)
    {
        String sTmp = MixString(sMain, sName1, sName2);
        return MixString(sTmp, sName3);
    }

    static String MixString(String sMain, int nParam1)
    {
        return MixString(sMain, Integer.toString(nParam1));
    }

    static String MixString(String sMain, int nParam1, int nParam2)
    {
        return MixString(sMain, Integer.toString(nParam1), Integer.toString(nParam2));
    }

    static String MixString(String sMain, int nParam1, int nParam2, int nParam3)
    {
        return MixString(sMain, Integer.toString(nParam1), Integer.toString(nParam2), Integer.toString(nParam3));
    }

    static String MixString(String sMain, int aParams[])
    {
        String sRet = sMain;
        if(aParams != null)
        {
            for(int i = 0; i < aParams.length; i++)
                sRet = MixString(sRet, Integer.toString(aParams[i]));

        }
        return sRet;
    }

    static String MixString(String sMain, String aParams[])
    {
        String sRet = sMain;
        if(aParams != null)
        {
            for(int i = 0; i < aParams.length; i++)
                sRet = MixString(sRet, aParams[i]);

        }
        return sRet;
    }

    static int NegateBits(int nBits)
    {
        return -1 - nBits;
    }

    static String RemoveLastSign(String sText)
    {
        if(sText != null && sText.length() > 1)
            sText = sText.substring(0, sText.length() - 1);
        return sText;
    }

    static String Bool2YesNo(boolean bVal)
    {
        if(bVal)
            return "Yes";
        else
            return "No";
    }

    static String Time2String(int nTime)
    {
        int nHour = nTime / 3600;
        int nMin = (nTime % 3600) / 60;
        int nSec = nTime % 60;
        String sTextTime = (nHour <= 0 ? "" : nHour + ":") + (nMin <= 9 ? "0" : "") + nMin + ":" + (nSec <= 9 ? "0" : "") + nSec;
        return sTextTime;
    }

    static String IP2String(int nIP)
    {
        return Integer.toString(nIP >> 24 & 0xff) + "." + Integer.toString(nIP >> 16 & 0xff) + "." + Integer.toString(nIP >> 8 & 0xff) + "." + Integer.toString(nIP & 0xff);
    }

    static boolean DeleteDir(String sDir, int nMaxLevel)
    {
        return DeleteDir(sDir, 0, nMaxLevel);
    }

    private static boolean DeleteDir(String sDir, int nLevel, int nMaxLevel)
    {
        if(nLevel > nMaxLevel)
            return true;
        File cDir = new File(sDir);
        if(!cDir.exists())
            return nLevel > 0;
        nLevel++;
        String aFileNames[] = cDir.list();
        for(int i = 0; i < aFileNames.length; i++)
        {
            String sFile = sDir + "/" + aFileNames[i];
            File cFile = new File(sFile);
            if(cFile.exists())
                if(cFile.isDirectory())
                    DeleteDir(sFile, nLevel);
                else
                    cFile.delete();
        }

        cDir.delete();
        return true;
    }

    static int CntBits(int nBits)
    {
        int nBitsCnt = 0;
        for(int i = 0; i < 32; i++)
        {
            if(nBits % 2 != 0)
                nBitsCnt++;
            nBits /= 2;
        }

        return nBitsCnt;
    }

    static int IPString2Int(String sIP)
    {
        int nIP = 0;
        if(sIP != null)
        {
            StreamTokenizer st = new StreamTokenizer(new StringReader(sIP));
            st.whitespaceChars(0, 255);
            st.ordinaryChars(48, 57);
            st.wordChars(48, 57);
            try
            {
                st.nextToken();
                while(st.ttype != -1) 
                {
                    String s = st.sval;
                    try
                    {
                        int i = Integer.parseInt(s);
                        nIP = (nIP << 8) + i;
                    }
                    catch(NumberFormatException e)
                    {
                        nIP = 0;
                    }
                    st.nextToken();
                }
            }
            catch(IOException e) { }
        }
        return nIP;
    }

    static boolean IsInternetAddress(String sIP)
    {
        return !IsLANAddress(sIP);
    }

    static String RemoveWord(String sText, String sWord)
    {
        if(sText == null || sWord == null)
            return null;
        boolean bRemoveBrackets = false;
        int nBegin = GameObject.TextToUpperCase(sText).indexOf(GameObject.TextToUpperCase(sWord));
        int nEnd = nBegin + sWord.length();
        if(sText.length() > nEnd)
            nEnd++;
        if(nBegin > 0 && sText.startsWith("\""))
        {
            bRemoveBrackets = true;
            nBegin--;
        }
        if(nBegin < 0)
        {
            GameObject.LogD(" substring " + sWord + " not found in " + sText + "\n");
            return null;
        } else
        {
            return sText.substring(0, nBegin) + sText.substring(nEnd);
        }
    }

    static String sm_sNameSubstring = "%s";

}
