/** grupa laczy obiekty AI, aby dac interfejs do wolania akcji na wielu obiektach
 */
public class AIGroup extends EditorRenderableObject
	implements IGroup, IFreezable, IObjCount, iact_Appear, iact_EnableDisable, IDestroyTarget, iact_ChangeWaypoint
{
    public NewAI[] aMembers = null;//#MEMBERS

    /** referencja na pierwszy waipoint AI */
    public Waypoint cWaypoint = null;//#\AIGROUP;Reference to first group Waypoint; It not null, overwrites AIs cWaypoint
    public boolean bDisappearOnStart = false;//#\AIGROUP
    public boolean bAffectMembersPositions = true;//#\AIGROUP;When true members positions are calculated relative to startup positions in group
    
    // akcje odpalane kiedy aktor ginie
    public Action[] aOnDie = null;//#\AIGROUP;Actions preformed when all members of group die
	public Action[] aOnSingleMemberDie = null;//#\AIGROUP;Actions preformed when a single member of group dies
	boolean m_bDead = false;

    /** Lokalne pozycje czlonkow grupy wzgledem niej
     */
    protected Vector aMembersLocalPos[] = null;
    
    AIGroup()
    {
        EditorMeshName = "data\\interface\\AIGroup.3da";
        iRenderType = EEditorRenderType._MESH;
    }

    // definiujemy rysowanie kresek miedzy kolejnymi Waypointami
    public static void DefineVisualisation()
    {
        DefineDebugLine("cWaypoint", new Vector(1f,1f,1f));
        DefineDebugArrowArray("aMembers", new Vector(1f,1f,0), true);
    }
    
	public void OnCreate()
	{
		// ustawiamy referencje wsteczne na grupe w ai
		if (aMembers != null)
		{
			for (int i = 0; i < aMembers.length; i++)
            {
                aMembers[i].SetAIGroup(this);
            }
		}
        bSG = true;
	}
    
    public void OnPostCreate()
    {
        if (aMembers != null)
        {
            RecalcGroupPosition(); // ustaw pozycje grupy...
            Vector vPos = null;
            aMembersLocalPos = new Vector[aMembers.length];
            for (int i = 0; i < aMembers.length; i++)
            {
                if (cWaypoint != null)
                {
                    aMembers[i].cFirstWaypoint = cWaypoint; // ustaw waypoint startowy
                    aMembers[i].cWaypoint = null;
                }
                vPos = aMembers[i].GetPosition();
                if (vPos != null)
                    aMembersLocalPos[i] = PointWorldToLocal(vPos);
            }
        }
        if (bDisappearOnStart)
            CallMethodIn("Disappear", 0);
    }
	
    public void Disappear()
    {
        Disappear(null);
    }
    
    public void OnDestroy()
    {
    }
	
	public ControlObject [] GetAllMembers()
	{
		return aMembers;
	}
    
	public void MemberDie(NewAI member)
	{
		boolean bAllDead = true;
		boolean bMemberFound = false;
		
		if (m_bDead)
			return; // juz wszyscy nie zyja...
		
		for (int i = 0; i < aMembers.length; i++)
		{
			if (aMembers[i] == member)
				bMemberFound = true;
			if (!aMembers[i].bDead)
				bAllDead = false; // ktos w tej grupie jeszcze zyje
		}
			
		if (!bMemberFound)
			return; // ups, cos poszlo nie tak, member-parametr nie jest czlonkiem grupy
		
		InvokeActionChain(aOnSingleMemberDie);
		
		if (bAllDead)
		{
			m_bDead = true;
			InvokeActionChain(aOnDie);
		}
	}
    
    /** Ustawienie pozycji grupy jako srodka ciezkosci jej czlonkow traktowanych 
     *  z rownymi wagami
     */
    public void RecalcGroupPosition()
    {
        if (aMembers != null)
        {
            Vector vMedPos = null;
            Vector vPos = null;
            int nPositions = 0;
            for (int i = 0; i < aMembers.length; i++)
            {
                vPos = aMembers[i].GetPosition();
                if (vPos != null)
                {
                    if (vMedPos == null)
                        vMedPos = vPos;
                    else
                        vMedPos.Add(vPos);
                    nPositions++;
                }
            }
            if (vMedPos != null)
            {
                if (nPositions > 0)
                    vMedPos.Mul(1.f / (float)nPositions);
                SetPositionWorld(vMedPos);
            }
        }
    }
    
    /** Pobranie pozycji konkretnego AI gdy srodek grupy ma sie znajezc w okreslonym punkcie
     *  i ma byc ustawiony w okreslonym kierunku
     *  @param member AI, ktore pyta o swoja pozycje
     *  @param groupDestPos docelowa pozycja grupy, dla ktorej AI pyta o swoja pozycje
     *  @param groupDestDir kierunek, w jakim ma byc ustawiona grupa w ponkcie groupDestPos
     *  @return pozycja w swiecie, lub null, jesli cos jest nie tak (np. AI bylo nieaktywne 
     *          w momencie badania pozycji w OnPostCreate)
     */
    public Vector GetMemberFormationPosition(NewAI member, Vector groupDestPos, Vector groupDestDir)
    {
        if (bAffectMembersPositions)
        {
            if (groupDestPos != null)
                SetPositionWorld(groupDestPos);
            if (groupDestDir != null)
                SetDir(groupDestDir);
            
            // TODO: teraz przeszukiwanie liniowe, jesli bedzie tego wiecej, przerobic na hashmape....
            if (aMembers != null)
            {
                for (int i = 0; i < aMembers.length; i++)
                    if (member == aMembers[i])
                    {
                        if (aMembersLocalPos[i] != null)
                            return PointLocalToWorld(aMembersLocalPos[i]);
                        else
                            return null;
                    }
            }
            return null;
        }
        else
            return groupDestPos;
    }

    
    public boolean IsFrozen()
    {
        if(aMembers==null) return false;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                if( !aMembers[i].IsFrozen() ) return false;
        
        return true;
    }
    
    public void Die()
    {
        if(aMembers==null) return;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                aMembers[i].Die();
    }
    
	public void Respawn()
    {
        if(aMembers==null) return;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                aMembers[i].Respawn();
    }
    
	public boolean IsDead()
    {
        if(aMembers==null) return false;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                if( !aMembers[i].IsDead() ) return false;
        
        return true;
    }
    
    public void Freeze()
    {
        if(aMembers==null) return;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                aMembers[i].Freeze();
    }
    
	public void Unfreeze()
    {
        if(aMembers==null) return;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                aMembers[i].Unfreeze();
    }
    
    public void Appear(ControlObject cDestination)
    {
        if(aMembers==null) return;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                aMembers[i].Appear(cDestination);
    }
    
    public void Disappear(ControlObject cDestination)
    {
		// TODO: wykorzystac parametr cDestination!!!
        if(aMembers==null) return;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                aMembers[i].Disappear(cDestination);
    }
    
    public void Enable()
    {
        if(aMembers==null) return;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                aMembers[i].Enable();
    }
    
    public void Disable()
    {
        if(aMembers==null) return;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                aMembers[i].Disable();
    }
    
    public boolean IsEnabled()
    {
        return false;
    }
	
	public ChromeActor [] GetObjectsToDestroy()
	{
        if (aMembers == null)
			return null;
		int iAliveMembers = 0;

        for(int i = 0; i < aMembers.length; i++)
            if ( (aMembers[i] != null) && (!aMembers[i].IsDead()) )
				iAliveMembers++;
		if (iAliveMembers == 0)
			return null;
		
		ChromeActor retTab [] = new ChromeActor[iAliveMembers];
        for(int i = 0; i < aMembers.length; i++)
		{
            if ( (aMembers[i] != null) && (!aMembers[i].IsDead()) )
			{
				iAliveMembers--;
				retTab[iAliveMembers] =  aMembers[i];
			}
		}
		return retTab;
	}
    
    public void ChangeWaypoint(Waypoint cNewWaypoint, boolean bStopCurrActivity)
    {
        if(aMembers==null) return;
        for(int i=0; i<aMembers.length; ++i)
            if(aMembers[i]!=null)
                aMembers[i].ChangeWaypoint(cNewWaypoint, bStopCurrActivity);
    }
    
    /////////////////////////////////////////////////////////////////////////////
    // Implementacja IObjCount
    public int GetObjsCount()
    {
        return aMembers.length;
    }
    
    public int GetDeadObjsCount()
    {
        int iDeadObjects = 0;
        
        for (int i = 0; i<aMembers.length; ++i)
        {
            if ( (aMembers[i] != null) && (aMembers[i].bDead) )
                iDeadObjects++;
        }
        return iDeadObjects;
    }
    
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[AIGroup.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_AIGROUP)
        {
			super.SGLoadChunk(cFC);
            m_bDead = cFC.LoadBool();
        }
		
		cFC.delete();
	}
    

    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[AIGroup.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_AIGROUP, cFCParent);
		
		super.SGSaveChunk(cFC);

        cFC.SaveBool(m_bDead);
        
        cFC.delete(); 
	}				
	
	public void SGUpdate()
	{
		FileChunk.Log("[AIGroup.SGUpdate]\n");
		super.SGUpdate();
	}

}
