public class AIControllerWalker extends AIController
{
    /////////////////////////////////////////////////////////////////////
    // BRON I STRZELANIE
    int METHOD_FireOneBullet = 0;
    int METHOD_FireDone = 0;
    int METHOD_Fire = 0;
    int METHOD_UpdateShootDecision = 0;
    
    int m_nBulletsToFire = 0;
    
    float _SHARP_ANGLE_TO_SHOOT = 10f;
    float _SHARP_ANGLE_TO_SHOOT_COS = 0;
    
    boolean m_bIsShooting = false;
    float m_fNextShootTime = 0;

    /** Wplyw (mnoznik) ShootVisibilityFactor celu na celnosc ai */
    float m_fTargetShootVisibilityFactorImportance = 0.2f;

    ///////////////////////////////////////////////
    // Parametry ustawiane na podstawie broni
    /** Prawdopodobienstwo, ze stojace odpowiednio daleko AI przyceluje przed strzalem
     *  (gdy 100 - zawsze probuje przycelowac, nawet blisko) [%] */
    int m_nReadinessToStartAiming = 100;
    
    /** Prawdopodobienstwo, ze konczac serie AI przestanie przycelowywac */
    int m_nReadinessToStopAiming = 100;
    
    /** Dystans ^2, ponizej ktorego ai uznaje cel za bliski
     * (do przycelowania i dlugosci serii)*/
    float m_fNearTargetDistance2 = 1;

    /** Dystans ^2, powyzej ktorego ai uznaje cel za odlegly 
     * (do przycelowania i dlugosci serii)*/
    float m_fMaxAimingChanceDistance2 = 2;

    /** Dystans ^2, powyzej ktorego ai uznaje cel za odlegly 
     * (do przycelowania i dlugosci serii)*/
    float m_fFarTargetDistance2 = 3;
    
    /** Maksymalna odleglosc ^2, ponizej ktorej AI strzela z broni w trybie auto [cm^2] */
    float m_fMaxBurstFireDistance2 = 1000*1000;

    /** Maksymalna ilosc kul do wystrzelenia z broni w trybie single (min = 1) */
    int m_nMaxBulletsToFire;
    
    /** Minimalny czas strzelania seria */
    float m_fMinBurstFiringTime;
    /** Losowy modyfikator fla m_fMinBurstFiringTime */
    float m_fRandomBurstFiringTime;
    
    /** Minimalna przerwa miedzy 'seriami' pojedynczych strzalow */
    float m_fMinSingleFireDelay;
    /** Losowy modyfikator dla m_fMinSingleFireDelay */
    float m_fRandomSingleFireDelay;
    
    /** Minimalna przerwa miedzy seriami strzalow w trybie auto */
    float m_fMinBurstFireDelay;
    /** Losowy modyfikator dla m_fMinBurstFireDelay */
    float m_fRandomBurstFireDelay;
    
    /** Efektywy zasieg broni ^2 */
    float m_fWeaponEffectiveRange2;
    
    /** Minimalne prawdopodobienstwo trafiania przez AI w cel */
    public float m_fMinDesiredHitRatio;
    
    /** Maksymalne prawdopodobienstwo trafiania przez AI w cel */
    public float m_fMaxDesiredHitRatio;
    
    
    AIControllerWalker()
    {    
        super();
        
        _MANY_ATTACKING_ENEMIES = 10;
    }
    
    public void OnCreate()
    {
        super.OnCreate();
        
        METHOD_FireOneBullet = RegisterCallMethod("FireOneBullet");
        METHOD_FireDone = RegisterCallMethod("FireDone");
        METHOD_Fire = RegisterCallMethod("Fire");
        METHOD_UpdateShootDecision = RegisterCallMethod("UpdateShootDecision");
        
        _SHARP_ANGLE_TO_SHOOT_COS = (float) Math.cos(_SHARP_ANGLE_TO_SHOOT * Math.PI / 180f);
    }
    
    public void InitController(NewAI ai)
    {
        super.InitController(ai);
        
        m_HearingSense = (HearingSense) CreateObject("HearingSense");
        m_HearingSense.Init(this);

        m_PainSense = (PainSense) CreateObject("PainSense");
        m_PainSense.Init(this);

        m_ObjectsSense = (ObjectsSense) CreateObjectNoInit("ObjectsSense");
        m_ObjectsSense._AI_SIGHT_RANGE = ((NewAIWalker) ai).fSightRange;
        m_ObjectsSense._AI_RECOGNIZE_RANGE = ((NewAIWalker) ai).fSightRange * ((NewAIWalker) ai).fSightRangeRecognitionMult;
        m_ObjectsSense._AI_SIGHT_ANGLE = ((NewAIWalker) ai).fSightAngle;
        m_ObjectsSense.InitObject();
        m_ObjectsSense.Init(this);
    }

    public void AddGenerators()
    {
        super.AddGenerators();

#if AI_DEBUG
        Generator debugFilter = new ScenariosFilterWalker(this);
        m_DebugScenariosFilter.Set(debugFilter.m_GenScenarios);
#endif

        m_aGenerators = new Generator[] {
                           new CanShoot_W(this),
                           new EnemySpotted_W(this),
                           new HasToKeepDistance_W(this),
                           new IdleAllowed_W(this),
                           new PerformEditorActivity_W(this),
                           new ScenariosFilterWalker(this),
                           new ShortEnemyDistance_W(this), 
                           new ShouldLookForEnemy_W(this), 
                           new ShouldWaitForEnemyAction_W(this),
                           new SilenceBroken_W(this),								   
                           };
    }
    
    public void Die()
    {
        super.Die();
        
        FireDone();
        DisableCallMethod(METHOD_UpdateShootDecision);
    }
    
    /** wrzucenie przekazanych przez zmysl ObjectsSense obiektow */
    void RedistributeObjects(ControlObject [] aObjs)
    {
        float fTime = GetTime();
        
        for (int i = 0; i < aObjs.length; i++)
        {
            if (aObjs[i] == m_MeshAI)
                continue;
            
            if ( (aObjs[i] instanceof NewAI) ||
                 (aObjs[i] instanceof Vehicle) ||
                 (aObjs[i] instanceof Player) )
            {
                AddToProperList((ChromeActor) aObjs[i], fTime);
            }
        }
    }

    /** implementacja interfejsu hideoutow */
    public boolean CanAttackFromCurrentHideout() { return false; }
    public ChromeActor HideoutsNear() { return null; }
    public Vector GetDestHideoutPos() { return null; }
    public boolean IsHidingBehindHideout() { return false; }
    public boolean IsBehindHideout() { return false; }
    public boolean CanHitActiveEnemyFromHideout() { return false; }
    
    /** implementacja interfejsu do pojazdow */
    void SetObjectToGetIn(ISeatVehicle cObjectToGetIn, EnumGetInType cGetInAs) {}
    SeatInfo GetSeat() { return null; }
    public void GotIn() {}
    public void GotOut() {}
    boolean InsideVehicle() { return false; }
    void SetObjectToDrive(IDrivenVehicle cObjectToDrive) {}
    IDrivenVehicle GetDrivenVehicle() { return null; }
    void Drive() {}
    public void Checkpoint() {}
    public void MovingEnd() {}
    
    public boolean InsideCannon() { return false; }
    public void SetInCannon(boolean bInCannon) {}
    
    /////////////////////////////////////////////////////////////////////
    // STRZELANIE
    /** wlaczenie strzelania */
    void SetCanFire(boolean bCanAim)
    {
        super.SetCanFire(bCanAim);
        
        m_bIsShooting = false;
        m_fNextShootTime = 0;
        UpdateShootDecision();
        CallMethodEvery(METHOD_UpdateShootDecision, 0.5f, 0.5f * GetMesh().GetTimeDeviation());
    }

    /** wylaczenie strzelania */
    void ResetCanFire()
    {
        super.ResetCanFire();
        DisableCallMethod(METHOD_UpdateShootDecision);
        FireDone();
    }	

    /** Zwraca 0 jesli aktywny wrog jest blisko (ponizej fNearDistance2),
     *  1 jesli daleko (powyzej fFarDistance2) i wartosc pomiedzy 0-1 gdy 
     *  pomiedzy tymi odleglosciami */
    float CalcActiveEnemyDistCoef(float fNearDistance2, float fFarDistance2)
    {
        return Tools.InterpLinear(fNearDistance2,
                                  fFarDistance2, 
                                  GetActiveEnemyDistance2());
    }
    
    public void UpdateShootDecision()
    {
    }
    
    public void Fire()
    {
        ComputeAccuracy();
        switch (ChooseBestFireMode())
        {
        case Weapon._FIREMODE_SINGLE:
            FireSingle();
            break;
        case Weapon._FIREMODE_BURST:
            FireAuto();
            break;
        default:
            // chyba sobie nie postrzela...
            break;
        }
        m_bIsShooting = true;
    }

    int ChooseBestFireMode()
    {
        int nWeaponFireModes = GetActiveWeapon().nAIFireModes;
        
        // jesli jest tryb burst
        if ((nWeaponFireModes & Weapon._FIREMODE_BURST) != 0)
        {
            // jesli nie ma trybu single albo przeciwnik jest blisko to burst
            if ((nWeaponFireModes & Weapon._FIREMODE_SINGLE) == 0 ||
                GetActiveEnemyDistance2() < m_fMaxBurstFireDistance2)
            {
                return Weapon._FIREMODE_BURST;
            }
        }
        
        // jesli jest tryb single
        if ((nWeaponFireModes & Weapon._FIREMODE_SINGLE) != 0)
            return Weapon._FIREMODE_SINGLE;
        else
            return Weapon._FIREMODE_NONE;
    }
    
    
    /** Wystrzelenie jednego pocisku */
    void FireSingle()
    {
        FireBullets(1 + (int)(RandomFloat(m_nMaxBulletsToFire) *
                              (1 - CalcActiveEnemyDistCoef(m_fNearTargetDistance2,
                                                           m_fFarTargetDistance2))));
    }
    
    void FireBullets(int nBullets)
    {
        m_nBulletsToFire = nBullets;
        FireOneBullet();
    }
    
    public void FireOneBullet()
    {
        if (m_nBulletsToFire > 0)
        {
            m_nBulletsToFire--;
            GetActiveWeapon().StartFire(Weapon.MODE_SINGLE);
            CallMethodOnceIn(METHOD_FireOneBullet, 60f / GetActiveWeapon().fRateOfFire);
        }
        else
            FireDone();
    }
    
    /** Wystrzelenie serii o losowej dlugosci */
    void FireAuto()
    {
        float fFiringTime = m_fMinBurstFiringTime + 
                            RandomFloat(m_fRandomBurstFiringTime) *
                            (1 - CalcActiveEnemyDistCoef(m_fNearTargetDistance2,
                                                         m_fFarTargetDistance2));
        GetActiveWeapon().StartFire(Weapon.MODE_AUTO);
        CallMethodOnceIn(METHOD_FireDone, fFiringTime);
    }
    
    /** Zakonczenie strzelania, wolane po pojedynczym strzale lub po uplywie czasu */
    public void FireDone()
    {
        DisableCallMethod(METHOD_Fire);
        DisableCallMethod(METHOD_FireDone);
        DisableCallMethod(METHOD_FireOneBullet);
        m_nBulletsToFire = 0;
        if (GetActiveWeapon() != null)
            GetActiveWeapon().StopFire();
        m_bIsShooting = false;
        
        float fFireDelay = 1;
        if (GetActiveWeapon() != null)
        {
            switch (GetActiveWeapon().nCurrentMode)
            {
            case Weapon.MODE_SINGLE:
                fFireDelay = m_fMinSingleFireDelay +
                             RandomFloat(m_fRandomSingleFireDelay) * 
                             CalcActiveEnemyDistCoef(m_fNearTargetDistance2,
                                                     m_fFarTargetDistance2);
                break;
            case Weapon.MODE_AUTO:
                fFireDelay = m_fMinBurstFireDelay +
                             RandomFloat(m_fRandomBurstFireDelay) * 
                             CalcActiveEnemyDistCoef(m_fNearTargetDistance2,
                                                     m_fFarTargetDistance2);
                break;
            }
        }
        m_fNextShootTime = GetTime() + fFireDelay;
    }
    
    void MessageWeapon(int nState)
    {
        // zgloszenie zdarzenia ze skonczyla sie amunicja oraz nie zgloszone
        // jeszcze zdarzenie
        if ((nState & Weapon.STATE_NO_AMMO) != 0)//  && !GetMesh().m_bAmmoStateKnown)      
        {
            RequestEvent(AIEvents._EV_NO_AMMO);
        }
    }
    
    boolean SharpAngleToShoot(Vector vPointToShoot)
    {
        boolean bIsSharp = false;
        
        Vector vDir = new Vector(vPointToShoot);
        vDir.Sub(m_MeshAI.GetFireOrigin());
        vDir.fY = 0;
        vDir.Normalize();
        
        
        if (GetActiveWeapon() != null)
        {
            Vector vWpn = GetActiveWeapon().GetForwardVector();
            vWpn.fY = 0;
            vWpn.Normalize();

            bIsSharp = vWpn.Dot(vDir) > _SHARP_ANGLE_TO_SHOOT_COS;
        }
        
        return bIsSharp;
    }
    
    public void ActiveWeaponChanged()
    {
        super.ActiveWeaponChanged();
        
        Weapon weapon = GetActiveWeapon();
        if (weapon != null)
        {
            m_nReadinessToStartAiming = weapon.nAIReadinessToAim;
            m_nReadinessToStopAiming = weapon.nAIReadinessToStopAiming;

            m_fNearTargetDistance2 = weapon.fAINearTargetDistance * 
                                     weapon.fAINearTargetDistance;
            m_fMaxAimingChanceDistance2 = weapon.fAIMaxAimingChanceDistance * 
                                          weapon.fAIMaxAimingChanceDistance;
            m_fFarTargetDistance2 = weapon.fAIFarTargetDistance * 
                                    weapon.fAIFarTargetDistance;

            m_fMaxBurstFireDistance2 = weapon.fAIMaxBurstDistance * 
                                       weapon.fAIMaxBurstDistance;

            m_nMaxBulletsToFire = (int)Math.max(1, weapon.nAIMaxSingleFireBullets * m_MeshAI.mod_fFiringTimeMult);
            
            m_fMinBurstFiringTime = weapon.fAIBurstTime * m_MeshAI.mod_fFiringTimeMult;
            m_fRandomBurstFiringTime = weapon.fAIRandomBurstTime * m_MeshAI.mod_fFiringTimeMult;
            
            m_fMinSingleFireDelay = weapon.fAISingleFireDelay * m_MeshAI.mod_fFireDelayMult;
            m_fRandomSingleFireDelay = weapon.fAIRandomSingleFireDelay * m_MeshAI.mod_fFireDelayMult;
            
            m_fMinBurstFireDelay = weapon.fAIBurstFireDelay * m_MeshAI.mod_fFireDelayMult;
            m_fRandomBurstFireDelay = weapon.fAIRandomBurstFireDelay * m_MeshAI.mod_fFireDelayMult;
            
            m_fWeaponEffectiveRange2 = weapon.fAIEffectiveRange *
                                       weapon.fAIEffectiveRange;
            
            m_fMinDesiredHitRatio = MathUtils.crop(weapon.fAIMinDesiredHitRatio *
                                                   m_MeshAI.mod_fMinHitRatioMult / 100, 0, 1);
            m_fMaxDesiredHitRatio = MathUtils.crop(weapon.fAIMaxDesiredHitRatio *
                                                   m_MeshAI.mod_fMaxHitRatioMult / 100, 0, 1);

        }
    }
    
    
    public void ComputeAccuracy()
    {
        Weapon weapon = GetActiveWeapon();
        NewAIWalker mesh = (NewAIWalker)GetMesh();
        if (weapon != null)
        {
            ChromeActor enemy = GetActiveEnemy();
            if (enemy != null)
            {
                m_fCurrAccuracy = 1 - Tools.InterpLinear(weapon.fAINearTargetDistance, weapon.fAIEffectiveRange,
                                                         mesh.GetDistanceTo(enemy));
                m_fCurrAccuracy += m_fTargetShootVisibilityFactorImportance * (enemy.GetShootVisibilityFactor(mesh) - 1);
            }
            else
                m_fCurrAccuracy = 0;
            
            m_fCurrAccuracy *= mesh.fOverallAIAccMult * mesh.mod_fAccuracyMult;

//            Tools.ShowInfoTop("Accuracy: " + m_fCurrAccuracy, GetModule());

            //clamp to 0-1
            if (m_fCurrAccuracy > 1.0f) m_fCurrAccuracy = 1.0f;
            else if (m_fCurrAccuracy < 0.0f) m_fCurrAccuracy = 0.0f;

            mesh.m_fDesiredHitRatio = (m_fMinDesiredHitRatio +
                                       m_fCurrAccuracy *
                                       (m_fMaxDesiredHitRatio - m_fMinDesiredHitRatio));
        }
        else
        {
            m_fCurrAccuracy = 0;
            mesh.m_fDesiredHitRatio = 0;
        }
    }
    
    //*********************************************************
    //                     SaveGame
    //*********************************************************
    
    
    public void SGLoadChunk(FileChunk cParentFC)
    {
        FileChunk.Log("[AIControllerWalker.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_AICONTROLLER_WALKER)
        {
            super.SGLoadChunk(cFC);

            m_nBulletsToFire = cFC.LoadInt();
            m_bIsShooting = cFC.LoadBool();
            m_fNextShootTime = cFC.LoadFloat();
            
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[AIControllerWalker.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_AICONTROLLER_WALKER, cFCParent);
        
        super.SGSaveChunk(cFC);

        cFC.SaveInt(m_nBulletsToFire);
        cFC.SaveBool(m_bIsShooting);
        cFC.SaveFloat(m_fNextShootTime);
        
        cFC.delete(); 
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[AIControllerWalker.SGUpdate]\n");
        super.SGUpdate();
        ActiveWeaponChanged();
    }

}
