public class LookForEnemy extends Scenario
{
    static int _ID = _NUM_OF_SCENARIOS++;
    
    static float _MIN_WALK_SEARCH_DISTANCE = 200f;
    static float _MAX_WALK_SEARCH_DISTANCE = 1000f;
    static float _MAX_SEARCH_TIME = 60f;
    static float _MAX_RICOCHET_DIR_DISTORTION_ANGLE = 90.f;//maksymalny kat odchylenia od kierunku do przeciwnika, ktory wydal noise'a
    static float _MAX_SHOT_DIR_DISTORTION_ANGLE = 30.f;
	static float _MIN_DIST_TO_LAST_ENEMY_POS = 500f;
	static float _DEST_DIST_TO_ENEMY = 200f; // odleglosc od ai do przeciwnika, na jaka ai ma dobiec;
											 // musi byc wieksza niz szerokosc elipsoid i mniejsza od
											 // _MIN_DIST_TO_LAST_ENEMY_POS
    static float _MIN_DISTANCE_TO_FRIEND = 400f; // odleglosc od przyjaciela, ponizej ktorej staramy sie nie zblizac
    static float _MAX_DISTANCE_TO_FRIEND = 2000; // odleglosc do przyjaciela, powyzej ktorej AI do niego nie podbiega

    static float _MIN_AREA_RADIUS_TO_RUN = 350.f; // minimalny promien fGuardedAreaRadius aby AI biegalo losowo, gdy nie moze sie w guardzie ruszyc...
    
    private static Vector m_vDestPos;
    void PrepareScenario(AIController ai)
	{
        m_vDestPos = null;
        NewAI mesh = ai.GetMesh();
        Vector vVolMeshPos = mesh.GetPositionVectorVolatile();
        ChromeActor cEnemy = ai.GetActiveEnemy();
        if (cEnemy != null)
        {
			EnemyInfo info = (EnemyInfo) ai.m_aEnemies.get(cEnemy);
			if ( (info != null) && (info.m_vLastKnownPos != null) )
			{
				float fDist = mesh.GetDistanceTo(info.m_vLastKnownPos);
				if (fDist > _MIN_DIST_TO_LAST_ENEMY_POS)
				{
					Vector vPos = vVolMeshPos;
    	            m_vDestPos = new Vector(info.m_vLastKnownPos);
					m_vDestPos.Sub(vPos);
					m_vDestPos.Mul( (fDist - _DEST_DIST_TO_ENEMY) / fDist);
					m_vDestPos.Add(vPos);
				}
			}
        }
		else 
        {
			if (ai.JustHeardEnemySteps() && ai.m_vLastHeardEnemyStepsPos != null)
            {
                m_vDestPos = new Vector(ai.m_vLastHeardEnemyStepsPos);
            }
            else if (ai.m_vLastPassageEnemyNoisePos != null &&
                     !ai.TimePassed(ai.m_fLastPassageEnemyNoiseTime, ai._VERY_LONG_SOUND_IMPORTANCE_TIME, ai.GetTime()))
            {
                m_vDestPos = new Vector(ai.m_vLastPassageEnemyNoisePos);
            }
            else if (ai.IsValidLastHeardEnemyShotPos())
            {
                m_vDestPos = new Vector(ai.GetLastHeardEnemyShotPos());
            }
            else
			{
				Vector vPos = null;
                float fDistortionAngle = _MAX_RICOCHET_DIR_DISTORTION_ANGLE;
                // sprobuj ze sprawdzeniem strzelejacego przyjaciela
                if (ai.IsValidLastHeardFriendShotPos())
                {
                    // pobierz jego pozycje
                    vPos = ai.m_vLastHeardFriendShotPos;
                    // jesli jestes za blisko - idz na przeciwnika 
                    float dist2 = 0;
                    if (vPos != null)
                        dist2 = mesh.GetDistance2To(vPos);
                    
                    if (dist2 < _MAX_DISTANCE_TO_FRIEND * // jesli za daleko strzela kolega - ignoruj
                                _MAX_DISTANCE_TO_FRIEND)
                    {
                        if (vPos == null || dist2 < _MIN_DISTANCE_TO_FRIEND * // jak blisko - wez jego cel
                                                    _MIN_DISTANCE_TO_FRIEND)
                        {
                            vPos = ai.m_vLastFriendsTargetPos;
                        }
                        if (vPos != null)
                            fDistortionAngle = _MAX_SHOT_DIR_DISTORTION_ANGLE;
                    }
                }
                if (vPos == null && ai.JustHeardRicochet())
                    vPos = ai.GetLastRicochetMakerPos();
				if (vPos == null && ai.JustHeardExplosion())
					vPos = ai.GetLastExplosionMakerPos();
				
                if (vPos != null)
				{
					m_vDestPos = new Vector(vPos);
					m_vDestPos.Sub(vVolMeshPos);
					// gdy nie mamy wroga i odleglosc do przyjaciela ktory spowodowal dzwiek jest mala
					// nie chcemy przebiegac b. krotkich odcinkow do przyjaciela, tylko sie oddalamy
					if (m_vDestPos.Len() < _MIN_DISTANCE_TO_FRIEND)
						m_vDestPos = ai.RotateVector(m_vDestPos, 
													 180f,
													 Vector.Up);
					else
						m_vDestPos.Mul(GameObject.RandomFloat(0.2f, 0.8f)); // nie chcemy podbiegac do samego celu
					m_vDestPos = ai.RotateVector(m_vDestPos, 
												 GameObject.RandomFloat(-fDistortionAngle, 
																		fDistortionAngle),
												 Vector.Up);
					m_vDestPos.Add(vVolMeshPos);
				}
			}
		}
		ai.AddTask(new TaskTrackingStop());
		ai.AddTask(new TaskStopFire());
        if (m_vDestPos != null)
        {
            ai.AddTask(new TaskMotionStateChange(MotionState.SPEED_RUN));
            ai.AddTask(new TaskTurnAtPos(m_vDestPos));
            if (mesh.m_GuardedObject == null ||
                mesh.GetDistanceTo(mesh.GetValidGoToPosition(vVolMeshPos, m_vDestPos)) > _MIN_WALK_SEARCH_DISTANCE)
            {                
                if (ai.GetMesh().GetDistanceTo(m_vDestPos) > _DEST_DIST_TO_ENEMY)
                {
                    ai.AddTask(new TaskGotoPos(m_vDestPos, _MAX_SEARCH_TIME));
                }
                else
                {
                    ai.AddTask(new TaskMotionStateChange(MotionState.SPEED_WALK));
                    m_vDestPos.Add(MathUtils.CalcRandomMoveVolatile(_MIN_WALK_SEARCH_DISTANCE, 
                                                                    _MAX_WALK_SEARCH_DISTANCE));
                    ai.AddTask(new TaskGotoPos(m_vDestPos, _MAX_SEARCH_TIME));
                }
            }
            else
            {
                // tutaj hak - jesli ai przez guarda nie moze podejsc blizej - niech biega losowo 
                // w obszarze guarda jesli jest on odpowiednio duzy...
                if (mesh.m_fGuardedAreaRadius > _MIN_AREA_RADIUS_TO_RUN)
                {
                    m_vDestPos.Set(Vector.Forward);
                    m_vDestPos.Mul(ai.RandomFloat(_MIN_WALK_SEARCH_DISTANCE, mesh.m_fGuardedAreaRadius));
                    m_vDestPos = ai.RotateVector(m_vDestPos, ai.RandomFloat(360), Vector.Up);
                    m_vDestPos.Add(mesh.m_GuardedObject.GetPositionVectorVolatile());
                    ai.AddTask(new TaskGotoPos(m_vDestPos, _MAX_SEARCH_TIME));
                }
                ai.AddTask(new TaskWait(ai.RandomFloat(0.5f, 2.f)));
            }
        }
        else
            ai.AddTask(new TaskWait(ai.RandomFloat(0.5f, 2.f)));

        super.PrepareScenario(ai);
	}
	
    public String toString()
    {
        return "LFE";
    }
    
    // pt:. -------------   TESTY -----------  
    int GetID()
    {
        return _ID;   
    }        
}