public class TaskGoToObjectZigZag extends Task
{
    /** dzielnik odleglosci, by obliczyc czas przebiegniecia z punktu do punktu	 */
    final static float _SPEED = 500;  
    /** minimalna deviacja	 */
    final static float _MIN_DEVIATION = 250;
    /** docelowy obiekt */
    ControlObject m_Object = null;
    
    /** czas trwaia zadania */
    float m_fEndTime = 0.0f;

    /** aktualny sumaryczny czas calego zadania 	 */
    float m_fEntireTime = 0.0f;
    /** czas ostatniego ustalenie aktualnej docelowej pozycji	 */
    float m_fLastUpdateTime = 0.0f;
    
    /** minimalna odleglosc, na jaka nalezy podejsc do obiektu */
    float m_fMinDist = 0.0f;
    
    /** czas, co jaki updateuje sie pozycja obiektu */	
    final float _UPDATE_TIME = 1.f;
    
    /**   maksymalne odchylenie od toru */
    float m_fMaxDeviation = 0.0f;
    /** jakimi odcinkami ma sie poruszac do przodu     */
    float m_fProgress = 0.0f;
    
    /** aktualna pozycja do ktorej dazy     */
    Vector m_vCurrPos = null;
    
    
    public TaskGoToObjectZigZag()
    {
        //SG
    }
    
    public TaskGoToObjectZigZag(ControlObject cObject, float fEndTime,
                                float fMinDist, float fMaxDeviation, float fProgress)
    {
        m_Object = cObject;
        m_fEndTime = fEndTime;
        m_fMinDist = fMinDist;    
        m_fMaxDeviation = fMaxDeviation;        
        m_fProgress = fProgress;
        SetUpdating(_UPDATE_TIME);
    }
    
    /** wykonuje zadanie     */
    public void Perform()
    {
        // oszacowanie czasu dobiegniecia
        float fDist = m_cAI.GetMesh().GetDistanceTo(m_Object);		
        
        if (m_fEndTime <= 0f)
            m_fEndTime = (fDist / _SPEED) * 2f;
        
        super.Perform();
#if AI_DEBUG
        DebugUtils.CrashLog(m_cAI.m_MeshAI, "\n START ZigZag "+m_cAI.GetTime()+"\n");		
#endif
        m_cAI.StartTaskTime(m_fEndTime);
        UpdateNextCurPos();
        Update();
        m_fEntireTime = 0;
    }
    
    public void Cancel(boolean bFast)
    {
        m_cAI.GetMesh().StopGoTo();
#if AI_DEBUG
        DebugUtils.CrashLog(m_cAI.m_MeshAI, "\n CANCEL ZigZag "+m_cAI.GetTime()+"\n");
#endif
        super.Cancel(bFast);
    }    
    
    /** obsluguje komunikat wyslany z AIControllera     */
    public void ServeNotice(int nNotice)
    {
        super.ServeNotice(nNotice);
        switch(nNotice)
        {
        case AIEvents._NOTICE_DEST_POS:
            m_cAI.ClearTaskTime();						
            m_fEntireTime += m_cAI.GetTime() - m_fLastUpdateTime;	
            if(m_fEntireTime>=m_fEndTime) // jesli minal czas zadania
                m_cAI.Notify(AIEvents._NOTICE_END_TASK);
            else
                UpdateNextCurPos();			
            break;
        case AIEvents._NOTICE_TASK_TIME_PASSED:
            m_fEntireTime += m_cAI.GetTime() - m_fLastUpdateTime;
            if(m_fEntireTime>=m_fEndTime) // jesli minal czas zadania
            {
                m_cAI.GetMesh().StopGoTo();
                m_cAI.Notify(AIEvents._NOTICE_END_TASK);
            }
            else
                UpdateNextCurPos();
            break;            
        }
    }	 
    
    public void Update()
    {
        if (m_Object != null)
        {
            Vector vPos = m_Object.GetPosition();
            if (vPos == null)
                Cancel(false);
            else
                vPos = m_cAI.GetMesh().GetInFormationPosition(vPos, null);

            
            boolean bMove = m_cAI.GetMesh().m_Motion.IsMovingLinear();
            
            if(m_Object.GetDistanceTo(m_cAI.GetMesh()) > m_fMinDist)
            {
                if(!bMove)
                    m_cAI.GetMesh().GoToPosition(m_vCurrPos);  
            }
            else
            {
                if(bMove)
                    m_cAI.GetMesh().StopGoTo();
            }
        }
    }    
    /** ustalenie nastepnej docelowej  pozycji  */
    void UpdateNextCurPos()
    {
        if (m_Object == null)
            return;
        Vector vObjectPos = m_Object.GetPosition();
        if (vObjectPos == null)
            return;
        else
            vObjectPos = m_cAI.GetMesh().GetInFormationPosition(vObjectPos, null);

        float fDistance = m_cAI.GetMesh().GetDistanceTo(vObjectPos);
        Vector vDelta = new Vector(vObjectPos);
        vDelta.Sub(m_cAI.GetMesh().GetPositionVectorVolatile());
        vDelta.fY = 0.0f;
        vDelta = m_cAI.RotateVector( vDelta, 90.0f, Vector.Up );
        vDelta.Normalize();
        // mamy wektor w poprzek ruchu, 
        
        // teraz policzmy losowy rozrzut
        // bedzie to wartosc z przedzialu -fMaxDelta/2 .. +fMaxDelta/2 
        
        float fMaxDelta = fDistance * m_fMaxDeviation;
        float fDelta    = fMaxDelta * (GameObject.RandomFloat(1.0f)-0.5f);
        if(fDelta<0 && fDelta>-_MIN_DEVIATION)
            fDelta = -_MIN_DEVIATION;
        if(fDelta>=0 && fDelta<_MIN_DEVIATION)
            fDelta = _MIN_DEVIATION;
        
        vDelta.Mul( fDelta );
        
        // nalozmy ten rozrzut na cel
        Vector vDist = new Vector(vObjectPos);
        vDist.Sub(m_cAI.GetMesh().GetPositionVectorVolatile());
        
        vDist.Normalize();
        vDist.Mul(m_fProgress * fDistance);
        
        vDist.Add(vDelta);    
        float fPartDistance = vDist.Len(); // odleglosc od nastepnego punktu
        m_vCurrPos = m_cAI.GetMesh().GetPosition();;
        m_vCurrPos.Add(vDist);
        
        
        if(m_Object.GetDistanceTo(m_cAI.GetMesh()) > m_fMinDist)		
            m_cAI.GetMesh().GoToPosition(m_vCurrPos);  
        
        float fPartTime = (fPartDistance / _SPEED)*1.3f;
        m_cAI.StartTaskTime(fPartTime);
        m_fLastUpdateTime = m_cAI.GetTime();
    }

    public String toString()
    {
        return "GoToObjectZigZag("+m_Object.GetName()+","+m_fEndTime+","+m_fMinDist+")";
    }
    
    //*********************************************************
    //                      SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[TaskGoToObjectZigZag.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_TASK_GO_TO_OBJECT_ZIG_ZAG)
        {
            super.SGLoadChunk(cFC);
            m_Object = (ControlObject)cFC.LoadObjectRef();
            m_fEndTime = cFC.LoadFloat();
            m_fEntireTime = cFC.LoadFloat();
            m_fLastUpdateTime =cFC.LoadFloat();
            m_fMinDist = cFC.LoadFloat();
            m_fMaxDeviation = cFC.LoadFloat();
            m_fProgress = cFC.LoadFloat();
            m_vCurrPos = cFC.LoadVector();
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[TaskGoToObjectZigZag.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_TASK_GO_TO_OBJECT_ZIG_ZAG, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveObjectRef(m_Object);
        cFC.SaveFloat(m_fEndTime);
        cFC.SaveFloat(m_fEntireTime);
        cFC.SaveFloat(m_fLastUpdateTime);
        cFC.SaveFloat(m_fMinDist);
        cFC.SaveFloat(m_fMaxDeviation);
        cFC.SaveFloat(m_fProgress);
        cFC.SaveVector(m_vCurrPos);

        cFC.delete(); 
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[TaskGoToObjectZigZag.SGUpdate]\n");
        super.SGUpdate();
        //
    }

}
