public class TaskLean extends Task
{
    public final static int _LEAN_LEFT  = 0;
    public final static int _LEAN_RIGHT = 1;
        
    
    /** Punkt, na jaki ma patrzec AI po wychyleniu sie */
    protected Vector m_vPointToLookAt = new Vector();
    /** Czas, przez jaki AI ma pozostawac wychylone */
    protected float m_fLeaningTime;
    
    /** Kierunek, w jakim ma sie wychylic AI (_LEAN_LEFT | _LEAN_RIGHT) */
    protected int m_nLeanDir;
    /** Znacznik, ze nalezy sprawdzac, kiedy konczymy strzelac (serie) */
    protected boolean m_bStopFireCheck;
    /** Znacznik, ze AI task moze wlaczac/wylaczas strzelania */
    protected boolean m_bFireControlAllowed;
    
    /** Zapamietana pozycja, z jakiej startuje AI */
    protected Vector m_vStartPos = new Vector();
    /** Zapamietany kierunek, z jakiego startuje AI */
    protected Vector m_vStartDir = new Vector();
    /** Odleglosc, na jaka ma sie wychylic AI */
    protected float m_fLeanDist = 100.f;
    /** Punkt docelowy wychylenia */
    protected Vector m_vDestLeanPos = new Vector();
    /** docelowy kierunek, do jakiego ma sie obrocic AI */
    protected Vector m_vDestDir = new Vector();
    /** posredni kierunek, przez jaki ma przejsc AI aby dojsc do m_vDestDir */
    protected Vector m_vIntermediateDir = new Vector();
    /** kat o jaki ma sie obrocic AI, zeby osiagnac docelowy kierunek */
    protected float m_fTurnAngle;
    
    
    /** Znacznik, ze bedzie to wychylenie z obrotem o 180[deg] */
    protected boolean m_bBackLean;
    
    /** Tablica animacji dla wychylania sie 
     *  Kolejne wiersze zawieraja animacje dla wychylania sie w:
     *    lewo, prawo, lewo_tyl, prawo_tyl
     *  Pozycje w wierszu to animacje: wychyl sie, wroc
     */
    protected static int m_aLeanAnims[][] = {
                 {AIAnimationsHuman.A_LEAN_RIGHT, AIAnimationsHuman.A_LEAN_RIGHT_RET},
                 {AIAnimationsHuman.A_LEAN_LEFT, AIAnimationsHuman.A_LEAN_LEFT_RET},
                 {AIAnimationsHuman.A_BACK_LEAN_LEFT, AIAnimationsHuman.A_BACK_LEAN_LEFT_RET},
                 {AIAnimationsHuman.A_BACK_LEAN_RIGHT, AIAnimationsHuman.A_BACK_LEAN_RIGHT_RET},
    };
    /** Aktualnie wybrane animacje do leana */
    protected static int m_aCurrLeanAnims[] = null;
    
    
    /** Aktualnie wykonywana faza wychylania sie (_PHASE_*) */
    protected int m_nPhase;
    
    protected final static int _PHASE_LEAN_START    = 0;
    protected final static int _PHASE_LEAN_START_2  = 1;
    protected final static int _PHASE_LEAN_HOLD     = 2;
    protected final static int _PHASE_LEAN_BACK     = 3;
    protected final static int _PHASE_LEAN_BACK_2   = 4;
    protected final static int _PHASE_LEAN_END      = 5;
    
    
    /////////////////////////////////////////////////////////////////////////////
    public TaskLean()
    {
        //SG
    }
    
    /** Ustawienie parametrow taska
     * @param nLeanDir kierunek wychylenia sie (_LEAN_LEFT | _LEAN_RIGHT)
     * @param vPointToLookAt punkt, na ktory nalezy patrzec po wychyleniu
     * @param fMinLeanTime minimalny czas pozostawania w wychyleniu
     * @param fMaxLeanTime maksymalny dopuszczalny czas pozostawania w wychyleniu
     * @param bStopFireCheck znacznik, ze po zakonczeniu strzelania wychylenie ma sie zakonczyc
     */
    public TaskLean(int nLeanDir, Vector vPointToLookAt,
                    float fLeanTime,
                    boolean bStopFireCheck, boolean bFireControlAllowed)
    {
        m_nLeanDir = nLeanDir;
        m_vPointToLookAt.Set(vPointToLookAt);
        m_bStopFireCheck = bStopFireCheck;
        m_bFireControlAllowed = bFireControlAllowed;

        m_fLeaningTime = fLeanTime;
    }
    
    /** wykonuje zadanie     */
    public void Perform()
    {
        super.Perform();
        
        //Task z animacja - zawolaj najpierw update motion, zeby zawolane z opoznieniem czegos nie popsulo
        m_cAI.GetMesh().CallOrRemoveUpdateMotion();
        m_cAI.GetMesh().GetPosition(m_vStartPos);
        m_cAI.GetMesh().GetForwardVector(m_vStartDir);
        m_vStartDir.Normalize();
        
        // oblicz przyblizony kierunek do docelowego punktu do patrzenia
        m_vDestDir.Set(m_vPointToLookAt);
        m_vDestDir.Sub(m_vStartPos);
        m_vDestDir.Normalize();
        
        //sprawdz, czy bedzie to obrot w tyl...
        m_bBackLean = (m_vDestDir.Dot(m_vStartDir) < 0);

        // ustal zestaw animacji
        int nAnimIdx = m_nLeanDir + ((m_bBackLean) ? 2 : 0);
        m_aCurrLeanAnims = m_aLeanAnims[nAnimIdx];
        
        // oblicz punkt docelowy przesuniecia
        m_cAI.GetMesh().GetRightVector(m_vDestLeanPos);
        m_vDestLeanPos.Normalize();

        if (m_bBackLean ^ m_nLeanDir == _LEAN_LEFT)
            m_vDestLeanPos.Mul(-m_fLeanDist);
        else
            m_vDestLeanPos.Mul(m_fLeanDist);

        m_vDestLeanPos.Add(m_vStartPos);

        // oblicz kierunek do docelowego punktu do patrzenia
        m_vDestDir.Set(m_vPointToLookAt);
        m_vDestDir.Sub(m_vDestLeanPos);
        m_vDestDir.Normalize();
        
        float fDot = MathUtils.crop(m_vDestDir.Dot(m_vStartDir), -1, 1);
        
        m_fTurnAngle = (float)(180 / Math.PI * Math.acos(fDot));
        
        if (m_bBackLean)
        {
            m_cAI.GetMesh().GetRightVector(m_vIntermediateDir);
            if (m_nLeanDir != _LEAN_LEFT)
                m_vIntermediateDir.Negate();
        }
        else
        {
            m_vIntermediateDir.Set(m_vDestDir); 
            m_vIntermediateDir.Add(m_vStartDir);
        }

        m_nPhase = _PHASE_LEAN_START;
        NextLeanPhase(m_nPhase++);
    }	
    
    
    public void Cancel(boolean bFast)
    {
//        GameObject.CrashLog("### CANCEL ###\n");
        NewAI mesh = m_cAI.GetMesh();
        mesh.SetMaxAngSpeed(mesh.m_aMotionAngularVelocity[mesh.m_Motion.GetSpeed()]);
        CancelLeanPhase(m_nPhase-1, bFast);
        if (m_bFireControlAllowed)
            m_cAI.ResetCanFire();
        super.Cancel(bFast);
    }

    /** Wykonuje nastepna faze wychylenia
     *  @return false, gdy zakonczono ostatnia faze
     */
    boolean NextLeanPhase(int nPhase)
    {
        NewAI cMesh = m_cAI.GetMesh();
        switch (nPhase)
        {
        case _PHASE_LEAN_START:
            if (m_bFireControlAllowed)
                m_cAI.ResetCanFire();
            cMesh.PlayAIAnim(m_aCurrLeanAnims[0]);
            float fTime = m_cAI.GetMesh().GetAIAnimLength(m_aCurrLeanAnims[0]) / 2;
            cMesh.InterpolatePositionTo(m_vDestLeanPos, null, m_fLeanDist / fTime, true);
            cMesh.SetMaxAngSpeed(m_fTurnAngle / fTime);
            cMesh.SetFacingDestDir(m_vIntermediateDir);            
            m_cAI.StartTaskTime(fTime);
            break;
            
        case _PHASE_LEAN_START_2:
            fTime = m_cAI.GetMesh().GetAIAnimLength(m_aCurrLeanAnims[0]) / 2;
            cMesh.SetMaxAngSpeed(m_fTurnAngle / fTime);
            cMesh.SetFacingDestDir(m_vDestDir);
            m_cAI.StartTaskTime(fTime);
            break;
            
        case _PHASE_LEAN_HOLD:
            if (m_bBackLean)
                cMesh.m_Motion.SetSpeed(MotionState.SPEED_WALK);
            else
                cMesh.m_Motion.SetSpeed(MotionState.SPEED_DUCK);
            if (m_bFireControlAllowed)
                m_cAI.SetCanFire(true);
            cMesh.UpdateMotion();
            m_cAI.StartTaskTime(m_fLeaningTime);
            break;
            
        case _PHASE_LEAN_BACK:
            if (m_bFireControlAllowed)
                m_cAI.ResetCanFire();
            cMesh.PlayAIAnim(m_aCurrLeanAnims[1]);
            fTime = m_cAI.GetMesh().GetAIAnimLength(m_aCurrLeanAnims[1]);
            cMesh.InterpolatePositionTo(m_vStartPos, null, m_fLeanDist / fTime, true);
            cMesh.SetMaxAngSpeed(m_fTurnAngle / fTime);
            cMesh.SetFacingDestDir(m_vIntermediateDir);            
            m_cAI.StartTaskTime(fTime);
            break;
            
        case _PHASE_LEAN_BACK_2:
            fTime = m_cAI.GetMesh().GetAIAnimLength(m_aCurrLeanAnims[1]);

            cMesh.SetMaxAngSpeed(m_fTurnAngle / fTime);
            cMesh.SetFacingDestDir(m_vStartDir);
            m_cAI.StartTaskTime(fTime);
            break;
            
        case _PHASE_LEAN_END:
            cMesh.m_Motion.SetSpeed(MotionState.SPEED_WALK);
            cMesh.UpdateMotion();
            return false;
        }
        return true;
    }

    
    void CancelLeanPhase(int nPhase, boolean bFast)
    {
        NewAI cMesh = m_cAI.GetMesh();

        switch (nPhase)
        {
        case _PHASE_LEAN_START:
            if (!bFast)
                cMesh.SetFacingDestDir(m_vDestDir);
        case _PHASE_LEAN_START_2:
            cMesh.StopInterpolatePosition(false);
            if (m_bBackLean)
                cMesh.m_Motion.SetSpeed(MotionState.SPEED_WALK);
            else
                cMesh.m_Motion.SetSpeed(MotionState.SPEED_DUCK);
            cMesh.UpdateMotion();
            break;
            
        case _PHASE_LEAN_HOLD:
            break;
            
        case _PHASE_LEAN_BACK:
            if (!bFast)
                cMesh.SetFacingDestDir(m_vStartDir);
        case _PHASE_LEAN_BACK_2:
            cMesh.StopInterpolatePosition(false);
            cMesh.m_Motion.SetSpeed(MotionState.SPEED_WALK);
            cMesh.UpdateMotion();
            break;
            
        case _PHASE_LEAN_END:
            break;
        }
    }
    
    
    /** obsluguje komunikat wyslany z AIControllera     */
    public void ServeNotice(int nNotice)
    {
        super.ServeNotice(nNotice);
        switch(nNotice)
        {
        case AIEvents._NOTICE_WEAPON_STATE_CHANGED:
            if (!m_bStopFireCheck || 
                m_nPhase - 1 != _PHASE_LEAN_HOLD || 
                m_cAI.GetActiveWeapon().nCurrentMode != Weapon.MODE_IDLE)
                break; // jesli nie przestal strzelac - zignoruj
            m_cAI.ClearTaskTime(); 
            // kontynuacja _NOTICE_WEAPON_STATE_CHANGED
        case AIEvents._NOTICE_TASK_TIME_PASSED:
            if (!NextLeanPhase(m_nPhase++))
                m_cAI.Notify(AIEvents._NOTICE_END_TASK);
            break;
        }
    }	    	
    
    
    public String toString()
    {
        return "TaskLean(" + ((m_nLeanDir == _LEAN_LEFT) ? "_LEAN_LEFT" : "_LEAN_RIGHT") +
               "," + m_vPointToLookAt + "," + m_fLeaningTime + "," + 
               m_bFireControlAllowed + "," + m_bStopFireCheck + ")";
    }

    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[TaskLean.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_TASK_LEAN)
        {
            super.SGLoadChunk(cFC);
            m_vPointToLookAt = cFC.LoadVector();
            m_fLeaningTime = cFC.LoadFloat();
            m_nLeanDir = cFC.LoadInt();
            m_bStopFireCheck = cFC.LoadBool();
            m_bFireControlAllowed = cFC.LoadBool();
            m_vStartPos = cFC.LoadVector();
            m_vStartDir = cFC.LoadVector();
            m_fLeanDist = cFC.LoadFloat();
            m_vDestLeanPos = cFC.LoadVector();
            m_vDestDir = cFC.LoadVector();
            m_vIntermediateDir = cFC.LoadVector();
            m_fTurnAngle = cFC.LoadFloat();
            m_bBackLean = cFC.LoadBool();
            m_nPhase = cFC.LoadInt();
      }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[TaskLean.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_TASK_LEAN, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveVector(m_vPointToLookAt);
        cFC.SaveFloat(m_fLeaningTime);
        cFC.SaveInt(m_nLeanDir);
        cFC.SaveBool(m_bStopFireCheck);
        cFC.SaveBool(m_bFireControlAllowed);
        cFC.SaveVector(m_vStartPos);
        cFC.SaveVector(m_vStartDir);
        cFC.SaveFloat(m_fLeanDist);
        cFC.SaveVector(m_vDestLeanPos);
        cFC.SaveVector(m_vDestDir);
        cFC.SaveVector(m_vIntermediateDir);
        cFC.SaveFloat(m_fTurnAngle);
        cFC.SaveBool(m_bBackLean);
        cFC.SaveInt(m_nPhase);

        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[TaskLean.SGUpdate]\n");
        super.SGUpdate();
        //
    }
}
