public class TaskThrowGrenade extends Task
{
    /** W jakiej czesci czasu animacji nalezy 'pojawic' granata */
    public final static float _SHOW_PHASE_TIME_PART   = 0.2f;
    /** W jakiej czesci czasu animacji nalezy wyrzucic granata */
    public final static float _THROW_PHASE_TIME_PART  = 0.7f;
    
    Vector m_vDest = null;
    float m_fMaxHeight = 0;
    Grenade m_cGrenade;
    
    boolean m_bGrenadeVisible = false;
    boolean m_bGrenadeThrown  = false;
    
    float m_fShowThrowInterval;
    
    /** czy task byl juz przerwany - bez tego granat powodowal noisemsg,
     * canceltask, start, noisemsg... */
    boolean m_bAlreadyCancelled = false;
    
    
    
    public TaskThrowGrenade()
    {
        //SG
    }
    
    public TaskThrowGrenade(Vector vDest, float fMaxHeight)
    {
        m_vDest = new Vector(vDest);
        m_fMaxHeight = fMaxHeight;
    }
    
    /** wykonuje zadanie */
    public void Perform()
    {
        super.Perform();

        m_cAI.GetMesh().CallOrRemoveUpdateMotion();
        // tworzymy granat
        m_cGrenade = (Grenade) m_cAI.CreateObject("Grenade");
        m_cGrenade.EnableRendering(false);
        m_cGrenade.SetPhysicsActive(false);
        m_cGrenade.ClearFlag(EFlags._COLLISION);
        m_cGrenade.Init(null);
        m_cGrenade.Owner = m_cAI.GetMesh();
        m_cGrenade.ChangeConflictSide(m_cAI.GetMesh().GetConflictSide());
        
        NewAIHuman cThrower = (NewAIHuman)m_cAI.GetMesh();

        cThrower.AttachChild(m_cGrenade, cThrower.m_nLeftHandID);
        cThrower.CallOrRemoveUpdateMotion();
        cThrower.PlayAIAnim(AIAnimationsHuman.A_THROW_GRENADE);
        float fThrowAnimTime = cThrower.GetAIAnimLength(AIAnimationsHuman.A_THROW_GRENADE);
        
        float fStartShowInterval = fThrowAnimTime * _SHOW_PHASE_TIME_PART;
        m_fShowThrowInterval = fThrowAnimTime * _THROW_PHASE_TIME_PART - fStartShowInterval;
        
        m_cAI.StartTaskTime(fStartShowInterval);
    }
    
    public void Cancel(boolean bFast)
    {
        if (m_bAlreadyCancelled)
            return;
        m_bAlreadyCancelled = true;
        
        m_cGrenade.DetachFromParentAndKeepWorldPos();
        
        if (m_bGrenadeVisible)
        {
            if (!m_bGrenadeThrown)
            {
                m_cGrenade.SetPhysicsActive(true);
                m_cGrenade.Start(0.1f, Vector.Down); //upusc granata
            }
        }
        else
        {
            m_cGrenade.DeactivateOnModuleAlways(); //usun granata
            m_cGrenade.delete();
            m_cGrenade = null;
        }
        
        super.Cancel(bFast);
    }
    
    public void ServeNotice(int nNotice)
    {
        super.ServeNotice(nNotice);
        
        switch(nNotice)
        {
        case AIEvents._NOTICE_TASK_TIME_PASSED:
            if (!m_bGrenadeVisible)
            {
                m_bGrenadeVisible = true;
                m_cGrenade.EnableRendering(true);
                m_cAI.StartTaskTime(m_fShowThrowInterval);
            }
            else
            {
                NewAIHuman cThrower = (NewAIHuman)m_cAI.GetMesh();
                
                Vector vStartPos = m_cGrenade.GetPosition();
                m_cGrenade.DetachFromParentAndKeepWorldPos();
                m_cGrenade.SetPhysicsActive(true);
                
                ((AIControllerHuman) m_cAI).ThrowGrenade(m_cGrenade, vStartPos, m_vDest, m_fMaxHeight);
                m_bGrenadeThrown = true;
            }
            break;
        case AIEvents._NOTICE_ANIM_STOPPED:
            Cancel(false);
            // cont...
        case AIEvents._NOTICE_ANIM_END:
            m_cAI.Notify(AIEvents._NOTICE_END_TASK);
            break;
            
        }
    }
    
    public String toString()
    {
        return "ThrowGrenade";
    }
    
    //*********************************************************
    //                        SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[TaskThrowGrenade.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_TASK_THROW_GRENADE)
        {
            super.SGLoadChunk(cFC);
            m_vDest = cFC.LoadVector();
            m_fMaxHeight = cFC.LoadFloat();
            m_cGrenade = (Grenade)cFC.LoadObjectRef();
            m_bGrenadeVisible = cFC.LoadBool();
            m_bGrenadeThrown = cFC.LoadBool();
            m_fShowThrowInterval = cFC.LoadFloat();
            m_bAlreadyCancelled = cFC.LoadBool();
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[TaskThrowGrenade.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_TASK_THROW_GRENADE, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveVector(m_vDest);
        cFC.SaveFloat(m_fMaxHeight);
        cFC.SaveObjectRef(m_cGrenade);
        cFC.SaveBool(m_bGrenadeVisible);
        cFC.SaveBool(m_bGrenadeThrown);
        cFC.SaveFloat(m_fShowThrowInterval);
        cFC.SaveBool(m_bAlreadyCancelled);

        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[TaskThrowGrenade.SGUpdate]\n");
        super.SGUpdate();
        //
    }

}
