/** klasa latajacego pojazdu - pojazd potrafi wystartowac, wyladowac,
 * doleciec do punktu i przeleciec przez kilka punktow (wychylajac sie podczas lotu)
 */
public class FlyingVehicle extends ChromeActor implements ipar_MovieVehicle
{
	/** zakresy blendowania i wycinania obiektu */
	public float fBlendStart = 30000f;//#\FLYINGVEHICLE;
	public float fVisibilityEnd = 35000f;//#\FLYINGVEHICLE;
    public boolean bGridIndependent = false;//#FLYINGVEHICLE;
	
    /** dzwieki startu i lotu */
	String _START_SOUND_NAME = "Data/Sounds/Vehicles/shuttle start";
	String _FLY_SOUND_NAME = "Data/Sounds/Vehicles/shuttle";
	
    public float fMinSound = 5000f;//#SOUND
    public float fMaxSound = 15000f;//#SOUND
    
	/** fizyka pojazdu */
	FlyingVehiclePhysics m_Physics = null;
	
	/**	numerek dzwieku lotu */
	int m_iFlySound = 0;
	boolean m_bRestoreFlySoundAfterDeactivate = false;

    /** Jak czesto 'podsmazani' sa Actor'zy w czasie ladowania / startowania */
    float m_fActorFryingUpdateTime = 0.5f;
    /** Promien, w ktorym wyciagani sa aktorzy do 'posmazania' (0 - nieaktywne)*/
    float m_fActorFryingDistance = 1200f;
    /** Damage zadawany aktorom przy podpiekaniu ich  */
    float m_fActorFryingDamage = 500f;
    
	/** domyslna klasa fizyki wahadlowca */
    String m_sPhysicsClassName = "FlyingVehiclePhysics";

	/** czy pojazd jest w tej chwili aktywny (czy widac go i czy ma fizyke wlaczona) */
	boolean m_bVehicleActive = false;
    
    static String _VEHICLE_DUST_START = "DustStartFlyingVehicle";
    static String _VEHICLE_DUST_LAND = "DustLandFlyingVehicle";

    boolean m_bStarting = false;
    boolean m_bLanding = false;
    
    FlyingVehicle()
	{
        MeshName = "Data/Meshes/Vehicles/TransportowiecMaly.3DA";
        
        bIsAITarget = true;
	}
	
	public void OnCreate()
	{
		super.OnCreate();
		
		bSG = true;
		
        SetCollType(ECollFlags.OBJCOL_TYPE_PERFACEOBJECT);
        SetCollAction(ECollFlags.OBJCOL_ACTION_SLIDE);
        SetCollHandler(ECollFlags.OBJCOL_HANDLER_NONE);
        SetTraceCollType(ECollTraceFlags._HULL);
        
        SetFlag(EFlags._COLLISION);
        SetFlag(EFlags._BLOCK_TRACE);
        SetFlag(EFlags._BLOCK_MOVE);
		
        SetPhysicsType(EPhysics._NONE);
        SetPhysicsActive(false);

		// tworzymy fizyke pojazdu
		m_Physics = (FlyingVehiclePhysics) CreateObject(m_sPhysicsClassName);
		m_Physics.InitPhysics(this);

		// stan poczatkowy to stan idle
		m_Physics.SetState("IdleState");
		
		SetBlobMapSize(1300f);
		
		// na samym poczatku deaktywujemy pojazd
		DeactivateOnModuleAlways();
	}
	
    public void OnDestroy()
    {
        if (m_Physics != null)
        {
            m_Physics.delete();
            m_Physics = null;
        }
		DetachFlySound();
    }
	
	/** aktywowanie wahadlowca - spowoduje jego pokazanie i aktywowanie fizyki */
	public void ActivateVehicle()
	{
		m_bVehicleActive = true;
		
		SetMeshVisibilityRange(fBlendStart, fVisibilityEnd);

		if (bGridIndependent)
			ActivateOnModuleGridIndependent();
		else
			ActivateOnModule();
		
		if (m_Physics != null)
			m_Physics.ActivatePhysics();
		if (m_bRestoreFlySoundAfterDeactivate)
		{
			m_bRestoreFlySoundAfterDeactivate = false;
			AttachFlySound();
		}
        EnableRendering(true);
	}
	
	/** deaktywowanie wahadlowca - wahadlowiec zostanie ukryty i zdezaktywowana
	 * bedzie jego fizyka */
	public void DeactivateVehicle()
	{
		m_bVehicleActive = false;
		
        if (m_Physics != null)
			m_Physics.DeactivatePhysics();
		if (m_iFlySound != 0)
		{
			DetachFlySound();
			m_bRestoreFlySoundAfterDeactivate = true;
		}
		DeactivateOnModuleAlways();
        
        EnableRendering(false);
	}
	
	/** zostanie zawolane, jak wahadlowiec wystartuje */
	void VehicleStarted(float fTime)
	{
		m_Physics.SetState("IdleState");
		AttachFlySound();
	}
	
	/** zostanie zawolane, jak wahadlowiec wyladuje */
	void VehicleLanded(float fTime)
	{
		m_Physics.SetState("IdleState");
		DetachFlySound();
	}

	/** zostanie wywolane, jak wahadlowiec doleci do koncowego punktu */
	void OnDestinationPos()
	{
		m_Physics.SetState("IdleState");
	}

	/** zostanie wywolane, jak wahadlowiec doleci do nastepnego z punktow patrolowych */
	void OnDestinationPos(int iNdx)
	{
	}
	
    public void Die() 
    {
        if (bDead)
            return;
        
    	DetachFlySound();
        DetachAllSounds();
            
        if (m_Physics != null)
			m_Physics.DeactivatePhysics();
        
        super.Die();
    }
	
    public boolean IsActive(ControlObject Activator)
    {
        return false;
    }
	
	void SetPreciseMoving(boolean bPrecise)
	{
		m_Physics.m_bPreciseMoving = bPrecise;
	}
	
    public void PlayStartSound(float fTimeToIgnoreInSinglePlayer)
    {
        PlayStartSound();
    }
	
	public void PlayStartSound()
	{
		if (_START_SOUND_NAME != null)
			AttachOneShotSound(_START_SOUND_NAME, fMinSound, fMaxSound);
	}
	
	void AttachFlySound()
	{
		if ( (m_iFlySound == 0) && (_FLY_SOUND_NAME != null) )
			m_iFlySound = AttachSound(_FLY_SOUND_NAME, fMinSound, fMaxSound);
	}
	
	void DetachFlySound()
	{
		if (m_iFlySound != 0)
		{
			DetachSound(m_iFlySound);
			m_iFlySound = 0;
		}
	}
	
	void ActivatePhysics()
	{
        if (m_Physics != null)
			m_Physics.ActivatePhysics();
	}
	
	void DeactivatePhysics()
	{
        if (m_Physics != null)
			m_Physics.DeactivatePhysics();
	}
    
    /** podsmazanie aktorow, ktorzy znalezli sie pod silnikami */
    void FryActors()
    {
        if (m_fActorFryingDistance > 0)
        {
            GameObject aObjs[] = GetObjectsInRadiusOfFlagsFromClass(GetPosition(),
                                                                    m_fActorFryingDistance,
                                                                    EFlags._AI_TARGET,
                                                                    "ChromeActor");
            if (aObjs != null)
			{
                for (int i = 0; i < aObjs.length; i++)
                {
                    Actor cActor = (Actor) aObjs[i];
                    if (cActor.fHealth > 0)
                    {
                        ControlObject parent = cActor;
                        boolean bFound = false;
                        do
                        {
                            parent = parent.GetParent();
                            if (parent == this)
                            {
                                bFound = true;
                                break;
                            }
                        } while (parent != null);
                            
                        if (!bFound)
                        {
                            //CrashLog("Fry: " + cActor + "\n");
                            cActor.TakeDamage(m_fActorFryingDamage, m_fActorFryingDamage, this, null, null,
						    				  EBones._NON_BONE, EnumDamageType._HEAT);
                        }
                    }
                }
			}
        }
    }
    
    public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
	{
        // powiadom przeciwnika, ze trafil
        if (Assassin instanceof ChromeActor)
            ((ChromeActor)Assassin).NotifyHit(this, nDamageType);
        
        if (bDead)
            return;
        
        Damage(fDamage, nDamageType);
    }
    
    void PlayFXDustStart()
    {
    }
    
    void PlayFXDustLand()
    {
    }
	
    public void SetVehicleForMovie(boolean bEnable, boolean bPlayEngineStartStopSound)
    {
        if (bEnable)
        {
            DeactivatePhysics();
            if (bPlayEngineStartStopSound)
                PlayStartSound();
            //AttachFlySound();
        }
        else
        {
            //ActivatePhysics();
            //DetachFlySound();
        }
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
	int m_iFlySoundSG = 0;
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[FlyingVehicle.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_FLYING_VEHICLE)
        {	
			super.SGLoadChunk(cFC);
			
			// fizyka pojazdu 
			m_Physics = (FlyingVehiclePhysics)cFC.LoadObject(m_Physics,this);
	
			m_bRestoreFlySoundAfterDeactivate = cFC.LoadBool();
 
			// czy pojazd jest w tej chwili aktywny (czy widac go i czy ma fizyke wlaczona) 
			m_bVehicleActive = cFC.LoadBool();
		
			m_bStarting = cFC.LoadBool();
			m_bLanding = cFC.LoadBool();			
			m_iFlySoundSG = cFC.LoadInt();
        }
		
		cFC.delete();
    }
	    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[FlyingVehicle.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_FLYING_VEHICLE, cFCParent);
		
		super.SGSaveChunk(cFC);
			
		// fizyka pojazdu 
		cFC.SaveObject(m_Physics);
	
		cFC.SaveBool(m_bRestoreFlySoundAfterDeactivate);
 
		// czy pojazd jest w tej chwili aktywny (czy widac go i czy ma fizyke wlaczona) 
		cFC.SaveBool(m_bVehicleActive);
		
		cFC.SaveBool(m_bStarting);
		cFC.SaveBool(m_bLanding);
		
		cFC.SaveInt(m_iFlySound);
				
        cFC.delete();
    }	    				
	
    /** Metoda wywolujaca odpowiednie akcje po odczycie danych
     */		
	public void SGUpdate()
	{
		FileChunk.Log("[FlyingVehicle.SGUpdate]\n");
		
		DetachFlySound();
		
		if (m_iFlySoundSG != 0)
			AttachFlySound();
		
		super.SGUpdate();			
		
		if (m_Physics != null)
			m_Physics.SGUpdate();			
		
		/*
		if (m_bVehicleActive)
			ActivateVehicle();
		else
			DeactivateVehicle();
		*/
	}			
    
    public void OnAudioReinitialize()
    {
        if (m_iFlySound != 0)
        {
            m_iFlySound = 0;
            AttachFlySound();
        }		
    }
}
