public class Huey extends FlyingTransporter
{
    int m_iWingLeft = -1;
    String sWingLeft = "Huey_skrzydloL";
    int m_iWingRight = -1;
    String sWingRight = "Huey_skrzydloP";
    
    String sShowWingsAnim = "wysuwa_skrzydla";
    String sHideWingsAnim = "chowa_skrzydla";
    
    String [] m_aGlowDummies = new String[] { "Glow01", "Glow02", "Glow03", "Glow04", "Glow05", "Glow06"};
    int [] m_aGlowDummiesIDs = null;
    int [] m_aGlowFXsIDs = null;
    int METHOD_UpdateGlowFXs = 0;
    public float _GLOW_FX_UPDATE_TIME = 60f;
    public String _GLOW_FX_NAME = "FXHueyGlow";
    int METHOD_TurnGlowsOn = 0;
    int METHOD_TurnGlowsOff = 0;

    String m_sWreckMeshName = null;
    MeshObject m_Wreck = null;
    boolean m_bWreckVisible = false;
    
    public Huey()
    {    
        fHealth = fMaxHealth = 2000;
        fDamageThreshold = 100;
        m_sWreckMeshName = "Data/Meshes/Vehicles/HueyWreck.3da";

        fBlowUpShakeStrength = 1500;
        fBlowUpShakeDistMin = 1000;
        fBlowUpShakeDistMax = 4000;
        
    	_VEHICLE_SMOKE = "SmokeFlyingVehicleSmall";
    }
    
	public void OnCreate()
	{
        SetGlows();
        
		super.OnCreate();

		SetBlobMapSize(800f);
        SetWings();
        CreateWreck();
	}

    public void Die() 
    {
        if (bDead)
            return;
        
        TurnGlowsOff();
        
        if (m_bLanded)
            ShowWreck();
        
        super.Die();
    }
    
    public void ActivateVehicle()
    {
        super.ActivateVehicle();

        DisableCallMethod(METHOD_TurnGlowsOn);
        DisableCallMethod(METHOD_TurnGlowsOff);
        CallMethodIn(METHOD_TurnGlowsOn, 0f);
        UnfreezeCallMethod(METHOD_TurnGlowsOn);
    }

    public void DeactivateVehicle()
    {
        super.DeactivateVehicle();
        
        DisableCallMethod(METHOD_TurnGlowsOn);
        DisableCallMethod(METHOD_TurnGlowsOff);
        CallMethodIn(METHOD_TurnGlowsOff, 0f);
        UnfreezeCallMethod(METHOD_TurnGlowsOff);
    }
    
    void SetDoors()
    {
		m_iDoorElem = -1;
    }
    
    void SetEngines()
    {
		m_iEngine1Elem = -1;
		m_iEngine2Elem = -1;
        
		for (int i = 0; i < _ENGINE_EXHAUST_NUM; i++)
			m_EnginesExhaust[i] = -1;
		m_EnginesExhaust[0] = GetElementID("Engine");

        sEnginesExhaust = "FXFlyingEngine";
    }
    
    void SetGlows()
    {
        m_aGlowDummiesIDs = new int[m_aGlowDummies.length];
        for (int i = 0; i < m_aGlowDummies.length; i++)
            m_aGlowDummiesIDs[i] = GetElementID(m_aGlowDummies[i]);
        
        m_aGlowFXsIDs = new int[m_aGlowDummiesIDs.length];
        for (int i = 0; i < m_aGlowFXsIDs.length; i++)
            m_aGlowFXsIDs[i] = -1;
        METHOD_UpdateGlowFXs = RegisterCallMethod("UpdateGlowFXs");
        METHOD_TurnGlowsOn = RegisterCallMethod("TurnGlowsOn");
        METHOD_TurnGlowsOff = RegisterCallMethod("TurnGlowsOff");
    }
    
    void TurnGlowsOn()
    {
        CallMethodEvery(METHOD_UpdateGlowFXs, _GLOW_FX_UPDATE_TIME);
        UpdateGlowFXs();
    }
    
    void TurnGlowsOff()
    {
        DisableCallMethod(METHOD_UpdateGlowFXs);
        for (int i = 0; i < m_aGlowDummiesIDs.length; i++)
        {
            if (m_aGlowFXsIDs[i] >= 0)
            {
                DeleteFXSequence(m_aGlowFXsIDs[i]);
                m_aGlowFXsIDs[i] = -1;
            }
        }
    }
    
    void UpdateGlowFXs()
    {
        for (int i = 0; i < m_aGlowDummiesIDs.length; i++)
        {
            if (m_aGlowFXsIDs[i] >= 0)
                DeleteFXSequence(m_aGlowFXsIDs[i]);
            m_aGlowFXsIDs[i] = ExecuteFX(this, m_aGlowDummiesIDs[i], _GLOW_FX_NAME, new Vector());
        }
    }
    
    void SetWheels()
    {
        sForwardWheel = "Huey_plozaT";
        sBackwardRightWheel = "Huey_plozaR";
        sBackwardLeftWheel = "Huey_plozaL";
        sHideWheelsAnim = "chowa_podwozie";
        sShowWheelsAnim = "wysuwa_podwozie";
        
        super.SetWheels();
    }
    
    void SetWings()
    {
        m_iWingLeft = GetElementID(sWingLeft);
        m_iWingRight = GetElementID(sWingRight);
    }
    
    void ShowWings()
    {
        if (m_iWingLeft != -1)
            PlayAnim(sWingLeft, sShowWingsAnim, 1);
        if (m_iWingRight != -1)
            PlayAnim(sWingRight, sShowWingsAnim, 1);
    }
    
    void HideWings()
    {
        if (m_iWingLeft != -1)
            PlayAnim(sWingLeft, sHideWingsAnim, 1);
        if (m_iWingRight != -1)
            PlayAnim(sWingRight, sHideWingsAnim, 1);
    }
    
	public void EditorStartFly(mt_StartFlying motionType)
    {
        super.EditorStartFly(motionType);

        ShowWings();
    }
    
	void VehicleLanded(float fTime)
	{
        super.VehicleLanded(fTime);
        
        HideWings();
    }
    
    public void DeleteFlying()
    {
        DeactivateVehicle();
        
        if (m_Physics != null)
		{
            m_Physics.delete();
			m_Physics = null;
		}
        
        DeactivateOnModuleAlways();
        
        TurnGlowsOff();
        
        CallMethodIn("DeleteFXes", 10f);
    }
    
    void CreateWreck()
    {
        if (m_Wreck != null)
            return;
        if (m_sWreckMeshName == null)
            return;
        m_Wreck = (MeshObject) CreateObject("MeshObject");
		m_Wreck.LoadMesh(m_sWreckMeshName);
		m_Wreck.SetFlag(EFlags._BLOCK_MOVE | EFlags._BLOCK_TRACE |
                      EFlags._NO_ANIMATION | EFlags._NO_ANIMATION_REFRESH);
        m_Wreck.ClearFlag(EFlags._COLLISION);
        m_Wreck.ClearFlag(EFlags._BLOCK_PARTICLES);
        m_Wreck.SetCollType(ECollFlags.OBJCOL_TYPE_PERFACEOBJECT);
        m_Wreck.SetCollHandler(ECollFlags.OBJCOL_HANDLER_NONE);
        m_Wreck.SetTraceCollType(ECollTraceFlags._HULL);
		m_Wreck.SetLighting(true);
        m_Wreck.SetFlag(EFlags._CAST_SHADOWS);
		m_Wreck.EnableRendering(false);
    }
    
    void ShowWreck()
    {
        if (m_Wreck == null)
            return;
        
        m_Wreck.ActivateOnModule();
		m_Wreck.SetFlag(EFlags._COLLISION);
        m_Wreck.EnableRendering(true);
        m_Wreck.FromUpForwardPos(Vector.Up, GetForwardVector(), GetPosition());
        m_bWreckVisible = true;
    }
    
    void HideWreck()
    {
        if (m_Wreck == null)
            return;
        
        m_Wreck.DeactivateOnModuleAlways();
		m_Wreck.ClearFlag(EFlags._COLLISION);
        m_Wreck.EnableRendering(false);
        m_bWreckVisible = false;
    }

    public void SetVehicleForMovie(boolean bEnable, boolean bPlayEngineStartStopSound)
    {
        super.SetVehicleForMovie(bEnable, bPlayEngineStartStopSound);
        
        if (bEnable)
        {
            //ShowWings();
        }
        else
        {
            TurnGlowsOff();
           //HideWings();
        }
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[Huey.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_FLYING_VEHICLE)
        {	
			super.SGLoadChunk(cFC);
            
			if (cFC.LoadBool())
				m_Wreck = (MeshObject) cFC.LoadObject(m_Wreck, this);
            m_bWreckVisible = cFC.LoadBool();
        }
		
		cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Huey.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_FLYING_VEHICLE, cFCParent);
		
		super.SGSaveChunk(cFC);

		cFC.SaveBool(m_Wreck != null);
		if (m_Wreck != null)
			cFC.SaveObject(m_Wreck);
        cFC.SaveBool(m_bWreckVisible);
        
        cFC.delete();
    }	    			
    
	public void SGUpdate()
	{
		FileChunk.Log("[Huey.SGSaveChunk]\n");
		
		super.SGUpdate();

		if (m_bWreckVisible)
            ShowWreck();
	}
    
}
