/** Stan ruchu pojazdow
 */
public class VehicleMotionState implements ISaveGame
{
    /** Znacznik przyspieszenia */
    public byte iAcc = 0;

    /** Skrecanie (znacznik czy jest jakas predkosc katowa) */
	public float fTurn = 0f;
    
    VehicleMotionState()
    {
        Stop();
    }

    VehicleMotionState(VehicleMotionState motion)
    {
        iAcc = motion.iAcc;
        fTurn = motion.fTurn;
    }
    
    boolean IsEqual(VehicleMotionState motion)
    {
        return ( (motion.iAcc == iAcc) &&
				 (motion.fTurn == fTurn) );
    }
    
    void Stop()
    {
        iAcc = 0;
		fTurn = 0f;
    }

    boolean IsMoving()
    {
        return ( (fTurn != 0f) || (iAcc != 0) );
    }
    
    boolean IsMovingLinear()
    {
        return (iAcc != 0);
    }

    void Accelerate()
    {
        if (iAcc < 1)
			iAcc++;
    }

    void Decelerate()
    {
        if (iAcc > -1)
			iAcc--;
    }

    void TurnRight()
    {
        if (fTurn < 0f)
			fTurn = 0f;
		else
		if (fTurn < 1f)
			fTurn = 1f;
    }
    
    void TurnLeft()
    {
        if (fTurn > 0f)
			fTurn = 0f;
		else
        if (fTurn > -1f)
			fTurn = -1f;
    }
    
    void SetMoveFromKeysState(boolean bForward,
                              boolean bBackward,
                              boolean bRight,
                              boolean bLeft)
    {
        Stop();
        if ( bForward )  Accelerate();
        if ( bBackward ) Decelerate();
        if ( bRight )    TurnRight();
        if ( bLeft )     TurnLeft();
	}

	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[VehicleMotionState.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_VEHICLE_MOTION_STATE)
        {
			// Znacznik przyspieszenia
			iAcc = (byte)cFC.LoadInt();
			// Skrecanie (znacznik czy jest jakas predkosc katowa)
			fTurn = cFC.LoadFloat();						
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[VehicleMotionState.SGSaveChunk]\n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_VEHICLE_MOTION_STATE, cFCParent);
		
		// Znacznik przyspieszenia
		cFC.SaveInt((int)iAcc);
		// Skrecanie (znacznik czy jest jakas predkosc katowa)
		cFC.SaveFloat(fTurn);
						
        cFC.delete();
    }
	
	public void SGUpdate() 
	{}	
}
